@@ -26,21 +26,14 @@
 package org.broad.igv;
 
 import org.apache.commons.io.FileUtils;
-import org.apache.log4j.LogManager;
-import org.apache.log4j.Logger;
-import org.apache.logging.log4j.Level;
-import org.apache.logging.log4j.core.LoggerContext;
-import org.apache.logging.log4j.core.appender.RollingFileAppender;
-import org.apache.logging.log4j.core.appender.rolling.SizeBasedTriggeringPolicy;
-import org.apache.logging.log4j.core.config.Configuration;
-import org.apache.logging.log4j.core.config.LoggerConfig;
-import org.apache.logging.log4j.core.layout.PatternLayout;
 import org.broad.igv.exceptions.DataLoadException;
+import org.broad.igv.logging.LogFileHandler;
+import org.broad.igv.logging.LogManager;
+import org.broad.igv.logging.Logger;
 import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.ui.util.FileDialogUtils;
 import org.broad.igv.ui.util.MessageUtils;
-import org.broad.igv.util.RuntimeUtils;
 
 import javax.swing.*;
 import javax.swing.filechooser.FileSystemView;
@@ -56,7 +49,7 @@ import java.util.prefs.Preferences;
  */
 public class DirectoryManager {
 
-    private static Logger log = Logger.getLogger(DirectoryManager.class);
+    private static Logger log = LogManager.getLogger(DirectoryManager.class);
 
     private static File USER_HOME;
     private static File USER_DIRECTORY;    // FileSystemView.getFileSystemView().getDefaultDirectory();
@@ -81,7 +74,6 @@ public class DirectoryManager {
      */
     public static synchronized File getUserDirectory() {
         if (USER_DIRECTORY == null) {
-            log.info("Fetching user directory... ");
             USER_DIRECTORY = FileSystemView.getFileSystemView().getDefaultDirectory();
             //Mostly for testing, in some environments USER_DIRECTORY can be null
             if (USER_DIRECTORY == null) {
@@ -116,20 +108,6 @@ public class DirectoryManager {
                 File rootDir = getUserHome();
                 IGV_DIRECTORY = new File(rootDir, "igv");
 
-                if (!IGV_DIRECTORY.exists()) {
-                    // See if a pre-2.1 release directory exists, if so copy it
-                    File legacyDirectory = null;
-                    try {
-                        legacyDirectory = getLegacyIGVDirectory();
-                        if (legacyDirectory.exists()) {
-                            log.info("Copying " + legacyDirectory + " => " + IGV_DIRECTORY);
-                            FileUtils.copyDirectory(legacyDirectory, IGV_DIRECTORY);
-                        }
-                    } catch (IOException e) {
-                        log.error("Error copying igv directory " + legacyDirectory + " => " + IGV_DIRECTORY, e);
-                    }
-                }
-
                 if (!IGV_DIRECTORY.exists()) {
                     try {
                         boolean wasSuccessful = IGV_DIRECTORY.mkdir();
@@ -175,8 +153,6 @@ public class DirectoryManager {
             } else if (!canWrite(IGV_DIRECTORY)) {
                 throw new DataLoadException("Cannot write to user directory", IGV_DIRECTORY.getAbsolutePath());
             }
-
-            log.info("IGV Directory: " + IGV_DIRECTORY.getAbsolutePath());
         }
         return IGV_DIRECTORY;
     }
@@ -229,7 +205,7 @@ public class DirectoryManager {
             directory = new File(cachePref);
         }
 
-        if(directory == null || !directory.exists() || !directory.isDirectory()) {
+        if (directory == null || !directory.exists() || !directory.isDirectory()) {
 
             directory = new File(getGenomeCacheDirectory(), "seq");
             if (!directory.exists()) {
@@ -328,23 +304,27 @@ public class DirectoryManager {
      * is successfully moved, irrespective of any errors that might occur later (e.g. when attempting to
      * remove the old directory).
      *
-     * @param newIGVDirectory
+     * @param newIGVDirectoryParent -- parent directory for "igv" folder
      * @return True if the directory is successfully moved, false otherwise
      */
 
-    public static Boolean moveIGVDirectory(final File newIGVDirectory) {
+    public static void moveIGVDirectory(final File newIGVDirectoryParent) {
+
+        File newIGVDirectory = new File(newIGVDirectoryParent, "igv");
 
         if (newIGVDirectory.equals(IGV_DIRECTORY)) {
-            return false; // Nothing to do
+            return; // Nothing to do
         }
 
-        if (IGV_DIRECTORY != null && IGV_DIRECTORY.exists()) {
+
+        if (IGV_DIRECTORY != null) {
 
             File oldDirectory = IGV_DIRECTORY;
 
             try {
                 log.info("Moving igv directory from " + oldDirectory.getParent() + " to " +
                         newIGVDirectory.getAbsolutePath());
+
                 FileUtils.copyDirectory(IGV_DIRECTORY, newIGVDirectory);
                 IGV_DIRECTORY = newIGVDirectory;
 
@@ -358,30 +338,19 @@ public class DirectoryManager {
             } catch (IOException e) {
                 log.error("Error copying IGV directory", e);
                 MessageUtils.showMessage("<html>Error moving IGV directory:<br/>&nbsp;nbsp;" + e.getMessage());
-                return false;
+                return;
             }
 
-            // Restart the log
-            LogManager.shutdown();
             initializeLog();
 
             // Try to delete the old directory
-            try {
-                deleteDirectory(oldDirectory);
-            } catch (IOException e) {
-                log.error("An error was encountered deleting the previous IGV directory", e);
-                MessageUtils.showMessage("<html>An error was encountered deleting the previous IGV directory (" +
-                        e.getMessage() + "):<br>&nbsp;nbsp;nbsp;" + oldDirectory.getAbsolutePath() +
-                        "<br>Remaining files should be manually deleted.");
-            }
+            org.broad.igv.util.FileUtils.deleteDir(oldDirectory);
 
         }
 
         GENOME_CACHE_DIRECTORY = null;
         GENE_LIST_DIRECTORY = null;
         BAM_CACHE_DIRECTORY = null;
-        return true;
-
     }
 
     public static void moveDirectoryContents(File oldDirectory, File newDirectory) {
@@ -401,7 +370,7 @@ public class DirectoryManager {
         }
     }
 
-    public static  boolean isChildOf(File base, File child)
+    public static boolean isChildOf(File base, File child)
             throws IOException {
 
         File parent = child.getParentFile();
@@ -414,26 +383,6 @@ public class DirectoryManager {
         return false;
     }
 
-    /**
-     * Delete the directory and all contents recursively.  The apache FileUtils is hanging on Linux.
-     *
-     * @param oldDirectory
-     * @throws IOException
-     */
-    private static void deleteDirectory(File oldDirectory) throws IOException {
-        if (Globals.IS_LINUX || Globals.IS_MAC) {
-            //System.out.println("Deleting: " + oldDirectory);
-            String[] cmd = new String[]{"rm", "-rf", oldDirectory.getAbsolutePath()};
-            String result = RuntimeUtils.executeShellCommand(cmd, null, null);
-            if (result != null && result.trim().length() > 0) {
-                log.info("Response from 'rm -rf': " + result);
-            }
-        } else {
-            // The apache commons FileUtils is not working reliably
-            org.broad.igv.util.FileUtils.deleteDir(oldDirectory);
-        }
-    }
-
 
     /**
      * Return the pre 2.1 release properties file.  This may or may not exist.
@@ -486,32 +435,7 @@ public class DirectoryManager {
     }
 
     public static void initializeLog() {
-
-        // Create a log file that is ready to have text appended to it
-        try {
-            File logFile = getLogFile();
-            LoggerContext context = LoggerContext.getContext(false);
-            Configuration configuration = context.getConfiguration();
-            LoggerConfig rootLogger = configuration.getRootLogger();
-
-            rootLogger.removeAppender("IGV_ROLLING_APPENDER");
-            PatternLayout layout = PatternLayout.newBuilder().withConfiguration(configuration)
-                    .withPattern("%p [%d{ISO8601}] [%F:%L]  %m%n").build();
-            RollingFileAppender appender = RollingFileAppender.newBuilder().withName("IGV_ROLLING_APPENDER")
-                    .setConfiguration(configuration)
-                    .withFileName(logFile.getAbsolutePath()).withAppend(true)
-                    .withFilePattern(getIgvDirectory().getAbsolutePath() + File.pathSeparator + "igv-%i.log")
-                    .withLayout(layout)
-                    .withPolicy(SizeBasedTriggeringPolicy.createPolicy("1000K"))
-                    .build();
-            appender.start();
-            configuration.addAppender(appender);
-            rootLogger.addAppender(appender, Level.ALL, null);
-            context.updateLoggers();
-        } catch (IOException e) {
-            // Can't create log file, just log to console as set in log4j2.xml
-            log.error("Error creating log file", e);
-        }
+        LogFileHandler.getInstance().updateHandler();
     }
 
 }
