@@ -25,7 +25,7 @@
 
 package org.broad.igv;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.renderer.SequenceRenderer;
 
 import java.io.IOException;
@@ -39,18 +39,13 @@ import java.util.regex.Pattern;
  * Date: Feb 3, 2010
  */
 public class Globals {
-    private static Logger log = Logger.getLogger(Globals.class);
+    private static Logger log = LogManager.getLogger(Globals.class);
 
     public static final int DESIGN_DPI = 96;
     public static final DecimalFormat DECIMAL_FORMAT = new DecimalFormat();
     final static public String HISTORY_DELIMITER = ";";
     public static final String DEFAULT_GENOME = "hg19";
 
-    // External resoure URLs
-    public static final String DEFAULT_GENOME_URL = "http://igv.broadinstitute.org/genomes/genomes.txt";
-    public static final String DEFAULT_DATA_URL = "https://data.broadinstitute.org/igvdata/$$_dataServerRegistry.txt";
-
-
     public static final String CHR_ALL = "All";
     public static final String TRACK_NAME_ATTRIBUTE = "NAME";
     public static final String TRACK_DATA_FILE_ATTRIBUTE = "DATA FILE";
@@ -62,6 +57,7 @@ public class Globals {
     private static boolean testing = false;
     public static int CONNECT_TIMEOUT = 20000;        // 20 seconds
     public static int READ_TIMEOUT = 1000 * 3 * 60;   // 3 minutes
+    public static int TOKEN_EXPIRE_GRACE_TIME = 1000 * 1 * 60; // 1 minute
 
     /**
      * Field description
@@ -78,6 +74,7 @@ public class Globals {
 
     final static public Pattern commaPattern = Pattern.compile(",");
     final static public Pattern tabPattern = Pattern.compile("\t");
+    final static public Pattern multiTabPattern = Pattern.compile("\t+");
     final static public Pattern colonPattern = Pattern.compile(":");
     final static public Pattern dashPattern = Pattern.compile("-");
     final static public Pattern equalPattern = Pattern.compile("=");
@@ -117,12 +114,12 @@ public class Globals {
     public static boolean development = false;
 
     public static String versionURL = "https://data.broadinstitute.org/igv/projects/current/version.txt";
-    public static String downloadURL = "http://www.broadinstitute.org/igv/download";
+    public static String downloadURL = "https://software.broadinstitute.org/software/igv/download";
     static {
         Properties properties = new Properties();
         try {
             properties.load(Globals.class.getResourceAsStream("/resources/about.properties"));
-        } catch (IOException e) {
+        } catch (Exception e) {
             log.error("*** Error retrieving version and build information! ***", e);
         }
         VERSION = properties.getProperty("version", "???");
@@ -135,9 +132,8 @@ public class Globals {
         final String developmentProperty = System.getProperty("development", properties.getProperty("development", "false"));
         development = Boolean.parseBoolean(developmentProperty);
         if(development){
-            log.warn("Development mode is enabled");
+            log.info("Development mode is enabled");
         }
-
     }
 
     public static void setHeadless(boolean bool) {
