@@ -26,11 +26,15 @@
 package org.broad.igv.batch;
 
 import biz.source_code.base64Coder.Base64Coder;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.LogManager;
+import org.broad.igv.logging.Logger;
 import org.broad.igv.Globals;
 import org.broad.igv.feature.genome.GenomeManager;
 import org.broad.igv.google.OAuthUtils;
+import org.broad.igv.prefs.Constants;
+import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.ui.IGV;
+import org.broad.igv.ui.util.UIUtilities;
 import org.broad.igv.util.StringUtils;
 
 import java.awt.*;
@@ -52,7 +56,7 @@ public class CommandListener implements Runnable {
 
     public static int currentListenerPort = -1;
 
-    private static Logger log = Logger.getLogger(CommandListener.class);
+    private static Logger log = LogManager.getLogger(CommandListener.class);
 
     private static CommandListener listener;
     private static final String CRLF = "\r\n";
@@ -78,11 +82,20 @@ public class CommandListener implements Runnable {
         indexParams = new HashSet<String>(Arrays.asList("index"));
     }
 
-    public static synchronized void start(int port) {
-        listener = new CommandListener(port);
-        listener.listenerThread.start();
+    public static synchronized void start() {
+        start(PreferencesManager.getPreferences().getAsInt(Constants.PORT_NUMBER));
     }
 
+    public static synchronized void start(int port) {
+        try {
+            listener = new CommandListener(port);
+            listener.listenerThread.start();
+        } catch (Exception e) {
+            listener.closeSockets();
+            listener = null;
+            log.error(e);
+        }
+    }
 
     public static synchronized void halt() {
         if (listener != null) {
@@ -98,13 +111,14 @@ public class CommandListener implements Runnable {
         listenerThread = new Thread(this);
     }
 
-    
+
     /**
      * Return true if the listener is currently enabled
+     *
      * @return state of listener
      */
     public static boolean isListening() {
-    	return CommandListener.isListening;
+        return listener != null;
     }
 
     /**
@@ -113,40 +127,34 @@ public class CommandListener implements Runnable {
      */
     public void run() {
 
-        CommandExecutor cmdExe = new CommandExecutor();
-
         try {
+            CommandExecutor cmdExe = new CommandExecutor(IGV.getInstance());
+
             serverSocket = new ServerSocket(port);
             log.info("Listening on port " + port);
             currentListenerPort = port;
-            isListening = true;
             while (!halt) {
-            	
                 clientSocket = serverSocket.accept();
                 processClientSession(cmdExe);
                 if (clientSocket != null) {
                     try {
                         clientSocket.close();
                         clientSocket = null;
-                        isListening = false;
+                        // We do NOT set isListening = false here, otherwise logout/login state change falls back to OOB
                     } catch (IOException e) {
                         log.error("Error in client socket loop", e);
-                        isListening = false;
                     }
                 }
             }
-
-
         } catch (java.net.BindException e) {
             log.error(e);
             currentListenerPort = -1;
-            isListening = false;
+            listener = null;
         } catch (ClosedByInterruptException e) {
             log.error(e);
-            isListening = false;
-
+            listener = null;
         } catch (IOException e) {
-        	isListening = false;
+            listener = null;
             if (!halt) {
                 log.error("IO Error on port socket ", e);
             }
@@ -167,78 +175,91 @@ public class CommandListener implements Runnable {
             in = new BufferedReader(new InputStreamReader(clientSocket.getInputStream()));
             String inputLine;
 
-
             while (!halt && (inputLine = in.readLine()) != null) {
 
                 String cmd = inputLine;
-                if (cmd.startsWith("GET")) {
-
-                    // Consume the remainder of the request, if any.   This is important to free the connection.
-                    Map<String, String> headers = new HashMap<String, String>();
-                    String nextLine = in.readLine();
-                    while (nextLine != null && nextLine.length() > 0) {
-                        nextLine = in.readLine();
-                        String[] tokens = Globals.colonPattern.split(nextLine, 2);
-                        if (tokens.length == 2) {
-                            headers.put(tokens[0].trim(), tokens[1].trim());
-                        }
-                    }
-
+                if(!cmd.contains("/oauthCallback")) {
                     log.info(cmd);
+                }
 
-                    String command = null;
-                    Map<String, String> params = null;
-                    String[] tokens = inputLine.split(" ");
-                    if (tokens.length < 2) {
-                        sendTextResponse(out, "ERROR unexpected command line: " + inputLine);
-                        return;
-                    } else {
-                        String[] parts = tokens[1].split("\\?");
-                        command = parts[0];
-                        params = parts.length < 2 ? new HashMap() : parseParameters(parts[1]);
-                    }
+                boolean isHTTP = cmd.startsWith("OPTIONS") || cmd.startsWith("HEAD") || cmd.startsWith("GET");
 
-                    // Detect google oauth callback
-                    if (command.equals("/oauthCallback")) {
-                        if (params.containsKey("code")) {
-                            OAuthUtils.getInstance().setAuthorizationCode(params.get("code"));
-                        } else if (params.containsKey("token")) {
-                            OAuthUtils.getInstance().setAccessToken(params.get("token"));
-                        }
-                        sendTextResponse(out, "OK");
+                if (isHTTP) {
+                    if (cmd.startsWith("OPTIONS")) {
+                        sendHTTPOptionsResponse(out);
                     } else {
 
-                        // If a callback (javascript) function is specified write it back immediately.  This function
-                        // is used to cancel a timeout handler
-                        String callback = params.get("callback");
-                        if (callback != null) {
-                            sendJavascriptResponse(out, callback);
+                        String result = null;
+                        String command = null;
+                        Map<String, String> params = null;
+
+                        String[] tokens = inputLine.split(" ");
+                        if (tokens.length < 2) {
+                            result = "ERROR unexpected command line: " + inputLine;
+                        } else {
+                            String[] parts = tokens[1].split("\\?");
+                            command = parts[0];
+                            params = parts.length < 2 ? new HashMap() : parseParameters(parts[1]);
+                        }
+                        // Consume the remainder of the request, if any.   This is important to free the connection.
+                        Map<String, String> headers = new HashMap<>();
+                        String nextLine = in.readLine();
+                        while (nextLine != null && nextLine.length() > 0) {
+                            nextLine = in.readLine();
+                            String[] headerTokens = Globals.colonPattern.split(nextLine, 2);
+                            if (headerTokens.length == 2) {
+                                headers.put(headerTokens[0].trim(), headerTokens[1].trim());
+                            }
                         }
 
-                        // Process the request.
+                        if (cmd.startsWith("HEAD")) {
+                            sendHTTPResponse(out, result, "text/html", "HEAD");
+                        } else {
+
+                            if (command != null) {
+
+                                // Detect google oauth callback
+                                if (command.equals("/oauthCallback")) {
+                                    if (params.containsKey("code")) {
+                                        OAuthUtils.getInstance().setAuthorizationCode(params);
+                                    } else if (params.containsKey("token")) {
+                                        OAuthUtils.getInstance().setAccessToken(params);
+                                    }
+                                    sendTextResponse(out, "SUCCESS");
+                                    if(PreferencesManager.getPreferences().getAsBoolean(Constants.PORT_ENABLED) == false) {
+                                        // Turn off port
+                                        halt();
+                                    }
+                                } else {
+                                    // Process the request.
+                                    result = processGet(command, params, cmdExe); // Send no response if result is "OK".
+                                    if ("OK".equals(result)) result = null;
+                                    sendTextResponse(out, result);
+                                }
+                            }
 
 
-                        String result = processGet(command, params, cmdExe);
-
-                        // If no callback was specified write back response now
-                        if (callback == null) {
-                            // We send no response if result is "ok".
-                            if (result.equals(OK)) result = null;
-                            sendTextResponse(out, result);
                         }
                     }
-
-                    // http sockets are used for one request only
+                    // http sockets are used for one request only => return will close the socket
                     return;
-                    // }
+
 
                 } else {
                     // Port command
-                    Globals.setBatch(true);
-                    Globals.setSuppressMessages(true);
-                    final String response = cmdExe.execute(inputLine);
-                    out.println(response);
-                    out.flush();
+                    try {
+                        Globals.setBatch(true);
+                        String finalInputLine = inputLine;
+                        PrintWriter finalOut = out;
+                        UIUtilities.invokeAndWaitOnEventThread(() -> {
+                            final String response = cmdExe.execute(finalInputLine);
+                            finalOut.println(response);
+                            finalOut.flush();
+                        });
+
+                    } finally {
+                        Globals.setBatch(false);
+                    }
                 }
             }
         } catch (IOException e) {
@@ -279,16 +300,11 @@ public class CommandListener implements Runnable {
     private static final String NO_CACHE = "Cache-Control: no-cache, no-store";
     private static final String ACCESS_CONTROL_ALLOW_ORIGIN = "Access-Control-Allow-Origin: *";
 
-    private void sendJavascriptResponse(PrintWriter out, String result) {
-        sendHTTPResponse(out, result, "application/javascript");
-
-    }
-
     private void sendTextResponse(PrintWriter out, String result) {
-        sendHTTPResponse(out, result, "text/html");
+        sendHTTPResponse(out, result, "text/html", "GET");
     }
 
-    private void sendHTTPResponse(PrintWriter out, String result, String contentType) {
+    private void sendHTTPResponse(PrintWriter out, String result, String contentType, String method) {
 
         out.print(result == null ? HTTP_NO_RESPONSE : HTTP_RESPONSE);
         out.print(CRLF);
@@ -303,13 +319,29 @@ public class CommandListener implements Runnable {
             out.print(CRLF);
             out.print(CONNECTION_CLOSE);
             out.print(CRLF);
-            out.print(CRLF);
-            out.print(result);
-            out.print(CRLF);
+
+            if (!method.equals("HEAD")) {
+                out.print(CRLF);
+                out.print(result);
+                out.print(CRLF);
+            }
         }
         out.close();
     }
 
+    private void sendHTTPOptionsResponse(PrintWriter out) {
+
+        out.print(HTTP_NO_RESPONSE);
+        out.print(CRLF);
+        out.print(ACCESS_CONTROL_ALLOW_ORIGIN);
+        out.print(CRLF);
+        out.println("Access-Control-Allow-Methods: HEAD, GET, OPTIONS");
+        out.print(CRLF);
+
+        out.close();
+    }
+
+
     /**
      * Process an http get request.
      */
@@ -317,7 +349,7 @@ public class CommandListener implements Runnable {
     private String processGet(String command, Map<String, String> params, CommandExecutor cmdExe) throws IOException {
 
         String result = OK;
-        final Frame mainFrame = IGV.getMainFrame();
+        final Frame mainFrame = IGV.getInstance().getMainFrame();
 
         // Trick to force window to front, the setAlwaysOnTop works on a Mac,  toFront() does nothing.
         mainFrame.toFront();
@@ -385,6 +417,8 @@ public class CommandListener implements Runnable {
     }
 
 
+
+
     /**
      * Parse the html parameter string into a set of key-value pairs.  Parameter values are
      * url decoded with the exception of the "locus" parameter.
