@@ -26,7 +26,7 @@
 package org.broad.igv.bbfile;
 
 import htsjdk.samtools.seekablestream.SeekableStream;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.util.stream.IGVSeekableBufferedStream;
 import org.broad.igv.util.stream.IGVSeekableStreamFactory;
 
@@ -91,7 +91,7 @@ public class BBFileReader {
 
     public static final long BBFILE_HEADER_OFFSET = 0;
 
-    private static Logger log = Logger.getLogger(BBFileReader.class);
+    private static Logger log = LogManager.getLogger(BBFileReader.class);
 
     private SeekableStream fis;      // BBFile input stream handle
     private long fileOffset;           // file offset for next item to be read
@@ -183,8 +183,7 @@ public class BBFileReader {
         chromDataTreeOffset = fileHeader.getFullIndexOffset();
         if (chromDataTreeOffset != 0) {
             fileOffset = chromDataTreeOffset;
-            boolean forceDescend = false;
-            chromosomeDataTree = new RPTree(fis, fileOffset, isLowToHigh, uncompressBufSize, forceDescend);
+            chromosomeDataTree = new RPTree(fis, fileOffset, isLowToHigh, uncompressBufSize);
         }
 
 
