@@ -1,8 +1,11 @@
 package org.broad.igv.bedpe;
 
-import org.apache.log4j.Logger;
 import org.broad.igv.Globals;
+import org.broad.igv.feature.Range;
 import org.broad.igv.feature.genome.Genome;
+import org.broad.igv.jbrowse.CircularViewUtilities;
+import org.broad.igv.logging.LogManager;
+import org.broad.igv.logging.Logger;
 import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.track.AbstractTrack;
@@ -10,10 +13,11 @@ import org.broad.igv.track.RenderContext;
 import org.broad.igv.track.TrackClickEvent;
 import org.broad.igv.track.TrackMenuUtils;
 import org.broad.igv.ui.FontManager;
-import org.broad.igv.ui.IGV;
+import org.broad.igv.ui.panel.FrameManager;
 import org.broad.igv.ui.panel.IGVPopupMenu;
 import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.ui.util.MessageUtils;
+import org.broad.igv.util.Downsampler;
 import org.broad.igv.util.FeatureCache;
 import org.broad.igv.util.ResourceLocator;
 import org.w3c.dom.Document;
@@ -21,21 +25,19 @@ import org.w3c.dom.Element;
 
 import javax.swing.*;
 import java.awt.*;
-import java.awt.event.ActionEvent;
-import java.awt.event.ActionListener;
-import java.util.*;
 import java.util.List;
+import java.util.*;
+import java.util.function.Function;
 
 import static org.broad.igv.bedpe.InteractionTrack.Direction.UP;
-import static org.broad.igv.track.TrackMenuUtils.refresh;
-
 
 /**
  * Created by jrobinso on 6/29/18.
  */
 public class InteractionTrack extends AbstractTrack {
 
-    private static Logger log = Logger.getLogger(InteractionTrack.class);
+    public static final int MAX_WG_COUNT = 1000;
+    private static Logger log = LogManager.getLogger(InteractionTrack.class);
 
     protected static final int AXIS_AREA_WIDTH = 60;
     protected static Color axisLineColor = new Color(255, 180, 180);
@@ -43,6 +45,9 @@ public class InteractionTrack extends AbstractTrack {
     private JMenuItem maxScoreItem;
     private List<BedPE> wgFeatures;
 
+    // TODO -- for jbrowse experiment
+    private List<BedPEFeature> allFeatures;
+
 
     enum Direction {UP, DOWN}
 
@@ -131,35 +136,12 @@ public class InteractionTrack extends AbstractTrack {
 
         wgFeatures = createWGFeatures(featureList, genome);
 
+        allFeatures = featureList;
+
 
     }
 
 
-    private List<BedPE> createWGFeatures(List<BedPEFeature> features, Genome genome) {
-
-        List<BedPE> wgFeatures = new ArrayList<>(features.size());
-
-        for (BedPEFeature f : features) {
-
-            int start1 = genome.getGenomeCoordinate(f.chr1, f.start1);
-            int end1 = genome.getGenomeCoordinate(f.chr1, f.end1);
-            int start2 = genome.getGenomeCoordinate(f.chr2, f.start2);
-            int end2 = genome.getGenomeCoordinate(f.chr2, f.end2);
-            BedPEFeature wgFeature = new BedPEFeature(Globals.CHR_ALL, start1, end1, Globals.CHR_ALL, start2, end2);
-
-            wgFeature.name = f.name;
-            wgFeature.score = f.score;
-            wgFeature.thickness = f.thickness;
-            wgFeature.color = f.color;
-            wgFeature.attributes = f.attributes;
-
-
-            wgFeatures.add(wgFeature);
-
-        }
-        return wgFeatures;
-    }
-
     @Override
     public boolean isReadyToPaint(ReferenceFrame frame) {
         return true;
@@ -172,7 +154,7 @@ public class InteractionTrack extends AbstractTrack {
 
     private List<BedPE> getFeaturesOverlapping(String chr, double start, double end) {
 
-        if(chr.equals(Globals.CHR_ALL)) {
+        if (chr.equals(Globals.CHR_ALL)) {
             return wgFeatures;
         } else {
             return featureCache.getFeatures(chr, (int) start, (int) end);
@@ -194,7 +176,7 @@ public class InteractionTrack extends AbstractTrack {
             if (features != null && features.size() > 0) {
 
                 if (graphType == GraphType.PROPORTIONAL_ARC) {
-                    if(autoscale || maxScore <= 0) {
+                    if (autoscale || maxScore <= 0) {
                         maxScore = autoscale(features);
                     }
                     drawScale(context, trackRectangle);
@@ -215,11 +197,11 @@ public class InteractionTrack extends AbstractTrack {
 
     /**
      * Draw scale in top left of rectangle
-
+     *
      * @param context
      * @param arect
      */
-    public  void drawScale(RenderContext context, Rectangle arect){
+    public void drawScale(RenderContext context, Rectangle arect) {
         if (context.multiframe == false) {
             Graphics2D g = context.getGraphic2DForColor(Color.black);
             Font font = g.getFont();
@@ -258,6 +240,26 @@ public class InteractionTrack extends AbstractTrack {
 
         IGVPopupMenu menu = new IGVPopupMenu();
 
+
+        // Experimental JBrowse.
+        if (PreferencesManager.getPreferences().getAsBoolean(Constants.CIRC_VIEW_ENABLED) && CircularViewUtilities.ping()) {
+            menu.addSeparator();
+            JMenuItem item = new JMenuItem("Add Pairs to Circular View");
+            item.addActionListener(e -> {
+                List<? extends BedPE> visibleFeatures;
+                if (te.getFrame() == null || te.getFrame().getChrName().equals(Globals.CHR_ALL)) {
+                    visibleFeatures = allFeatures;
+                } else {
+                    List<ReferenceFrame> frames = te.getFrame() != null ?
+                            Arrays.asList(te.getFrame()) :
+                            FrameManager.getFrames();
+                    visibleFeatures = getVisibleFeatures(frames);
+                }
+                CircularViewUtilities.sendBedpeToJBrowse(visibleFeatures, InteractionTrack.this.getName(), InteractionTrack.this.getColor());
+            });
+            menu.add(item);
+        }
+
         menu.add(TrackMenuUtils.getTrackRenameItem(Collections.singleton(InteractionTrack.this)));
 
         JMenuItem item = new JMenuItem("Set Track Height...");
@@ -286,7 +288,7 @@ public class InteractionTrack extends AbstractTrack {
                 PreferencesManager.getPreferences().put(Constants.ARC_TYPE, entry.getValue().toString());
                 autoscaleCB.setEnabled(graphType == GraphType.PROPORTIONAL_ARC);
                 maxScoreItem.setEnabled(graphType == GraphType.PROPORTIONAL_ARC);
-                refresh();
+                repaint();
             });
             group.add(mm);
             menu.add(mm);
@@ -298,7 +300,7 @@ public class InteractionTrack extends AbstractTrack {
         showBlocksCB.addActionListener(e -> {
             showBlocks = showBlocksCB.isSelected();
             PreferencesManager.getPreferences().put(Constants.ARC_BLOCKS, String.valueOf(showBlocksCB.isSelected()));
-            refresh();
+            repaint();
         });
         menu.add(showBlocksCB);
 
@@ -307,7 +309,7 @@ public class InteractionTrack extends AbstractTrack {
         autoscaleCB.setSelected(autoscale);
         autoscaleCB.addActionListener(e -> {
             autoscale = autoscaleCB.isSelected();
-            refresh();
+            repaint();
         });
         menu.add(autoscaleCB);
 
@@ -320,7 +322,7 @@ public class InteractionTrack extends AbstractTrack {
                     if (ms > 0) {
                         maxScore = ms;
                         autoscale = false;
-                        refresh();
+                        repaint();
                     } else {
                         MessageUtils.showMessage("maximum score must be > 0");
                     }
@@ -343,7 +345,7 @@ public class InteractionTrack extends AbstractTrack {
             } else {
                 direction = UP;
             }
-            IGV.getInstance().repaint();
+            repaint();
         });
         menu.add(item);
 
@@ -353,7 +355,7 @@ public class InteractionTrack extends AbstractTrack {
             if (t != null) {
                 try {
                     thickness = Integer.parseInt(t);
-                    IGV.getInstance().repaint();
+                    repaint();
                 } catch (NumberFormatException e1) {
                     MessageUtils.showErrorMessage("Line thickness must be an integer", e1);
                 }
@@ -449,7 +451,105 @@ public class InteractionTrack extends AbstractTrack {
         if (element.hasAttribute("maxScore")) {
             this.maxScore = Double.parseDouble(element.getAttribute("maxScore"));
         }
-
     }
 
+
+    /**
+     * Return features visible in the supplied frames
+     */
+    public List<BedPE> getVisibleFeatures(List<ReferenceFrame> frames) {
+
+        Function<ReferenceFrame, List<BedPE>> frameFeatures = (f) -> {
+            String chr = f.getChrName();
+            Range r = f.getCurrentRange();
+            return getFeaturesOverlapping(chr, r.getStart(), r.getEnd());
+        };
+
+        if (frames.size() == 0) {
+            return Collections.emptyList();
+        } else if (frames.size() == 1) {
+            return frameFeatures.apply(frames.get(0));
+        } else {
+            List<BedPE> inView = new ArrayList<>();
+            for (ReferenceFrame f : frames) {
+                inView.addAll(frameFeatures.apply(f));
+            }
+            return inView;
+        }
+    }
+
+
+    private List<BedPE> createWGFeatures(List<BedPEFeature> features, Genome genome) {
+
+        int size = Math.min(features.size(), MAX_WG_COUNT);
+        List<BedPE> wgFeatures = new ArrayList<>(size);
+
+        List<BedPEFeature> sampledFeatures;
+        if (features.size() < MAX_WG_COUNT) {
+            sampledFeatures = features;
+        } else {
+            sampledFeatures = downsampleWGFeatures(features);
+        }
+
+        for (BedPEFeature f : sampledFeatures) {
+
+            int start1 = genome.getGenomeCoordinate(f.chr1, f.start1);
+            int end1 = genome.getGenomeCoordinate(f.chr1, f.end1);
+            int start2 = genome.getGenomeCoordinate(f.chr2, f.start2);
+            int end2 = genome.getGenomeCoordinate(f.chr2, f.end2);
+            BedPEFeature wgFeature = new BedPEFeature(Globals.CHR_ALL, start1, end1, Globals.CHR_ALL, start2, end2);
+
+            wgFeature.name = f.name;
+            wgFeature.score = f.score;
+            wgFeature.thickness = f.thickness;
+            wgFeature.color = f.color;
+            wgFeature.attributes = f.attributes;
+
+            wgFeatures.add(wgFeature);
+
+        }
+        return wgFeatures;
+    }
+
+    private List<BedPEFeature> downsampleWGFeatures(List<BedPEFeature> features) {
+
+        // First pass -- find the max score feature
+        BedPEFeature maxScoreFeature = features.get(0);
+        for (BedPEFeature f : features) {
+            if (f.score > maxScoreFeature.score) {
+                maxScoreFeature = f;
+            }
+        }
+
+        int nBins = maxScoreFeature.score > 0 ? 5 : 1;  // TODO make a function of total # of features & maxCount?
+        double binSize = nBins > 1 ? Math.log10(maxScoreFeature.score) / nBins : Integer.MAX_VALUE;
+
+        // Divide features into bins\
+        Object [] binnedFeatures = new Object[nBins];
+        int counts [] = new int[nBins];
+        for(int i=0; i<nBins; i++) {
+            binnedFeatures[i] = new ArrayList<BedPEFeature> ();
+            counts[i] = 0;
+        }
+        for (BedPEFeature f : features) {
+            int bin = f.score <= 0 ? 0 : (int) Math.min(nBins - 1, Math.floor(Math.log10(f.score) / binSize));
+            ((List<BedPEFeature>) binnedFeatures[bin]).add(f);
+            counts[bin]++;
+        }
+
+        // Add sampled features from each bin
+        int featuresPerBin = MAX_WG_COUNT / nBins;
+        List<BedPEFeature> sampledFeatures = new ArrayList<>(MAX_WG_COUNT);
+        for(int i=0; i<nBins; i++) {
+            List<BedPEFeature> bfs = (List<BedPEFeature>) binnedFeatures[i];
+            sampledFeatures.addAll(Arrays.asList(new Downsampler<BedPEFeature>().sample(bfs.toArray(BedPEFeature[]::new), featuresPerBin)));
+        }
+
+        // Be sure we keep the maximum feature
+        if(maxScoreFeature != null) {
+            sampledFeatures.add(maxScoreFeature);
+        }
+
+        return sampledFeatures;
+    }
 }
