@@ -28,7 +28,7 @@ package org.broad.igv.data;
 //~--- non-JDK imports --------------------------------------------------------
 
 import htsjdk.samtools.seekablestream.SeekableStream;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.exceptions.ParserException;
 import org.broad.igv.feature.genome.Genome;
@@ -60,7 +60,7 @@ import java.util.Map;
  */
 public class IGVDatasetParser {
 
-    private static Logger log = Logger.getLogger(IGVDatasetParser.class);
+    private static Logger log = LogManager.getLogger(IGVDatasetParser.class);
     private ResourceLocator dataResourceLocator;
     private int chrColumn = -1;
     private int startColumn = -1;
@@ -83,78 +83,48 @@ public class IGVDatasetParser {
     }
 
     private void setColumnDefaults() {
-        String tmp = (dataResourceLocator.getPath().endsWith(".txt")
-                ? dataResourceLocator.getPath().substring(0,
-                dataResourceLocator.getPath().length() - 4) : dataResourceLocator.getPath()).toLowerCase();
 
-        if (tmp.endsWith(".igv")) {
-            chrColumn = 0;
-            startColumn = 1;
-            endColumn = 2;
-            probeColumn = 3;
-            firstDataColumn = 4;
-            hasEndLocations = true;
-            hasCalls = false;
-        } else if (tmp.endsWith(".xcn") || tmp.endsWith("cn") || tmp.endsWith(".snp") || tmp.endsWith(".loh")) {
-            probeColumn = 0;
-            chrColumn = 1;
-            startColumn = 2;
-            endColumn = -1;
-            firstDataColumn = 3;
-            hasEndLocations = false;
-            hasCalls = tmp.endsWith(".xcn") || tmp.endsWith(".snp");
-        } else if (tmp.endsWith(".expr")) {
-            //gene_id	bundle_id	chr	left	right	FPKM	FPKM_conf_lo	FPKM_conf_hi
-            probeColumn = 0;
-            chrColumn = 2;
-            startColumn = 3;
-            endColumn = 4;
-            startBase = 1;
-            firstDataColumn = 5;
-            lastDataColumn = 5;
-            hasEndLocations = true;
+        String format = dataResourceLocator.getFormat();
+        switch (format) {
+            case "igv":
+                chrColumn = 0;
+                startColumn = 1;
+                endColumn = 2;
+                probeColumn = 3;
+                firstDataColumn = 4;
+                hasEndLocations = true;
+                hasCalls = false;
+                break;
+            case "cn":
+            case "snp":
+            case "xcn":
+            case "loh":
+                probeColumn = 0;
+                chrColumn = 1;
+                startColumn = 2;
+                endColumn = -1;
+                firstDataColumn = 3;
+                hasEndLocations = false;
+                hasCalls = format.equals("xcn") || format.equals("snp");
+                break;
+            case ".expr":
+                //gene_id	bundle_id	chr	left	right	FPKM	FPKM_conf_lo	FPKM_conf_hi
+                probeColumn = 0;
+                chrColumn = 2;
+                startColumn = 3;
+                endColumn = 4;
+                startBase = 1;
+                firstDataColumn = 5;
+                lastDataColumn = 5;
+                hasEndLocations = true;
+                break;
+            default:
+                // TODO -- popup dialog and ask user to define columns,  and csv vs tsv?
+                throw new ParserException("Unknown file type: ", 0);
 
-        } else {
-            // TODO -- popup dialog and ask user to define columns,  and csv vs tsv?
-            throw new ParserException("Unknown file type: ", 0);
         }
     }
 
-    /*
-     */
-
-    public static boolean parsableMAGE_TAB(ResourceLocator file) throws IOException {
-        AsciiLineReader reader = null;
-        try {
-            reader = ParsingUtils.openAsciiReader(file);
-            String nextLine = null;
-
-            //skip first row
-            reader.readLine();
-
-            //check second row for MAGE_TAB identifiers
-            if ((nextLine = reader.readLine()) != null && (nextLine.contains("Reporter REF") || nextLine.contains("Composite Element REF") || nextLine.contains("Term Source REF") || nextLine.contains("CompositeElement REF") || nextLine.contains("TermSource REF") || nextLine.contains("Coordinates REF"))) {
-                int count = 0;
-                // check if this mage_tab data matrix can be parsed by this class
-                while ((nextLine = reader.readLine()) != null && count < 5) {
-                    nextLine = nextLine.trim();
-                    if (nextLine.startsWith("SNP_A") || nextLine.startsWith("CN_")) {
-                        return true;
-                    }
-
-                    count++;
-                }
-                return false;
-            }
-        } finally {
-            if (reader != null) {
-                reader.close();
-            }
-        }
-
-        return false;
-    }
-
     /**
      * Scan the datafile for chromosome breaks.
      *
@@ -395,13 +365,14 @@ public class IGVDatasetParser {
     public ChromosomeData loadChromosomeData(ChromosomeSummary chrSummary, String[] dataHeaders) {
 
         // InputStream is = null;
+        SeekableStream is = null;
         try {
             int skipColumns = hasCalls ? 2 : 1;
 
             // Get an estimate of the number of snps (rows).  THIS IS ONLY AN ESTIMATE
             int nRowsEst = chrSummary.getNDataPts();
 
-            SeekableStream is = IGVSeekableStreamFactory.getInstance().getStreamFor(dataResourceLocator.getPath());
+            is = IGVSeekableStreamFactory.getInstance().getStreamFor(dataResourceLocator.getPath());
             is.seek(chrSummary.getStartPosition());
             AsciiLineReader reader = new AsciiLineReader(is);
 
@@ -420,7 +391,7 @@ public class IGVDatasetParser {
             String chromosome = chrSummary.getName();
             boolean chromosomeStarted = false;
             String nextLine = reader.readLine();
-
+            int skippedLineCount = 0;
             while ((nextLine != null) && (nextLine.trim().length() > 0)) {
 
                 if (!nextLine.startsWith("#")) {
@@ -444,7 +415,7 @@ public class IGVDatasetParser {
 
                             startLocations.add(start);
 
-                            if(tokens.length <= firstDataColumn + (dataHeaders.length - 1)*skipColumns){
+                            if (tokens.length <= firstDataColumn + (dataHeaders.length - 1) * skipColumns) {
                                 String msg = "Line has too few data columns: " + nextLine;
                                 log.error(msg);
                                 throw new RuntimeException(msg);
@@ -463,9 +434,12 @@ public class IGVDatasetParser {
                         }
 
                     } catch (NumberFormatException numberFormatException) {
-
-                        // Skip line
-                        log.info("Skipping line (NumberFormatException) " + nextLine);
+                        if (skippedLineCount < 5) {
+                            skippedLineCount++;
+                            if (skippedLineCount == 5) {
+                                log.warn("Skipping line: " + nextLine + (skippedLineCount < 5 ? "" : " Further skipped lines will not be logged"));
+                            }
+                        }
                     }
                 }
 
@@ -487,8 +461,14 @@ public class IGVDatasetParser {
             return cd;
 
         } catch (IOException ex) {
-            log.error("Error parsing cn file", ex);
-            throw new RuntimeException("Error parsing cn file", ex);
+            log.error("Error parsing igv file " + this.dataResourceLocator.getPath(), ex);
+            throw new RuntimeException("Error parsing igv file", ex);
+        } finally {
+            try {
+                is.close();
+            } catch (IOException e) {
+                log.error("Error closing igv file " + this.dataResourceLocator.getPath(), e);
+            }
         }
 
     }
@@ -690,10 +670,6 @@ public class IGVDatasetParser {
                 data.put(h, new FloatArrayList(50000));
             }
         }
-
-        int size() {
-            return locations.size();
-        }
     }
 
 }
