@@ -25,7 +25,7 @@
 
 package org.broad.igv.feature.tribble;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.feature.*;
 import org.broad.igv.feature.genome.Genome;
@@ -47,13 +47,14 @@ import java.util.regex.Pattern;
  */
 public class IGVBEDCodec extends UCSCCodec<BasicFeature>  {
 
-    private static final Logger log = Logger.getLogger(IGVBEDCodec.class);
+    private static final Logger log = LogManager.getLogger(IGVBEDCodec.class);
 
     static final Pattern BR_PATTERN = Pattern.compile("<br>");
     static final Pattern EQ_PATTERN = Pattern.compile("=");
 
 
-    Genome genome;
+    private Genome genome;
+    private Pattern delimiter = null;
 
     public IGVBEDCodec() {
         this(null);
@@ -265,7 +266,12 @@ public class IGVBEDCodec extends UCSCCodec<BasicFeature>  {
             return null;
         }
 
-        tokens = Globals.whitespacePattern.split(trimLine);
+        // Bed files can be tab or whitespace delimited
+        if(delimiter == null) {
+            delimiter = trimLine.contains("\t") ?  Globals.multiTabPattern : Globals.whitespacePattern;
+        }
+
+        tokens = delimiter.split(trimLine);
         BasicFeature feature = decode(tokens);
         feature.setRepresentation(nextLine);
         return feature;
