@@ -29,7 +29,7 @@ import htsjdk.tribble.AsciiFeatureCodec;
 import htsjdk.tribble.readers.AsciiLineReader;
 import htsjdk.tribble.readers.LineIterator;
 import htsjdk.tribble.readers.LineIteratorImpl;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.exceptions.DataLoadException;
 import org.broad.igv.feature.Mutation;
@@ -40,6 +40,7 @@ import org.broad.igv.util.collections.MultiMap;
 
 import java.io.BufferedReader;
 import java.io.IOException;
+import java.io.InputStream;
 import java.util.Arrays;
 import java.util.HashSet;
 
@@ -52,7 +53,7 @@ import java.util.HashSet;
  */
 public class MUTCodec extends AsciiFeatureCodec<Mutation> {
 
-    private static Logger log = Logger.getLogger(MUTCodec.class);
+    private static Logger log = LogManager.getLogger(MUTCodec.class);
 
     private String path;  // for error messages
     private boolean isMAF;
@@ -74,11 +75,18 @@ public class MUTCodec extends AsciiFeatureCodec<Mutation> {
         super(Mutation.class);
         this.path = path;
         this.genome = genome;
+        InputStream is = null;
         try {
-            LineIterator reader = new LineIteratorImpl(new AsciiLineReader(ParsingUtils.openInputStream(path)));
+            is = ParsingUtils.openInputStream(path);
+            LineIterator reader = new LineIteratorImpl(new AsciiLineReader(is));
             readActualHeader(reader);
         } catch (IOException e) {
             log.error(e.getMessage(), e);
+        } finally {
+            try {
+                is.close();
+            } catch (IOException e) {
+            }
         }
     }
 
@@ -150,7 +158,7 @@ public class MUTCodec extends AsciiFeatureCodec<Mutation> {
                 if (errorCount > 100) {
                     throw new DataLoadException("Column " + (startColumn + 1) + " must be a numeric value.", path);
                 } else {
-                    log.info("Error parsing line: " + line);
+                    log.error("Error parsing line: " + line);
                     return null;
                 }
             }
@@ -163,7 +171,7 @@ public class MUTCodec extends AsciiFeatureCodec<Mutation> {
                 if (errorCount > 100) {
                     throw new DataLoadException("Column " + (endColumn + 1) + " must be a numeric value.", path);
                 } else {
-                    log.info("Error parsing line: " + line);
+                    log.warn("Error parsing line: " + line);
                     return null;
                 }
             }
@@ -355,17 +363,7 @@ Match_Norm_Seq_Allele2
      */
     public static boolean isMutationAnnotationFile(ResourceLocator locator) {
 
-        String ext;
-        {
-            //Only want pathLC when checking extension, so we limit its scope
-            String typeStringLC = locator.getTypeString().toLowerCase();
-            if (typeStringLC.endsWith(".maf.annotated")) {
-                // TCGA extension
-                return true;
-            }
-
-            ext = ParsingUtils.getIGVExtension(typeStringLC);
-        }
+        String ext = locator.getFormat();
 
         if (ext.equals("mut")) {
             return true;
