@@ -7,7 +7,8 @@ import java.util.Arrays;
  */
 final public class Constants {
 
-    private Constants() {}  // Prevent instantiation
+    private Constants() {
+    }  // Prevent instantiation
 
     // Preference sets
     public static final String NULL_CATEGORY = "NULL";
@@ -48,6 +49,7 @@ final public class Constants {
 
     public static final String GENOMES_SERVER_URL = "IGV.genome.sequence.dir";
     public static final String BLAT_URL = "BLAT_URL";
+    public static final String BLAT_SERVER_TYPE = "BLAT_SERVER_TYPE";
     public static final String EXTVIEW_URL = "EXTVIEW_URL";
     public static final String DATA_SERVER_URL_KEY = "MASTER_RESOURCE_FILE_KEY";
 
@@ -97,6 +99,7 @@ final public class Constants {
     public static final String SAM_ALLELE_USE_QUALITY = "SAM.ALLELE_USE_QUALITY";
     public static final String SAM_QUALITY_THRESHOLD = "SAM.QUALITY_THRESHOLD";
     public static final String SAM_ALIGNMENT_SCORE_THRESHOLD = "SAM.ALIGNMENT_SCORE_THRESHOLD";
+    public static final String SAM_BASEMOD_THRESHOLD = "SAM.BASEMOD_THRESHOLD";
     public static final String SAM_COMPUTE_ISIZES = "SAM.COMPUTE_ISIZES";
     public static final String SAM_MAX_INSERT_SIZE_THRESHOLD = "SAM.INSERT_SIZE_THRESHOLD";
     public static final String SAM_MIN_INSERT_SIZE_THRESHOLD = "SAM.MIN_INSERT_SIZE_THRESHOLD";
@@ -117,9 +120,9 @@ final public class Constants {
     public static final String SAM_HIDDEN_TAGS = "SAM.HIDDEN_TAGS";
     public static final String SAM_MAX_VISIBLE_RANGE = "SAM.MAX_VISIBLE_RANGE";
     public static final String SAM_FILTER_DUPLICATES = "SAM.FILTER_DUPLICATES";
-    public static final String SAM_SHOW_DUPLICATES = "SAM.SHOW_DUPLICATES";
     public static final String SAM_QUICK_CONSENSUS_MODE = "SAM.QUICK_CONSENSUS_MODE";
     public static final String SAM_SHOW_SOFT_CLIPPED = "SAM.SHOW_SOFT_CLIPPED";
+    public static final String SAM_MAX_SOFT_CLIP = "SAM.MAX_SOFT_CLIP";
     public static final String SAM_FLAG_UNMAPPED_PAIR = "SAM.FLAG_UNMAPPED_PAIR";
     public static final String SAM_SAMPLING_COUNT = "SAM.MAX_LEVELS"; // Sampling count
     public static final String SAM_SAMPLING_WINDOW = "SAM.SAMPLING_WINDOW";
@@ -132,6 +135,7 @@ final public class Constants {
     public static final String SAM_COLOR_BY_TAG = "SAM.COLOR_BY_TAG";
     public static final String SAM_SORT_BY_TAG = "SAM.SORT_BY_TAG";
     public static final String SAM_GROUP_BY_TAG = "SAM.GROUP_BY_TAG";
+    public static final String SAM_LINK_BY_TAGS = "SAM.LINK_BY_TAGS";
     public static final String SAM_GROUP_BY_POS = "SAM.GROUP_BY_POS";
     public static final String SAM_BISULFITE_CONTEXT = "SAM.BISULFITE_CONTEXT";
     public static final String SAM_FILTER_FAILED_READS = "SAM.FILTER_FAILED_READS";
@@ -151,17 +155,17 @@ final public class Constants {
     public static final String SAM_REDUCED_MEMORY_MODE = "SAM.REDUCED_MEMORY_MODE";
     public static final String SAM_HIDE_SMALL_INDEL = "SAM.HIDE_SMALL_INDEL";
     public static final String SAM_SMALL_INDEL_BP_THRESHOLD = "SAM.SMALL_INDEL_BP_THRESHOLD";
-    public static final String SAM_LINK_READS = "SAM.LINK_READS";
-    public static final String SAM_LINK_TAG = "SAM.LINK_TAG";
     public static final String SAM_SHOW_ALIGNMENT_TRACK = "SAM.SHOW_ALIGNMENT_TRACK";
     public static final String SAM_COLOR_A = "SAM.COLOR.A";
     public static final String SAM_COLOR_C = "SAM.COLOR.C";
     public static final String SAM_COLOR_T = "SAM.COLOR.T";
     public static final String SAM_COLOR_G = "SAM.COLOR.G";
     public static final String SAM_COLOR_N = "SAM.COLOR.N";
+    public static final String SAM_DISPLAY_MODE = "SAM.DISPLAY_MODE";
     public static final String KNOWN_SNPS = "KNOWN_SNPS_FILE";
 
     // Sequence track settings
+    public static final String SEQUENCE_TRANSLATION_STRAND = "SEQUENCE_TRANSLATION_STRAND";
     public static final String SHOW_SEQUENCE_TRANSLATION = "SHOW_SEQUENCE_TRANSLATION";
     public static final String MAX_SEQUENCE_RESOLUTION = "MAX_SEQUENCE_RESOLUTION";
     public static final String COLOR_A = "COLOR.A";
@@ -179,6 +183,10 @@ final public class Constants {
     public static final String AF_REF_COLOR = "AF_REF.COLOR";
     public static final String AF_VAR_COLOR = "AF_VAR.COLOR";
 
+    // Heatmap settings
+    public static final String NO_DATA_COLOR = "NO_DATA.COLOR";
+    public static final String NO_CALL_COLOR = "NO_CALL.COLOR";
+
     // "Mut" and "MAF" mutation track settings
     public static final String MUTATION_COLOR_TABLE = "MUTATION_COLOR_TABLE";
     public static final String MUTATION_INDEL_COLOR_KEY = "MUTATION_INDEL_COLOR_KEY";
@@ -250,15 +258,18 @@ final public class Constants {
     public static final String DB_NAME = "DB_NAME";
     public static final String DB_PORT = "DB_PORT";
 
-    // Letter of support dialog
-    public static final String SHOW_LOS = "showLOS";
+    // OAuth provisioning
+    public static final String PROVISIONING_URL = "PROVISIONING.URL";
+
+    // JBrowse circular view integration
+    public static final String CIRC_VIEW_ENABLED = "CIRC_VIEW_ENABLED";
+    public static final String CIRC_VIEW_PORT = "CIRC_VIEW_PORT";
+    public static final String CIRC_VIEW_HOST = "CIRC_VIEW_HOST";
 
     // Experimental
     public static final String SCORE_VARIANTS = "SCORE_VARIANTS";
 
 
-
-
     /**
      * List of keys that affect the alignments loaded.  This list is used to trigger a reload, if required.
      * Not all alignment preferences need trigger a reload, this is a subset.
@@ -269,7 +280,7 @@ final public class Constants {
             SAM_FILTER_ALIGNMENTS,
             SAM_FILTER_URL,
             SAM_MAX_VISIBLE_RANGE,
-            SAM_SHOW_DUPLICATES,
+            SAM_FILTER_DUPLICATES,
             SAM_SHOW_SOFT_CLIPPED,
             SAM_SAMPLING_COUNT,
             SAM_SAMPLING_WINDOW,
@@ -278,7 +289,8 @@ final public class Constants {
             SAM_FILTER_SECONDARY_ALIGNMENTS,
             SAM_FILTER_SUPPLEMENTARY_ALIGNMENTS,
             SAM_JUNCTION_MIN_FLANKING_WIDTH,
-            SAM_JUNCTION_MIN_COVERAGE
+            SAM_JUNCTION_MIN_COVERAGE,
+            SAM_BASEMOD_THRESHOLD
     );
 
     /**
@@ -290,7 +302,23 @@ final public class Constants {
             SAM_ALLELE_THRESHOLD,
             SAM_FLAG_LARGE_INDELS,
             SAM_LARGE_INDELS_THRESHOLD,
-            SAM_SHOW_INSERTION_MARKERS
+            SAM_SHOW_INSERTION_MARKERS,
+            SAM_GROUP_OPTION,
+            SAM_GROUP_BY_TAG
+    );
+
+    /**
+     * List of keys that affect proxy usage
+     */
+    static java.util.List<String> PROXY_KEYS = Arrays.asList(
+            USE_PROXY,
+            PROXY_AUTHENTICATE,
+            PROXY_HOST,
+            PROXY_PORT,
+            PROXY_PW,
+            PROXY_TYPE,
+            PROXY_USER,
+            PROXY_WHITELIST
     );
 
 }
