@@ -26,8 +26,10 @@
 package org.broad.igv.renderer;
 
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.feature.*;
+import org.broad.igv.feature.aa.AminoAcidSequence;
+import org.broad.igv.feature.aa.CodonAA;
 import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.feature.genome.GenomeManager;
 import org.broad.igv.prefs.PreferencesManager;
@@ -51,7 +53,7 @@ import java.util.List;
  */
 public class IGVFeatureRenderer extends FeatureRenderer {
 
-    private static Logger log = Logger.getLogger(IGVFeatureRenderer.class);
+    private static Logger log = LogManager.getLogger(IGVFeatureRenderer.class);
 
     // Constants
     static protected final int NORMAL_STRAND_Y_OFFSET = 14;
@@ -137,6 +139,9 @@ public class IGVFeatureRenderer extends FeatureRenderer {
 
 
             boolean alternateExonColor = (track instanceof FeatureTrack && ((FeatureTrack) track).isAlternateExonColor());
+            Color trackPosColor = track.getColor();
+            Color trackNegColor = alternateExonColor ? track.getColor() : track.getAltColor();
+
 
             for (IGVFeature feature : featureList) {
 
@@ -186,7 +191,7 @@ public class IGVFeatureRenderer extends FeatureRenderer {
                     continue;
                 }
 
-                Color color = getFeatureColor(feature, track);
+                Color color = getFeatureColor(feature, track, trackPosColor, trackNegColor);
                 Graphics2D g2D = context.getGraphic2DForColor(color);
 
                 // Draw block representing entire feature
@@ -238,7 +243,7 @@ public class IGVFeatureRenderer extends FeatureRenderer {
 
 
                 // Draw name , if there is room
-                if (displayMode != Track.DisplayMode.SQUISHED) {
+                if (displayMode != Track.DisplayMode.SQUISHED && track.isShowFeatureNames()) {
                     String name = feature.getName();
                     if (name != null) {
                         // Limit name display length
@@ -416,6 +421,7 @@ public class IGVFeatureRenderer extends FeatureRenderer {
 
             Graphics2D blockGraphics = g2D;
             Graphics2D edgeGraphics = context.getGraphic2DForColor(Color.gray);
+
             if (alternateExonColor) {
                 Color color = colorToggle ? color1 : color2;
                 blockGraphics = context.getGraphic2DForColor(color);
@@ -429,14 +435,13 @@ public class IGVFeatureRenderer extends FeatureRenderer {
             int pEnd = getPixelFromChromosomeLocation(exon.getChr(), exon.getEnd(), theOrigin, locationScale);
 
 
-            Graphics2D arrowGraphics = context.getGraphic2DForColor(Color.blue);
             //We draw connecting lines/arrows from previous exon to this one.
             //Exons may not be strictly sorted, we avoid double-drawing using maxLineEndX
             if (drawConnectingLine && lastExonEndX > Integer.MIN_VALUE && lastY > Integer.MIN_VALUE
                     && lastExonEndX >= maxLineEndX) {
                 drawConnectingLine(lastExonEndX, lastY, pStart, curYOffset, exon.getStrand(), blockGraphics);
                 double angle = Math.atan(-(curYOffset - lastY) / ((pStart - lastExonEndX) + 1e-12));
-                drawStrandArrows(gene.getStrand(), lastExonEndX, pStart, lastY, angle, mode, trackRectangle, arrowGraphics);
+                drawStrandArrows(gene.getStrand(), lastExonEndX, pStart, lastY, angle, mode, trackRectangle, blockGraphics);
                 maxLineEndX = Math.max(maxLineEndX, pStart);
             }
             lastExonEndX = pEnd;
@@ -572,8 +577,12 @@ public class IGVFeatureRenderer extends FeatureRenderer {
 
     }
 
-    final private int drawFeatureName(IGVFeature feature, Track.DisplayMode mode, int pixelStart, int pixelEnd,
-                                      int lastFeatureEndedAtPixelX, Graphics2D g2D,
+    final private int drawFeatureName(IGVFeature feature,
+                                      Track.DisplayMode mode,
+                                      int pixelStart,
+                                      int pixelEnd,
+                                      int lastFeatureEndedAtPixelX,
+                                      Graphics2D g2D,
                                       int textBaselineY) {
 
         String name = feature.getName();
@@ -632,7 +641,7 @@ public class IGVFeatureRenderer extends FeatureRenderer {
             int aaSeqStartPosition = aaSequence.getStart();
             boolean odd =  exon.getAminoAcidNumber(exon.getCdStart()) % 2 == 1;
 
-            for (AminoAcid acid : aaSequence.getSequence()) {
+            for (CodonAA acid : aaSequence.getSequence()) {
                 if (acid != null) {
 
                     int start = Math.max(exon.getStart(), aaSeqStartPosition);
@@ -647,17 +656,18 @@ public class IGVFeatureRenderer extends FeatureRenderer {
 
 
                         Graphics2D bgGraphics = context.getGraphic2DForColor(odd ? AA_COLOR_1 : AA_COLOR_2);
-                        if (((acid.getSymbol() == 'M') && (((gene.getStrand() == Strand.POSITIVE) &&
+                        char aaSymbol = acid.getAminoAcid().getSymbol();
+                        if (((aaSymbol == 'M') && (((gene.getStrand() == Strand.POSITIVE) &&
                                 (aaSeqStartPosition == exon.getCdStart())) || ((gene.getStrand() == Strand.NEGATIVE) &&
                                 (aaSeqStartPosition == exon.getCdEnd() - 3))))) {
                             bgGraphics = context.getGraphic2DForColor(Color.green);
-                        } else if (acid.getSymbol() == '*') {
+                        } else if (aaSymbol == '*') {
                             bgGraphics = context.getGraphic2DForColor(Color.RED);
                         }
 
                         bgGraphics.fill(aaRect);
 
-                        String tmp = new String(new char[]{acid.getSymbol()});
+                        String tmp = new String(new char[]{aaSymbol});
                         GraphicUtils.drawCenteredText(tmp, aaRect, fontGraphics);
                     }
                     odd = !odd;
@@ -689,26 +699,34 @@ public class IGVFeatureRenderer extends FeatureRenderer {
         return "Basic Feature";
     }
 
-    protected Color getFeatureColor(IGVFeature feature, Track track) {
-        // Set color used to draw the feature
+    protected Color getFeatureColor(IGVFeature feature, Track track, Color defaultPosColor, Color defaultNegColor) {
 
-        Color color = null;
-        if (track.isItemRGB()) {
+        // Set color used to draw the feature
+         Color color = null;
+
+        // If an alt color is explicitly set use it for negative strand features;
+        if(feature.getStrand() == Strand.NEGATIVE) {
+            color = track.getExplicitAltColor();
+        }
+
+        // If color is explicitly set use it
+        if(color == null) {
+             color = track.getExplicitColor();
+        }
+
+        // No explicitly set color, try the feature itself
+        if(color == null && track.isItemRGB()) {
             color = feature.getColor();
         }
 
+        // If still no color use defaults
         if (color == null) {
             // TODO -- hack, generalize this
             if (track.getTrackType() == TrackType.CNV) {
                 color = feature.getName().equals("gain") ? DULL_RED : DULL_BLUE;
             } else {
-                // Only used if feature color is not set
-                Color altColor = track.getAltColor();
-                color =  feature.getStrand() == Strand.NEGATIVE && altColor != null ?
-                        track.getAltColor() :
-                        track.getColor();
+                color = feature.getStrand() == Strand.NEGATIVE ? defaultNegColor : defaultPosColor;
             }
-
         }
 
         if (track.isUseScore()) {
