@@ -25,8 +25,10 @@
 
 package org.broad.igv.sam;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
+import org.broad.igv.event.IGVEventBus;
+import org.broad.igv.event.IGVEventObserver;
 import org.broad.igv.event.RefreshEvent;
 import org.broad.igv.feature.Chromosome;
 import org.broad.igv.feature.Range;
@@ -34,15 +36,12 @@ import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.prefs.IGVPreferences;
 import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.sam.AlignmentTrack.SortOption;
+import org.broad.igv.sam.reader.AlignmentReader;
 import org.broad.igv.sam.reader.AlignmentReaderFactory;
-import org.broad.igv.track.RenderContext;
-import org.broad.igv.event.IGVEventBus;
-import org.broad.igv.event.IGVEventObserver;
 import org.broad.igv.track.Track;
 import org.broad.igv.ui.panel.FrameManager;
 import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.util.ResourceLocator;
-import org.broad.igv.util.collections.IntArrayList;
 
 import java.io.IOException;
 import java.util.*;
@@ -56,24 +55,29 @@ import static org.broad.igv.prefs.Constants.*;
 
 public class AlignmentDataManager implements IGVEventObserver {
 
-    private static Logger log = Logger.getLogger(AlignmentDataManager.class);
+    private static Logger log = LogManager.getLogger(AlignmentDataManager.class);
+    private final AlignmentReader reader;
 
 
+    private AlignmentTrack alignmentTrack;
+    private CoverageTrack coverageTrack;
+    private Set<Track> subscribedTracks;
+
     private List<AlignmentInterval> intervalCache;
     private ResourceLocator locator;
     private HashMap<String, String> chrMappings = new HashMap();
-    private Set<Range> isLoading = new HashSet<>();
-    private AlignmentTileLoader reader;
-    private CoverageTrack coverageTrack;
+    private AlignmentTileLoader loader;
     private Map<String, PEStats> peStats;
     private SpliceJunctionHelper.LoadOptions loadOptions;
-    private Object loadLock = new Object();
-    AlignmentTrack.ExperimentType inferredExperimentType;
-    private Set<Track> subscribedTracks;
+    private Range currentlyLoading;
 
     public AlignmentDataManager(ResourceLocator locator, Genome genome) throws IOException {
         this.locator = locator;
-        reader = new AlignmentTileLoader(AlignmentReaderFactory.getReader(locator));
+        // The time-gated limit for an AWS signed URL has expired, we need to re-sign the URL with the newly acquired
+        // access token, otherwise we will face an Access Denied error. CheckReader() provides a very low overhead
+        // mechanism to refresh expired presigned URLs.
+        reader = AlignmentReaderFactory.getReader(locator);
+        loader = new AlignmentTileLoader(reader);
         peStats = new HashMap();
         initLoadOptions();
         initChrMap(genome);
@@ -85,13 +89,12 @@ public class AlignmentDataManager implements IGVEventObserver {
     }
 
     public void receiveEvent(Object event) {
-
         if (event instanceof FrameManager.ChangeEvent) {
             trimCache();
         } else if (event instanceof RefreshEvent) {
             clear();
         } else {
-            log.info("Unknown event type: " + event.getClass());
+            log.warn("Unknown event type: " + event.getClass());
         }
     }
 
@@ -102,6 +105,7 @@ public class AlignmentDataManager implements IGVEventObserver {
     public void unsubscribe(Track track) {
         subscribedTracks.remove(track);
         if (subscribedTracks.isEmpty()) {
+            dispose();
             dumpAlignments();
             IGVEventBus.getInstance().unsubscribe(this);
         }
@@ -122,17 +126,17 @@ public class AlignmentDataManager implements IGVEventObserver {
             // Build a chr size -> name lookup table.   We will assume sizes are unique.  This will be used if no alias
             // is defined for a sequence.
             Map<Long, String> inverseDict = null;
-            Map<String, Long> sequenceDictionary = reader.getSequenceDictionary();
+            Map<String, Long> sequenceDictionary = getLoader().getSequenceDictionary();
 
             if (sequenceDictionary != null) {
 
                 Set<Long> nonUnique = new HashSet<>();
                 Set<Long> seen = new HashSet<>();
                 // First find sequences whose size are not unique,  we'll filter these
-                for(Long size : sequenceDictionary.values()) {
-                    if(seen.contains(size)) {
+                for (Long size : sequenceDictionary.values()) {
+                    if (seen.contains(size)) {
                         nonUnique.add(size);
-                    } else{
+                    } else {
                         seen.add(size);
                     }
                 }
@@ -141,20 +145,20 @@ public class AlignmentDataManager implements IGVEventObserver {
 
                 for (Chromosome chromosome : genome.getChromosomes()) {
 
-                    Long size = new Long(chromosome.getLength());
+                    Long size = (long) chromosome.getLength();
                     if (!nonUnique.contains(size)) {
                         if (inverseDict.containsKey(size)) {
                             inverseDict.remove(size);
                             nonUnique.add(size);
                         } else {
-                            inverseDict.put(new Long(size), chromosome.getName());
+                            inverseDict.put(size, chromosome.getName());
                         }
                     }
                 }
             }
 
 
-            List<String> seqNames = reader.getSequenceNames();
+            List<String> seqNames = getLoader().getSequenceNames();
             if (seqNames != null) {
                 for (String seq : seqNames) {
 
@@ -177,8 +181,8 @@ public class AlignmentDataManager implements IGVEventObserver {
         return chrMappings.size() > 0;
     }
 
-    public AlignmentTileLoader getReader() {
-        return reader;
+    public AlignmentTileLoader getLoader() {
+        return loader;
     }
 
     public ResourceLocator getLocator() {
@@ -190,23 +194,19 @@ public class AlignmentDataManager implements IGVEventObserver {
     }
 
     public boolean isPairedEnd() {
-        return reader.isPairedEnd();
+        return getLoader().isPairedEnd();
     }
 
     public boolean hasYCTags() {
-        return reader.hasYCTags();
+        return getLoader().hasYCTags();
     }
 
     public boolean hasIndex() {
-        return reader.hasIndex();
+        return getLoader().hasIndex();
     }
 
-    public void setInferredExperimentType(AlignmentTrack.ExperimentType inferredExperimentType) {
-        if (inferredExperimentType != this.inferredExperimentType) {
-            ExperimentTypeChangeEvent event = new ExperimentTypeChangeEvent(this, inferredExperimentType);
-            this.inferredExperimentType = inferredExperimentType;
-            IGVEventBus.getInstance().post(event);
-        }
+    public void setAlignmentTrack(AlignmentTrack alignmentTrack) {
+        this.alignmentTrack = alignmentTrack;
     }
 
     public void setCoverageTrack(CoverageTrack coverageTrack) {
@@ -218,9 +218,22 @@ public class AlignmentDataManager implements IGVEventObserver {
     }
 
     public double getMinVisibleScale() {
-        IGVPreferences prefs = PreferencesManager.getPreferences();
-        float maxRange = prefs.getAsFloat(SAM_MAX_VISIBLE_RANGE);
-        return (maxRange * 1000) / 700;
+        return getVisibilityWindow() / 700;
+    }
+
+    public double getVisibilityWindow() {
+        return getPreferences().getAsFloat(SAM_MAX_VISIBLE_RANGE) * 1000;
+    }
+
+    private IGVPreferences getPreferences() {
+        String category = NULL_CATEGORY;
+        AlignmentTrack.ExperimentType experimentType = getExperimentType();
+        if (experimentType == AlignmentTrack.ExperimentType.RNA) {
+            category = RNA;
+        } else if (experimentType == AlignmentTrack.ExperimentType.THIRD_GEN) {
+            category = THIRD_GEN;
+        }
+        return PreferencesManager.getPreferences(category);
     }
 
 
@@ -231,22 +244,31 @@ public class AlignmentDataManager implements IGVEventObserver {
      * @return
      */
     public List<String> getSequenceNames() throws IOException {
-        return reader.getSequenceNames();
+        return getLoader().getSequenceNames();
     }
 
     public Map<String, Long> getSequenceDictionary() {
-        return reader.getSequenceDictionary();
+        return getLoader().getSequenceDictionary();
     }
 
-
     public AlignmentInterval getLoadedInterval(ReferenceFrame frame) {
+        return getLoadedInterval(frame, false);
+    }
 
+    public AlignmentInterval getLoadedInterval(ReferenceFrame frame, boolean includeOverlaps) {
         for (AlignmentInterval interval : intervalCache) {
             if (interval.contains(frame.getCurrentRange())) {
                 return interval;
             }
         }
-
+        // No contains, look for overlap
+        if (includeOverlaps) {
+            for (AlignmentInterval interval : intervalCache) {
+                if (interval.overlaps(frame.getCurrentRange())) {
+                    return interval;
+                }
+            }
+        }
         return null;
     }
 
@@ -271,7 +293,7 @@ public class AlignmentDataManager implements IGVEventObserver {
                 for (Row row : alignmentRows) {
                     row.updateScore(option, location, interval, tag);
                 }
-                Collections.sort(alignmentRows);
+                Collections.sort(alignmentRows, (o1, o2) -> (int) Math.signum(o1.getScore() - o2.getScore()));
             }
             return true;
         }
@@ -303,50 +325,44 @@ public class AlignmentDataManager implements IGVEventObserver {
         return getLoadedInterval(frame) != null;
     }
 
-    public boolean isLoading(ReferenceFrame frame) {
-
-//        Range range = frame.getCurrentRange();
-//        for (Range r : isLoading) {
-//            if (r.contains(range)) return true;
-//        }
-        return false;
-    }
-
-
-    public void load(ReferenceFrame referenceFrame,
+    public void load(ReferenceFrame frame,
                      AlignmentTrack.RenderOptions renderOptions,
                      boolean expandEnds) {
 
-        if (isLoaded(referenceFrame)) return;  // Already loaded
+        if (frame.getChrName().equals(Globals.CHR_ALL) || frame.getScale() > getMinVisibleScale())
+            return; // should not happen
 
-        if (isLoading(referenceFrame)) return;   // Already oading
+        if (isLoaded(frame)) {
+            return;  // Already loaded
+        }
 
-        synchronized (loadLock) {
-            Range range = referenceFrame.getCurrentRange();
+        Range range = frame.getCurrentRange();
+        final String chr = frame.getChrName();
+        final int start = range.getStart();
+        final int end = range.getEnd();
+        int adjustedStart = start;
+        int adjustedEnd = end;
+        Range adjustedRange = new Range(chr, start, end);
 
-            isLoading.add(range);
-
-            final String chr = referenceFrame.getChrName();
-
-            final int start = (int) range.getStart();
-            final int end = (int) range.getEnd();
-            int adjustedStart = start;
-            int adjustedEnd = end;
+        if (currentlyLoading != null && currentlyLoading.contains(adjustedRange)) {
+            return;  // Already loading
+        }
+        try {
+            currentlyLoading = adjustedRange;
 
             // Expand the interval by the lesser of  +/- a 2 screens, or max visible range
             int windowSize = Math.min(4 * (end - start), PreferencesManager.getPreferences().getAsInt(SAM_MAX_VISIBLE_RANGE) * 1000);
-            int center = (end + start) / 2;
-            int expand = Math.max(end - start, windowSize / 2);
+            int center = start + (end - start) / 2;     // Be careful how you calculate this -- potential overflow for large chromosomes
+            int expand = Math.min(Integer.MAX_VALUE - center, Math.max(end - start, windowSize / 2));
 
             if (expandEnds) {
                 adjustedStart = Math.max(0, Math.min(start, center - expand));
                 adjustedEnd = Math.max(end, center + expand);
+                if(adjustedEnd < 0) {
+                    adjustedEnd = Integer.MAX_VALUE;  // Overflow
+                }
             }
 
-
-            log.debug("Loading alignments: " + chr + ":" + adjustedStart + "-" + adjustedEnd + " for " + AlignmentDataManager.this);
-
-
             AlignmentInterval loadedInterval = loadInterval(chr, adjustedStart, adjustedEnd, renderOptions);
 
             trimCache();
@@ -354,27 +370,27 @@ public class AlignmentDataManager implements IGVEventObserver {
             intervalCache.add(loadedInterval);
 
             packAlignments(renderOptions);
-            isLoading.remove(range);
-
-            //  IGVEventBus.getInstance().post(new DataLoadedEvent(referenceFrame));
 
+        } finally {
+            currentlyLoading = null;
         }
-    }
 
+        //  IGVEventBus.getInstance().post(new DataLoadedEvent(frame));
+
+    }
 
     /**
      * Remove out-of-view intervals from the cache.  This is O(N) where N = #frames X #intervals.   It is assumed
      * that N is small
      */
-    private synchronized void trimCache() {
-
-        Iterator<AlignmentInterval> iter = intervalCache.iterator();
-        while (iter.hasNext()) {
-            AlignmentInterval interval = iter.next();
-            if (!intervalInView(interval)) {
-                iter.remove();
+    private void trimCache() {
+        List<AlignmentInterval> trimmedIntervals = new ArrayList<>();
+        for (AlignmentInterval interval : intervalCache) {
+            if (intervalInView(interval)) {
+                trimmedIntervals.add(interval);
             }
         }
+        intervalCache = trimmedIntervals;
     }
 
 
@@ -400,43 +416,45 @@ public class AlignmentDataManager implements IGVEventObserver {
 
         SpliceJunctionHelper spliceJunctionHelper = new SpliceJunctionHelper(this.loadOptions);
 
-        ReadStats readStats = new ReadStats();
-
-        AlignmentTileLoader.AlignmentTile t = reader.loadTile(sequence, start, end, spliceJunctionHelper,
-                downsampleOptions, readStats, peStats, bisulfiteContext);
-//
-        if (inferredExperimentType == null) {
-            readStats.compute();
-            inferType(readStats);
-        }
-
-        List<Alignment> alignments = t.getAlignments();
+        AlignmentTileLoader.AlignmentTile t = getLoader().loadTile(sequence, start, end, spliceJunctionHelper,
+                downsampleOptions, peStats, bisulfiteContext, renderOptions);
+      List<Alignment> alignments = t.getAlignments();
         List<DownsampledInterval> downsampledIntervals = t.getDownsampledIntervals();
         return new AlignmentInterval(chr, start, end, alignments, t.getCounts(), spliceJunctionHelper, downsampledIntervals);
     }
 
-    /**
-     * Some empirical metrics for determining experiment type
-     *
-     * @param readStats
-     */
-    private void inferType(ReadStats readStats) {
+    public AlignmentTrack.ExperimentType inferType() {
+        ReadStats readStats = new ReadStats();
+        List<Alignment> sample = AlignmentUtils.firstAlignments(reader, 100);
+        for(Alignment a : sample) {
+            readStats.addAlignment(a);
+        }
+        AlignmentTrack.ExperimentType type = readStats.inferType();
 
-        if (readStats.readLengthStdDev > 100 || readStats.medianReadLength > 1000) {
-            setInferredExperimentType(AlignmentTrack.ExperimentType.THIRD_GEN);  // Could also use fracReadsWithIndels
-        } else if (readStats.medianRefToReadRatio > 10) {
-            setInferredExperimentType(AlignmentTrack.ExperimentType.RNA);
+        if(type == AlignmentTrack.ExperimentType.THIRD_GEN) {
+            return type;
         } else {
-            setInferredExperimentType(AlignmentTrack.ExperimentType.OTHER);
+            // Get a larger sample to distinguish RNA-Seq
+            readStats = new ReadStats();
+            sample = AlignmentUtils.firstAlignments(reader, 2000);
+            for(Alignment a : sample) {
+                readStats.addAlignment(a);
+            }
+            return readStats.inferType();
         }
     }
 
+    private AlignmentTrack.ExperimentType getExperimentType() {
+        return alignmentTrack == null ? null : alignmentTrack.getExperimentType();
+    }
 
-    public synchronized PackedAlignments getGroups(RenderContext context, AlignmentTrack.RenderOptions renderOptions) {
-        //   load(context.getReferenceFrame(), renderOptions, false);
-        //   Range range = context.getReferenceFrame().getCurrentRange();
+    private void setExperimentType(AlignmentTrack.ExperimentType type) {
+        if (alignmentTrack != null) alignmentTrack.setExperimentType(type);
+    }
 
-        AlignmentInterval interval = getLoadedInterval(context.getReferenceFrame());
+
+    public PackedAlignments getGroups(AlignmentInterval interval, AlignmentTrack.RenderOptions renderOptions) {
+        //AlignmentInterval interval = getLoadedInterval(context.getReferenceFrame());
         if (interval != null) {
             return interval.getPackedAlignments();
         } else {
@@ -459,15 +477,15 @@ public class AlignmentDataManager implements IGVEventObserver {
      * return the grouped alignments
      *
      * @param position
-     * @param referenceFrame
+     * @param frame
      * @return alignmentRows, grouped and ordered by key
      */
-    public PackedAlignments getGroupedAlignmentsContaining(double position, ReferenceFrame referenceFrame) {
-        String chr = referenceFrame.getChrName();
+    public PackedAlignments getGroupedAlignmentsContaining(double position, ReferenceFrame frame) {
+        String chr = frame.getChrName();
         int start = (int) position;
         int end = start + 1;
 
-        AlignmentInterval interval = getLoadedInterval(referenceFrame);
+        AlignmentInterval interval = getLoadedInterval(frame);
         if (interval == null) {
             return null;
         } else {
@@ -511,17 +529,15 @@ public class AlignmentDataManager implements IGVEventObserver {
         return groupCount;
     }
 
-    @Override
-    protected void finalize() throws Throwable {
-        super.finalize();
-        if (reader != null) {
+
+    private void dispose() {
+        if (loader != null) {
             try {
-                reader.close();
+                loader.close();
             } catch (IOException ex) {
                 log.error("Error closing AlignmentQueryReader. ", ex);
             }
         }
-
     }
 
     public void updatePEStats(AlignmentTrack.RenderOptions renderOptions) {
@@ -548,22 +564,21 @@ public class AlignmentDataManager implements IGVEventObserver {
     }
 
     public boolean isTenX() {
-        return reader.isTenX();
+        return getLoader().isTenX();
     }
 
     public boolean isPhased() {
-        return reader.isPhased();
+        return getLoader().isPhased();
     }
 
     public boolean isMoleculo() {
-        return reader.isMoleculo();
+        return getLoader().isMoleculo();
     }
 
     public Collection<AlignmentInterval> getLoadedIntervals() {
         return intervalCache;
     }
 
-
     public static class DownsampleOptions {
         private boolean downsample;
         private int sampleWindowSize;
