@@ -29,10 +29,9 @@
  */
 package org.broad.igv.sam;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.feature.Range;
 import org.broad.igv.feature.Strand;
-import org.broad.igv.sam.AlignmentTrack.GroupOption;
 
 import java.util.*;
 
@@ -43,7 +42,7 @@ import java.util.*;
  */
 public class AlignmentPacker {
 
-    private static final Logger log = Logger.getLogger(AlignmentPacker.class);
+    private static final Logger log = LogManager.getLogger(AlignmentPacker.class);
 
     /**
      * Minimum gap between the end of one alignment and start of another.
@@ -266,15 +265,11 @@ public class AlignmentPacker {
 
         for (Alignment a : alList) {
 
-            if(a.isPrimary()) {
+            if (a.isPrimary()) {
                 Object bc;
-                if("READNAME".equals(tag)) {
+                if ("READNAME".equals(tag)) {
                     bc = a.getReadName();
-                    if(a.isPaired()) {
-                        bc += a.isFirstOfPair() ? "/1" : "/2";
-                    }
-                }
-                else {
+                } else {
                     bc = a.getAttribute(tag);
                 }
 
@@ -289,27 +284,24 @@ public class AlignmentPacker {
                     }
                     linkedAlignment.addAlignment(a);
                 }
-            }
-            else {
-                // Don't link secondary reads
+            } else {
+                // Don't link secondary (i.e alternative) alignments
                 bcList.add(a);
             }
         }
 
         // Now copy list, de-linking orhpaned alignments (alignments with no linked mates)
         List<Alignment> delinkedList = new ArrayList<>(alList.size());
-        for(Alignment a : bcList) {
-            if(a instanceof LinkedAlignment) {
+        for (Alignment a : bcList) {
+            if (a instanceof LinkedAlignment) {
                 final List<Alignment> alignments = ((LinkedAlignment) a).alignments;
-                if(alignments.size() == 1) {
+                if (alignments.size() == 1) {
                     delinkedList.add(alignments.get(0));
-                }
-                else {
+                } else {
                     a.finish();
                     delinkedList.add(a);
                 }
-            }
-            else {
+            } else {
                 delinkedList.add(a);
             }
         }
@@ -364,16 +356,13 @@ public class AlignmentPacker {
                                 if (o1 instanceof Integer && o2 instanceof Integer) {
                                     Integer i1 = (Integer) o1, i2 = (Integer) o2;
                                     return i1.compareTo(i2);
-                                }
-                                else if (o1 instanceof Float && o2 instanceof Float) {
+                                } else if (o1 instanceof Float && o2 instanceof Float) {
                                     Float f1 = (Float) o1, f2 = (Float) o2;
                                     return f1.compareTo(f2);
-                                }
-                                else if (o1 instanceof Double && o2 instanceof Double) {
+                                } else if (o1 instanceof Double && o2 instanceof Double) {
                                     Double d1 = (Double) o1, d2 = (Double) o2;
                                     return d1.compareTo(d2);
-                                }
-                                else {
+                                } else {
                                     String s1 = o1.toString(), s2 = o2.toString();
                                     return s1.compareToIgnoreCase(s2);
                                 }
@@ -402,21 +391,31 @@ public class AlignmentPacker {
                 return al.getLibrary();
             case READ_GROUP:
                 return al.getReadGroup();
+            case LINKED:
+                return (al instanceof LinkedAlignment) ? "Linked" : "";
+            case PHASE:
+                return al.getAttribute("HP");
             case TAG:
-                Object tagValue = al.getAttribute(tag);
+                Object tagValue = tag == null ? null : al.getAttribute(tag);
                 if (tagValue == null) {
                     return null;
-                }
-                else if (tagValue instanceof Integer || tagValue instanceof Float || tagValue instanceof Double) {
+                } else if (tagValue instanceof Integer || tagValue instanceof Float || tagValue instanceof Double) {
                     return tagValue;
-                }
-                else {
+                } else {
                     return tagValue.toString();
                 }
             case FIRST_OF_PAIR_STRAND:
                 Strand strand = al.getFirstOfPairStrand();
                 String strandString = strand == Strand.NONE ? null : strand.toString();
                 return strandString;
+            case READ_ORDER:
+                if (al.isPaired() && al.isFirstOfPair()) {
+                    return "FIRST";
+                } else if (al.isPaired() && al.isSecondOfPair()) {
+                    return "SECOND";
+                } else {
+                    return "";
+                }
             case PAIR_ORIENTATION:
                 PEStats peStats = AlignmentRenderer.getPEStats(al, renderOptions);
                 AlignmentTrack.OrientationType type = AlignmentRenderer.getOrientationType(al, peStats);
@@ -436,25 +435,29 @@ public class AlignmentPacker {
                 }
             case SUPPLEMENTARY:
                 return al.isSupplementary() ? "SUPPLEMENTARY" : "";
+            case REFERENCE_CONCORDANCE:
+                return !al.isProperPair() ||
+                        al.getCigarString().toUpperCase().contains("S") ||
+                        al.isSupplementary() ?
+                        "DISCORDANT": "";
             case BASE_AT_POS:
                 // Use a string prefix to enforce grouping rules:
                 //    1: alignments with a base at the position
                 //    2: alignments with a gap at the position
                 //    3: alignment that do not overlap the position (or are on a different chromosome)
 
-                if (al.getChr().equals(pos.getChr()) &&
-                    al.getAlignmentStart() <= pos.getStart() &&
-                    al.getAlignmentEnd() > pos.getStart()) {
+                if (pos != null &&
+                        al.getChr().equals(pos.getChr()) &&
+                        al.getAlignmentStart() <= pos.getStart() &&
+                        al.getAlignmentEnd() > pos.getStart()) {
 
-                    byte[] baseAtPos = new byte[] {al.getBase(pos.getStart())};
+                    byte[] baseAtPos = new byte[]{al.getBase(pos.getStart())};
                     if (baseAtPos[0] == 0) { // gap at position
                         return "2:";
-                    }
-                    else { // base at position
+                    } else { // base at position
                         return "1:" + new String(baseAtPos);
                     }
-                }
-                else { // does not overlap position
+                } else { // does not overlap position
                     return "3:";
                 }
             case MOVIE: // group PacBio reads by movie
@@ -541,7 +544,7 @@ public class AlignmentPacker {
             while (bucketNumber < bucketArray.length) {
 
                 if (bucketNumber < 0) {
-                    log.info("Negative bucket number: " + bucketNumber);
+                    log.warn("Negative bucket number: " + bucketNumber);
                 }
 
                 bucket = bucketArray[bucketNumber];
