@@ -27,7 +27,7 @@ package org.broad.igv.sam;
 
 import htsjdk.samtools.SAMFileHeader;
 import htsjdk.samtools.util.CloseableIterator;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.prefs.IGVPreferences;
 import org.broad.igv.prefs.PreferencesManager;
@@ -44,6 +44,7 @@ import org.broad.igv.util.RuntimeUtils;
 import javax.swing.*;
 import java.io.IOException;
 import java.lang.ref.WeakReference;
+import java.text.DecimalFormat;
 import java.util.*;
 
 import static org.broad.igv.prefs.Constants.*;
@@ -55,7 +56,9 @@ import static org.broad.igv.prefs.Constants.*;
  */
 public class AlignmentTileLoader implements IGVEventObserver {
 
-    private static Logger log = Logger.getLogger(AlignmentTileLoader.class);
+    private static Logger log = LogManager.getLogger(AlignmentTileLoader.class);
+
+    static DecimalFormat df = new DecimalFormat("###,###,###");
 
     private static Set<WeakReference<AlignmentTileLoader>> activeLoaders = Collections.synchronizedSet(new HashSet());
 
@@ -103,7 +106,7 @@ public class AlignmentTileLoader implements IGVEventObserver {
         return reader.getSequenceNames();
     }
 
-    public CloseableIterator<Alignment> iterator() {
+    public CloseableIterator<Alignment> iterator() throws IOException {
         return reader.iterator();
     }
 
@@ -120,15 +123,16 @@ public class AlignmentTileLoader implements IGVEventObserver {
                            int end,
                            SpliceJunctionHelper spliceJunctionHelper,
                            AlignmentDataManager.DownsampleOptions downsampleOptions,
-                           ReadStats readStats, Map<String, PEStats> peStats,
-                           AlignmentTrack.BisulfiteContext bisulfiteContext) {
+                           Map<String, PEStats> peStats,
+                           AlignmentTrack.BisulfiteContext bisulfiteContext,
+                           AlignmentTrack.RenderOptions renderOptions) {
 
         final IGVPreferences prefMgr = PreferencesManager.getPreferences();
         boolean filterFailedReads = prefMgr.getAsBoolean(SAM_FILTER_FAILED_READS);
         boolean filterSecondaryAlignments = prefMgr.getAsBoolean(SAM_FILTER_SECONDARY_ALIGNMENTS);
         boolean filterSupplementaryAlignments = prefMgr.getAsBoolean(SAM_FILTER_SUPPLEMENTARY_ALIGNMENTS);
         ReadGroupFilter filter = ReadGroupFilter.getFilter();
-        boolean showDuplicates = prefMgr.getAsBoolean(SAM_SHOW_DUPLICATES) || !prefMgr.getAsBoolean(SAM_FILTER_DUPLICATES);
+        boolean filterDuplicates = prefMgr.getAsBoolean(SAM_FILTER_DUPLICATES);
         int qualityThreshold = prefMgr.getAsInt(SAM_QUALITY_THRESHOLD);
         int alignmentScoreTheshold = prefMgr.getAsInt(SAM_ALIGNMENT_SCORE_THRESHOLD);
 
@@ -136,13 +140,11 @@ public class AlignmentTileLoader implements IGVEventObserver {
 
         AlignmentTile t = new AlignmentTile(start, end, spliceJunctionHelper, downsampleOptions, bisulfiteContext, reducedMemory);
 
-
         //assert (tiles.size() > 0);
         if (corruptIndex) {
             return t;
         }
 
-
         CloseableIterator<Alignment> iter = null;
 
         //log.debug("Loading : " + start + " - " + end);
@@ -159,7 +161,10 @@ public class AlignmentTileLoader implements IGVEventObserver {
                 IGV.getInstance().enableStopButton(true);
             }
 
+            MessageUtils.setStatusBarMessage("Reading...");
+
             iter = reader.query(chr, start, end, false);
+            MessageUtils.setStatusBarMessage("Iterating...");
 
             while (iter != null && iter.hasNext()) {
 
@@ -169,11 +174,6 @@ public class AlignmentTileLoader implements IGVEventObserver {
 
                 Alignment record = iter.next();
 
-                if (readStats != null) {
-                    readStats.addAlignment(record);
-                }
-                ;
-
                 // Set mate sequence of unmapped mates
                 // Put a limit on the total size of this collection.
                 String readName = record.getReadName();
@@ -218,7 +218,8 @@ public class AlignmentTileLoader implements IGVEventObserver {
                 }
 
 
-                if (!record.isMapped() || (!showDuplicates && record.isDuplicate()) ||
+                if (!record.isMapped() ||
+                        (filterDuplicates && record.isDuplicate()) ||
                         (filterFailedReads && record.isVendorFailedRead()) ||
                         (filterSecondaryAlignments && !record.isPrimary()) ||
                         (filterSupplementaryAlignments && record.isSupplementary()) ||
@@ -228,13 +229,13 @@ public class AlignmentTileLoader implements IGVEventObserver {
                 }
 
                 // Alignment score (optional tag)
-                if(alignmentScoreTheshold > 0) {
+                if (alignmentScoreTheshold > 0) {
 
                     Object alignmentScoreObj = record.getAttribute("AS");
 
-                    if(alignmentScoreObj != null) {
+                    if (alignmentScoreObj != null) {
                         int as = ((Number) alignmentScoreObj).intValue();
-                        if(as < alignmentScoreTheshold) {
+                        if (as < alignmentScoreTheshold) {
                             continue;
                         }
                     }
@@ -247,8 +248,10 @@ public class AlignmentTileLoader implements IGVEventObserver {
                 int interval = Globals.isTesting() ? 100000 : 1000;
                 if (alignmentCount % interval == 0) {
                     String msg = "Reads loaded: " + alignmentCount;
+                    //System.out.println(msg);
                     MessageUtils.setStatusBarMessage(msg);
                     if (memoryTooLow()) {
+                        Runtime.getRuntime().gc();
                         cancelReaders();
                         t.finish();
                         return t;
@@ -281,11 +284,6 @@ public class AlignmentTileLoader implements IGVEventObserver {
                 }
             }
 
-            if (readStats != null) {
-                readStats.compute();
-            }
-
-
             // Clean up any remaining unmapped mate sequences
             for (String mappedMateName : mappedMates.getKeys()) {
                 Alignment mappedMate = mappedMates.get(mappedMateName);
@@ -299,7 +297,7 @@ public class AlignmentTileLoader implements IGVEventObserver {
             t.finish();
 
             // TODO -- make this optional (on a preference)
-            InsertionManager.getInstance().processAlignments(chr, t.alignments);
+            InsertionManager.getInstance().processAlignments(chr, t.alignments, renderOptions);
 
 
         } catch (java.nio.BufferUnderflowException e) {
@@ -340,7 +338,7 @@ public class AlignmentTileLoader implements IGVEventObserver {
     }
 
 
-    private static synchronized boolean memoryTooLow() {
+    private static boolean memoryTooLow() {
         if (RuntimeUtils.getAvailableMemoryFraction() < 0.2) {
             System.gc();
             if (RuntimeUtils.getAvailableMemoryFraction() < 0.2) {
@@ -383,9 +381,7 @@ public class AlignmentTileLoader implements IGVEventObserver {
     @Override
     public void receiveEvent(Object event) {
         if (event instanceof StopEvent) {
-            System.out.println("Canceled");
             cancel = true;
-
             reader.cancelQuery();
         }
     }
