@@ -29,13 +29,12 @@
  */
 package org.broad.igv.sam;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.data.CoverageDataSource;
 import org.broad.igv.feature.FeatureUtils;
 import org.broad.igv.feature.LocusScore;
 import org.broad.igv.feature.genome.Genome;
-import org.broad.igv.goby.GobyCountArchiveDataSource;
 import org.broad.igv.prefs.IGVPreferences;
 import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.renderer.*;
@@ -50,7 +49,6 @@ import org.broad.igv.ui.panel.IGVPopupMenu;
 import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.ui.util.FileDialogUtils;
 import org.broad.igv.ui.util.MessageUtils;
-import org.broad.igv.ui.util.UIUtilities;
 import org.broad.igv.util.ResourceLocator;
 import org.broad.igv.util.StringUtils;
 import org.w3c.dom.Document;
@@ -63,6 +61,7 @@ import java.awt.event.ActionListener;
 import java.io.File;
 import java.text.DecimalFormat;
 import java.util.ArrayList;
+import java.util.Arrays;
 import java.util.Collection;
 import java.util.List;
 
@@ -74,30 +73,26 @@ import static org.broad.igv.prefs.Constants.*;
 
 public class CoverageTrack extends AbstractTrack implements ScalableTrack {
 
-    private static Logger log = Logger.getLogger(CoverageTrack.class);
+    private static Logger log = LogManager.getLogger(CoverageTrack.class);
+
     public static final int TEN_MB = 10000000;
     static DecimalFormat locationFormatter = new DecimalFormat();
 
     char[] nucleotides = {'a', 'c', 'g', 't', 'n'};
-    public static Color lightBlue = new Color(0, 0, 150);
-    private static Color coverageGrey = new Color(175, 175, 175);
-    public static final Color negStrandColor = new Color(140, 140, 160);
-    public static final Color posStrandColor = new Color(160, 140, 140);
 
     public static final boolean DEFAULT_AUTOSCALE = true;
-    public static final boolean DEFAULT_SHOW_REFERENCE = false;
 
+    private Color _color = null;   // Explicit color setting
     private float snpThreshold;
-
     private AlignmentTrack alignmentTrack;
     private AlignmentDataManager dataManager;
     private CoverageDataSource dataSource;
     private DataRenderer dataSourceRenderer;
     private IntervalRenderer intervalRenderer;
     private IGVPreferences prefs;
-    private JMenuItem dataRangeItem;
     private Genome genome;
     private boolean removed = false;
+    IGV igv;
 
     /**
      * Whether to autoscale across all ReferenceFrames
@@ -105,6 +100,8 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
      */
     private boolean globalAutoScale = true;
 
+    public CoverageTrack() {
+    }
 
     /**
      * Copy constructor.  Used for Sashimi plot.
@@ -117,6 +114,7 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
         if (track.dataSource != null) this.setDataSource(track.dataSource);
         this.snpThreshold = track.snpThreshold;
         this.prefs = track.prefs;
+        this.igv = IGV.hasInstance() ? IGV.getInstance() : null;
     }
 
     public CoverageTrack(ResourceLocator locator, String name, AlignmentTrack alignmentTrack, Genome genome) {
@@ -126,19 +124,31 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
         this.genome = genome;
         intervalRenderer = new IntervalRenderer();
         setMaximumHeight(40);
-
-        setColor(coverageGrey);
-
         prefs = PreferencesManager.getPreferences();
         snpThreshold = prefs.getAsFloat(SAM_ALLELE_THRESHOLD);
         autoScale = DEFAULT_AUTOSCALE;
-
+        this.igv = IGV.hasInstance() ? IGV.getInstance() : null;
     }
 
-    public CoverageTrack() {
-
+    @Override
+    public Color getColor() {
+        return _color == null ?
+                ColorUtilities.slightlyDarker(alignmentTrack.getColor()) :
+                _color;
     }
 
+    @Override
+    public void setColor(Color color) {
+        _color = color;
+    }
+
+    @Override
+    public String getSample() {
+        if (sampleId != null) {
+            return sampleId;    // Explicitly set sample ID (e.g. from server load XML)
+        }
+        return alignmentTrack == null ? null : alignmentTrack.getSample();
+    }
 
     @Override
     public boolean isNumeric() {
@@ -159,7 +169,6 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
 
     @Override
     public boolean isReadyToPaint(ReferenceFrame frame) {
-
         if (frame.getChrName().equals(Globals.CHR_ALL) || frame.getScale() > dataManager.getMinVisibleScale()) {
             return true;   // Nothing to paint
         } else {
@@ -186,21 +195,24 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
     }
 
     @Override
-    public void dispose() {
-        super.dispose();
+    public boolean isVisible() {
+        return super.isVisible() && !removed;
+    }
+
+    @Override
+    public void unload() {
+        super.unload();
         removed = true;
         if (dataManager != null) {
             dataManager.unsubscribe(this);
         }
-        dataManager = null;
-        dataSource = null;
-        alignmentTrack = null;
         setVisible(false);
     }
 
     public void render(RenderContext context, Rectangle rect) {
 
-        if (context.getScale() > dataManager.getMinVisibleScale() && dataSource == null) {
+        int viewWindowSize = context.getReferenceFrame().getCurrentRange().getLength();
+        if (viewWindowSize > dataManager.getVisibilityWindow() && dataSource == null) {
             Rectangle visibleRect = context.getVisibleRect().intersection(rect);
             Graphics2D g = context.getGraphic2DForColor(Color.gray);
             GraphicUtils.drawCenteredText("Zoom in to see coverage.", visibleRect, g);
@@ -223,22 +235,16 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
 
     public void drawData(RenderContext context, Rectangle rect) {
 
-        float maxRange = PreferencesManager.getPreferences().getAsFloat(SAM_MAX_VISIBLE_RANGE);
-        float minVisibleScale = (maxRange * 1000) / 700;
-
-        if (context.getScale() < minVisibleScale && !context.getChr().equals(Globals.CHR_ALL)) {
+        int viewWindowSize = context.getReferenceFrame().getCurrentRange().getLength();
+        if (viewWindowSize < dataManager.getVisibilityWindow() && !context.getChr().equals(Globals.CHR_ALL)) {
             //Show coverage calculated from intervals if zoomed in enough
             AlignmentInterval interval = null;
             if (dataManager != null) {
-                dataManager.load(context.getReferenceFrame(), alignmentTrack.renderOptions, true);
-                interval = dataManager.getLoadedInterval(context.getReferenceFrame());
+                interval = dataManager.getLoadedInterval(context.getReferenceFrame(), true);
             }
             if (interval != null) {
-                if (interval.contains(context.getChr(), (int) context.getOrigin(), (int) context.getEndLocation())) {
-                    //if (autoScale) rescale(context.getReferenceFrame());
-                    intervalRenderer.paint(context, rect, interval.getCounts());
-                    return;
-                }
+                intervalRenderer.paint(context, rect, interval.getCounts());
+                return;
             }
         }
 
@@ -290,8 +296,8 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
     @Override
     public Range getInViewRange(ReferenceFrame frame) {
 
-        if (dataManager == null || frame.getScale() > dataManager.getMinVisibleScale()) {
-
+        int viewWindowSize = frame.getCurrentRange().getLength();
+        if (dataManager == null || viewWindowSize > dataManager.getVisibilityWindow()) {
             List<LocusScore> scores = getInViewScores(frame);
             if (scores != null && scores.size() > 0) {
                 float min = scores.get(0).getScore();
@@ -308,18 +314,15 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
             }
 
         } else {
-
             AlignmentInterval interval = dataManager.getLoadedInterval(frame);
             if (interval == null) return null;
 
             int origin = (int) frame.getOrigin();
             int end = (int) frame.getEnd() + 1;
-
             int intervalMax = interval.getMaxCount(origin, end);
 
             return new Range(0, Math.max(10, intervalMax));
         }
-
     }
 
 
@@ -362,6 +365,11 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
                 AlignmentCounts counts = interval.getCounts();
                 if (counts != null) {
                     buf.append(counts.getValueStringAt((int) position));
+                    boolean baseModMode = alignmentTrack.renderOptions.getColorOption() == AlignmentTrack.ColorOption.BASE_MODIFICATION;
+                    if (baseModMode && counts.getModifiedBaseCounts() != null) {
+                        buf.append("<hr>");
+                        buf.append(counts.getModifiedBaseCounts().getValueString((int) position));
+                    }
                 }
             }
         } else {
@@ -370,15 +378,6 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
         return buf.toString();
     }
 
-    public AlignmentCounts getCounts(String chr, double position, ReferenceFrame frame) {
-        AlignmentInterval interval = dataManager.getLoadedInterval(frame);
-        if (interval != null && interval.contains(chr, (int) position, (int) position)) {
-            return interval.getCounts();
-        } else {
-            return null;
-        }
-    }
-
     private String getPrecomputedValueString(String chr, double position, ReferenceFrame frame) {
 
         if (dataSource == null) {
@@ -457,6 +456,7 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
             final double scale = context.getScale();
 
             boolean bisulfiteMode = alignmentTrack.renderOptions.getColorOption() == AlignmentTrack.ColorOption.BISULFITE;
+            boolean baseModMode = alignmentTrack.renderOptions.getColorOption() == AlignmentTrack.ColorOption.BASE_MODIFICATION;
 
 
             // First pass, coverage
@@ -552,7 +552,7 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
                             }
                         }
 
-                        if (!mismatch) {
+                        if (!(mismatch || baseModMode)) {
                             continue;
                         }
 
@@ -574,6 +574,9 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
                                     drawBarBisulfite(context, pos, rect, totalCount, maxRange,
                                             pY, pX, dX, bc, range.isLog());
                                 }
+                            } else if (baseModMode) {
+                                drawModifiedBaseBar(context, pos, rect, totalCount, maxRange,
+                                        pY, pX, dX, alignmentCounts, range.isLog());
                             } else {
                                 drawBar(context, pos, rect, totalCount, maxRange,
                                         pY, pX, dX, alignmentCounts, range.isLog());
@@ -616,20 +619,16 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
         for (char nucleotide : nucleotides) {
 
             int count = interval.getCount(pos, (byte) nucleotide);
-
-            Color c = SequenceRenderer.nucleotideColors.get(nucleotide);
-
-            Graphics2D tGraphics = context.getGraphic2DForColor(c);
-
             double tmp = isLog ?
                     (count / totalCount) * Math.log10(totalCount + 1) / max :
                     count / max;
             int height = (int) (tmp * rect.getHeight());
-
             height = Math.min(pY - rect.y, height);
             int baseY = pY - height;
 
             if (height > 0) {
+                Color c = SequenceRenderer.nucleotideColors.get(nucleotide);
+                Graphics2D tGraphics = context.getGraphic2DForColor(c);
                 tGraphics.fillRect(pX, baseY, dX, height);
             }
 
@@ -638,6 +637,44 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
         return pX + dX;
     }
 
+    int drawModifiedBaseBar(RenderContext context,
+                            int pos,
+                            Rectangle rect,
+                            double totalCount,
+                            double max,
+                            int pY,
+                            int pX,
+                            int dX,
+                            AlignmentCounts interval,
+                            boolean isLog) {
+
+        ModifiedBaseCounts baseCounts = interval.getModifiedBaseCounts();
+
+        if (baseCounts != null) {
+
+            byte likelihood = (byte) 255;
+
+            for (String modification : baseCounts.getAllModifications()) {
+
+                int count = baseCounts.getCount(pos, modification);
+                double tmp = isLog ?
+                        (count / totalCount) * Math.log10(totalCount + 1) / max :
+                        count / max;
+                int height = (int) (tmp * rect.getHeight());
+                height = Math.min(pY - rect.y, height);
+                int baseY = pY - height;
+                if (height > 0) {
+                    Color c = BaseModification.getModColor(modification, likelihood);
+                    Graphics2D tGraphics = context.getGraphic2DForColor(c);
+                    tGraphics.fillRect(pX, baseY, dX, height);
+                }
+
+                pY = baseY;
+            }
+        }
+        return pX + dX;
+    }
+
     int drawBarBisulfite(RenderContext context,
                          int pos,
                          Rectangle rect,
@@ -649,8 +686,7 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
                          BisulfiteCounts.Count count,
                          boolean isLog) {
 
-        // If bisulfite mode, we expand the rectangle to make it more visible.  This code is copied from
-        // AlignmentRenderer
+        // If bisulfite mode, we expand the rectangle to make it more visible.  This code is copied from AlignmentRenderer
         int pX = pX0;
         if (dX < 3) {
             int expansion = dX;
@@ -790,8 +826,10 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
         popupMenu.addSeparator();
         addCopyDetailsItem(popupMenu, te);
 
-        popupMenu.addSeparator();
-        addShowItems(popupMenu);
+        if (alignmentTrack != null) {
+            popupMenu.addSeparator();
+            addShowItems(popupMenu);
+        }
 
         return popupMenu;
     }
@@ -854,75 +892,60 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
 
     public JMenuItem addSnpTresholdItem(JPopupMenu menu) {
         JMenuItem maxValItem = new JMenuItem("Set allele frequency threshold...");
-
-        maxValItem.addActionListener(new ActionListener() {
-
-            public void actionPerformed(ActionEvent e) {
-
-                String value = JOptionPane.showInputDialog("Allele frequency threshold: ", Float.valueOf(snpThreshold));
-                if (value == null) {
-                    return;
-                }
-                try {
-                    float tmp = Float.parseFloat(value);
-                    snpThreshold = tmp;
-                    IGV.getInstance().revalidateTrackPanels();
-                } catch (Exception exc) {
-                    //log
-                }
-
+        maxValItem.addActionListener(e -> {
+            String value = JOptionPane.showInputDialog("Allele frequency threshold: ", Float.valueOf(snpThreshold));
+            if (value == null) {
+                return;
+            }
+            try {
+                float tmp = Float.parseFloat(value);
+                snpThreshold = tmp;
+                repaint();
+            } catch (Exception exc) {
+                //log
             }
         });
         menu.add(maxValItem);
-
         return maxValItem;
     }
 
     public void addShowItems(JPopupMenu menu) {
 
+        final SpliceJunctionTrack spliceJunctionTrack = alignmentTrack.getSpliceJunctionTrack();
+
+        final JMenuItem item = new JCheckBoxMenuItem("Show Coverage Track");
+        item.setSelected(true);
+        item.addActionListener(e -> {
+            CoverageTrack.this.setVisible(item.isSelected());
+            IGV.getInstance().repaint(Arrays.asList(CoverageTrack.this));
+        });
+        //If this is the only track visible, disable option to hide it.
+        if (!(alignmentTrack.isVisible() ||
+                (spliceJunctionTrack != null && spliceJunctionTrack.isVisible()))) {
+            item.setEnabled(false);
+        }
+        menu.add(item);
+
+        if (spliceJunctionTrack != null) {
+            final JMenuItem junctionItem = new JCheckBoxMenuItem("Show Splice Junction Track");
+            junctionItem.setSelected(spliceJunctionTrack.isVisible());
+            junctionItem.setEnabled(!spliceJunctionTrack.isRemoved());
+            junctionItem.addActionListener(e -> {
+                spliceJunctionTrack.setVisible(junctionItem.isSelected());
+                IGV.getInstance().repaint(Arrays.asList(spliceJunctionTrack));
+            });
+            menu.add(junctionItem);
+        }
+
         if (alignmentTrack != null) {
             final JMenuItem alignmentItem = new JCheckBoxMenuItem("Show Alignment Track");
             alignmentItem.setSelected(alignmentTrack.isVisible());
             alignmentItem.setEnabled(!alignmentTrack.isRemoved());
-            alignmentItem.addActionListener(new ActionListener() {
-                @Override
-                public void actionPerformed(ActionEvent e) {
-                    alignmentTrack.setVisible(alignmentItem.isSelected());
-                }
+            alignmentItem.addActionListener(e -> {
+                alignmentTrack.setVisible(alignmentItem.isSelected());
+                IGV.getInstance().repaint(Arrays.asList(alignmentTrack));
             });
             menu.add(alignmentItem);
-
-            final SpliceJunctionTrack spliceJunctionTrack = alignmentTrack.getSpliceJunctionTrack();
-            if (spliceJunctionTrack != null) {
-                final JMenuItem junctionItem = new JCheckBoxMenuItem("Show Splice Junction Track");
-                junctionItem.setSelected(spliceJunctionTrack.isVisible());
-                junctionItem.setEnabled(!spliceJunctionTrack.isRemoved());
-                junctionItem.addActionListener(new ActionListener() {
-                    @Override
-                    public void actionPerformed(ActionEvent e) {
-                        spliceJunctionTrack.setVisible(junctionItem.isSelected());
-                    }
-                });
-
-                menu.add(junctionItem);
-            }
-
-            final JMenuItem coverageItem = new JMenuItem("Hide Coverage Track");
-            coverageItem.setEnabled(!isRemoved());
-            coverageItem.addActionListener(new ActionListener() {
-                @Override
-                public void actionPerformed(ActionEvent e) {
-                    UIUtilities.invokeOnEventThread(new Runnable() {
-
-                        public void run() {
-                            setVisible(false);
-                            if (IGV.hasInstance()) IGV.getInstance().getMainPanel().revalidate();
-
-                        }
-                    });
-                }
-            });
-            menu.add(coverageItem);
         }
     }
 
@@ -930,28 +953,20 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
     public void addLoadCoverageDataItem(JPopupMenu menu) {
         // Change track height by attribute
         final JMenuItem item = new JCheckBoxMenuItem("Load pre-computed coverage data...");
-        item.addActionListener(new ActionListener() {
-
-            public void actionPerformed(ActionEvent e) {
-
-                final IGVPreferences prefs = PreferencesManager.getPreferences();
-                File initDirectory = prefs.getLastTrackDirectory();
-                File file = FileDialogUtils.chooseFile("Select coverage file", initDirectory, FileDialog.LOAD);
-                if (file != null) {
-                    prefs.setLastTrackDirectory(file.getParentFile());
-                    String path = file.getAbsolutePath();
-                    if (path.endsWith(".tdf") || path.endsWith(".tdf")) {
-                        TDFReader reader = TDFReader.getReader(file.getAbsolutePath());
-                        TDFDataSource ds = new TDFDataSource(reader, 0, getName() + " coverage", genome);
-                        setDataSource(ds);
-                        IGV.getInstance().revalidateTrackPanels();
-                    } else if (path.endsWith(".counts")) {
-                        CoverageDataSource ds = new GobyCountArchiveDataSource(file);
-                        setDataSource(ds);
-                        IGV.getInstance().revalidateTrackPanels();
-                    } else {
-                        MessageUtils.showMessage("Coverage data must be in .tdf format");
-                    }
+        item.addActionListener(e -> {
+            final IGVPreferences prefs = PreferencesManager.getPreferences();
+            File initDirectory = prefs.getLastTrackDirectory();
+            File file = FileDialogUtils.chooseFile("Select coverage file", initDirectory, FileDialog.LOAD);
+            if (file != null) {
+                prefs.setLastTrackDirectory(file.getParentFile());
+                String path = file.getAbsolutePath();
+                if (path.endsWith(".tdf") || path.endsWith(".tdf")) {
+                    TDFReader reader = TDFReader.getReader(file.getAbsolutePath());
+                    TDFDataSource ds = new TDFDataSource(reader, 0, getName() + " coverage", genome);
+                    setDataSource(ds);
+                    repaint();
+                } else {
+                    if (igv != null) MessageUtils.showMessage("Coverage data must be in .tdf format");
                 }
             }
         });
@@ -971,17 +986,22 @@ public class CoverageTrack extends AbstractTrack implements ScalableTrack {
     public void marshalXML(Document document, Element element) {
 
         super.marshalXML(document, element);
-
         element.setAttribute("snpThreshold", String.valueOf(snpThreshold));
+        if (_color != null) {
+            element.setAttribute("_color", ColorUtilities.colorToString(_color));
+        }
 
     }
 
     @Override
     public void unmarshalXML(Element element, Integer version) {
-
+        super.unmarshalXML(element, version);
         if (element.hasAttribute("snpThreshold")) {
             snpThreshold = Float.parseFloat(element.getAttribute("snpThreshold"));
         }
+        if (element.hasAttribute("_color")) {
+            _color = ColorUtilities.stringToColor(element.getAttribute("_color"));
+        }
 
     }
 
