@@ -28,7 +28,7 @@ package org.broad.igv.sashimi;
 //~--- non-JDK imports --------------------------------------------------------
 
 import htsjdk.tribble.Feature;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.feature.IExon;
 import org.broad.igv.feature.IGVFeature;
@@ -48,22 +48,22 @@ import java.awt.*;
 import java.awt.geom.Ellipse2D;
 import java.awt.geom.GeneralPath;
 import java.awt.geom.Rectangle2D;
-import java.util.*;
 import java.util.List;
+import java.util.*;
 
 import static org.broad.igv.prefs.Constants.SASHIMI_SHOW_COVERAGE;
 
+
 /**
  * Renderer for splice junctions. Draws a filled-in arc for each junction, with the width of the
  * arc representing the depth of coverage. If coverage information is present for the flanking
  * regions, draws that, too; otherwise indicates flanking regions with rectangles
- *
  */
 public class SashimiJunctionRenderer extends IGVFeatureRenderer {
 
-    private static Logger log = Logger.getLogger(SashimiJunctionRenderer.class);
+    private static Logger log = LogManager.getLogger(SashimiJunctionRenderer.class);
 
-    public enum ShapeType{
+    public enum ShapeType {
         CIRCLE,
         ELLIPSE,
         TEXT,
@@ -106,20 +106,20 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
      * want to make sure they don't overlap
      */
     //private HashBasedTable<Integer, Integer, Feature> featureStartEndTable = null;
-
     public void setSelectedExons(Set<IExon> selectedExons) {
         this.selectedExons = selectedExons;
     }
 
     /**
      * Set the data manager
+     *
      * @param dataManager
      */
     public void setDataManager(AlignmentDataManager dataManager) {
         this.dataManager = dataManager;
     }
 
-    public AlignmentDataManager getDataManager(){
+    public AlignmentDataManager getDataManager() {
         return this.dataManager;
     }
 
@@ -139,7 +139,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
         this.maxDepth = maxDepth;
     }
 
-    public static void setShapeType(ShapeType st){
+    public static void setShapeType(ShapeType st) {
         shapeType = st;
     }
 
@@ -153,7 +153,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
 
     public void setColor(Color color) {
         this.color = color;
-        if(this.coverageTrack != null) this.coverageTrack.setColor(color);
+        if (this.coverageTrack != null) this.coverageTrack.setColor(color);
     }
 
     public void setCoverageTrack(CoverageTrack covTrack) {
@@ -167,7 +167,6 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
     }
 
 
-
     /**
      * Note:  assumption is that featureList is sorted by pStart position.
      *
@@ -188,7 +187,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
 
         final boolean showCoverage = PreferencesManager.getPreferences().getAsBoolean(SASHIMI_SHOW_COVERAGE);
 
-        if(coverageTrack != null && showCoverage){
+        if (coverageTrack != null && showCoverage) {
             //Only want the coverage track to go so high so that the arcs still have room
             int newHeight = coverageRectangle.height / 2;
             int newY = coverageRectangle.y + coverageRectangle.height / 2 - newHeight;
@@ -220,7 +219,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
             // a feature's region
             Set<IExon> locselectedExons = selectedExons;
 
-            if(drawFeatureAbove == null) drawFeatureAbove = new HashMap<Feature, Boolean>(featureList.size());
+            if (drawFeatureAbove == null) drawFeatureAbove = new HashMap<Feature, Boolean>(featureList.size());
             //if(featureStartEndTable == null) featureStartEndTable = HashBasedTable.create();
             boolean lastDrawAbove = true;
             for (IGVFeature feature : featureList) {
@@ -228,18 +227,8 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
                 int junctionStart = junctionFeature.getJunctionStart();
                 int junctionEnd = junctionFeature.getJunctionEnd();
 
-//                //Make sure we flip features already drawn. This is for pos/neg strand
-//                Feature otherFeature = featureStartEndTable.get(junctionStart, junctionEnd);
-//                Boolean drawAbove;
-//                if(otherFeature != null){
-//                    drawAbove = !drawFeatureAbove.get(otherFeature);
-//                }else{
-//                    drawAbove = drawFeatureAbove.get(junctionFeature);
-//                    featureStartEndTable.put(junctionStart, junctionEnd, junctionFeature);
-//                }
-
                 Boolean drawAbove = drawFeatureAbove.get(junctionFeature);
-                if(drawAbove == null) {
+                if (drawAbove == null) {
                     drawAbove = !lastDrawAbove;
                     drawFeatureAbove.put(junctionFeature, drawAbove);
                 }
@@ -247,29 +236,23 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
                 //Only show arcs for the selected feature, if applicable
                 if (locselectedExons != null && locselectedExons.size() > 0) {
                     boolean inSelected = false;
-                    for(IExon selectedExon: locselectedExons){
-                        if((junctionStart >= selectedExon.getStart() && junctionStart <= selectedExon.getEnd())
-                                || (junctionEnd >= selectedExon.getStart() && junctionEnd <= selectedExon.getEnd())){
+                    for (IExon selectedExon : locselectedExons) {
+                        if ((junctionStart >= selectedExon.getStart() && junctionStart <= selectedExon.getEnd())
+                                || (junctionEnd >= selectedExon.getStart() && junctionEnd <= selectedExon.getEnd())) {
                             inSelected = true;
                             break;
                         }
                     }
 
-                    if(!inSelected) continue;
+                    if (!inSelected) continue;
                 }
 
-                //double virtualPixelStart = Math.round((flankingStart - origin) / locScale);
-                //double virtualPixelEnd = Math.round((flankingEnd - origin) / locScale);
-
                 double virtualPixelJunctionStart = Math.round((junctionStart - origin) / locScale);
                 double virtualPixelJunctionEnd = Math.round((junctionEnd - origin) / locScale);
 
                 // If the any part of the feature fits in the track rectangle draw it
                 if ((virtualPixelJunctionEnd >= trackRectangleX) && (virtualPixelJunctionStart <= trackRectangleMaxX)) {
 
-                    //int displayPixelEnd = (int) Math.min(trackRectangleMaxX, virtualPixelEnd);
-                    //int displayPixelStart = (int) Math.max(trackRectangleX, virtualPixelStart);
-
                     int depth = junctionFeature.getJunctionDepth();
                     Color color = feature.getColor();
 
@@ -278,11 +261,15 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
                     int pixelYstart = 0;
                     int pixelYend = 0;
 
-                    if(coverageTrack != null && showCoverage){
-                        pixelYstart = getYOffset(coverageRectangle, coverageTrack.getDataRange() ,getCoverage(junctionStart));
-                        pixelYend = getYOffset(coverageRectangle, coverageTrack.getDataRange() ,getCoverage(junctionEnd));
+                    if (coverageTrack != null && showCoverage) {
+                        pixelYstart = getYOffset(coverageRectangle, coverageTrack.getDataRange(), getCoverage(junctionStart));
+                        pixelYend = getYOffset(coverageRectangle, coverageTrack.getDataRange(), getCoverage(junctionEnd));
                     }
-
+//
+//                    System.out.println("{chr: 'chr12'" +
+//                            ", start: " + ((SpliceJunctionFeature) feature).getJunctionStart() +
+//                            ", end: " + ((SpliceJunctionFeature) feature).getJunctionEnd() +
+//                            ", value: " + ((SpliceJunctionFeature) feature).getJunctionDepth() + "},");
 
                     drawFeature(pixelYstart, pixelYend,
                             (int) virtualPixelJunctionStart, (int) virtualPixelJunctionEnd, depth,
@@ -304,6 +291,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
      * Get the coverage around this approximate genome position. We actually look
      * for a maximum around a certain window. This is intended for plotting, we just
      * want the arc to look like it's coming from the top
+     *
      * @param genomePos
      * @return
      */
@@ -311,15 +299,15 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
 //        Integer yOffset = yOffsetMap.get(genomePos);
 //        if(yOffset != null) return yOffset;
 
-        if(dataManager == null) return 0;
+        if (dataManager == null) return 0;
         Collection<AlignmentInterval> intervals = dataManager.getLoadedIntervals();
-        if(intervals == null) return 0;
+        if (intervals == null) return 0;
 
         int buffer = 4;
         int coverage = 0;
-        for(AlignmentInterval interval: intervals){
-            if(interval.contains(interval.getChr(), genomePos - buffer, genomePos + buffer)){
-                for(int loc= genomePos - buffer; loc < genomePos + buffer; loc++){
+        for (AlignmentInterval interval : intervals) {
+            if (interval.overlaps(interval.getChr(), genomePos - buffer, genomePos + buffer)) {
+                for (int loc = genomePos - buffer; loc < genomePos + buffer; loc++) {
                     coverage = Math.max(coverage, interval.getTotalCount(loc));
                 }
                 return coverage;
@@ -329,7 +317,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
         return 0;
     }
 
-    private int getYOffset(Rectangle rect, DataRange range, int totalCount){
+    private int getYOffset(Rectangle rect, DataRange range, int totalCount) {
         //int pY = (int) rect.getMaxY() - 1;
         double maxRange = range.isLog() ? Math.log10(range.getMaximum()) : range.getMaximum();
         double tmp = range.isLog() ? Math.log10(totalCount) / maxRange : totalCount / maxRange;
@@ -343,15 +331,15 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
      * Draw a filled arc representing a single feature. The thickness and height of the arc are proportional to the
      * depth of coverage.  Some of this gets a bit arcane -- the result of lots of visual tweaking.
      *
-     * @param pixelYStartOffset    the y offset from center line that the arc should start at
-     * @param pixelYEndOffset      thy y offset from center line that the arc should end at
+     * @param pixelYStartOffset  the y offset from center line that the arc should start at
+     * @param pixelYEndOffset    thy y offset from center line that the arc should end at
      * @param pixelJunctionStart the starting position of the junction, whether on-screen or not
      * @param pixelJunctionEnd   the ending position of the junction, whether on-screen or not
      * @param depth              coverage depth
      * @param trackRectangle
      * @param context
      * @param featureColor       the color specified for this feature.  May be null.
-     * @param drawAbove         Whether to draw the arc above or below the centerline
+     * @param drawAbove          Whether to draw the arc above or below the centerline
      */
     protected void drawFeature(int pixelYStartOffset, int pixelYEndOffset,
                                int pixelJunctionStart, int pixelJunctionEnd, int depth,
@@ -402,7 +390,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
         GeneralPath arcPath = new GeneralPath();
         arcPath.moveTo(pixelJunctionStart, arcBeginY);
         arcPath.curveTo(pixelJunctionStart, arcControlPeakY,
-                        pixelJunctionEnd, arcControlPeakY,
+                pixelJunctionEnd, arcControlPeakY,
                 pixelJunctionEnd, arcEndY);
 
         Graphics2D g2D = context.getGraphic2DForColor(color);
@@ -415,7 +403,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
 
         //Setting maxDepth to 1 should just max everything out, but 1/0 tends
         //to make things crash
-        if(maxDepth >= 2){
+        if (maxDepth >= 2) {
             double scale = (maxStrokeSize - minStrokeSize) / Math.log(maxDepth);
             strokeSize = scale * Math.log(effDepth) + minStrokeSize;
         }
@@ -436,7 +424,7 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
         float maxPossibleShapeHeight = maxPossibleArcHeight / 2;
         Shape shape = null;
 
-        switch(shapeType){
+        switch (shapeType) {
             case CIRCLE:
                 shape = createDepthCircle(maxPossibleShapeHeight, depthProportionOfMax, midX, actArcPeakY);
                 break;
@@ -464,15 +452,15 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
                 break;
         }
 
-        if(shape != null){
+        if (shape != null) {
             g2D.draw(shape);
             g2D.fill(shape);
         }
     }
 
-    private Shape createDepthEllipse(double maxPossibleShapeHeight, double depthProportionOfMax, double arcMidX, double actArcPeakY){
+    private Shape createDepthEllipse(double maxPossibleShapeHeight, double depthProportionOfMax, double arcMidX, double actArcPeakY) {
         double w = 5f;
-        double x = arcMidX - w/2;
+        double x = arcMidX - w / 2;
 
         double h = maxPossibleShapeHeight * depthProportionOfMax;
 
@@ -482,11 +470,11 @@ public class SashimiJunctionRenderer extends IGVFeatureRenderer {
         return new Ellipse2D.Double(x, y, w, h);
     }
 
-    private Shape createDepthCircle(double maxPossibleShapeHeight, double depthProportionOfMax, double arcMidX, double actArcPeakY){
+    private Shape createDepthCircle(double maxPossibleShapeHeight, double depthProportionOfMax, double arcMidX, double actArcPeakY) {
 
         double h = maxPossibleShapeHeight * Math.sqrt(depthProportionOfMax);
         double w = h;
-        double x = arcMidX - w/2;
+        double x = arcMidX - w / 2;
 
         //The ellipse is always specified from the top left corner
         double y = actArcPeakY - h / 2;
