@@ -25,6 +25,9 @@
 
 package org.broad.igv.sashimi;
 
+import org.broad.igv.event.IGVEventBus;
+import org.broad.igv.event.IGVEventObserver;
+import org.broad.igv.event.ViewChange;
 import org.broad.igv.feature.IExon;
 import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.PreferencesManager;
@@ -33,10 +36,8 @@ import org.broad.igv.track.*;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.color.ColorPalette;
 import org.broad.igv.ui.color.ColorUtilities;
-import org.broad.igv.event.IGVEventBus;
-import org.broad.igv.event.IGVEventObserver;
-import org.broad.igv.event.ViewChange;
 import org.broad.igv.ui.panel.*;
+import org.broad.igv.ui.util.IGVMouseInputAdapter;
 import org.broad.igv.ui.util.UIUtilities;
 
 import javax.swing.*;
@@ -45,12 +46,11 @@ import javax.swing.event.PopupMenuListener;
 import java.awt.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.awt.event.MouseAdapter;
 import java.awt.event.MouseEvent;
 import java.awt.geom.Rectangle2D;
 import java.io.File;
-import java.util.*;
 import java.util.List;
+import java.util.*;
 
 /**
  * Window for displaying sashimi style junction plot
@@ -90,6 +90,7 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
 
     public SashimiPlot(ReferenceFrame iframe, Collection<? extends AlignmentTrack> alignmentTracks, FeatureTrack geneTrack) {
 
+        setContentPane(new SashimiContentPane());
         getGlassPane().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
         int minJunctionCoverage = PreferencesManager.getPreferences().getAsInt(Constants.SAM_JUNCTION_MIN_COVERAGE);
 
@@ -158,9 +159,9 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
         ZoomSliderPanel zoomSliderPanel = new ZoomSliderPanel(frame);
         zoomSliderPanel.setMinZoomLevel(frame.getZoom());
 
-        zoomSliderPanel.addMouseListener(new MouseAdapter() {
+        zoomSliderPanel.addMouseListener(new IGVMouseInputAdapter() {
             @Override
-            public void mouseClicked(MouseEvent e) {
+            public void igvMouseClicked(MouseEvent e) {
                 SashimiPlot.this.repaint();
             }
         });
@@ -211,7 +212,7 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
         geneTrack.load(context.getReferenceFrame());
 
 
-        Dimension maxGeneDim = new Dimension(Integer.MAX_VALUE, geneTrack.getNumberOfFeatureLevels() * geneTrack.getSquishedRowHeight() + 10);
+        Dimension maxGeneDim = new Dimension(Integer.MAX_VALUE,  Math.max(1, geneTrack.getNumberOfFeatureLevels()) * geneTrack.getSquishedRowHeight() + 10);
         geneComponent.setMaximumSize(maxGeneDim);
         Dimension prefGeneDim = new Dimension(maxGeneDim);
         prefGeneDim.setSize(prefWidth, prefGeneDim.height);
@@ -332,6 +333,7 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
             });
 
             CoverageTrack covTrack = getRenderer(this.trackComponent.track).getCoverageTrack();
+            covTrack.setWindowFunction(WindowFunction.max);
             JMenuItem setCoverageDataRange = CoverageTrack.addDataRangeItem(SashimiPlot.this, null, Arrays.asList(covTrack));
             setCoverageDataRange.setText("Set Exon Coverage Max");
 
@@ -423,8 +425,8 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
             strandGroup.add(minusStrand);
 
 
-            JMenuItem saveImageItem = new JMenuItem("Save Image...");
-            saveImageItem.addActionListener(new ActionListener() {
+            JMenuItem savePngImageItem = new JMenuItem("Save PNG Image...");
+            savePngImageItem.addActionListener(new ActionListener() {
                 @Override
                 public void actionPerformed(ActionEvent e) {
                     File defaultFile = new File("Sashimi.png");
@@ -432,6 +434,15 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
                 }
             });
 
+            JMenuItem saveSvgImageItem = new JMenuItem("Save SVG Image...");
+            saveSvgImageItem.addActionListener(new ActionListener() {
+                @Override
+                public void actionPerformed(ActionEvent e) {
+                    File defaultFile = new File("Sashimi.svg");
+                    IGV.getInstance().createSnapshot(SashimiPlot.this.getContentPane(), defaultFile);
+                }
+            });
+
             // Coverage ranges -- these apply to current plot only
             menu.add(new JLabel("Junction Coverage Display"));
             menu.add(setCoverageDataRange);
@@ -446,7 +457,7 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
             menu.addSeparator();
             menu.add(textShape);
             menu.add(circleShape);
-           // menu.add(ellipseShape);
+            // menu.add(ellipseShape);
             menu.add(noShape);
 
             // Strand options -- applies to all plots
@@ -456,7 +467,8 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
             menu.add(minusStrand);
             menu.addSeparator();
 
-            menu.add(saveImageItem);
+            menu.add(savePngImageItem);
+            menu.add(saveSvgImageItem);
 
             return menu;
         }
@@ -525,7 +537,7 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
         }
     }
 
-    private abstract class TrackComponentMouseAdapter<T extends Track> extends MouseAdapter {
+    private abstract class TrackComponentMouseAdapter<T extends Track> extends IGVMouseInputAdapter {
 
         protected TrackComponent<T> trackComponent;
         protected PanTool currentTool;
@@ -555,6 +567,7 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
 
         @Override
         public void mouseReleased(MouseEvent e) {
+            super.mouseReleased(e);
             if (e.isPopupTrigger()) {
                 doPopupMenu(e);
             } else {
@@ -564,6 +577,7 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
 
         @Override
         public void mousePressed(MouseEvent e) {
+            super.mousePressed(e);
             if (e.isPopupTrigger()) {
                 doPopupMenu(e);
             } else {
@@ -583,7 +597,7 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
         }
 
         @Override
-        public void mouseClicked(MouseEvent e) {
+        public void igvMouseClicked(MouseEvent e) {
             if (e.isPopupTrigger()) {
                 doPopupMenu(e);
                 return;
@@ -627,7 +641,7 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
             if (nonJunctionTracks.size() == 1) {
                 geneTrack = nonJunctionTracks.get(0);
             } else {
-                FeatureTrackSelectionDialog dlg = new FeatureTrackSelectionDialog(IGV.getMainFrame(), nonJunctionTracks);
+                FeatureTrackSelectionDialog dlg = new FeatureTrackSelectionDialog(IGV.getInstance().getMainFrame(), nonJunctionTracks);
                 dlg.setTitle("Select Gene Track");
                 dlg.setVisible(true);
                 if (dlg.getIsCancelled()) return;
@@ -643,7 +657,7 @@ public class SashimiPlot extends JFrame implements IGVEventObserver {
 
             if (alignmentTracks.size() > 1) {
                 TrackSelectionDialog<AlignmentTrack> alDlg =
-                        new TrackSelectionDialog<AlignmentTrack>(IGV.getMainFrame(), TrackSelectionDialog.SelectionMode.MULTIPLE, alignmentTracks);
+                        new TrackSelectionDialog<AlignmentTrack>(IGV.getInstance().getMainFrame(), TrackSelectionDialog.SelectionMode.MULTIPLE, alignmentTracks);
                 alDlg.setTitle("Select Alignment Tracks");
                 alDlg.setVisible(true);
                 if (alDlg.getIsCancelled()) return;
