@@ -25,20 +25,24 @@
 
 package org.broad.igv.session;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.feature.genome.GenomeListItem;
+import org.broad.igv.logging.*;
 import org.broad.igv.feature.RegionOfInterest;
 import org.broad.igv.feature.genome.GenomeManager;
 import org.broad.igv.lists.GeneList;
 import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.PreferencesManager;
-import org.broad.igv.track.*;
+import org.broad.igv.track.AttributeManager;
+import org.broad.igv.track.Track;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.TrackFilter;
 import org.broad.igv.ui.TrackFilterElement;
+import org.broad.igv.ui.commandbar.GenomeListManager;
 import org.broad.igv.ui.panel.FrameManager;
 import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.ui.panel.TrackPanel;
 import org.broad.igv.util.FileUtils;
+import org.broad.igv.util.ParsingUtils;
 import org.broad.igv.util.ResourceLocator;
 import org.broad.igv.util.Utilities;
 import org.w3c.dom.DOMException;
@@ -46,28 +50,26 @@ import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 
 import javax.swing.*;
-
 import javax.xml.parsers.DocumentBuilder;
 import javax.xml.parsers.DocumentBuilderFactory;
 import java.io.*;
 import java.util.*;
+import java.util.stream.Collectors;
 
 /**
  * @author jrobinso
  */
 public class SessionWriter {
 
-    static Logger log = Logger.getLogger(SessionWriter.class);
+    static Logger log = LogManager.getLogger(SessionWriter.class);
 
-    Session session;
+    private Session session;
     private static int CURRENT_VERSION = 8;
-
-    private static final String TRACK_TAG = SessionElement.TRACK;
-
-    Document document;
+    private File outputFile;
+    private Document document;
 
     /**
-     * Method description
+     * Save the session as an XML document
      *
      * @param session
      * @param outputFile
@@ -83,8 +85,7 @@ public class SessionWriter {
         this.session = session;
 
         if (outputFile == null) {
-            RuntimeException e = new RuntimeException("Can't save session file: " + outputFile);
-            log.error("Session Management Error", e);
+            log.error("Session Management Error: NULL outputFile");
         }
 
         String xmlString = createXmlFromSession(session, outputFile);
@@ -104,7 +105,8 @@ public class SessionWriter {
 
     public String createXmlFromSession(Session session, File outputFile) throws RuntimeException {
 
-        String xmlString = null;
+        this.session = session;
+        this.outputFile = outputFile;
 
         try {
 
@@ -120,6 +122,10 @@ public class SessionWriter {
 
             String genomeId = GenomeManager.getInstance().getGenomeId();
             if (genomeId != null) {
+                // If genomeId is a file try to write it out as a relative path
+                if ((new File(genomeId)).exists() && isUseRelative(outputFile)) {
+                    genomeId = FileUtils.getRelativePath(outputFile.getAbsolutePath(), genomeId);
+                }
                 globalElement.setAttribute(SessionAttribute.GENOME, genomeId);
             }
 
@@ -128,7 +134,7 @@ public class SessionWriter {
                 globalElement.setAttribute(SessionAttribute.LOCUS, locus);
             }
 
-            String groupBy = IGV.getInstance().getGroupByAttribute();
+            String groupBy = session.getGroupByAttribute();
             if (groupBy != null) {
                 globalElement.setAttribute(SessionAttribute.GROUP_TRACKS_BY, groupBy);
             }
@@ -142,12 +148,6 @@ public class SessionWriter {
                 globalElement.setAttribute("removeEmptyTracks", "true");
             }
 
-            globalElement.setAttribute(SessionAttribute.HAS_GENE_TRACK, "" + IGV.getInstance().hasGeneTrack());
-            globalElement.setAttribute(SessionAttribute.HAS_SEQ_TRACK, "" + IGV.getInstance().hasSequenceTrack());
-
-            globalElement.setAttribute("path", outputFile.getAbsolutePath());
-
-
             // Resource Files
             writeResources(outputFile, globalElement, document);
 
@@ -157,7 +157,6 @@ public class SessionWriter {
             // Panel layout
             writePanelLayout(globalElement, document);
 
-
             // Regions of Interest
             writeRegionsOfInterest(globalElement, document);
 
@@ -169,23 +168,20 @@ public class SessionWriter {
             }
 
             // Hidden attributes
-            if (session.getHiddenAttributes() != null && session.getHiddenAttributes().size() > 0) {
+            if (session.getHiddenAttributes() != null) {
                 writeHiddenAttributes(session, globalElement, document);
             }
 
-
             document.appendChild(globalElement);
 
             // Transform document into XML
-            xmlString = Utilities.getString(document);
+            return Utilities.getString(document);
         } catch (Exception e) {
-            String message = "An error has occurred while trying to create the session!";
+            String message = "Error creating session.";
             log.error(message, e);
-            JOptionPane.showMessageDialog(IGV.getMainFrame(), message);
+            JOptionPane.showMessageDialog(IGV.getInstance().getMainFrame(), message);
             throw new RuntimeException(e);
         }
-
-        return xmlString;
     }
 
 
@@ -262,7 +258,6 @@ public class SessionWriter {
             hiddenAttributes.appendChild(regionElement);
         }
         globalElement.appendChild(hiddenAttributes);
-
     }
 
     private void writeGeneList(Element globalElement, Document document) {
@@ -273,21 +268,18 @@ public class SessionWriter {
 
             Element geneListElement = document.createElement(SessionElement.GENE_LIST);
             geneListElement.setAttribute(SessionAttribute.NAME, geneList.getName());
-
             StringBuffer genes = new StringBuffer();
             for (String gene : geneList.getLoci()) {
                 genes.append(gene);
                 genes.append("\n");
             }
-
             geneListElement.setTextContent(genes.toString());
-
             globalElement.appendChild(geneListElement);
 
 
-            // Now store the list of frames visible
+            // Now store the list of frames visible.  This seems redundant, but frame extent can be changed after
+            // "gene list" definition, for example by zooming out or panning in a frame
             for (ReferenceFrame frame : FrameManager.getFrames()) {
-
                 Element frameElement = document.createElement(SessionElement.FRAME);
                 frameElement.setAttribute(SessionAttribute.NAME, frame.getName());
                 frameElement.setAttribute(SessionAttribute.CHR, frame.getChrName());
@@ -314,18 +306,13 @@ public class SessionWriter {
                     //RESOURCE ELEMENT
                     Element dataFileElement = document.createElement(SessionElement.RESOURCE);
 
-                    //REQUIRED ATTRIBUTES - Cannot be null
-
-                    boolean useRelative = PreferencesManager.getPreferences().getAsBoolean(Constants.SESSION_RELATIVE_PATH);
-
-                    String relativePath = useRelative ?
-                            FileUtils.getRelativePath(outputFile.getAbsolutePath(), resourceLocator.getPath()) :
-                            resourceLocator.getPath();
-
-                    dataFileElement.setAttribute(SessionAttribute.PATH, relativePath);
+                    String resourcePath = resourceLocator.getPath();
+                    if (isUseRelative(outputFile)) {
+                        resourcePath = FileUtils.getRelativePath(outputFile.getAbsolutePath(), resourcePath);
+                    }
+                    dataFileElement.setAttribute(SessionAttribute.PATH, resourcePath);
 
                     //OPTIONAL ATTRIBUTES
-
                     if (resourceLocator.getName() != null) {
                         dataFileElement.setAttribute(SessionAttribute.NAME, resourceLocator.getName());
                     }
@@ -341,8 +328,8 @@ public class SessionWriter {
                     if (resourceLocator.getDescription() != null) {
                         dataFileElement.setAttribute(SessionAttribute.DESCRIPTION, resourceLocator.getDescription());
                     }
-                    if (resourceLocator.getType() != null) {
-                        dataFileElement.setAttribute(SessionAttribute.TYPE, resourceLocator.getType());
+                    if (resourceLocator.format != null) {
+                        dataFileElement.setAttribute(SessionAttribute.TYPE, resourceLocator.format);
                     }
                     if (resourceLocator.getIndexPath() != null) {
                         dataFileElement.setAttribute(SessionAttribute.INDEX, resourceLocator.getIndexPath());
@@ -363,6 +350,11 @@ public class SessionWriter {
         }
     }
 
+    private boolean isUseRelative(File outputFile) {
+        return outputFile != null &&
+                PreferencesManager.getPreferences().getAsBoolean(Constants.SESSION_RELATIVE_PATH);
+    }
+
     private void writePanels(Element globalElement, Document document) throws DOMException {
 
         for (TrackPanel trackPanel : IGV.getInstance().getTrackPanels()) {
@@ -382,9 +374,15 @@ public class SessionWriter {
                     Element element = document.createElement("Track");
                     element.setAttribute("clazz", SessionElement.getXMLClassName(track.getClass()));
 
+                    String id = track.getId();
+                    if (isUseRelative(outputFile) && !FileUtils.isRemote(id)) {
+                        id = FileUtils.getRelativePath(outputFile.getAbsolutePath(), id);
+                    }
+                    element.setAttribute("id", id);
+
                     track.marshalXML(document, element);
 
-                    if(track.isNumeric() && track.getDataRange() != null) {
+                    if (track.isNumeric() && track.getDataRange() != null) {
                         Element dataRangeElement = document.createElement(SessionElement.DATA_RANGE);
                         track.getDataRange().marshalXML(document, dataRangeElement);
                         element.appendChild(dataRangeElement);
@@ -408,15 +406,12 @@ public class SessionWriter {
             globalElement.appendChild(panelLayout);
 
             StringBuffer locString = new StringBuffer();
-            locString.append(String.valueOf(dividerFractions[0]));
+            locString.append(dividerFractions[0]);
             for (int i = 1; i < dividerFractions.length; i++) {
                 locString.append("," + dividerFractions[i]);
             }
             panelLayout.setAttribute("dividerFractions", locString.toString());
-
-
         }
-
     }
 
     /**
@@ -430,8 +425,16 @@ public class SessionWriter {
                 IGV.getInstance().getDataResourceLocators();
 
         if (currentTrackFileLocators != null) {
+
+            // Filter data files that are included in genome annotations
+            List<ResourceLocator> genomeResources = GenomeManager.getInstance().getCurrentGenome().getAnnotationResources();
+            Set<String> absoluteGenomeAnnotationPaths = genomeResources == null ? Collections.emptySet() :
+                    genomeResources.stream().map(rl -> rl.getPath()).collect(Collectors.toSet());
+
             for (ResourceLocator locator : currentTrackFileLocators) {
-                locators.add(locator);
+                if (!absoluteGenomeAnnotationPaths.contains(locator.getPath())) {
+                    locators.add(locator);
+                }
             }
         }
 
