@@ -40,23 +40,22 @@ import htsjdk.tribble.index.Index;
 import htsjdk.tribble.index.IndexFactory;
 import htsjdk.tribble.util.LittleEndianOutputStream;
 import org.apache.commons.io.FilenameUtils;
-import org.apache.log4j.Logger;
-import org.apache.logging.log4j.Level;
-import org.apache.logging.log4j.core.LoggerContext;
-import org.apache.logging.log4j.core.appender.ConsoleAppender;
-import org.apache.logging.log4j.core.appender.ConsoleAppender.Target;
-import org.apache.logging.log4j.core.config.Configuration;
-import org.apache.logging.log4j.core.config.LoggerConfig;
-import org.apache.logging.log4j.core.layout.PatternLayout;
+import org.broad.igv.logging.*;
+//import org.broad.igv.logging.Level;
+//import org.broad.igv.logging.core.LoggerContext;
+//import org.broad.igv.logging.core.appender.ConsoleAppender;
+//import org.broad.igv.logging.core.appender.ConsoleAppender.Target;
+//import org.broad.igv.logging.core.config.Configuration;
+//import org.broad.igv.logging.core.config.LoggerConfig;
+//import org.broad.igv.logging.core.layout.PatternLayout;
 import org.broad.igv.Globals;
 import org.broad.igv.exceptions.DataLoadException;
 import org.broad.igv.feature.FeatureFileUtils;
 import org.broad.igv.feature.GFFParser;
 import org.broad.igv.feature.Mutation;
-import org.broad.igv.feature.genome.Genome;
-import org.broad.igv.feature.genome.GenomeDescriptor;
-import org.broad.igv.feature.genome.GenomeManager;
+import org.broad.igv.feature.genome.*;
 import org.broad.igv.feature.genome.fasta.FastaUtils;
+import org.broad.igv.feature.genome.load.GenomeDescriptor;
 import org.broad.igv.feature.tribble.CodecFactory;
 import org.broad.igv.feature.tribble.GFFCodec;
 import org.broad.igv.feature.tribble.IGVBEDCodec;
@@ -94,7 +93,7 @@ public class IgvTools {
     public static final String CMD_VCFTOBED = "vcftobed";
     public static final String CMD_DENSITIESTOBEDGRAPH = "densitiestobedgraph";
     public static final String CMD_GEN_GENOME_LIST = "genGenomeList";
-    static private Logger log = Logger.getLogger(IgvTools.class);
+    static private Logger log = LogManager.getLogger(IgvTools.class);
 
     static final String CMD_TILE = "tile";
     static final String CMD_TOTDF = "totdf";
@@ -220,23 +219,23 @@ public class IgvTools {
     }
 
     private static void initLogger() {
-        LoggerContext context = LoggerContext.getContext(false);
-        Configuration configuration = context.getConfiguration();
-        LoggerConfig rootLogger = configuration.getRootLogger();
-
-        if (Logger.getRootLogger().getAppender(CONSOLE_APPENDER_NAME) == null) {
-            PatternLayout layout = PatternLayout.newBuilder().withConfiguration(configuration)
-                    .withPattern("%m%n").build();
-            ConsoleAppender consoleAppender = ConsoleAppender.newBuilder().withName(CONSOLE_APPENDER_NAME)
-                    .setConfiguration(configuration)
-                    .setFollow(true)
-                    .withLayout(layout)
-                    .build();
-            consoleAppender.start();
-            configuration.addAppender(consoleAppender);
-            rootLogger.addAppender(consoleAppender, Level.INFO, null);
-            context.updateLoggers();
-        }
+//        LoggerContext context = LoggerContext.getContext(false);
+//        Configuration configuration = context.getConfiguration();
+//        LoggerConfig rootLogger = configuration.getRootLogger();
+//
+//        //if (LogManager.getRootLogger().getAppender(CONSOLE_APPENDER_NAME) == null) {
+//            PatternLayout layout = PatternLayout.newBuilder().withConfiguration(configuration)
+//                    .withPattern("%m%n").build();
+//            ConsoleAppender consoleAppender = ConsoleAppender.newBuilder().withName(CONSOLE_APPENDER_NAME)
+//                    .setConfiguration(configuration)
+//                    .setFollow(true)
+//                    .withLayout(layout)
+//                    .build();
+//            consoleAppender.start();
+//            configuration.addAppender(consoleAppender);
+//            rootLogger.addAppender(consoleAppender, Level.INFO, null);
+//            context.updateLoggers();
+//       // }
     }
 
 
@@ -525,8 +524,7 @@ public class IgvTools {
     }
 
     /**
-     * if ofile.equals(STDOUT_FILE_STR), write output to stdout. This also means redirecting log statements
-     * to someplace other than stdout, we use stderr
+     * if ofile.equals(STDOUT_FILE_STR), write output to stdout. This also means redirecting user messages to stderr
      *
      * @param ofile
      * @return Whether output will be written to stdout
@@ -535,28 +533,7 @@ public class IgvTools {
         //Output will be written to stdout instead of file,
         //need to redirect user messages
         if (ofile.equals(STDOUT_FILE_STR)) {
-
             userMessageWriter = System.err;
-
-            LoggerContext context = LoggerContext.getContext(false);
-            Configuration configuration = context.getConfiguration();
-            LoggerConfig rootLogger = configuration.getRootLogger();
-            PatternLayout layout = PatternLayout.newBuilder().withConfiguration(configuration)
-                    .withPattern("%m%n").build();
-            ConsoleAppender consoleAppender = ConsoleAppender.newBuilder().withName(CONSOLE_APPENDER_NAME)
-                    .setConfiguration(configuration)
-                    .setFollow(true)
-                    .setTarget(Target.SYSTEM_ERR)
-                    .withLayout(layout)
-                    .build();
-            consoleAppender.start();
-            rootLogger.removeAppender(CONSOLE_APPENDER_NAME);
-            configuration.addAppender(consoleAppender);
-            rootLogger.addAppender(consoleAppender, Level.INFO, null);
-            
-            //See log4j2.xml file
-            rootLogger.removeAppender("stdout");
-            context.updateLoggers();
             return true;
         }
         return false;
@@ -660,7 +637,7 @@ public class IgvTools {
         }
 
         ResourceLocator locator = new ResourceLocator(ifile);
-        locator.setType(typeString);
+        locator.setFormat(typeString);
         GCTtoIGVConverter.convert(locator, ofile, probefile, maxRecords, tmpDir, genome);
 
     }
@@ -671,16 +648,16 @@ public class IgvTools {
 
         if (!ifile.endsWith(".affective.csv")) validateIsTilable(typeString);
 
-        log.info("toTDF.  File = " + ifile);
-        log.info("Max zoom = " + maxZoomValue);
-        if (probeFile != null && probeFile.trim().length() > 0) {
-            log.info("Probe file = " + probeFile);
-        }
-        String wfString = "Window functions: ";
-        for (WindowFunction wf : windowFunctions) {
-            wfString += wf.toString() + " ";
-        }
-        log.info(wfString);
+//        log.warn("toTDF.  File = " + ifile);
+//        log.warn("Max zoom = " + maxZoomValue);
+//        if (probeFile != null && probeFile.trim().length() > 0) {
+//            log.warn("Probe file = " + probeFile);
+//        }
+//        String wfString = "Window functions: ";
+//        for (WindowFunction wf : windowFunctions) {
+//            wfString += wf.toString() + " ";
+//        }
+//        log.warn(wfString);
 
         boolean isGCT = isGCT(typeString);
         Genome genome = loadGenome(genomeId);
@@ -835,15 +812,15 @@ public class IgvTools {
                         String trackLine, String queryString, int minMapQuality, int countFlags) throws IOException {
 
 
-        log.info("Computing coverage.  File = " + ifile);
-        log.info("Max zoom = " + maxZoomValue);
-        log.info("Window size = " + windowSizeValue);
-        String wfString = "Window functions: ";
-        for (WindowFunction wf : windowFunctions) {
-            wfString += wf.toString() + " ";
-        }
-        log.info(wfString);
-        log.info("Ext factor = " + extFactorValue);
+//        log.warn("Computing coverage.  File = " + ifile);
+//        log.warn("Max zoom = " + maxZoomValue);
+//        log.warn("Window size = " + windowSizeValue);
+//        String wfString = "Window functions: ";
+//        for (WindowFunction wf : windowFunctions) {
+//            wfString += wf.toString() + " ";
+//        }
+//        log.warn(wfString);
+//        log.warn("Ext factor = " + extFactorValue);
 
 
         Genome genome = loadGenome(genomeId);
@@ -1024,7 +1001,7 @@ public class IgvTools {
         while(iter.hasNext()) {
             SAMRecord rec = iter.next();
             if (++totalRecords % 1000000 == 0) {
-                if (null != log) log.info(totalRecords + " reads processed ...");
+                System.out.println(totalRecords + " reads processed ...");
             }
             indexer.processAlignment(rec);
         }
@@ -1165,11 +1142,11 @@ public class IgvTools {
 
         File genomeFile = new File(genomeFileOrID);
         if (!genomeFile.exists()) {
-            genomeFile = new File(rootDir, "genomes" + File.separator + genomeFileOrID + ".genome");
-
+            String pre = genomeFileOrID.replace("/", "_");
+            genomeFile = new File(rootDir, "genomes" + File.separator + pre + ".chrom.sizes");
         }
         if (!genomeFile.exists()) {
-            genomeFile = new File(rootDir, "genomes" + File.separator + genomeFileOrID + ".chrom.sizes");
+            genomeFile = new File(rootDir, "genomes" + File.separator + genomeFileOrID + ".genome");
         }
         if (!genomeFile.exists()) {
             genomeFile = new File(rootDir, "genomes" + File.separator + genomeFileOrID);
@@ -1183,7 +1160,7 @@ public class IgvTools {
         //TODO Prevents loading genome again if loading from path.
         //May or may not want this, for now we just use it for testing
         if (Globals.isTesting() && genomeFile.getAbsolutePath().endsWith(".genome")) {
-            GenomeDescriptor genomeDescriptor = GenomeManager.parseGenomeArchiveFile(genomeFile);
+            GenomeDescriptor genomeDescriptor = GenomeDescriptor.parseGenomeArchiveFile(genomeFile);
             if (genome != null && genomeDescriptor.getId().equals(genome.getId())) {
                 return genome;
             }
