@@ -25,8 +25,9 @@
 
 package org.broad.igv.tools;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.track.WindowFunction;
+import org.broad.igv.ui.util.FileDialogUtils;
 
 import javax.swing.*;
 import javax.swing.border.EtchedBorder;
@@ -42,9 +43,7 @@ import java.util.Collection;
 
 public class IgvToolsGui extends JDialog {
 
-    private static Logger log = Logger.getLogger(IgvToolsGui.class);
-
-    static JFileChooser fileDialog;
+    private static Logger log = LogManager.getLogger(IgvToolsGui.class);
 
     public enum Tool {
         COUNT("Count"),
@@ -80,71 +79,51 @@ public class IgvToolsGui extends JDialog {
         initUI();
         updateUI();
 
-        closeButton.addActionListener(new ActionListener() {
-            public void actionPerformed(ActionEvent e) {
-                close();
-                setVisible(false);
-            }
+        closeButton.addActionListener(e -> {
+            close();
+            setVisible(false);
         });
-        inputButton.addActionListener(new ActionListener() {
-            public void actionPerformed(ActionEvent e) {
-                try {
-                    File chosenFile = chooseFile();
-                    inputField.setText(chosenFile.getAbsolutePath());
-                    updateUI();
-                } catch (NullPointerException npe) {
-                }
-            }
-        });
-        outputButton.addActionListener(new ActionListener() {
-            public void actionPerformed(ActionEvent e) {
-                try {
-                    File chosenFile = chooseFile();
-                    outputField.setText(chosenFile.getAbsolutePath());
-                    updateUI();
-                } catch (NullPointerException npe) {
-                }
-            }
-        });
-        genomeButton.addActionListener(new ActionListener() {
-            public void actionPerformed(ActionEvent e) {
-                try {
-                    File chosenFile = chooseFile();
-                    genomeField.setText(chosenFile.getAbsolutePath());
-                    updateUI();
-                } catch (NullPointerException npe) {
-                }
-            }
-        });
-        probeButton.addActionListener(new ActionListener() {
-            public void actionPerformed(ActionEvent e) {
-                try {
-                    File chosenFile = chooseFile();
-                    probeField.setText(chosenFile.getAbsolutePath());
-                    updateUI();
-                } catch (NullPointerException npe) {
-                }
-            }
-        });
-        toolCombo.addActionListener(new ActionListener() {
-            public void actionPerformed(ActionEvent e) {
+        inputButton.addActionListener(e -> {
+            try {
+                File chosenFile = chooseFile();
+                inputField.setText(chosenFile.getAbsolutePath());
                 updateUI();
+            } catch (NullPointerException npe) {
             }
         });
-        runButton.addActionListener(new ActionListener() {
-            public void actionPerformed(ActionEvent e) {
-                run();
+        outputButton.addActionListener(e -> {
+            try {
+                File chosenFile = chooseFile();
+                outputField.setText(chosenFile.getAbsolutePath());
+                updateUI();
+            } catch (NullPointerException npe) {
             }
         });
+        genomeButton.addActionListener(e -> {
+            try {
+                File chosenFile = chooseFile();
+                genomeField.setText(chosenFile.getAbsolutePath());
+                updateUI();
+            } catch (NullPointerException npe) {
+            }
+        });
+        probeButton.addActionListener(e -> {
+            try {
+                File chosenFile = chooseFile();
+                probeField.setText(chosenFile.getAbsolutePath());
+                updateUI();
+            } catch (NullPointerException npe) {
+            }
+        });
+        toolCombo.addActionListener(e -> updateUI());
+        runButton.addActionListener(e -> run());
 
-        tempButton.addActionListener(new ActionListener() {
-            public void actionPerformed(ActionEvent e) {
-                try {
-                    File chosenFile = chooseFile();
-                    tmpDirectoryField.setText(chosenFile.getAbsolutePath());
-                    updateUI();
-                } catch (NullPointerException npe) {
-                }
+        tempButton.addActionListener(e -> {
+            try {
+                File chosenFile = FileDialogUtils.chooseDirectory("Choose Directory:", lastDirectory);
+                tmpDirectoryField.setText(chosenFile.getAbsolutePath());
+                updateUI();
+            } catch (NullPointerException npe) {
             }
         });
     }
@@ -553,6 +532,9 @@ public class IgvToolsGui extends JDialog {
         if (maxCheckBox.isSelected()) {
             wfs.add(WindowFunction.max);
         }
+        if (absoluteMaxCheckBox.isSelected()) {
+            wfs.add(WindowFunction.absoluteMax);
+        }
         if (meanCheckBox.isSelected()) {
             wfs.add(WindowFunction.mean);
         }
@@ -582,29 +564,7 @@ public class IgvToolsGui extends JDialog {
      * @return
      */
     private File chooseFile() {
-
-        //TODO Override so you can specify the file type with a string array ex: {".wig", ".tdf"}
-
-        // if (fileDialog == null) {
-        fileDialog = new JFileChooser();
-        // }
-
-        if (lastDirectory != null) {
-            fileDialog.setCurrentDirectory(lastDirectory);
-        }
-
-        fileDialog.setMultiSelectionEnabled(false);
-
-        fileDialog.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
-
-        int returnVal = fileDialog.showDialog(this, "Select File");
-        if (returnVal == JFileChooser.CANCEL_OPTION) {
-            return null;
-        } else {
-            File selected = fileDialog.getSelectedFile();
-            lastDirectory = selected.isDirectory() ? selected : selected.getParentFile();
-            return selected;
-        }
+       return FileDialogUtils.chooseFile("Select File: ", lastDirectory, FileDialogUtils.LOAD);
     }
 
     public static void main(String[] args) {
@@ -706,6 +666,7 @@ public class IgvToolsGui extends JDialog {
         windowFunctionPanel = new JPanel();
         minCheckBox = new JCheckBox();
         maxCheckBox = new JCheckBox();
+        absoluteMaxCheckBox = new JCheckBox();
         meanCheckBox = new JCheckBox();
         medianCheckBox = new JCheckBox();
         a2CheckBox = new JCheckBox();
@@ -865,6 +826,10 @@ public class IgvToolsGui extends JDialog {
                     maxCheckBox.setText("Max");
                     windowFunctionPanel.add(maxCheckBox);
 
+                    //---- absoluteMaxCheckBox ----
+                    absoluteMaxCheckBox.setText("Absolute Max");
+                    windowFunctionPanel.add(absoluteMaxCheckBox);
+
                     //---- meanCheckBox ----
                     meanCheckBox.setSelected(true);
                     meanCheckBox.setText("Mean");
@@ -1069,6 +1034,7 @@ public class IgvToolsGui extends JDialog {
     private JPanel windowFunctionPanel;
     private JCheckBox minCheckBox;
     private JCheckBox maxCheckBox;
+    private JCheckBox absoluteMaxCheckBox;
     private JCheckBox meanCheckBox;
     private JCheckBox medianCheckBox;
     private JCheckBox a2CheckBox;
