@@ -25,7 +25,7 @@
 
 package org.broad.igv.tools;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.feature.Chromosome;
 import org.broad.igv.feature.genome.Genome;
@@ -49,7 +49,7 @@ import java.util.*;
  */
 public class Preprocessor implements DataConsumer {
 
-    private static Logger log = Logger.getLogger(Preprocessor.class);
+    private static Logger log = LogManager.getLogger(Preprocessor.class);
 
     boolean compressed = true;
     private boolean skipZeroes = false;
@@ -81,6 +81,7 @@ public class Preprocessor implements DataConsumer {
             WindowFunction.median,
             WindowFunction.min,
             WindowFunction.max,
+            WindowFunction.absoluteMax,
             WindowFunction.percentile2,
             WindowFunction.percentile10,
             WindowFunction.percentile90,
@@ -122,7 +123,7 @@ public class Preprocessor implements DataConsumer {
     public void setTrackParameters(TrackType trackType, String trackLine, String[] trackNames, boolean computeWholeGenome) {
 
         if (trackLine != null) {
-            log.info(trackLine);
+            // log.warn(trackLine);
         }
 
         if (outputFile != null && writer == null) {
@@ -328,16 +329,16 @@ public class Preprocessor implements DataConsumer {
             log.warn("No features were found that matched chromosomes in genome: " + genome.getId());
         } else {
             rawData.close();
-
-            // Record max/min
-            allDataStats.finish();
-            TDFGroup group = writer.getGroup("/");
-            group.setAttribute(TDFGroup.USE_PERCENTILE_AUTOSCALING, "true");
-            for (WindowFunction wf : allDataFunctions) {
-                group.setAttribute(wf.getValue(), String.valueOf(allDataStats.getValue(wf)));
-            }
-            writer.closeFile();
         }
+        // Record max/min
+        allDataStats.finish();
+        TDFGroup group = writer.getGroup("/");
+        group.setAttribute(TDFGroup.USE_PERCENTILE_AUTOSCALING, "true");
+        for (WindowFunction wf : allDataFunctions) {
+            group.setAttribute(wf.getValue(), String.valueOf(allDataStats.getValue(wf)));
+        }
+        writer.closeFile();
+
 
         if (statusMonitor != null) {
             statusMonitor.setPercentComplete(100);
