@@ -28,17 +28,18 @@ package org.broad.igv.track;
 //~--- non-JDK imports --------------------------------------------------------
 
 import htsjdk.tribble.Feature;
-import org.apache.log4j.Logger;
 import org.broad.igv.Globals;
 import org.broad.igv.event.IGVEventBus;
 import org.broad.igv.event.IGVEventObserver;
+import org.broad.igv.logging.LogManager;
+import org.broad.igv.logging.Logger;
+import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.renderer.*;
 import org.broad.igv.session.SessionAttribute;
 import org.broad.igv.ui.FontManager;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.TooltipTextFrame;
-import org.broad.igv.ui.UIConstants;
 import org.broad.igv.ui.color.ColorUtilities;
 import org.broad.igv.ui.panel.AttributeHeaderPanel;
 import org.broad.igv.ui.panel.IGVPopupMenu;
@@ -52,8 +53,8 @@ import org.w3c.dom.NodeList;
 
 import java.awt.*;
 import java.awt.event.MouseEvent;
-import java.util.*;
 import java.util.List;
+import java.util.*;
 
 import static org.broad.igv.prefs.Constants.*;
 
@@ -67,39 +68,13 @@ public abstract class AbstractTrack implements Track {
     public static final DisplayMode DEFAULT_DISPLAY_MODE = DisplayMode.COLLAPSED;
     public static final int DEFAULT_HEIGHT = -1;
     public static final int VISIBILITY_WINDOW = -1;
-    private static Logger log = Logger.getLogger(AbstractTrack.class);
-
-    /**
-     * Classes which we have tried to marshal/unmarshal
-     * and have failed. Since we just use exception catching (slow),
-     * we don't want to repeat failures
-     */
-    public static final Set<Class> knownUnknownTrackClasses = new HashSet<Class>();
-    public static final Class defaultTrackClass = AbstractTrack.class;
-
-    /**
-     * Set default renderer classes by track type.
-     */
-    private static Class defaultRendererClass = BarChartRenderer.class;
-    private static Map<TrackType, Class> defaultRendererMap = new HashMap();
-
-    static {
-        defaultRendererMap.put(TrackType.RNAI, HeatmapRenderer.class);
-        defaultRendererMap.put(TrackType.COPY_NUMBER, HeatmapRenderer.class);
-        defaultRendererMap.put(TrackType.CNV, HeatmapRenderer.class);
-        defaultRendererMap.put(TrackType.ALLELE_SPECIFIC_COPY_NUMBER, HeatmapRenderer.class);
-        defaultRendererMap.put(TrackType.GENE_EXPRESSION, HeatmapRenderer.class);
-        defaultRendererMap.put(TrackType.DNA_METHYLATION, HeatmapRenderer.class);
-        defaultRendererMap.put(TrackType.LOH, HeatmapRenderer.class);
-        defaultRendererMap.put(TrackType.OTHER, BarChartRenderer.class);
-        defaultRendererMap.put(TrackType.CHIP_CHIP, HeatmapRenderer.class);
-    }
-
+    public static final boolean DEFAULT_SHOW_FEATURE_NAMES = true;
+    private static Logger log = LogManager.getLogger(AbstractTrack.class);
 
     protected String id;
 
+    private String attributeKey;
     private String name;
-
     private String url;
     private boolean itemRGB = true;
 
@@ -109,7 +84,7 @@ public abstract class AbstractTrack implements Track {
 
     protected int fontSize = PreferencesManager.getPreferences().getAsInt(DEFAULT_FONT_SIZE);
     private boolean showDataRange = true;
-    private String sampleId;
+    protected String sampleId;
 
     private ResourceLocator resourceLocator;
 
@@ -137,8 +112,8 @@ public abstract class AbstractTrack implements Track {
 
     String autoscaleGroup;
 
-    protected Color posColor = DEFAULT_COLOR;
-    protected Color altColor = posColor;
+    protected Color posColor = null;
+    protected Color altColor = null;
 
     protected int visibilityWindow = VISIBILITY_WINDOW;
     private DisplayMode displayMode = DEFAULT_DISPLAY_MODE;
@@ -146,6 +121,7 @@ public abstract class AbstractTrack implements Track {
     protected Integer height = DEFAULT_HEIGHT;
 
     protected DataRange dataRange;
+    private boolean showFeatureNames = DEFAULT_SHOW_FEATURE_NAMES;
 
     public AbstractTrack() {
     }
@@ -157,26 +133,14 @@ public abstract class AbstractTrack implements Track {
         this.resourceLocator = dataResourceLocator;
         this.id = id;
         this.name = name;
+        this.attributeKey = this.name;
         init();
     }
 
-    public AbstractTrack(ResourceLocator dataResourceLocator, String id) {
-        this(dataResourceLocator, id, dataResourceLocator.getTrackName());
-    }
-
     public AbstractTrack(ResourceLocator locator) {
         this(locator, locator != null ? locator.getPath() : null, locator != null ? locator.getTrackName() : null);
     }
 
-    public AbstractTrack(String id) {
-        this(null, id, id);
-    }
-
-
-    public AbstractTrack(String id, String name) {
-        this(null, id, name);
-    }
-
     private void init() {
         showDataRange = PreferencesManager.getPreferences().getAsBoolean(CHART_SHOW_DATA_RANGE);
         if (PreferencesManager.getPreferences().getAsBoolean(EXPAND_FEAUTRE_TRACKS)) {
@@ -214,7 +178,7 @@ public abstract class AbstractTrack implements Track {
         return name;
     }
 
-    private String getDisplayName() {
+    public String getDisplayName() {
 
         String sampleKey = IGV.getInstance().getSession().getTrackAttributeName();
         if (sampleKey != null && sampleKey.trim().length() > 0) {
@@ -314,12 +278,19 @@ public abstract class AbstractTrack implements Track {
 
 
     public Color getColor() {
+        return posColor == null ? DEFAULT_COLOR : posColor;
+    }
+
+    public Color getExplicitColor() {
         return posColor;
     }
 
     public Color getAltColor() {
-        return altColor;
+        return altColor == null ? getColor() : altColor;
+    }
 
+    public Color getExplicitAltColor() {
+        return altColor;
     }
 
     public ResourceLocator getResourceLocator() {
@@ -366,7 +337,7 @@ public abstract class AbstractTrack implements Track {
      * <p/>
      * (1) the track attribute table
      * (2) by sampleId, as set in the Resource element of a session or load-from-server menu
-     * (3) by track name, the visible display name
+     * (3) by attributeKey, set from the original track name
      * (4) by full path to the file associated with this track
      *
      * @param attributeName
@@ -398,7 +369,7 @@ public abstract class AbstractTrack implements Track {
             value = attributeManager.getAttribute(getSample(), key);
         }
         if (value == null) {
-            value = attributeManager.getAttribute(getName(), key);
+            value = attributeManager.getAttribute(this.attributeKey, key);
         }
         if (value == null && getResourceLocator() != null && getResourceLocator().getPath() != null) {
             value = attributeManager.getAttribute(getResourceLocator().getPath(), key);
@@ -407,16 +378,11 @@ public abstract class AbstractTrack implements Track {
     }
 
     public String getSample() {
-
         if (sampleId != null) {
-            return sampleId;
+            return sampleId;    // Explicitly set sample ID (e.g. from server load XML)
         }
-//        String sample = AttributeManager.getInstance().getSampleFor(getName());
-//        return sample != null ? sample : getName();
-
         sampleId = AttributeManager.getInstance().getSampleFor(getName());
         return sampleId != null ? sampleId : getName();
-
     }
 
 
@@ -428,7 +394,7 @@ public abstract class AbstractTrack implements Track {
      * @return
      */
     private int getDefaultHeight() {
-        if (XYPlotRenderer.class.isAssignableFrom(getDefaultRendererClass())) {
+        if (getDefaultRenderer() instanceof XYPlotRenderer) {
             return PreferencesManager.getPreferences().getAsInt(CHART_TRACK_HEIGHT_KEY);
         } else {
             return PreferencesManager.getPreferences().getAsInt(TRACK_HEIGHT_KEY);
@@ -451,7 +417,6 @@ public abstract class AbstractTrack implements Track {
         }
     }
 
-
     public void setMinimumHeight(int minimumHeight) {
         this.minimumHeight = minimumHeight;
     }
@@ -460,7 +425,6 @@ public abstract class AbstractTrack implements Track {
         this.maximumHeight = maximumHeight;
     }
 
-
     /**
      * Return the actual minimum height if one has been set, otherwise get the default for the current renderer.
      *
@@ -478,7 +442,6 @@ public abstract class AbstractTrack implements Track {
         this.trackType = type;
     }
 
-
     public TrackType getTrackType() {
         return trackType;
     }
@@ -504,32 +467,29 @@ public abstract class AbstractTrack implements Track {
         this.posColor = color;
     }
 
-
     public void setAltColor(Color color) {
         altColor = color;
     }
 
-
-    public void setVisible(boolean isVisible) {
-        this.visible = isVisible;
+    public void setVisible(boolean visible) {
+        if (this.visible != visible) {
+            this.visible = visible;
+            if (IGV.hasInstance()) IGV.getInstance().getMainPanel().revalidate();
+        }
     }
 
-
     public void setOverlayed(boolean bool) {
         this.overlaid = bool;
     }
 
-
     public void setSelected(boolean selected) {
         this.selected = selected;
     }
 
-
     public boolean isSelected() {
         return selected;
     }
 
-
     public void setHeight(int height) {
         setHeight(height, false);
     }
@@ -574,10 +534,20 @@ public abstract class AbstractTrack implements Track {
         this.dataRange = axisDefinition;
     }
 
-
-    protected Class getDefaultRendererClass() {
-        Class def = defaultRendererMap.get(getTrackType());
-        return (def == null) ? defaultRendererClass : def;
+    protected Renderer getDefaultRenderer() {
+        TrackType trackType = getTrackType();
+        switch (trackType) {
+            case RNAI:
+            case COPY_NUMBER:
+            case ALLELE_SPECIFIC_COPY_NUMBER:
+            case GENE_EXPRESSION:
+            case DNA_METHYLATION:
+            case LOH:
+            case CHIP_CHIP:
+                return new HeatmapRenderer();
+            default:
+                return new BarChartRenderer();
+        }
     }
 
     public Collection<WindowFunction> getAvailableWindowFunctions() {
@@ -598,16 +568,15 @@ public abstract class AbstractTrack implements Track {
         String popupText = getValueStringAt(frame.getChrName(), e.getChromosomePosition(), e.getMouseEvent().getX(), e.getMouseEvent().getY(), frame);
 
         if (popupText != null) {
+            Color color = IGV.getInstance().getRootPane().getJMenuBar().getForeground();
+            String htmlColor = String.format("#%02x%02x%02x", color.getRed(), color.getGreen(), color.getBlue());
+            popupText = "<div style=\"color: " + htmlColor + "\">" + popupText + "</div>";
 
             final TooltipTextFrame tf = new TooltipTextFrame(getName(), popupText);
             Point p = me.getComponent().getLocationOnScreen();
             tf.setLocation(Math.max(0, p.x + me.getX() - 150), Math.max(0, p.y + me.getY() - 150));
 
-            UIUtilities.invokeOnEventThread(new Runnable() {
-                public void run() {
-                    tf.setVisible(true);
-                }
-            });
+            UIUtilities.invokeOnEventThread(() -> tf.setVisible(true));
             return true;
         }
         return false;
@@ -782,7 +751,7 @@ public abstract class AbstractTrack implements Track {
             } else {
                 colorScale = new ContinuousColorScale(min, max, minColor, c);
             }
-            colorScale.setNoDataColor(UIConstants.NO_DATA_COLOR);
+            colorScale.setNoDataColor(PreferencesManager.getPreferences().getAsColor(Constants.NO_DATA_COLOR));
         }
         return colorScale;
     }
@@ -840,23 +809,13 @@ public abstract class AbstractTrack implements Track {
         return null;
     }
 
-    /**
-     * Special normalization function for linear (non logged) copy number data
-     *
-     * @param value
-     * @param norm
-     * @return
-     */
-    public static float getLogNormalizedValue(float value, double norm) {
-        if (norm == 0) {
-            return Float.NaN;
-        } else {
-            return (float) (Math.log(Math.max(Float.MIN_VALUE, value) / norm) / Globals.log2);
-        }
-    }
 
     public float logScaleData(float dataY) {
 
+        if (Float.isNaN(dataY)) {
+            return dataY;
+        }
+
         // Special case for copy # -- centers data around 2 copies (1 for allele
         // specific) and log normalizes
         if (((getTrackType() == TrackType.COPY_NUMBER) ||
@@ -866,11 +825,10 @@ public abstract class AbstractTrack implements Track {
             double centerValue = (getTrackType() == TrackType.ALLELE_SPECIFIC_COPY_NUMBER)
                     ? 1.0 : 2.0;
 
-            dataY = getLogNormalizedValue(dataY, centerValue);
+            return (float) (Math.log(Math.max(Float.MIN_VALUE, dataY) / centerValue) / Globals.log2);
+        } else {
+            return dataY;
         }
-
-
-        return dataY;
     }
 
     public boolean isRegionScoreType(RegionScoreType type) {
@@ -988,7 +946,7 @@ public abstract class AbstractTrack implements Track {
     }
 
     @Override
-    public void dispose() {
+    public void unload() {
         if (this instanceof IGVEventObserver) {
             IGVEventBus.getInstance().unsubscribe((IGVEventObserver) this);
         }
@@ -1014,16 +972,19 @@ public abstract class AbstractTrack implements Track {
     @Override
     public void marshalXML(Document document, Element element) {
 
-        element.setAttribute("id", id);
         element.setAttribute("name", name);
+        element.setAttribute("attributeKey", attributeKey);
+        element.setAttribute("id", id);
         element.setAttribute("fontSize", String.valueOf(fontSize));
         element.setAttribute("visible", String.valueOf(visible));
 
-
-        if (posColor != DEFAULT_COLOR) {
+        if (showFeatureNames != DEFAULT_SHOW_FEATURE_NAMES) {
+            element.setAttribute("showFeatureNames", Boolean.toString(showFeatureNames));
+        }
+        if (posColor != null) {
             element.setAttribute(SessionAttribute.COLOR, ColorUtilities.colorToString(posColor));
         }
-        if (altColor != DEFAULT_COLOR) {
+        if (altColor != null) {
             element.setAttribute(SessionAttribute.ALT_COLOR, ColorUtilities.colorToString(altColor));
         }
 
@@ -1048,6 +1009,10 @@ public abstract class AbstractTrack implements Track {
             element.setAttribute("height", String.valueOf(this.height));
         }
 
+        if (showDataRange == false) {
+            element.setAttribute("showDataRange", Boolean.toString(showDataRange));
+        }
+
         if (isNumeric()) {
             if (autoscaleGroup != null) {
                 element.setAttribute("autoscaleGroup", this.autoscaleGroup);
@@ -1055,7 +1020,7 @@ public abstract class AbstractTrack implements Track {
 
             element.setAttribute("autoScale", String.valueOf(this.autoScale));
 
-            if(this.getWindowFunction() != null) {
+            if (this.getWindowFunction() != null) {
                 element.setAttribute("windowFunction", String.valueOf(this.getWindowFunction()));
             }
         }
@@ -1063,6 +1028,15 @@ public abstract class AbstractTrack implements Track {
     }
 
 
+    public void setShowFeatureNames(boolean b) {
+        this.showFeatureNames = b;
+    }
+
+    @Override
+    public boolean isShowFeatureNames() {
+        return showFeatureNames;
+    }
+
     /**
      * Restore track from XML serialization -- work in progress
      * //        <renderer="BASIC_FEATURE" sortable="false" visible="true" windowFunction="count">
@@ -1073,8 +1047,19 @@ public abstract class AbstractTrack implements Track {
     @Override
     public void unmarshalXML(Element element, Integer version) {
 
-        this.name = element.getAttribute("name");
-        this.id = element.getAttribute("id");
+        if (element.hasAttribute("name")) {
+            this.name = element.getAttribute("name");
+        }
+
+        if (element.hasAttribute("id")) {
+            this.id = element.getAttribute("id");
+        }
+
+        if (element.hasAttribute("attributeKey")) {
+            this.attributeKey = element.getAttribute("attributeKey");
+        } else {
+            this.attributeKey = this.name;
+        }
 
         if (element.hasAttribute("displayMode")) {
             try {
@@ -1089,7 +1074,6 @@ public abstract class AbstractTrack implements Track {
             try {
                 Color c = ColorUtilities.stringToColor(element.getAttribute("color"));
                 this.posColor = c;
-                this.altColor = c;  // default
             } catch (Exception e) {
                 log.error("Unrecognized color: " + element.getAttribute("color"));
             }
@@ -1129,7 +1113,16 @@ public abstract class AbstractTrack implements Track {
         }
 
         if (element.hasAttribute("autoscaleGroup")) {
-            this.autoscaleGroup = element.getAttribute("autoscaleGroup");
+            String autoscaleGroup = element.getAttribute("autoscaleGroup");
+            this.setAttributeValue(AttributeManager.GROUP_AUTOSCALE, "" + autoscaleGroup);
+        }
+
+        if (element.hasAttribute("showDataRange")) {
+            try {
+                this.showDataRange = Boolean.valueOf(element.getAttribute("showDataRange"));
+            } catch (Exception e) {
+                log.error("Unrecognized showDataRange: " + element.getAttribute("showDataRange"));
+            }
         }
 
         if (element.hasAttribute("featureVisibilityWindow")) {
@@ -1140,6 +1133,15 @@ public abstract class AbstractTrack implements Track {
             }
         }
 
+        if (element.hasAttribute("showFeatureNames")) {
+            try {
+                this.showFeatureNames = Boolean.valueOf(element.getAttribute("showFeatureNames"));
+            } catch (Exception e) {
+                log.error("Unrecognized showDataRange: " + element.getAttribute("showFeatureNames"));
+            }
+
+        }
+
         if (element.hasAttribute("fontSize")) {
             try {
                 this.fontSize = Integer.parseInt(element.getAttribute("fontSize"));
