@@ -33,7 +33,7 @@
  */
 package org.broad.igv.track;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.event.IGVEventBus;
 import org.broad.igv.event.IGVEventObserver;
@@ -68,7 +68,7 @@ import java.util.List;
 
 public abstract class DataTrack extends AbstractTrack implements ScalableTrack, IGVEventObserver {
 
-    private static Logger log = Logger.getLogger(DataTrack.class);
+    private static Logger log = LogManager.getLogger(DataTrack.class);
 
     private DataRenderer renderer;
 
@@ -97,7 +97,7 @@ public abstract class DataTrack extends AbstractTrack implements ScalableTrack,
 
 
         } else {
-            log.info("Unknown event type: " + event.getClass());
+            log.warn("Unknown event type: " + event.getClass());
         }
     }
 
@@ -141,6 +141,10 @@ public abstract class DataTrack extends AbstractTrack implements ScalableTrack,
         int delta = multiLocus ? 1 : (end - start) / 2;
         int expandedStart = Math.max(0, start - delta);
         int expandedEnd = Math.min(maxEnd, end + delta);
+        if(expandedEnd < 0) {
+            // overflow
+            expandedEnd = Integer.MAX_VALUE;
+        }
         LoadedDataInterval<List<LocusScore>> interval = getSummaryScores(queryChr, expandedStart, expandedEnd, zoom);
         loadedIntervalCache.put(referenceFrame.getName(), interval);
 
@@ -240,7 +244,7 @@ public abstract class DataTrack extends AbstractTrack implements ScalableTrack,
 
     public void setRendererClass(Class rc) {
         try {
-            renderer = (DataRenderer) rc.newInstance();
+            renderer = (DataRenderer) rc.getDeclaredConstructor().newInstance();
         } catch (Exception ex) {
             log.error("Error instantiating renderer ", ex);
         }
@@ -255,7 +259,7 @@ public abstract class DataTrack extends AbstractTrack implements ScalableTrack,
     @Override
     public DataRenderer getRenderer() {
         if (renderer == null) {
-            setRendererClass(getDefaultRendererClass());
+            renderer = (DataRenderer) getDefaultRenderer();
         }
         return renderer;
     }
