@@ -27,12 +27,13 @@ package org.broad.igv.track;
 
 import htsjdk.tribble.Feature;
 import htsjdk.tribble.TribbleException;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.event.DataLoadedEvent;
 import org.broad.igv.event.IGVEventBus;
 import org.broad.igv.event.IGVEventObserver;
 import org.broad.igv.feature.*;
+import org.broad.igv.feature.Range;
 import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.feature.genome.GenomeManager;
 import org.broad.igv.prefs.Constants;
@@ -47,6 +48,7 @@ import org.broad.igv.util.BrowserLauncher;
 import org.broad.igv.util.ResourceLocator;
 import org.broad.igv.util.StringUtils;
 import org.broad.igv.variant.VariantTrack;
+import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 import org.w3c.dom.NodeList;
 
@@ -55,6 +57,7 @@ import java.awt.event.MouseEvent;
 import java.io.IOException;
 import java.util.*;
 import java.util.List;
+import java.util.stream.Collectors;
 
 /**
  * Track which displays features, typically showing regions of the genome
@@ -67,7 +70,7 @@ import java.util.List;
 
 public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
-    private static Logger log = Logger.getLogger(FeatureTrack.class);
+    private static Logger log = LogManager.getLogger(FeatureTrack.class);
 
 
     public static final int MINIMUM_FEATURE_SPACING = 5;
@@ -84,14 +87,11 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
     Track.DisplayMode lastFeatureMode = null;  // Keeps track of the feature display mode before an auto-switch to COLLAPSE
 
-
-    protected List<Rectangle> levelRects = new ArrayList();
-
     // TODO -- this is a memory leak, this cache needs cleared when the reference frame collection (gene list) changes
     /**
      * Map of reference frame name -> packed features
      */
-    protected Map<String, PackedFeatures<IGVFeature>> packedFeaturesMap = Collections.synchronizedMap(new HashMap<String, PackedFeatures<IGVFeature>>());
+    protected Map<String, PackedFeatures<Feature>> packedFeaturesMap = Collections.synchronizedMap(new HashMap<>());
 
     protected Renderer renderer;
 
@@ -114,11 +114,9 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
     private boolean alternateExonColor = false;
 
-    private static final String PLUGIN_SOURCE = "PluginSource";
-    private static final String SEQUENCE_MATCH_SOURCE = "SequenceMatchSource";
+    private String trackLine = null;
 
-
-    String trackLine = null;
+    private boolean groupByStrand = false;
 
     public FeatureTrack() {
 
@@ -127,16 +125,12 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
     // TODO -- there are WAY too many constructors for this class
 
     /**
-     * Construct with no feature source.  Currently this is only used for the SpliceJunctionTrack subclass.
+     * Constructor used by SpliceJunctionTrack, BlatTrack, and MotifTrack
      *
+     * @param locator -- For splice junctions, ResourceLocator for associated alignment track.  Null otherwise
      * @param id
      * @param name
      */
-    public FeatureTrack(String id, String name) {
-        super(id, name);
-        setSortable(false);
-    }
-
     public FeatureTrack(ResourceLocator locator, String id, String name) {
         super(locator, id, name);
         setSortable(false);
@@ -144,20 +138,14 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
     /**
      * Constructor with no ResourceLocator.  Note:  tracks using this constructor will not be recorded in the
-     * "Resources" section of session files.
-     *
-     * @param id
-     * @param name
-     * @param source
-     * @api
+     * "Resources" section of session files.  This method is used by ".genome" and gbk genome loaders.
      */
     public FeatureTrack(String id, String name, FeatureSource source) {
-        super(id, name);
-        init(source, null);
+        super(null, id, name);
+        init(null, source);
         setSortable(false);
     }
 
-
     /**
      * Constructor specifically for BigWig data source
      *
@@ -168,51 +156,54 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
      */
     public FeatureTrack(ResourceLocator locator, String id, String name, FeatureSource source) {
         super(locator, id, name);
-        init(source, locator.getPath());
+        init(locator, source);
         setSortable(false);
     }
 
-
     public FeatureTrack(ResourceLocator locator, FeatureSource source) {
         super(locator);
-        init(source, locator != null ? locator.getPath() : null);
+        init(locator, source);
         setSortable(false);
     }
 
 
+    /**
+     * SMap files (bionano), MutationTrack
+     */
     public FeatureTrack(ResourceLocator locator, String id, FeatureSource source) {
-        super(locator, id);
-        init(source, locator.getPath());
+        super(locator, id, locator.getTrackName());
+        init(locator, source);
     }
 
     /**
-     * Create a new track which is a shallow copy of this one
-     *
-     * @param featureTrack
+     * Create a new track which is a shallow copy of this one.  Currently used by SashimiPlot.
      */
     public FeatureTrack(FeatureTrack featureTrack) {
         this(featureTrack.getId(), featureTrack.getName(), featureTrack.source);
     }
 
-    protected void init(FeatureSource source, String path) {
+    protected void init(ResourceLocator locator, FeatureSource source) {
 
         this.source = source;
         setMinimumHeight(10);
-        setColor(Color.blue.darker());
 
         coverageRenderer = new BarChartRenderer();
 
-        int sourceFeatureWindowSize = source.getFeatureWindowSize();
-        if (sourceFeatureWindowSize > 0) {  // Only apply a default if the feature source supports visibility window.
+        Integer vizWindow = locator == null ? null : locator.getVisibilityWindow();
+
+        if (vizWindow != null) {
+            visibilityWindow = vizWindow;
+        } else {
+            int sourceFeatureWindowSize = source.getFeatureWindowSize();
             int defVisibilityWindow = PreferencesManager.getPreferences().getAsInt(Constants.DEFAULT_VISIBILITY_WINDOW);
-            if (defVisibilityWindow > 0) {
-                setVisibilityWindow(defVisibilityWindow * 1000);
+            if (sourceFeatureWindowSize > 0 && defVisibilityWindow > 0) {  // Only apply a default if the feature source supports visibility window.
+                visibilityWindow = defVisibilityWindow * 1000;
             } else {
                 visibilityWindow = sourceFeatureWindowSize;
             }
         }
 
-        this.renderer = path != null && path.endsWith("junctions.bed") ?
+        this.renderer = locator != null && locator.getPath().endsWith("junctions.bed") ?
                 new SpliceJunctionRenderer() : new IGVFeatureRenderer();
 
         IGVEventBus.getInstance().subscribe(DataLoadedEvent.class, this);
@@ -220,11 +211,10 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
     }
 
     @Override
-    public void dispose() {
-        super.dispose();
+    public void unload() {
+        super.unload();
         if (source != null) {
-            source.dispose();
-            source = null;
+            source.close();
         }
     }
 
@@ -241,16 +231,10 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 //                IGV.getInstance().layoutMainPanel();
 //            }
         } else {
-            log.info("Unknown event type: " + e.getClass());
+            log.warn("Unknown event type: " + e.getClass());
         }
     }
 
-
-    @Override
-    public boolean isFilterable() {
-        return false; // Don't filter "feature" tracks
-    }
-
     @Override
     public int getHeight() {
         if (!isVisible()) {
@@ -277,13 +261,9 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
         this.squishedRowHeight = squishedRowHeight;
     }
 
-    public int getFeatureWindowSize() {
-        return source.getFeatureWindowSize();
-    }
-
     public void setRendererClass(Class rc) {
         try {
-            renderer = (Renderer) rc.newInstance();
+            renderer = (Renderer) rc.getDeclaredConstructor().newInstance();
         } catch (Exception ex) {
             log.error("Error instatiating renderer ", ex);
         }
@@ -316,15 +296,13 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
      * @return
      */
     public int getNumberOfFeatureLevels() {
-        if (areFeaturesStacked() && packedFeaturesMap.size() > 0) {
+        if (packedFeaturesMap.size() > 0) {
             int n = 0;
-            synchronized (packedFeaturesMap) {
-                for (PackedFeatures pf : packedFeaturesMap.values()) {
-                    //dhmay adding null check.  To my mind this shouldn't be necessary, but we're encountering
-                    //it intermittently.  Food for future thought
-                    if (pf != null) {
-                        n = Math.max(n, pf.getRowCount());
-                    }
+            for (PackedFeatures pf : packedFeaturesMap.values()) {
+                //dhmay adding null check.  To my mind this shouldn't be necessary, but we're encountering
+                //it intermittently.  Food for future thought
+                if (pf != null) {
+                    n = Math.max(n, pf.getRowCount());
                 }
             }
             return n;
@@ -332,14 +310,6 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
         return 1;
     }
 
-
-    /**
-     * @return Whether features are displayed stacked on top of one another, rather than overlapping
-     */
-    protected boolean areFeaturesStacked() {
-        return getDisplayMode() != DisplayMode.COLLAPSED;
-    }
-
     /**
      * Return a score over the interval.  This is required by the track interface to support sorting.
      */
@@ -417,7 +387,7 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
             StringBuffer buf = new StringBuffer();
             boolean firstFeature = true;
-            int maxNumber = 10;
+            int maxNumber = 100;
             int n = 1;
             for (Feature feature : allFeatures) {
                 if (feature != null && feature instanceof IGVFeature) {
@@ -534,13 +504,12 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
                 rowHeight = getHeight();
         }
 
-        return Math.max(0, Math.min(levelRects.size() - 1, (int) ((y - this.getY() - this.margin) / rowHeight)));
+        return Math.max(0, (y - this.getY() - this.margin) / rowHeight);
 
     }
 
     /**
-     * Knowing the feature row, figure out which feature is at {@code position}. If not expanded,
-     * featureRow is ignored
+     * Knowing the feature row, figure out which feature is at {@code position}.
      *
      * @param position
      * @param featureRow
@@ -549,20 +518,24 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
      */
     public List<Feature> getFeaturesAtPositionInFeatureRow(double position, int featureRow, ReferenceFrame frame) {
 
-        PackedFeatures<IGVFeature> packedFeatures = packedFeaturesMap.get(frame.getName());
+        PackedFeatures<Feature> packedFeatures = packedFeaturesMap.get(frame.getName());
 
         if (packedFeatures == null) {
             return null;
         }
 
+        List<PackedFeatures<Feature>.FeatureRow> rows = packedFeatures.getRows();
+        if (featureRow < 0 || featureRow >= rows.size()) {
+            return null;
+        }
+
         //If features are stacked we look at only the row.
         //If they are collapsed on top of each other, we get all features in all rows
-        int nLevels = areFeaturesStacked() ? packedFeatures.getRowCount() : 1;
-        List<IGVFeature> possFeatures = null;
-        if ((nLevels > 1) && (featureRow < nLevels)) {
-            possFeatures = packedFeatures.getRows().get(featureRow).getFeatures();
-        } else {
+        List<Feature> possFeatures;
+        if (getDisplayMode() == DisplayMode.COLLAPSED) {
             possFeatures = packedFeatures.getFeatures();
+        } else {
+            possFeatures = rows.get(featureRow).getFeatures();
         }
 
         List<Feature> featureList = null;
@@ -586,27 +559,20 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
         MouseEvent e = te.getMouseEvent();
 
-        //Selection of an expanded feature row
-        if (areFeaturesStacked()) {
-            if (levelRects != null) {
-                for (int i = 0; i < levelRects.size(); i++) {
-                    Rectangle rect = levelRects.get(i);
-                    if (rect.contains(e.getPoint())) {
-                        if (i == selectedFeatureRowIndex)
-                            setSelectedFeatureRowIndex(FeatureTrack.NO_FEATURE_ROW_SELECTED);
-                        else {
-                            //make this track selected
-                            setSelected(true);
-                            //select the appropriate row
-                            setSelectedFeatureRowIndex(i);
-                        }
-                        IGV.getInstance().doRefresh();
-                        break;
-                    }
-                }
+        // Toggle selection of row
+        if (getDisplayMode() != DisplayMode.COLLAPSED) {
+            int i = getFeatureRow(e.getY());
+            if (i == selectedFeatureRowIndex)
+                setSelectedFeatureRowIndex(FeatureTrack.NO_FEATURE_ROW_SELECTED);
+            else {
+                //make this track selected
+                setSelected(true);
+                //select the appropriate row
+                setSelectedFeatureRowIndex(i);
             }
         }
 
+
         //For feature selection
         selectedFeature = null;
 
@@ -671,12 +637,18 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
     public void setDisplayMode(DisplayMode mode) {
         // Explicity setting the display mode overrides the automatic switch
         lastFeatureMode = null;
+
+        for (PackedFeatures pf : packedFeaturesMap.values()) {
+            pf.pack(mode, groupByStrand);
+        }
+
         super.setDisplayMode(mode);
     }
 
     @Override
     public boolean isReadyToPaint(ReferenceFrame frame) {
         if (!isShowFeatures(frame)) {
+            packedFeaturesMap.clear();
             return true;  // Ready by definition (nothing to paint)
         } else {
             PackedFeatures packedFeatures = packedFeaturesMap.get(frame.getName());
@@ -688,7 +660,7 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
     }
 
     public void load(ReferenceFrame frame) {
-        loadFeatures(frame.getChrName(), (int) frame.getOrigin(), (int) frame.getEnd(), frame);
+        loadFeatures(frame.getChrName(), (int) frame.getOrigin(), (int) frame.getEnd(), frame.getName());
     }
 
     /**
@@ -698,13 +670,20 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
      * @param start
      * @param end
      */
-    protected void loadFeatures(final String chr, final int start, final int end, final ReferenceFrame frame) {
+    protected void loadFeatures(final String chr, final int start, final int end, final String frame) {
+
+        if (source == null) {
+            return;
+        }
 
         try {
 
             int delta = (end - start) / 2;
             int expandedStart = start - delta;
             int expandedEnd = end + delta;
+            if (expandedEnd < 0) {
+                expandedEnd = Integer.MAX_VALUE;  // overflow
+            }
 
             //Make sure we are only querying within the chromosome we allow for somewhat pathological cases of start
             //being negative and end being outside, but only if directly queried. Our expansion should not
@@ -719,24 +698,21 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
                 if (c != null && end < c.getLength()) expandedEnd = Math.min(c.getLength(), expandedEnd);
             }
 
-            if (source == null) {
-                System.out.println();
-            }
             Iterator<Feature> iter = source.getFeatures(chr, expandedStart, expandedEnd);
 
             if (iter == null) {
                 PackedFeatures pf = new PackedFeatures(chr, expandedStart, expandedEnd);
-                packedFeaturesMap.put(frame.getName(), pf);
+                packedFeaturesMap.put(frame, pf);
             } else {
-                PackedFeatures pf = new PackedFeatures(chr, expandedStart, expandedEnd, iter, getName());
-                packedFeaturesMap.put(frame.getName(), pf);
-                //log.info("Loaded " + chr + " " + expandedStart + "-" + expandedEnd);
+                PackedFeatures pf = new PackedFeatures(chr, expandedStart, expandedEnd, iter, this.getDisplayMode(), groupByStrand);
+                packedFeaturesMap.put(frame, pf);
+                //log.warn("Loaded " + chr + " " + expandedStart + "-" + expandedEnd);
             }
 
         } catch (Exception e) {
             // Mark the interval with an empty feature list to prevent an endless loop of load attempts.
             PackedFeatures pf = new PackedFeatures(chr, start, end);
-            packedFeaturesMap.put(frame.getName(), pf);
+            packedFeaturesMap.put(frame, pf);
             String msg = "Error loading features for interval: " + chr + ":" + start + "-" + end + " <br>" + e.toString();
             MessageUtils.showMessage(msg);
             log.error(msg, e);
@@ -758,7 +734,7 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
                 lastFeatureMode = null;
             }
             renderFeatures(context, renderRect);
-        } else {
+        } else if (coverageRenderer != null) {
             if (getDisplayMode() != DisplayMode.COLLAPSED) {
                 // An ugly hack, but we want to prevent this for vcf tracks
                 if (!(this instanceof VariantTrack)) {
@@ -792,7 +768,6 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
     protected void renderCoverage(RenderContext context, Rectangle inputRect) {
 
         final String chr = context.getChr();
-
         List<LocusScore> scores = (source != null && chr.equals(Globals.CHR_ALL)) ?
                 source.getCoverageScores(chr, (int) context.getOrigin(),
                         (int) context.getEndLocation(), context.getZoom()) :
@@ -866,53 +841,48 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
                         "<br>Unload track " + getName() + "?");
                 if (unload) {
                     Collection<Track> tmp = Arrays.asList((Track) this);
-                    IGV.getInstance().removeTracks(tmp);
-                    IGV.getInstance().doRefresh();
+                    IGV.getInstance().deleteTracks(tmp);
+                    IGV.getInstance().repaint();
                 } else {
                     fatalLoadError = false;
                 }
             }
         }
-
-
     }
 
     protected void renderFeatureImpl(RenderContext context, Rectangle inputRect, PackedFeatures packedFeatures) {
 
 
         Renderer renderer = getRenderer();
-        if (areFeaturesStacked()) {
-            List<PackedFeatures.FeatureRow> rows = packedFeatures.getRows();
-            if (rows != null && rows.size() > 0) {
 
-                int nLevels = rows.size();
-                synchronized (levelRects) {
-
-                    levelRects.clear();
-
-                    // Divide rectangle into equal height levels
-                    double h = getDisplayMode() == DisplayMode.SQUISHED ? squishedRowHeight : expandedRowHeight;
-                    Rectangle rect = new Rectangle(inputRect.x, inputRect.y, inputRect.width, (int) h);
-                    int i = 0;
-
-                    if (renderer instanceof FeatureRenderer) ((FeatureRenderer) renderer).reset();
-                    for (PackedFeatures.FeatureRow row : rows) {
-                        levelRects.add(new Rectangle(rect));
-                        renderer.render(row.features, context, levelRects.get(i), this);
-                        if (selectedFeatureRowIndex == i) {
-                            Graphics2D fontGraphics = context.getGraphic2DForColor(SELECTED_FEATURE_ROW_COLOR);
-                            fontGraphics.fillRect(rect.x, rect.y, rect.width, rect.height);
-                        }
-                        rect.y += h;
-                        i++;
-                    }
-                }
-            }
-        } else {
+        if (getDisplayMode() == DisplayMode.COLLAPSED) {
             List<Feature> features = packedFeatures.getFeatures();
             if (features != null) {
                 renderer.render(features, context, inputRect, this);
             }
+        } else {
+            List<PackedFeatures.FeatureRow> rows = packedFeatures.getRows();
+            if (rows != null && rows.size() > 0) {
+
+                // Divide rectangle into equal height levels
+                double h = getDisplayMode() == DisplayMode.SQUISHED ? squishedRowHeight : expandedRowHeight;
+                Rectangle rect = new Rectangle(inputRect.x, inputRect.y, inputRect.width, (int) h);
+                int i = 0;
+
+                if (renderer instanceof FeatureRenderer) ((FeatureRenderer) renderer).reset();
+                for (PackedFeatures.FeatureRow row : rows) {
+
+                    renderer.render(row.features, context, new Rectangle(rect), this);
+
+                    if (selectedFeatureRowIndex == i) {
+                        Graphics2D fontGraphics = context.getGraphic2DForColor(SELECTED_FEATURE_ROW_COLOR);
+                        fontGraphics.fillRect(rect.x, rect.y, rect.width, rect.height);
+                    }
+
+                    rect.y += h;
+                    i++;
+                }
+            }
         }
     }
 
@@ -935,10 +905,10 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
 
         if (packedFeatures != null && packedFeatures.containsInterval(chr, (int) center - 1, (int) center + 1)) {
             if (packedFeatures.getFeatures().size() > 0 && canScroll) {
-                f = (forward ? FeatureUtils.getFeatureAfter(center + 1, packedFeatures.getFeatures()) :
-                        FeatureUtils.getFeatureBefore(center - 1, packedFeatures.getFeatures()));
+                f = (forward ?
+                        FeatureUtils.getFeatureAfter(center, packedFeatures.getFeatures()) :
+                        FeatureUtils.getFeatureBefore(center, packedFeatures.getFeatures()));
             }
-
             if (f == null) {
                 FeatureSource rawSource = source;
                 if (source instanceof CachingFeatureSource) {
@@ -951,10 +921,10 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
         return f;
     }
 
+
     public void setVisibilityWindow(int windowSize) {
         super.setVisibilityWindow(windowSize);
         packedFeaturesMap.clear();
-        source.setFeatureWindowSize(visibilityWindow);
     }
 
     public int getSelectedFeatureRowIndex() {
@@ -1009,10 +979,18 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
      * @return
      */
     public List<Feature> getVisibleFeatures(ReferenceFrame frame) {
-        PackedFeatures packedFeatures = packedFeaturesMap.get(frame.getName());
-        return (packedFeatures == null) ? Collections.emptyList() : packedFeatures.getFeatures();
-
-
+        PackedFeatures<Feature> packedFeatures = packedFeaturesMap.get(frame.getName());
+        if (packedFeatures == null) {
+            return Collections.emptyList();
+        } else {
+            Range currentRange = frame.getCurrentRange();
+            return packedFeatures.getFeatures()
+                    .stream().filter(igvFeature ->
+                            igvFeature.getEnd() >= currentRange.getStart() &&
+                                    igvFeature.getStart() <= currentRange.getEnd() &&
+                                    igvFeature.getChr().equals(currentRange.getChr()))
+                    .collect(Collectors.toList());
+        }
     }
 
 
@@ -1030,14 +1008,32 @@ public class FeatureTrack extends AbstractTrack implements IGVEventObserver {
     }
 
 
-    //        <Track altColor="0,0,178" autoScale="false" clazz="org.broad.igv.track.FeatureTrack" color="255,0,0" displayMode="SQUISHED" featureVisibilityWindow="10000000" fontSize="10" id="tataaa Negative" name="tataaa Negative" renderer="BASIC_FEATURE" sortable="false" visible="true" windowFunction="count">
+    public void setGroupByStrand(boolean selected) {
+        this.groupByStrand = selected;
+        for (PackedFeatures pf : packedFeaturesMap.values()) {
+            pf.pack(getDisplayMode(), groupByStrand);
+        }
+    }
 
+    public boolean isGroupByStrand() {
+        return groupByStrand;
+    }
+
+    @Override
+    public void marshalXML(Document document, Element element) {
+        element.setAttribute("groupByStrand", String.valueOf(groupByStrand));
+
+        super.marshalXML(document, element);
+
+    }
 
     @Override
     public void unmarshalXML(Element element, Integer version) {
 
         super.unmarshalXML(element, version);
 
+        this.groupByStrand = "true".equals(element.getAttribute("groupByStrand"));
+
         NodeList tmp = element.getElementsByTagName("SequenceMatchSource");
         if (tmp.getLength() > 0) {
             Element sourceElement = (Element) tmp.item(0);
