@@ -26,7 +26,7 @@
 package org.broad.igv.track;
 
 import htsjdk.tribble.Feature;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.feature.Mutation;
 import org.broad.igv.prefs.Constants;
@@ -52,7 +52,7 @@ import java.net.URL;
  */
 public class MutationTrack extends FeatureTrack {
 
-    private static Logger log = Logger.getLogger(MutationTrack.class);
+    private static Logger log = LogManager.getLogger(MutationTrack.class);
 
     public MutationTrack(ResourceLocator locator, String id, FeatureSource source) {
         super(locator, id, source);
