@@ -29,13 +29,13 @@ import com.google.common.base.Predicate;
 import com.google.common.collect.Iterables;
 import com.google.common.collect.Lists;
 import htsjdk.tribble.Feature;
-import org.apache.commons.math.stat.StatUtils;
-import org.apache.log4j.Logger;
+import org.apache.commons.math3.stat.StatUtils;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.data.AbstractDataSource;
 import org.broad.igv.data.CombinedDataSource;
-import org.broad.igv.feature.*;
 import org.broad.igv.feature.Range;
+import org.broad.igv.feature.*;
 import org.broad.igv.feature.basepair.BasePairTrack;
 import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.feature.genome.GenomeManager;
@@ -48,11 +48,11 @@ import org.broad.igv.sam.AlignmentTrack;
 import org.broad.igv.sam.CoverageTrack;
 import org.broad.igv.sam.SAMWriter;
 import org.broad.igv.ui.*;
+import org.broad.igv.ui.action.OverlayTracksMenuAction;
 import org.broad.igv.ui.color.ColorUtilities;
 import org.broad.igv.ui.panel.FrameManager;
 import org.broad.igv.ui.panel.IGVPopupMenu;
 import org.broad.igv.ui.panel.ReferenceFrame;
-import org.broad.igv.ui.panel.TrackPanel;
 import org.broad.igv.ui.util.FileDialogUtils;
 import org.broad.igv.ui.util.MessageUtils;
 import org.broad.igv.ui.util.UIUtilities;
@@ -69,15 +69,15 @@ import java.awt.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.io.*;
-import java.util.*;
 import java.util.List;
+import java.util.*;
 
 /**
  * @author jrobinso
  */
 public class TrackMenuUtils {
 
-    static Logger log = Logger.getLogger(TrackMenuUtils.class);
+    static Logger log = LogManager.getLogger(TrackMenuUtils.class);
     final static String LEADING_HEADING_SPACER = "  ";
 
     private static List<TrackMenuItemBuilder> trackMenuItems = new ArrayList<TrackMenuItemBuilder>();
@@ -97,7 +97,7 @@ public class TrackMenuUtils {
         IGVPopupMenu menu = new IGVPopupMenu();
 
         JLabel popupTitle = new JLabel(LEADING_HEADING_SPACER + title, JLabel.CENTER);
-        popupTitle.setFont(UIConstants.boldFont);
+        popupTitle.setFont(FontManager.getFont(Font.BOLD, 12));
         if (popupTitle != null) {
             menu.add(popupTitle);
             menu.addSeparator();
@@ -239,7 +239,7 @@ public class TrackMenuUtils {
             };
 
             JLabel rendererHeading = new JLabel(LEADING_HEADING_SPACER + "Type of Graph", JLabel.LEFT);
-            rendererHeading.setFont(UIConstants.boldFont);
+            rendererHeading.setFont(FontManager.getFont(Font.BOLD, 12));
 
             menu.add(rendererHeading);
 
@@ -292,7 +292,7 @@ public class TrackMenuUtils {
 
             if (avaibleWindowFunctions.size() > 0) {
                 JLabel statisticsHeading = new JLabel(LEADING_HEADING_SPACER + "Windowing Function", JLabel.LEFT);
-                statisticsHeading.setFont(UIConstants.boldFont);
+                statisticsHeading.setFont(FontManager.getFont(Font.BOLD, 12));
 
                 menu.add(statisticsHeading);
 
@@ -337,18 +337,12 @@ public class TrackMenuUtils {
 
         if (tracks.size() > 1 || merged) {
             menu.addSeparator();
+
             final List<DataTrack> dataTrackList = Lists.newArrayList(Iterables.filter(tracks, DataTrack.class));
             final JMenuItem overlayGroups = new JMenuItem("Overlay Tracks");
-            overlayGroups.addActionListener(new ActionListener() {
-                @Override
-                public void actionPerformed(ActionEvent e) {
-                    MergedTracks mergedTracks = new MergedTracks(UUID.randomUUID().toString(), "Overlay", dataTrackList);
-                    Track firstTrack = tracks.iterator().next();
-                    TrackPanel panel = TrackPanel.getParentPanel(firstTrack);
-                    panel.addTrack(mergedTracks);
-                    panel.moveSelectedTracksTo(Arrays.asList(mergedTracks), firstTrack, false);
-                    panel.removeTracks(tracks);
-                }
+            overlayGroups.addActionListener(e -> {
+                OverlayTracksMenuAction.merge(dataTrackList, "Overlay");
+                IGV.getInstance().repaint();
             });
 
             int numDataTracks = dataTrackList.size();
@@ -361,20 +355,15 @@ public class TrackMenuUtils {
             menu.add(unmergeItem);
             if (merged) {
                 unmergeItem.addActionListener(e -> {
-                    Track firstTrack1 = tracks.iterator().next();
-                    TrackPanel panel = TrackPanel.getParentPanel(firstTrack1);
-                    final MergedTracks mergedTracks = (MergedTracks) firstTrack1;
-                    mergedTracks.setTrackAlphas(1.0);
-                    panel.addTracks(mergedTracks.getMemberTracks());
-                    panel.moveSelectedTracksTo(mergedTracks.getMemberTracks(), mergedTracks, true);
-                    IGV.getInstance().removeTracks(Arrays.asList(mergedTracks));
+                    OverlayTracksMenuAction.unmerge(tracks);
+                    IGV.getInstance().repaint();
                 });
             } else {
                 unmergeItem.setEnabled(false);
             }
 
             // Add color transparency item if merged
-            if(merged) {
+            if (merged) {
                 Track firstTrack1 = tracks.iterator().next();
                 final MergedTracks mergedTracks = (MergedTracks) firstTrack1;
 
@@ -399,34 +388,6 @@ public class TrackMenuUtils {
 
     }
 
-    private static List<JMenuItem> getCombinedDataSourceItems(final Collection<Track> tracks) {
-
-        Iterable<DataTrack> dataTracksIter = Iterables.filter(tracks, DataTrack.class);
-        final List<DataTrack> dataTracks = Lists.newArrayList(dataTracksIter);
-        JMenuItem addItem = new JMenuItem("Sum Tracks");
-        JMenuItem subItem = new JMenuItem("Subtract Tracks");
-        boolean enableComb = dataTracks.size() == 2;
-
-        addItem.setEnabled(enableComb);
-        addItem.setEnabled(enableComb);
-
-        addItem.addActionListener(new ActionListener() {
-            @Override
-            public void actionPerformed(ActionEvent e) {
-                addCombinedDataTrack(dataTracks, CombinedDataSource.Operation.ADD);
-            }
-        });
-
-        subItem.addActionListener(new ActionListener() {
-            @Override
-            public void actionPerformed(ActionEvent e) {
-                addCombinedDataTrack(dataTracks, CombinedDataSource.Operation.SUBTRACT);
-            }
-        });
-
-        return Arrays.asList(addItem, subItem);
-    }
-
     private static void addCombinedDataTrack(List<DataTrack> dataTracks, CombinedDataSource.Operation op) {
         String text = "";
         switch (op) {
@@ -455,6 +416,8 @@ public class TrackMenuUtils {
      */
     private static void addFeatureItems(JPopupMenu featurePopupMenu, final Collection<Track> tracks, TrackClickEvent te) {
 
+        TrackMenuUtils.addGroupByStrandItem(tracks, featurePopupMenu);
+        featurePopupMenu.addSeparator();
 
         addDisplayModeItems(tracks, featurePopupMenu);
 
@@ -496,12 +459,10 @@ public class TrackMenuUtils {
                     featurePopupMenu.add(getExtendViewItem(featureName, sequenceFeature, r));
                 }
 
-                featurePopupMenu.add(getBlatItem(sequenceFeature));
-            }
-            if (Globals.isDevelopment()) {
-                featurePopupMenu.addSeparator();
-                featurePopupMenu.add(getFeatureToGeneListItem(t));
+                final JMenuItem blatItem = getBlatItem(sequenceFeature);
+                featurePopupMenu.add(blatItem);
             }
+
             if (Globals.isDevelopment() && FrameManager.isGeneListMode() && tracks.size() == 1) {
                 featurePopupMenu.addSeparator();
                 featurePopupMenu.add(getShowSortFramesItem(tracks.iterator().next()));
@@ -512,6 +473,9 @@ public class TrackMenuUtils {
         featurePopupMenu.addSeparator();
         featurePopupMenu.add(getChangeFeatureWindow(tracks));
 
+        featurePopupMenu.addSeparator();
+        featurePopupMenu.add(getShowFeatureNames(tracks));
+
     }
 
     /**
@@ -529,7 +493,7 @@ public class TrackMenuUtils {
         }
 
         JLabel arcColorHeading = new JLabel(LEADING_HEADING_SPACER + "Arc colors (click to change)", JLabel.LEFT);
-        arcColorHeading.setFont(UIConstants.boldFont);
+        arcColorHeading.setFont(FontManager.getFont(Font.BOLD, 12));
 
         menu.add(arcColorHeading);
 
@@ -540,7 +504,7 @@ public class TrackMenuUtils {
             List<String> colors = track.getRenderOptions().getColors();
             List<String> colorLabels = track.getRenderOptions().getColorLabels();
             // iterate in reverse order so colors appearing first in list are the ones rendered on top
-            for (int i=colors.size()-1; i>=0; --i) {
+            for (int i = colors.size() - 1; i >= 0; --i) {
                 String key = colors.get(i) + ' ' + colorLabels.get(i);
                 if (!keys.contains(key)) {
                     keys.add(key);
@@ -554,14 +518,14 @@ public class TrackMenuUtils {
             final String label = pair.getSecond();
 
             JLabel colorBox = new JLabel(LEADING_HEADING_SPACER);
-            colorBox.setFont(UIConstants.boldFont);
+            colorBox.setFont(FontManager.getFont(Font.BOLD, 12));
             colorBox.setForeground(color);
 
             JPanel p = new JPanel();
             p.setLayout(new BoxLayout(p, BoxLayout.X_AXIS));
             p.add(colorBox);
             p.add(Box.createHorizontalStrut(1));
-            p.add(new JLabel(" "+label));
+            p.add(new JLabel(" " + label));
             p.add(Box.createGlue());
             p.setAlignmentX(Component.LEFT_ALIGNMENT);
 
@@ -570,7 +534,7 @@ public class TrackMenuUtils {
             double w = p.getPreferredSize().getWidth();
             double h = p.getPreferredSize().getHeight();
             Dimension size = new Dimension();
-            size.setSize(w, h+8);
+            size.setSize(w, h + 8);
             item.setPreferredSize(size);
 
             item.addActionListener(new ActionListener() {
@@ -585,7 +549,7 @@ public class TrackMenuUtils {
         menu.addSeparator();
 
         JLabel arcDirectionHeading = new JLabel(LEADING_HEADING_SPACER + "Arc direction", JLabel.LEFT);
-        arcDirectionHeading.setFont(UIConstants.boldFont);
+        arcDirectionHeading.setFont(FontManager.getFont(Font.BOLD, 12));
 
         menu.add(arcDirectionHeading);
 
@@ -593,12 +557,12 @@ public class TrackMenuUtils {
         int upCount = 0;
         int downCount = 0;
         BasePairTrack.ArcDirection currentArcDirection = null; // mixed up and down
-        for (BasePairTrack track: bpTracks) {
+        for (BasePairTrack track : bpTracks) {
             if (track.getRenderOptions().getArcDirection() == BasePairTrack.ArcDirection.UP) ++upCount;
             if (track.getRenderOptions().getArcDirection() == BasePairTrack.ArcDirection.DOWN) ++downCount;
         }
-        if (upCount==0) currentArcDirection = BasePairTrack.ArcDirection.DOWN;
-        if (downCount==0) currentArcDirection = BasePairTrack.ArcDirection.UP;
+        if (upCount == 0) currentArcDirection = BasePairTrack.ArcDirection.DOWN;
+        if (downCount == 0) currentArcDirection = BasePairTrack.ArcDirection.UP;
 
         ButtonGroup group = new ButtonGroup();
         Map<String, BasePairTrack.ArcDirection> arcDirections = new LinkedHashMap<String, BasePairTrack.ArcDirection>(3);
@@ -608,15 +572,13 @@ public class TrackMenuUtils {
         for (final Map.Entry<String, BasePairTrack.ArcDirection> entry : arcDirections.entrySet()) {
             JRadioButtonMenuItem mm = new JRadioButtonMenuItem(entry.getKey());
             mm.setSelected(currentArcDirection == entry.getValue());
-            mm.addActionListener(new ActionListener() {
-                public void actionPerformed(ActionEvent evt) {
-                    for (Track track : tracks) {
-                        if (track instanceof BasePairTrack) {
-                            ((BasePairTrack) track).getRenderOptions().setArcDirection(entry.getValue());
-                        }
+            mm.addActionListener(evt -> {
+                for (Track track : tracks) {
+                    if (track instanceof BasePairTrack) {
+                        ((BasePairTrack) track).getRenderOptions().setArcDirection(entry.getValue());
                     }
-                    refresh();
                 }
+                IGV.getInstance().repaint(tracks);
             });
             group.add(mm);
             menu.add(mm);
@@ -624,21 +586,6 @@ public class TrackMenuUtils {
         menu.addSeparator();
     }
 
-    private static JMenuItem getFeatureToGeneListItem(final Track t) {
-        JMenuItem mi = new JMenuItem("Use as loci list");
-
-        mi.addActionListener(new ActionListener() {
-            @Override
-            public void actionPerformed(ActionEvent e) {
-                // Current chromosome only for now
-
-
-            }
-        });
-
-        return mi;
-    }
-
     /**
      * Return a menu item which will export visible features
      * If {@code tracks} is not a single {@code FeatureTrack}, {@code null}
@@ -745,9 +692,6 @@ public class TrackMenuUtils {
 
                 //Can't trust FeatureTrack.getFeatures to limit itself, so we filter
                 List<Feature> features = fTrack.getVisibleFeatures(frame);
-                Range range = frame.getCurrentRange();
-                Predicate<Feature> pred = FeatureUtils.getOverlapPredicate(range.getChr(), range.getStart(), range.getEnd());
-                features = CollUtils.filter(features, pred);
                 IGVBEDCodec codec = new IGVBEDCodec();
                 for (Feature feat : features) {
                     String featString = codec.encode(feat);
@@ -773,29 +717,17 @@ public class TrackMenuUtils {
 
         menu.add(getTrackRenameItem(tracks));
 
-        String colorLabel = (hasFeatureTracks || hasCoverageTracks)
-                ? "Change Track Color..." : "Change Track Color (Positive Values)...";
+        String colorLabel = "Change Track Color...";
         JMenuItem item = new JMenuItem(colorLabel);
-        item.addActionListener(new ActionListener() {
-            public void actionPerformed(ActionEvent evt) {
-                changeTrackColor(tracks);
-            }
-        });
+        item.addActionListener(evt -> changeTrackColor(tracks));
         menu.add(item);
 
-        if (!(hasFeatureTracks || hasCoverageTracks)) {
-
-            // Change track color by attribute
-            item = new JMenuItem("Change Track Color (Negative Values)...");
-            item.setToolTipText(
-                    "Change the alternate track color.  This color is used when graphing negative values");
-            item.addActionListener(new ActionListener() {
-                public void actionPerformed(ActionEvent evt) {
-                    changeAltTrackColor(tracks);
-                }
-            });
-            menu.add(item);
-        }
+        // Change track color by attribute
+        item = new JMenuItem("Change Track Color (Negative Values or Strand)...");
+        item.setToolTipText(
+                "Change the alternate track color.  This color is used when drawing features with negative values or on the negative strand.");
+        item.addActionListener(evt -> changeAltTrackColor(tracks));
+        menu.add(item);
 
         menu.add(getChangeTrackHeightItem(tracks));
         menu.add(getChangeFontSizeItem(tracks));
@@ -806,7 +738,7 @@ public class TrackMenuUtils {
         for (Track track : selectedTracks) {
             track.setWindowFunction(WindowFunction.valueOf(statType));
         }
-        refresh();
+        IGV.getInstance().repaint(selectedTracks);
     }
 
 
@@ -840,7 +772,7 @@ public class TrackMenuUtils {
                 if (selectedTracks.size() > 0) {
 
                     ContinuousColorScale colorScale = selectedTracks.iterator().next().getColorScale();
-                    HeatmapScaleDialog dlg = new HeatmapScaleDialog(IGV.getMainFrame(), colorScale);
+                    HeatmapScaleDialog dlg = new HeatmapScaleDialog(IGV.getInstance().getMainFrame(), colorScale);
 
                     dlg.setVisible(true);
                     if (!dlg.isCanceled()) {
@@ -863,56 +795,36 @@ public class TrackMenuUtils {
     public static JMenuItem getDataRangeItem(final Collection<Track> selectedTracks) {
         JMenuItem item = new JMenuItem("Set Data Range...");
 
-        item.addActionListener(new ActionListener() {
+        item.addActionListener(evt -> {
+            if (selectedTracks.size() > 0) {
 
-            public void actionPerformed(ActionEvent evt) {
-                if (selectedTracks.size() > 0) {
+                // Create a datarange that spans the extent of prev tracks range
+                DataRange prevAxisDefinition = DataRange.getFromTracks(selectedTracks);
+                DataRangeDialog dlg = new DataRangeDialog(IGV.getInstance().getMainFrame(), prevAxisDefinition);
+                dlg.setVisible(true);
+                if (!dlg.isCanceled()) {
+                    float min = Math.min(dlg.getMax(), dlg.getMin());
+                    float max = Math.max(dlg.getMin(), dlg.getMax());
+                    float mid = dlg.getBase();
+                    mid = Math.max(min, Math.min(mid, max));
 
-                    // Create a datarange that spans the extent of prev tracks range
-                    DataRange prevAxisDefinition = DataRange.getFromTracks(selectedTracks);
-                    DataRangeDialog dlg = new DataRangeDialog(IGV.getMainFrame(), prevAxisDefinition);
-                    dlg.setVisible(true);
-                    if (!dlg.isCanceled()) {
-                        float min = Math.min(dlg.getMax(), dlg.getMin());
-                        float max = Math.max(dlg.getMin(), dlg.getMax());
-                        float mid = dlg.getBase();
-                        mid = Math.max(min, Math.min(mid, max));
+                    DataRange axisDefinition = new DataRange(dlg.getMin(), mid, dlg.getMax(),
+                            prevAxisDefinition.isDrawBaseline(), dlg.isLog());
 
-                        DataRange axisDefinition = new DataRange(dlg.getMin(), mid, dlg.getMax(),
-                                prevAxisDefinition.isDrawBaseline(), dlg.isLog());
-
-                        for (Track track : selectedTracks) {
-                            track.setDataRange(axisDefinition);
-                            track.setAutoScale(false);
-                            track.removeAttribute(AttributeManager.GROUP_AUTOSCALE);
-                        }
-                        IGV.getInstance().repaint();
+                    for (Track track : selectedTracks) {
+                        track.setDataRange(axisDefinition);
+                        track.setAutoScale(false);
+                        track.removeAttribute(AttributeManager.GROUP_AUTOSCALE);
                     }
-
+                    IGV.getInstance().repaint(selectedTracks);
                 }
+
             }
         });
 
         return item;
     }
 
-    private static JMenuItem getDrawBorderItem() {
-        // Change track height by attribute
-
-
-        final JCheckBoxMenuItem drawBorderItem = new JCheckBoxMenuItem("Draw borders");
-        drawBorderItem.setSelected(FeatureTrack.isDrawBorder());
-        drawBorderItem.addActionListener(new ActionListener() {
-
-            public void actionPerformed(ActionEvent evt) {
-                FeatureTrack.setDrawBorder(drawBorderItem.isSelected());
-                IGV.getInstance().revalidateTrackPanels();
-            }
-        });
-
-        return drawBorderItem;
-    }
-
 
     public static JMenuItem getLogScaleItem(final Collection<Track> selectedTracks) {
         // Change track height by attribute
@@ -930,7 +842,7 @@ public class TrackMenuUtils {
                 for (Track t : selectedTracks) {
                     t.getDataRange().setType(scaleType);
                 }
-                IGV.getInstance().revalidateTrackPanels();
+                IGV.getInstance().repaint(selectedTracks);
             }
         });
 
@@ -958,7 +870,7 @@ public class TrackMenuUtils {
                             t.removeAttribute(AttributeManager.GROUP_AUTOSCALE);
                         }
                     }
-                    IGV.getInstance().repaint();
+                    IGV.getInstance().repaint(selectedTracks);
                 }
             });
         }
@@ -969,24 +881,18 @@ public class TrackMenuUtils {
 
         final JMenuItem autoscaleItem = new JMenuItem("Group Autoscale");
 
+        autoscaleItem.addActionListener(evt -> {
 
-        autoscaleItem.addActionListener(new ActionListener() {
-
-            public void actionPerformed(ActionEvent evt) {
-
-                int nextAutoscaleGroup = IGV.getInstance().getSession().getNextAutoscaleGroup();
-
-                for (Track t : selectedTracks) {
-                    t.setAttributeValue(AttributeManager.GROUP_AUTOSCALE, "" + nextAutoscaleGroup);
-                    t.setAutoScale(false);
-                }
-
-                PreferencesManager.getPreferences().setShowAttributeView(true);
-                IGV.getInstance().getMainPanel().invalidate();
-                IGV.getInstance().doRefresh();
-
-
+            int nextAutoscaleGroup = IGV.getInstance().getSession().getNextAutoscaleGroup();
+            for (Track t : selectedTracks) {
+                t.setAttributeValue(AttributeManager.GROUP_AUTOSCALE, "" + nextAutoscaleGroup);
+                t.setAutoScale(false);
             }
+
+            PreferencesManager.getPreferences().setShowAttributeView(true);
+            IGV.getInstance().revalidateTrackPanels();
+            IGV.getInstance().repaint(selectedTracks);
+
         });
 
         return autoscaleItem;
@@ -1019,18 +925,14 @@ public class TrackMenuUtils {
             }
 
             item.setSelected(showDataRange);
-            item.addActionListener(new ActionListener() {
-
-                public void actionPerformed(ActionEvent evt) {
-
-                    boolean showDataRange = item.isSelected();
-                    for (Track t : selectedTracks) {
-                        if (t instanceof DataTrack) {
-                            ((DataTrack) t).setShowDataRange(showDataRange);
-                        }
+            item.addActionListener(evt -> {
+                boolean showDataRange1 = item.isSelected();
+                for (Track t : selectedTracks) {
+                    if (t instanceof DataTrack) {
+                        ((DataTrack) t).setShowDataRange(showDataRange1);
                     }
-                    IGV.getInstance().revalidateTrackPanels();
                 }
+                IGV.getInstance().repaint(selectedTracks);
             });
         }
         return item;
@@ -1069,27 +971,38 @@ public class TrackMenuUtils {
         for (final Map.Entry<String, Track.DisplayMode> entry : modes.entrySet()) {
             JRadioButtonMenuItem mm = new JRadioButtonMenuItem(entry.getKey());
             mm.setSelected(currentMode == entry.getValue());
-            mm.addActionListener(new ActionListener() {
-                public void actionPerformed(ActionEvent evt) {
-                    setTrackDisplayMode(tracks, entry.getValue());
-                    refresh();
+            mm.addActionListener(evt -> {
+                for (Track t : tracks) {
+                    t.setDisplayMode(entry.getValue());
                 }
+                IGV.getInstance().repaint(tracks);
             });
             group.add(mm);
             menu.add(mm);
         }
-
     }
 
+    public static void addGroupByStrandItem(final Collection<Track> tracks, JPopupMenu menu) {
 
-    private static void setTrackDisplayMode(Collection<Track> tracks, Track.DisplayMode mode) {
+        // Find "most representative" state from track collection
+        boolean allGrouped = tracks.stream().allMatch(track -> {
+            return track instanceof FeatureTrack && ((FeatureTrack) track).isGroupByStrand();
+        });
 
-        for (Track t : tracks) {
-            t.setDisplayMode(mode);
-        }
+        final JRadioButtonMenuItem groupByItem = new JRadioButtonMenuItem("Group by strand");
+        groupByItem.setSelected(allGrouped);
+        groupByItem.addActionListener(evt -> {
+            tracks.stream().forEach(track -> {
+                if (track instanceof FeatureTrack) {
+                    ((FeatureTrack) track).setGroupByStrand(groupByItem.isSelected());
+                }
+            });
+            IGV.getInstance().repaint(tracks);
+        });
+
+        menu.add(groupByItem);
     }
 
-
     public static JMenuItem getRemoveMenuItem(final Collection<Track> selectedTracks) {
 
         boolean multiple = selectedTracks.size() > 1;
@@ -1114,8 +1027,8 @@ public class TrackMenuUtils {
         if (selectedTracks.isEmpty()) {
             return;
         }
-        IGV.getInstance().removeTracks(selectedTracks);
-        IGV.getInstance().doRefresh();
+        IGV.getInstance().deleteTracks(selectedTracks);
+        IGV.getInstance().repaint();
     }
 
 
@@ -1123,7 +1036,7 @@ public class TrackMenuUtils {
         for (Track track : selectedTracks) {
             track.setRendererClass(rendererClass);
         }
-        refresh();
+        IGV.getInstance().repaint(selectedTracks);
     }
 
     public static void renameTrack(final Collection<Track> selectedTracks) {
@@ -1132,14 +1045,14 @@ public class TrackMenuUtils {
             return;
         }
         Track t = selectedTracks.iterator().next();
-        String newName = JOptionPane.showInputDialog(IGV.getMainFrame(), "Enter new name: ", t.getName());
+        String newName = JOptionPane.showInputDialog(IGV.getInstance().getMainFrame(), "Enter new name: ", t.getName());
 
         if (newName == null || newName.trim() == "") {
             return;
         }
 
         t.setName(newName);
-        refresh();
+        IGV.getInstance().repaintNamePanels();
     }
 
     public static void changeTrackHeight(final Collection<Track> selectedTracks) {
@@ -1157,7 +1070,7 @@ public class TrackMenuUtils {
         for (Track track : selectedTracks) {
             track.setHeight(value, true);
         }
-        refresh();
+        IGV.getInstance().repaint(selectedTracks);
     }
 
     public static void changeFeatureVisibilityWindow(final Collection<Track> selectedTracks) {
@@ -1185,7 +1098,7 @@ public class TrackMenuUtils {
             track.setVisibilityWindow((int) (value * 1000));
         }
 
-        refresh();
+        IGV.getInstance().repaint(featureTracks);
     }
 
     public static void changeFontSize(final Collection<Track> selectedTracks) {
@@ -1206,7 +1119,8 @@ public class TrackMenuUtils {
             track.setFontSize(value);
         }
 
-        refresh();
+        IGV.getInstance().repaintNamePanels();
+        IGV.getInstance().repaint(selectedTracks);
     }
 
 
@@ -1215,7 +1129,7 @@ public class TrackMenuUtils {
         while (true) {
 
             String strValue = JOptionPane.showInputDialog(
-                    IGV.getMainFrame(), parameter + ": ",
+                    IGV.getInstance().getMainFrame(), parameter + ": ",
                     String.valueOf(value));
 
             //strValue will be null if dialog cancelled
@@ -1227,7 +1141,7 @@ public class TrackMenuUtils {
                 value = Integer.parseInt(strValue);
                 return value;
             } catch (NumberFormatException numberFormatException) {
-                JOptionPane.showMessageDialog(IGV.getMainFrame(),
+                JOptionPane.showMessageDialog(IGV.getInstance().getMainFrame(),
                         parameter + " must be an integer number.");
             }
         }
@@ -1238,7 +1152,7 @@ public class TrackMenuUtils {
         while (true) {
 
             String strValue = JOptionPane.showInputDialog(
-                    IGV.getMainFrame(), parameter + ": ",
+                    IGV.getInstance().getMainFrame(), parameter + ": ",
                     String.valueOf(value));
 
             //strValue will be null if dialog cancelled
@@ -1264,7 +1178,7 @@ public class TrackMenuUtils {
         Color currentSelection = selectedTracks.iterator().next().getColor();
 
         Color color = UIUtilities.showColorChooserDialog(
-                "Select Track Color (Positive Values)",
+                "Select Track Color",
                 currentSelection);
 
         if (color == null) {
@@ -1275,8 +1189,7 @@ public class TrackMenuUtils {
             //We preserve the alpha value. This is motivated by MergedTracks
             track.setColor(ColorUtilities.modifyAlpha(color, currentSelection.getAlpha()));
         }
-        refresh();
-
+        IGV.getInstance().repaint(selectedTracks);
     }
 
     public static void changeAltTrackColor(final Collection<Track> selectedTracks) {
@@ -1285,7 +1198,7 @@ public class TrackMenuUtils {
             return;
         }
 
-        Color currentSelection = selectedTracks.iterator().next().getColor();
+        Color currentSelection = selectedTracks.iterator().next().getAltColor();
 
         Color color = UIUtilities.showColorChooserDialog(
                 "Select Track Color (Negative Values)",
@@ -1298,12 +1211,10 @@ public class TrackMenuUtils {
         for (Track track : selectedTracks) {
             track.setAltColor(ColorUtilities.modifyAlpha(color, currentSelection.getAlpha()));
         }
-        refresh();
-
+        IGV.getInstance().repaint(selectedTracks);
     }
 
     /**
-     *
      * @author stevenbusan
      */
     public static void changeBasePairTrackColor(final List<BasePairTrack> tracks,
@@ -1315,7 +1226,7 @@ public class TrackMenuUtils {
         }
 
         Color newColor = UIUtilities.showColorChooserDialog(
-                "Select Arc Color ("+currentLabel+")",
+                "Select Arc Color (" + currentLabel + ")",
                 currentColor);
 
         if (newColor == null) {
@@ -1325,7 +1236,7 @@ public class TrackMenuUtils {
         for (BasePairTrack t : tracks) {
             t.getRenderOptions().changeColor(currentColor, currentLabel, newColor);
         }
-        refresh();
+        IGV.getInstance().repaint(tracks);
 
     }
 
@@ -1412,10 +1323,10 @@ public class TrackMenuUtils {
     public static JMenuItem getCopySequenceItem(final Feature f) {
 
         final Strand strand;
-        if(f instanceof IGVFeature) {
+        if (f instanceof IGVFeature) {
             strand = ((IGVFeature) f).getStrand();
         } else {
-           strand = Strand.NONE;
+            strand = Strand.NONE;
         }
 
         JMenuItem item = new JMenuItem("Copy Sequence");
@@ -1440,21 +1351,25 @@ public class TrackMenuUtils {
     }
 
     public static JMenuItem getBlatItem(final Feature f) {
-        JMenuItem item = new JMenuItem("Blat Sequence");
-        item.addActionListener(new ActionListener() {
+        JMenuItem item = new JMenuItem("BLAT Sequence");
+        final int start = f.getStart();
+        final int end = f.getEnd();
 
-            public void actionPerformed(ActionEvent evt) {
+        if ((end - start) > 20 && (end - start) < 8000) {
+
+            item.addActionListener(evt -> {
 
                 final Strand strand;
-                if(f instanceof IGVFeature) {
+                if (f instanceof IGVFeature) {
                     strand = ((IGVFeature) f).getStrand();
                 } else {
                     strand = Strand.NONE;
                 }
-
-                BlatClient.doBlatQuery(f.getChr(), f.getStart(), f.getEnd(), strand);
-            }
-        });
+                BlatClient.doBlatQueryFromRegion(f.getChr(), start, end, strand);
+            });
+        } else {
+            item.setEnabled(false);
+        }
         return item;
     }
 
@@ -1482,13 +1397,6 @@ public class TrackMenuUtils {
 
     }
 
-    public static void refresh() {
-        if (IGV.hasInstance()) {
-            IGV.getInstance().showLoadedTrackCount();
-            IGV.getInstance().doRefresh();
-        }
-    }
-
     public static JMenuItem getChangeTrackHeightItem(final Collection<Track> selectedTracks) {
         // Change track height by attribute
         JMenuItem item = new JMenuItem("Change Track Height...");
@@ -1505,12 +1413,15 @@ public class TrackMenuUtils {
     public static JMenuItem getChangeFeatureWindow(final Collection<Track> selectedTracks) {
         // Change track height by attribute
         JMenuItem item = new JMenuItem("Set Feature Visibility Window...");
-        item.addActionListener(new ActionListener() {
+        item.addActionListener(evt -> changeFeatureVisibilityWindow(selectedTracks));
+        return item;
+    }
 
-            public void actionPerformed(ActionEvent evt) {
-                changeFeatureVisibilityWindow(selectedTracks);
-            }
-        });
+    public static JMenuItem getShowFeatureNames(final Collection<Track> selectedTracks) {
+        boolean currentValue = selectedTracks.stream().allMatch(t -> t.isShowFeatureNames());
+        String label = currentValue ? "Hide Feature Names" : "Show Feature Names";
+        JMenuItem item = new JMenuItem(label);
+        item.addActionListener(evt -> selectedTracks.stream().forEach(t -> t.setShowFeatureNames(!currentValue)));
         return item;
     }
 
