@@ -30,7 +30,7 @@
 package org.broad.igv.ui;
 
 import htsjdk.tribble.Feature;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.charts.ScatterPlotUtils;
 import org.broad.igv.feature.BasicFeature;
 import org.broad.igv.feature.Exon;
@@ -62,7 +62,7 @@ import static org.broad.igv.prefs.Constants.*;
  */
 public class GlobalKeyDispatcher implements KeyEventDispatcher {
 
-    private static Logger log = Logger.getLogger(GlobalKeyDispatcher.class);
+    private static Logger log = LogManager.getLogger(GlobalKeyDispatcher.class);
 
     private final InputMap inputMap = new InputMap();
     private final ActionMap actionMap = new ActionMap();
@@ -112,6 +112,7 @@ public class GlobalKeyDispatcher implements KeyEventDispatcher {
         final IGVPreferences prefMgr = PreferencesManager.getPreferences();
 
         // Next feature
+        final KeyStroke nextKey0 = KeyStroke.getKeyStroke(KeyEvent.VK_F, 0, false);
         final KeyStroke nextKey = KeyStroke.getKeyStroke(KeyEvent.VK_F, KeyEvent.CTRL_MASK, false);
         final Action nextAction = new EnableWrappedAction(new AbstractAction() {
             public void actionPerformed(ActionEvent e) {
@@ -119,10 +120,12 @@ public class GlobalKeyDispatcher implements KeyEventDispatcher {
             }
         });
         inputMap.put(nextKey, "nextFeature");
+        inputMap.put(nextKey0, "nextFeature");
         actionMap.put("nextFeature", nextAction);
 
         // Previous feature
         final KeyStroke prevKey = KeyStroke.getKeyStroke(KeyEvent.VK_B, KeyEvent.CTRL_MASK, false);
+        final KeyStroke prevKey0 = KeyStroke.getKeyStroke(KeyEvent.VK_B, 0, false);
         final Action prevAction = new EnableWrappedAction(new AbstractAction() {
 
             public void actionPerformed(ActionEvent e) {
@@ -130,20 +133,24 @@ public class GlobalKeyDispatcher implements KeyEventDispatcher {
             }
         });
         inputMap.put(prevKey, "prevFeature");
+        inputMap.put(prevKey0, "prevFeature");
         actionMap.put("prevFeature", prevAction);
 
         // Next exon
         final KeyStroke nextExonKey = KeyStroke.getKeyStroke(KeyEvent.VK_F, KeyEvent.CTRL_MASK + KeyEvent.SHIFT_MASK, false);
+        final KeyStroke nextExonKey0 = KeyStroke.getKeyStroke(KeyEvent.VK_F, KeyEvent.SHIFT_MASK, false);
         final Action nextExonAction = new EnableWrappedAction(new AbstractAction() {
             public void actionPerformed(ActionEvent e) {
                 nextExon(true);
             }
         });
         inputMap.put(nextExonKey, "nextExon");
+        inputMap.put(nextExonKey0, "nextExon");
         actionMap.put("nextExon", nextExonAction);
 
         // Previous exon
         final KeyStroke prevExonKey = KeyStroke.getKeyStroke(KeyEvent.VK_B, KeyEvent.CTRL_MASK + KeyEvent.SHIFT_MASK, false);
+        final KeyStroke prevExonKey0 = KeyStroke.getKeyStroke(KeyEvent.VK_B, KeyEvent.SHIFT_MASK, false);
         final Action prevExonAction = new EnableWrappedAction(new AbstractAction() {
 
             public void actionPerformed(ActionEvent e) {
@@ -151,6 +158,7 @@ public class GlobalKeyDispatcher implements KeyEventDispatcher {
             }
         });
         inputMap.put(prevExonKey, "prevExon");
+        inputMap.put(prevExonKey0, "prevExon");
         actionMap.put("prevExon", prevExonAction);
 
         // Set the focus to the "search" box
@@ -227,9 +235,7 @@ public class GlobalKeyDispatcher implements KeyEventDispatcher {
                     try {
                         AlignmentTrack.SortOption option = AlignmentTrack.SortOption.valueOf(sortOptionString);
                         String lastSortTag = prefMgr.get(SAM_SORT_BY_TAG);
-
                         igv.sortAlignmentTracks(option, lastSortTag);
-                        igv.revalidateTrackPanels();
                     } catch (IllegalArgumentException e1) {
                         log.error("Unrecognized sort option: " + sortOptionString);
                     }
@@ -368,7 +374,7 @@ public class GlobalKeyDispatcher implements KeyEventDispatcher {
 
 
         } else {
-            MessageUtils.showMessage("To use track panning you must first select a single feature track.");
+            //MessageUtils.showMessage("To use track panning you must first select a single feature track.");
         }
     }
 
@@ -421,7 +427,7 @@ public class GlobalKeyDispatcher implements KeyEventDispatcher {
 
             }
         } else {
-            MessageUtils.showMessage("To use track panning you must first select a single feature track.");
+            //MessageUtils.showMessage("To use track panning you must first select a single feature track.");
         }
 
 
