@@ -25,34 +25,38 @@
 
 package org.broad.igv.ui;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.DirectoryManager;
 import org.broad.igv.Globals;
 import org.broad.igv.annotations.ForTesting;
+import org.broad.igv.aws.S3LoadDialog;
+import org.broad.igv.batch.CommandExecutor;
 import org.broad.igv.charts.ScatterPlotUtils;
-import org.broad.igv.feature.genome.GenomeManager;
-import org.broad.igv.google.GoogleUtils;
-import org.broad.igv.google.OAuthUtils;
-import org.broad.igv.gs.GSOpenSessionMenuAction;
-import org.broad.igv.gs.GSSaveSessionMenuAction;
-import org.broad.igv.gs.GSUtils;
-import org.broad.igv.lists.GeneListManagerUI;
-import org.broad.igv.prefs.PreferencesManager;
-import org.broad.igv.tools.IgvToolsGui;
-import org.broad.igv.tools.motiffinder.MotifFinderPlugin;
-import org.broad.igv.track.CombinedDataSourceDialog;
-import org.broad.igv.ui.action.*;
 import org.broad.igv.event.GenomeChangeEvent;
 import org.broad.igv.event.IGVEventBus;
 import org.broad.igv.event.IGVEventObserver;
-import org.broad.igv.feature.genome.RemoveGenomesDialog;
+import org.broad.igv.feature.genome.GenomeManager;
+import org.broad.igv.feature.genome.GenomeUtils;
+import org.broad.igv.google.GoogleUtils;
+import org.broad.igv.google.OAuthProvider;
+import org.broad.igv.google.OAuthUtils;
+import org.broad.igv.lists.GeneListManagerUI;
+import org.broad.igv.prefs.PreferencesManager;
+import org.broad.igv.session.Session;
+import org.broad.igv.tools.IgvToolsGui;
+import org.broad.igv.tools.motiffinder.MotifFinderPlugin;
+import org.broad.igv.track.AttributeManager;
+import org.broad.igv.track.CombinedDataSourceDialog;
+import org.broad.igv.ui.action.*;
 import org.broad.igv.ui.commandbar.GenomeComboBox;
+import org.broad.igv.ui.commandbar.RemoveGenomesDialog;
 import org.broad.igv.ui.legend.LegendDialog;
 import org.broad.igv.ui.panel.FrameManager;
 import org.broad.igv.ui.panel.MainPanel;
 import org.broad.igv.ui.panel.ReferenceFrame;
 import org.broad.igv.ui.panel.ReorderPanelsDialog;
 import org.broad.igv.ui.util.*;
+import org.broad.igv.util.AmazonUtils;
 import org.broad.igv.util.BrowserLauncher;
 import org.broad.igv.util.HttpUtils;
 import org.broad.igv.util.LongRunningTask;
@@ -87,43 +91,27 @@ import static org.broad.igv.ui.UIConstants.*;
  */
 public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
-    private static Logger log = Logger.getLogger(IGVMenuBar.class);
-    public static final String GENOMESPACE_REG_TOOLTIP = "Register for GenomeSpace";
-    public static final String GENOMESPACE_REG_PAGE = "http://www.genomespace.org/register";
+    private static Logger log = LogManager.getLogger(IGVMenuBar.class);
+    private static final String LOAD_GENOME_SERVER_TOOLTIP = "Select genomes available on the server to appear in menu.";
+    private static final String CANNOT_LOAD_GENOME_SERVER_TOOLTIP = "Could not reach genome server";
+
+    private static IGVMenuBar instance;
 
     private JMenu extrasMenu;
+    private JMenu toolsMenu;
+    private JMenu googleMenu;
+    private JMenu AWSMenu;
     private FilterTracksMenuAction filterTracksAction;
     private JMenu viewMenu;
-    IGV igv;
-
-    private JMenu toolsMenu;
-
+    private IGV igv;
     /**
      * We store this as a field because we alter it if
      * we can't access genome server list
      */
     private JMenuItem loadFromServerMenuItem;
-
-    private static final String LOAD_GENOME_SERVER_TOOLTIP = "Select genomes available on the server to appear in menu.";
-    private static final String CANNOT_LOAD_GENOME_SERVER_TOOLTIP = "Could not reach genome server";
-
-    private static IGVMenuBar instance;
-    private JMenu googleMenu;
     private JMenuItem encodeMenuItem;
+    private JMenuItem reloadSessionItem;
 
-    public void notifyGenomeServerReachable(boolean reachable) {
-        if (loadFromServerMenuItem != null) {
-            UIUtilities.invokeOnEventThread(() -> {
-                loadFromServerMenuItem.setEnabled(reachable);
-                String tooltip = reachable ? LOAD_GENOME_SERVER_TOOLTIP : CANNOT_LOAD_GENOME_SERVER_TOOLTIP;
-                loadFromServerMenuItem.setToolTipText(tooltip);
-            });
-        }
-    }
-
-    public void showAboutDialog() {
-        (new AboutDialog(IGV.getMainFrame(), true)).setVisible(true);
-    }
 
     static IGVMenuBar createInstance(IGV igv) {
         if (instance != null) {
@@ -132,7 +120,7 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
             }
             throw new IllegalStateException("Cannot create another IGVMenuBar, use getInstance");
         }
-        UIUtilities.invokeAndWaitOnEventThread(() ->instance = new IGVMenuBar(igv));
+        UIUtilities.invokeAndWaitOnEventThread(() -> instance = new IGVMenuBar(igv));
         return instance;
     }
 
@@ -160,6 +148,22 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         }
     }
 
+
+    public void notifyGenomeServerReachable(boolean reachable) {
+        if (loadFromServerMenuItem != null) {
+            UIUtilities.invokeOnEventThread(() -> {
+                loadFromServerMenuItem.setEnabled(reachable);
+                String tooltip = reachable ? LOAD_GENOME_SERVER_TOOLTIP : CANNOT_LOAD_GENOME_SERVER_TOOLTIP;
+                loadFromServerMenuItem.setToolTipText(tooltip);
+            });
+        }
+    }
+
+    public void showAboutDialog() {
+        (new AboutDialog(igv.getMainFrame(), true)).setVisible(true);
+    }
+
+
     private List<AbstractButton> createMenus() {
 
         List<AbstractButton> menus = new ArrayList<AbstractButton>();
@@ -173,19 +177,30 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         refreshToolsMenu();
         menus.add(toolsMenu);
 
-        menus.add(createGenomeSpaceMenu());
         extrasMenu = createExtrasMenu();
         //extrasMenu.setVisible(false);
         menus.add(extrasMenu);
 
         try {
             googleMenu = createGoogleMenu();
-            googleMenu.setVisible(PreferencesManager.getPreferences().getAsBoolean(ENABLE_GOOGLE_MENU));
-            menus.add(googleMenu);
+            if (googleMenu != null) {
+                googleMenu.setVisible(PreferencesManager.getPreferences().getAsBoolean(ENABLE_GOOGLE_MENU));
+                menus.add(googleMenu);
+            }
         } catch (IOException e) {
             log.error("Error creating google menu: " + e.getMessage());
         }
 
+        try {
+            AWSMenu = createAWSMenu();
+            AWSMenu.setVisible(AmazonUtils.isAWSProviderPresent());
+            menus.add(AWSMenu);
+
+        } catch (IOException e) {
+            log.error("Error creating the Amazon AWS menu: " + e.getMessage());
+            AWSMenu.setVisible(false);
+        }
+
         menus.add(createHelpMenu());
 
         // Experimental -- remove for production release
@@ -193,11 +208,13 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         return menus;
     }
 
+    public void updateAWSMenu() {
+        AWSMenu.setVisible(AmazonUtils.isAWSProviderPresent());
+    }
+
     /**
      * Generate the "tools" menu.
-     * This is imperative, it is written to field {@code toolsMenu}.
-     * Reason being, when we add (TODO remove)
-     * a new tool, we need to refresh just this menu
+     * Legacy pattern -- at one times tools could be loaded dynamically as plug-ins
      */
     void refreshToolsMenu() {
         List<JComponent> menuItems = new ArrayList<JComponent>(10);
@@ -210,7 +227,7 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         menuAction = new SortTracksMenuAction("Run igvtools...", KeyEvent.VK_T, igv) {
             @Override
             public void actionPerformed(ActionEvent e) {
-                IgvToolsGui.launch(false, igv.getGenomeManager().getGenomeId());
+                IgvToolsGui.launch(false, GenomeManager.getInstance().getGenomeId());
             }
         };
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
@@ -219,14 +236,14 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         menuItems.add(MotifFinderPlugin.getMenuItem());
 
         // BLAT
-        menuItems.add(BlatClient.getMenuItem());
+        menuItems.add(createBlatMenuItem());
 
         // Combine data tracks
         JMenuItem combineDataItem = new JMenuItem("Combine Data Tracks");
         combineDataItem.addActionListener(new ActionListener() {
             @Override
             public void actionPerformed(ActionEvent e) {
-                CombinedDataSourceDialog dialog = new CombinedDataSourceDialog(IGV.getMainFrame());
+                CombinedDataSourceDialog dialog = new CombinedDataSourceDialog(igv.getMainFrame());
                 dialog.setVisible(true);
             }
         });
@@ -271,20 +288,21 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         menuAction.setToolTipText(UIConstants.LOAD_SERVER_DATA_TOOLTIP);
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
-//        menuAction = new LoadFromURLMenuAction(LoadFromURLMenuAction.LOAD_FROM_DAS, KeyEvent.VK_D, igv);
-//        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
-
         if (PreferencesManager.getPreferences().getAsBoolean(DB_ENABLED)) {
             menuAction = new LoadFromDatabaseAction("Load from Database...", 0, igv);
             menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
         }
 
-
-
         encodeMenuItem = MenuAndToolbarUtils.createMenuItem(new BrowseEncodeAction("Load from ENCODE (2012)...", KeyEvent.VK_E, igv));
         menuItems.add(encodeMenuItem);
-        String genomeId = IGV.getInstance().getGenomeManager().getGenomeId();
-        encodeMenuItem.setVisible (EncodeFileBrowser.genomeSupported(genomeId));
+        String genomeId = GenomeManager.getInstance().getGenomeId();
+        encodeMenuItem.setVisible(EncodeFileBrowser.genomeSupported(genomeId));
+
+
+        menuItems.add(new JSeparator());
+        menuAction = new ReloadTracksMenuAction("Reload Tracks", -1, igv);
+        menuAction.setToolTipText(RELOAD_SESSION_TOOLTIP);
+        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
         menuItems.add(new JSeparator());
 
@@ -304,7 +322,8 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
         menuAction = new ReloadSessionMenuAction("Reload Session", -1, igv);
         menuAction.setToolTipText(RELOAD_SESSION_TOOLTIP);
-        JMenuItem reloadSessionItem = MenuAndToolbarUtils.createMenuItem(menuAction);
+        reloadSessionItem = MenuAndToolbarUtils.createMenuItem(menuAction);
+        reloadSessionItem.setEnabled(false);
         menuItems.add(reloadSessionItem);
 
         menuItems.add(new JSeparator());
@@ -312,15 +331,27 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         // ***** Snapshots
         // Snapshot Application
         menuAction =
-                new MenuAction("Save Image ...", null, KeyEvent.VK_A) {
+                new MenuAction("Save PNG Image ...", null, KeyEvent.VK_A) {
                     @Override
                     public void actionPerformed(ActionEvent e) {
-                        igv.saveImage(igv.getMainPanel());
+                        igv.saveImage(igv.getMainPanel(), "png");
 
                     }
                 };
 
-        menuAction.setToolTipText(SAVE_IMAGE_TOOLTIP);
+        menuAction.setToolTipText(SAVE_PNG_IMAGE_TOOLTIP);
+        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+
+        menuAction =
+                new MenuAction("Save SVG Image ...", null) {
+                    @Override
+                    public void actionPerformed(ActionEvent e) {
+                        igv.saveImage(igv.getMainPanel(), "svg");
+
+                    }
+                };
+
+        menuAction.setToolTipText(SAVE_SVG_IMAGE_TOOLTIP);
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
         // TODO -- change "Exit" to "Close" for BioClipse
@@ -347,20 +378,19 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         if (recentSessions != null) {
             String[] sessions = recentSessions.split(";");
             for (String sessionPath : sessions) {
-                if (!igv.getRecentSessionList().contains(sessionPath)) {
+                if (!sessionPath.equals("null") &&
+                        !igv.getRecentSessionList().contains(sessionPath)) {
                     igv.getRecentSessionList().add(sessionPath);
                 }
 
             }
         }
 
-        if (!IGV.getInstance().getRecentSessionList().isEmpty()) {
-
+        if (!igv.getRecentSessionList().isEmpty()) {
             menuItems.add(new JSeparator());
-
             // Now add menu items
-            for (final String session : IGV.getInstance().getRecentSessionList()) {
-                OpenSessionMenuAction osMenuAction = new OpenSessionMenuAction(session, session, IGV.getInstance());
+            for (final String session : igv.getRecentSessionList()) {
+                OpenSessionMenuAction osMenuAction = new OpenSessionMenuAction(session, IGV.getInstance());
                 menuItems.add(MenuAndToolbarUtils.createMenuItem(osMenuAction));
             }
 
@@ -399,16 +429,16 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
                     }
                 };
 
-        menuAction.setToolTipText("Load a FASTA or .genome file...");
+        menuAction.setToolTipText("Load a FASTA, .json, or .genome file...");
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
         // Load genome from URL
         menuAction = new LoadFromURLMenuAction(LoadFromURLMenuAction.LOAD_GENOME_FROM_URL, 0, igv);
-        menuAction.setToolTipText("Load a FASTA or .genome file...");
+        menuAction.setToolTipText("Load a FASTA, .json, or .genome file...");
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
-        // Add genome to combo box from server
-        menuAction = new MenuAction("Load Genome From Server...", null) {
+        // Download genome from server 
+        menuAction = new MenuAction("Select Hosted Genome...", null) {
             @Override
             public void actionPerformed(ActionEvent event) {
                 GenomeComboBox.loadGenomeFromServer();
@@ -418,35 +448,34 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         loadFromServerMenuItem = MenuAndToolbarUtils.createMenuItem(menuAction);
         menuItems.add(loadFromServerMenuItem);
 
-
         menuItems.add(new JSeparator());
 
         // Add genome to combo box from server
-        menuAction = new MenuAction("Remove genomes ...", null) {
+        menuAction = new MenuAction("Remove Genomes...", null) {
             @Override
             public void actionPerformed(ActionEvent event) {
-                RemoveGenomesDialog dialog2 = new RemoveGenomesDialog(IGV.getMainFrame());
+                RemoveGenomesDialog dialog2 = new RemoveGenomesDialog(igv.getMainFrame());
                 dialog2.setVisible(true);
             }
         };
         menuAction.setToolTipText("Remove genomes which appear in the dropdown list");
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
-
-        menuItems.add(new JSeparator());
-
-        menuAction =
-                new MenuAction("Create .genome File...", null, KeyEvent.VK_D) {
-                    @Override
-                    public void actionPerformed(ActionEvent event) {
-                        javax.swing.ProgressMonitor monitor = new javax.swing.ProgressMonitor(IGV.getInstance().getMainPanel(),
-                                "Creating genome", null, 0, 100);
-                        igv.defineGenome(monitor);
-                    }
-                };
-
-        menuAction.setToolTipText(UIConstants.IMPORT_GENOME_TOOLTIP);
-        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+//
+//        menuItems.add(new JSeparator());
+//
+//        menuAction =
+//                new MenuAction("Create .genome File...", null, KeyEvent.VK_D) {
+//                    @Override
+//                    public void actionPerformed(ActionEvent event) {
+//                        javax.swing.ProgressMonitor monitor = new javax.swing.ProgressMonitor(igv.getMainPanel(),
+//                                "Creating genome", null, 0, 100);
+//                        igv.defineGenome(monitor);
+//                    }
+//                };
+//
+//        menuAction.setToolTipText(UIConstants.IMPORT_GENOME_TOOLTIP);
+//        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
         MenuAction genomeMenuAction = new MenuAction("Genomes", null);
         return MenuAndToolbarUtils.createMenu(menuItems, genomeMenuAction);
@@ -472,6 +501,17 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         filterTracksAction.setToolTipText(UIConstants.FILTER_TRACKS_TOOLTIP);
         menuItems.add(MenuAndToolbarUtils.createMenuItem(filterTracksAction));
 
+        // Rename tracks
+        menuAction = new RenameTracksMenuAction("Rename Tracks... ", KeyEvent.VK_R, IGV.getInstance());
+        menuAction.setToolTipText(UIConstants.RENAME_TRACKS_TOOLTIP);
+        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+
+        // Overlay tracks
+        menuAction = new OverlayTracksMenuAction("Overlay Data Tracks... ", KeyEvent.VK_O, IGV.getInstance());
+        menuAction.setToolTipText(UIConstants.OVERLAY_TRACKS_TOOLTIP);
+        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+
+
         menuItems.add(new JSeparator());
 
         // Reset Tracks
@@ -506,7 +546,7 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
                     public void actionPerformed(ActionEvent e) {
                         UIUtilities.invokeOnEventThread(new Runnable() {
                             public void run() {
-                                IGV.getInstance().doViewPreferences();
+                                igv.doViewPreferences();
                             }
                         });
                     }
@@ -518,7 +558,7 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
                 new MenuAction("Color Legends ...", null, KeyEvent.VK_H) {
                     @Override
                     public void actionPerformed(ActionEvent e) {
-                        (new LegendDialog(IGV.getMainFrame())).setVisible(true);
+                        (new LegendDialog(igv.getMainFrame())).setVisible(true);
                     }
                 };
         menuAction.setToolTipText(SHOW_HEATMAP_LEGEND_TOOLTIP);
@@ -532,14 +572,13 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
                 JCheckBoxMenuItem menuItem = (JCheckBoxMenuItem) e.getSource();
                 if (menuItem.isSelected()) {
-                    IGV.getInstance().getMainPanel().expandNamePanel();
+                    igv.getMainPanel().expandNamePanel();
                 } else {
-                    IGV.getInstance().getMainPanel().collapseNamePanel();
+                    igv.getMainPanel().collapseNamePanel();
                 }
-                IGV.getInstance().doRefresh();
             }
         };
-        boolean isShowing = IGV.getInstance().getMainPanel().isExpanded();
+        boolean isShowing = igv.getMainPanel().isExpanded();
         JCheckBoxMenuItem menuItem = new JCheckBoxMenuItem();
         menuItem.setSelected(isShowing);
         menuItem.setAction(menuAction);
@@ -549,7 +588,7 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         menuAction = new MenuAction("Set Name Panel Width...", null, KeyEvent.VK_A) {
             @Override
             public void actionPerformed(ActionEvent e) {
-                MainPanel mainPanel = IGV.getInstance().getMainPanel();
+                MainPanel mainPanel = igv.getMainPanel();
                 String currentValue = String.valueOf(mainPanel.getNamePanelWidth());
                 String newValue = MessageUtils.showInputDialog("Enter track name panel width: ", currentValue);
                 if (newValue != null) {
@@ -568,8 +607,8 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         menuItems.add(panelWidthmenuItem);
 
         // Hide or Show the attribute panels
-        boolean isShow = PreferencesManager.getPreferences().getAsBoolean(SHOW_ATTRIBUTE_VIEWS_KEY);
-        IGV.getInstance().doShowAttributeDisplay(isShow);  // <= WEIRD doing IGV.getInstance() here!
+        //boolean isShow = PreferencesManager.getPreferences().getAsBoolean(SHOW_ATTRIBUTE_VIEWS_KEY);
+        //igv.doShowAttributeDisplay(isShow);
 
         menuAction = new MenuAction("Show Attribute Display", null, KeyEvent.VK_A) {
             @Override
@@ -577,20 +616,18 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
                 JCheckBoxMenuItem menuItem = (JCheckBoxMenuItem) e.getSource();
                 PreferencesManager.getPreferences().setShowAttributeView(menuItem.getState());
-                IGV.getInstance().getMainPanel().invalidate();
-                IGV.getInstance().doRefresh();
+                igv.revalidateTrackPanels();
             }
         };
+        boolean isShow = PreferencesManager.getPreferences().getAsBoolean(SHOW_ATTRIBUTE_VIEWS_KEY);
         menuItem = MenuAndToolbarUtils.createMenuItem(menuAction, isShow);
         menuItems.add(menuItem);
 
 
         menuAction =
                 new MenuAction("Select Attributes to Show...", null, KeyEvent.VK_S) {
-
                     @Override
-                    public void actionPerformed(ActionEvent e) {
-                        IGV.getInstance().doSelectDisplayableAttribute();
+                    public void actionPerformed(ActionEvent e) {igv.doSelectDisplayableAttribute();
                     }
                 };
         menuAction.setToolTipText(SELECT_DISPLAYABLE_ATTRIBUTES_TOOLTIP);
@@ -602,11 +639,11 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
                 JCheckBoxMenuItem menuItem = (JCheckBoxMenuItem) e.getSource();
                 if (menuItem.isSelected()) {
-                    IGV.getInstance().getMainPanel().restoreHeader();
+                    igv.getMainPanel().restoreHeader();
                 } else {
-                    IGV.getInstance().getMainPanel().removeHeader();
+                    igv.getMainPanel().removeHeader();
                 }
-                IGV.getInstance().doRefresh();
+                igv.getMainPanel().revalidate();
             }
         };
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction, true));
@@ -617,12 +654,23 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
                     @Override
                     public void actionPerformed(ActionEvent e) {
-                        ReorderPanelsDialog dlg = new ReorderPanelsDialog(IGV.getMainFrame());
+                        ReorderPanelsDialog dlg = new ReorderPanelsDialog(igv.getMainFrame());
                         dlg.setVisible(true);
                     }
                 };
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
 
+        menuAction =
+                new MenuAction("Add New Panel", null, KeyEvent.VK_S) {
+
+                    @Override
+                    public void actionPerformed(ActionEvent e) {
+                        String newPanelName = "Panel" + System.currentTimeMillis();
+                        igv.addDataPanel(newPanelName);
+                    }
+                };
+        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+
         menuItems.add(new JSeparator());
         menuItems.add(new HistoryMenu("Go to"));
 
@@ -647,7 +695,7 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
                 new MenuAction("Gene Lists...", null, KeyEvent.VK_S) {
                     @Override
                     public void actionPerformed(ActionEvent e) {
-                        (GeneListManagerUI.getInstance(IGV.getMainFrame())).setVisible(true);
+                        (GeneListManagerUI.getInstance(igv.getMainFrame())).setVisible(true);
                     }
                 };
         menuAction.setToolTipText("Open gene list manager");
@@ -723,22 +771,12 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
             }
         }
 
-        menuAction =
-                new MenuAction("Check for Updates...") {
-
-                    @Override
-                    public void actionPerformed(ActionEvent e) {
-                        checkVersion();
-                    }
-                };
-        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
-
         menuAction =
                 new MenuAction("About IGV ") {
 
                     @Override
                     public void actionPerformed(ActionEvent e) {
-                        (new AboutDialog(IGV.getMainFrame(), true)).setVisible(true);
+                        (new AboutDialog(igv.getMainFrame(), true)).setVisible(true);
                     }
                 };
         menuAction.setToolTipText(ABOUT_TOOLTIP);
@@ -800,73 +838,29 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         }
     }
 
-    private JMenu createGenomeSpaceMenu() {
-
-        JMenu menu = new JMenu("GenomeSpace");
-
-        MenuAction menuAction = null;
-        menuAction = new LoadFromGSMenuAction("Load File from GenomeSpace...", KeyEvent.VK_U, igv);
-        menu.add(MenuAndToolbarUtils.createMenuItem(menuAction));
-
-        menu.addSeparator();
-        menuAction = new LoadGenomeFromGSMenuAction("Load Genome from GenomeSpace...", KeyEvent.VK_Z, igv);
-        menu.add(MenuAndToolbarUtils.createMenuItem(menuAction));
-
-        menu.addSeparator();
-
-        menuAction = new GSSaveSessionMenuAction("Save Session to GenomeSpace...", igv);
-        menu.add(MenuAndToolbarUtils.createMenuItem(menuAction));
-
-        menuAction = new GSOpenSessionMenuAction("Load Session from GenomeSpace...", igv);
-        menu.add(MenuAndToolbarUtils.createMenuItem(menuAction));
-
-        menu.add(new JSeparator());
-        menuAction = new MenuAction("Logout") {
-            @Override
-            public void actionPerformed(ActionEvent e) {
-                GSUtils.logout();
-                if (MessageUtils.confirm("You must shutdown IGV to complete the GenomeSpace logout. Shutdown now?")) {
-                    doExitApplication();
-                }
-            }
-        };
-        menu.add(MenuAndToolbarUtils.createMenuItem(menuAction));
-
-        menu.add(new JSeparator());
-        menuAction =
-                new MenuAction("Register... ") {
-
-                    @Override
-                    public void actionPerformed(ActionEvent e) {
-                        try {
-                            BrowserLauncher.openURL(GENOMESPACE_REG_PAGE);
-                        } catch (IOException ex) {
-                            log.error("Error opening browser", ex);
-                        }
-
-                    }
-                };
-        menuAction.setToolTipText(GENOMESPACE_REG_TOOLTIP);
-        menu.add(MenuAndToolbarUtils.createMenuItem(menuAction));
-
-
-        menu.setVisible(PreferencesManager.getPreferences().getAsBoolean(GENOME_SPACE_ENABLE));
-
-
-        return menu;
-    }
-
     private JMenu createExtrasMenu() {
 
         List<JComponent> menuItems = new ArrayList<JComponent>();
 
+        JMenuItem memTest = new JMenuItem("Memory test");
+        memTest.addActionListener(e -> {
+            CommandExecutor exe = new CommandExecutor(this.igv);
+            int count = 1;
+            int start = 0;
+            exe.execute("snapshotDirectory /Users/jrobinso/Downloads/tmp");
+            while (count++ < 10000) {
+                exe.execute("goto chr1:" + start + "-" + (start + 1000));
+                exe.execute("snapshot");
+                start += 1000;
+            }
+        });
+        menuItems.add(memTest);
+
         MenuAction menuAction = null;
 
         // Preferences reset
         menuAction = new ResetPreferencesAction("Reset Preferences", IGV.getInstance());
         menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
-
-
         menuItems.add(new JSeparator());
 
 
@@ -882,7 +876,7 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
                             if (vals.length == 2) {
                                 int w = Integer.parseInt(vals[0]);
                                 int h = Integer.parseInt(vals[1]);
-                                IGV.getMainFrame().setSize(w, h);
+                                igv.getMainFrame().setSize(w, h);
                             }
                         }
                     }
@@ -891,11 +885,23 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
 
         // Save entire window
         menuAction =
-                new MenuAction("Save Screenshot ...", null, KeyEvent.VK_A) {
+                new MenuAction("Save PNG Screenshot ...", null, KeyEvent.VK_A) {
 
                     @Override
                     public void actionPerformed(ActionEvent e) {
-                        IGV.getInstance().saveImage(IGV.getInstance().getContentPane());
+                        igv.saveImage(igv.getContentPane(), "png");
+
+                    }
+                };
+
+        menuItems.add(MenuAndToolbarUtils.createMenuItem(menuAction));
+
+        menuAction =
+                new MenuAction("Save SVG Screenshot ...", null) {
+
+                    @Override
+                    public void actionPerformed(ActionEvent e) {
+                        igv.saveImage(igv.getContentPane(), "svg");
 
                     }
                 };
@@ -952,51 +958,69 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         }
         menu.add(lfMenu);
 
+        JMenuItem updateCS = new JMenuItem("Update chrom sizes");
+        updateCS.addActionListener(e -> {
+            try {
+                GenomeUtils.main(new String [] {});
+            } catch (IOException ioException) {
+                ioException.printStackTrace();
+            }
+        });
+        menu.add(updateCS);
+
         menu.setVisible(false);
-
-
         return menu;
     }
 
-    private JMenu createGoogleMenu() throws IOException {
+    private JMenu createAWSMenu() throws IOException {
 
-    	// Dynamically name menu - dwm08
-        final OAuthUtils oauth = OAuthUtils.getInstance();
+        JMenu menu = new JMenu("Amazon");
 
-        JMenu menu =  new JMenu(oauth.authProvider);
-
-        final JMenuItem login = new JMenuItem("Login ... ");
+        final JMenuItem login = new JMenuItem("Login");
         login.addActionListener(e -> {
             try {
-                oauth.openAuthorizationPage();
+                OAuthProvider oauth = OAuthUtils.getInstance().getProvider("Amazon");
+                oauth.openAuthorizationPage(); // should trigger and event and UI takes over
             } catch (Exception ex) {
                 MessageUtils.showErrorMessage("Error fetching oAuth tokens.  See log for details", ex);
                 log.error("Error fetching oAuth tokens", ex);
             }
-
         });
         login.setEnabled(false);
         menu.add(login);
 
-
-        final JMenuItem logout = new JMenuItem("Logout ");
+        final JMenuItem logout = new JMenuItem("Logout");
         logout.addActionListener(e -> {
+            OAuthProvider oauth = OAuthUtils.getInstance().getProvider("Amazon");
             oauth.logout();
-            GoogleUtils.setProjectID(null);
         });
         logout.setEnabled(false);
         menu.add(logout);
 
-        final JMenuItem projectID = new JMenuItem("Enter Project ID ...");
-        projectID.addActionListener(e -> GoogleUtils.enterGoogleProjectID());
-        menu.add(projectID);
+        final JMenuItem loadS3 = new JMenuItem("Load from S3 bucket");
+        loadS3.addActionListener(e -> {
+            List<String> buckets = AmazonUtils.ListBucketsForUser();
+            log.debug(buckets);
 
+            UIUtilities.invokeOnEventThread(() -> {
+                S3LoadDialog dlg = new S3LoadDialog(igv.getMainFrame());
+                dlg.setModal(true);
+                dlg.setVisible(true);
+                dlg.dispose();
+            });
+        });
+        loadS3.setEnabled(false);
+        menu.add(loadS3);
 
         menu.addMenuListener(new MenuListener() {
             @Override
             public void menuSelected(MenuEvent e) {
                 Runnable runnable = () -> {
-                    boolean loggedIn = oauth.isLoggedIn();
+                    OAuthProvider oauth = OAuthUtils.getInstance().getProvider("Amazon");
+                    boolean loggedIn = false;
+                    loggedIn = oauth.isLoggedIn();
+                    log.debug("MenuBar is user loggedIn?: " + loggedIn);
+
                     if (loggedIn) {
                         login.setText(oauth.getCurrentUserName());
                     } else {
@@ -1004,6 +1028,7 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
                     }
                     login.setEnabled(!loggedIn);
                     logout.setEnabled(loggedIn);
+                    loadS3.setEnabled(loggedIn);
                 };
 
                 LongRunningTask.submit(runnable);
@@ -1018,12 +1043,82 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
             public void menuCanceled(MenuEvent e) {
 
             }
+
         });
 
-
         return menu;
     }
 
+    private JMenu createGoogleMenu() throws IOException {
+
+        // Dynamically name menu - dwm08
+        final OAuthProvider oauth = OAuthUtils.getInstance().getProvider();
+
+        if (oauth != null) {  // TODO -- how do we know this is a google provider?
+            oauth.setAuthProvider("Google");
+            JMenu menu = new JMenu(oauth.getAuthProvider());
+
+            final JMenuItem login = new JMenuItem("Login ... ");
+            login.addActionListener(e -> {
+                try {
+                    oauth.openAuthorizationPage();
+                } catch (Exception ex) {
+                    MessageUtils.showErrorMessage("Error fetching oAuth tokens.  See log for details", ex);
+                    log.error("Error fetching oAuth tokens", ex);
+                }
+
+            });
+            //login.setEnabled(false);
+            menu.add(login);
+
+
+            final JMenuItem logout = new JMenuItem("Logout ");
+            logout.addActionListener(e -> {
+                oauth.logout();
+                GoogleUtils.setProjectID(null);
+            });
+            logout.setEnabled(false);
+            menu.add(logout);
+
+            final JMenuItem projectID = new JMenuItem("Enter Project ID ...");
+            projectID.addActionListener(e -> GoogleUtils.enterGoogleProjectID());
+            menu.add(projectID);
+
+            menu.addMenuListener(new MenuListener() {
+                @Override
+                public void menuSelected(MenuEvent e) {
+                    Runnable runnable = () -> {
+                        boolean loggedIn = OAuthUtils.getInstance().getProvider().isLoggedIn();
+
+                        if (loggedIn) {
+                            login.setText(oauth.getCurrentUserName());
+                        } else {
+                            login.setText("Login ...");
+                        }
+                        login.setEnabled(!loggedIn);
+                        logout.setEnabled(loggedIn);
+                    };
+
+                    LongRunningTask.submit(runnable);
+                }
+
+                @Override
+                public void menuDeselected(MenuEvent e) {
+
+                }
+
+                @Override
+                public void menuCanceled(MenuEvent e) {
+
+                }
+
+            });
+            return menu;
+        } else {
+            return null;
+        }
+    }
+
 //    public void enableRemoveGenomes() {
 //        if (removeImportedGenomeAction != null) {
 //            removeImportedGenomeAction.setEnabled(true);
@@ -1067,17 +1162,11 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         return false;
     }
 
-    public JMenu getViewMenu() {
-        return viewMenu;
-    }
-
     final public void doExitApplication() {
 
         try {
-            IGV.getInstance().saveStateForExit();
-
-            Frame mainFrame = IGV.getMainFrame();
-
+            igv.saveStateForExit();
+            Frame mainFrame = igv.getMainFrame();
             PreferencesManager.getPreferences().setApplicationFrameBounds(mainFrame.getBounds());
 
             // Hide and close the application
@@ -1087,7 +1176,6 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
         } finally {
             System.exit(0);
         }
-
     }
 
     @ForTesting
@@ -1096,14 +1184,41 @@ public class IGVMenuBar extends JMenuBar implements IGVEventObserver {
     }
 
     public void enableGoogleMenu(boolean aBoolean) {
-        googleMenu.setVisible(aBoolean);
+        if (googleMenu != null) {
+            googleMenu.setVisible(aBoolean);
+        }
     }
 
     @Override
     public void receiveEvent(final Object event) {
 
-        if(event instanceof GenomeChangeEvent) {
-            UIUtilities.invokeOnEventThread(() -> encodeMenuItem.setVisible (EncodeFileBrowser.genomeSupported(((GenomeChangeEvent) event).genome.getId())));
+        if (event instanceof GenomeChangeEvent) {
+            UIUtilities.invokeOnEventThread(() -> encodeMenuItem.setVisible(EncodeFileBrowser.genomeSupported(((GenomeChangeEvent) event).genome.getId())));
         }
     }
+
+    public void enableReloadSession() {
+        this.reloadSessionItem.setEnabled(true);
+    }
+
+    public void disableReloadSession() {
+        this.reloadSessionItem.setEnabled(false);
+    }
+
+    public static JMenuItem createBlatMenuItem() {
+        JMenuItem menuItem = new JMenuItem("BLAT ...");
+        menuItem.addActionListener(e -> {
+
+            String blatSequence = MessageUtils.showInputDialog("Enter sequence to blat:");
+            if (blatSequence != null) {
+                if(blatSequence.length() < 20 || blatSequence.length() > 8000) {
+                    MessageUtils.showMessage("BLAT sequences must be between 20 and 8000 bases in length.");
+                } else {
+                    BlatClient.doBlatQuery(blatSequence, "BLAT");
+                }
+            }
+        });
+
+        return menuItem;
+    }
 }
