@@ -28,27 +28,32 @@ package org.broad.igv.ui;
 import com.jidesoft.plaf.LookAndFeelFactory;
 import com.sanityinc.jargs.CmdLineParser;
 import htsjdk.samtools.seekablestream.SeekableStreamFactory;
-
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.DirectoryManager;
 import org.broad.igv.Globals;
+import org.broad.igv.batch.CommandListener;
+import org.broad.igv.google.OAuthUtils;
 import org.broad.igv.prefs.IGVPreferences;
 import org.broad.igv.prefs.PreferencesManager;
+import org.broad.igv.ui.util.UIUtilities;
 import org.broad.igv.util.FileUtils;
 import org.broad.igv.util.HttpUtils;
 import org.broad.igv.util.RuntimeUtils;
 import org.broad.igv.util.stream.IGVSeekableStreamFactory;
+import org.broad.igv.util.stream.IGVUrlHelperFactory;
 
 import javax.swing.*;
 import java.awt.*;
 import java.awt.event.WindowAdapter;
 import java.awt.event.WindowEvent;
 import java.io.File;
+import java.io.FileWriter;
 import java.io.UnsupportedEncodingException;
 import java.net.URL;
 import java.net.URLDecoder;
 import java.util.ArrayList;
 import java.util.Arrays;
+import java.util.Collection;
 import java.util.HashSet;
 
 import static org.broad.igv.prefs.Constants.*;
@@ -73,7 +78,7 @@ import static org.broad.igv.prefs.Constants.*;
  */
 public class Main {
 
-    private static Logger log = Logger.getLogger(Main.class);
+    private static Logger log = LogManager.getLogger(Main.class);
 
     /**
      * Launch an igv instance as a stand-alone application in its own Frame.
@@ -90,26 +95,30 @@ public class Main {
         if (igvArgs.igvDirectory != null) {
             setIgvDirectory(igvArgs);
         }
+        checkDotIgvDirectory();
 
-        Runnable runnable = new Runnable() {
-            public void run() {
+        htsjdk.tribble.util.ParsingUtils.setURLHelperFactory(IGVUrlHelperFactory.getInstance());
 
-                // This is a workaround for an internal JVM crash that was happening on Windows 10 (Creators Update).
-                // TODO: remove when enough users have migrated to Java 8u141 or greater.
-                // http://bugs.java.com/bugdatabase/view_bug.do?bug_id=JDK-8179014
-                if (Globals.IS_WINDOWS && System.getProperty("os.name").contains("10")) {
-                    UIManager.put("FileChooser.useSystemExtensionHiding", false);
-                }
+        try {
+            OAuthUtils.getInstance();  // Initialize oauth
+        } catch (Exception e) {
+            log.error("Warning: Error fetching oAuth properties: " + e.getMessage());
+        }
 
-                DesktopIntegration.verifyJavaPlatform();
-                initApplication();
 
-                JFrame frame = new JFrame();
-                frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
-                ImageIcon icon = new ImageIcon(Main.class.getResource("mainframeicon.png"));
-                if (icon != null) frame.setIconImage(icon.getImage());
-                open(frame, igvArgs);
+        Runnable runnable = () -> {
+            if (Globals.IS_WINDOWS && System.getProperty("os.name").contains("10")) {
+                UIManager.put("FileChooser.useSystemExtensionHiding", false);
             }
+
+            DesktopIntegration.verifyJavaPlatform();
+            initApplication(igvArgs);
+
+            JFrame frame = new JFrame();
+            frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
+            ImageIcon icon = new ImageIcon(Main.class.getResource("mainframeicon.png"));
+            if (icon != null) frame.setIconImage(icon.getImage());
+            open(frame, igvArgs);
         };
 
         SwingUtilities.invokeLater(runnable);
@@ -143,7 +152,50 @@ public class Main {
         }
     }
 
-    private static void initApplication() {
+    private static void checkDotIgvDirectory() {
+        // Check if the .igv directory exists and create it if not.  This is a config
+        // file with a known name and location, not intended to be moved by the user.
+        // At present, this is only used by the launcher scripts and not the Java code.
+        String userHome = System.getProperty("user.home");
+        File dir = new File(userHome, ".igv");
+        if (!dir.exists()) {
+            // doesn't exist -- try to create it
+            try {
+                dir.mkdir();
+            } catch (Exception e) {
+                // Ignore the mkdir failure.  It's not necessary to even report this.
+                // We'll proceed without it.
+                return;
+            }
+        }
+
+        // Also check if the java_arguments file exists and create it if not.  This is
+        // likewise only used by the launcher scripts.  We create it here as a user
+        // convenience.  Note that we skip it if ~/.igv is not a directory.
+        if (dir.isDirectory()) {
+            File argsFile = new File(dir, "java_arguments");
+            if (!argsFile.exists()) {
+                // doesn't exist -- try to create it
+                try {
+                    FileWriter argsFileWriter = new FileWriter(argsFile);
+                    try {
+                        argsFileWriter.append("# See https://raw.githubusercontent.com/igvteam/igv/master/scripts/readme.txt for tips on using this file.");
+                        argsFileWriter.append(System.lineSeparator());
+                        argsFileWriter.append("# Uncomment the following line for an 8 GB memory spec for IGV.");
+                        argsFileWriter.append(System.lineSeparator());
+                        argsFileWriter.append("# -Xmx8G");
+                        argsFileWriter.append(System.lineSeparator());
+                    } finally {
+                        argsFileWriter.close();
+                    }
+                } catch (Exception e) {
+                    // As above, ignore the write failure.
+                }
+            }
+        }
+    }
+
+    private static void initApplication(Main.IGVArgs igvArgs) {
 
         long mem = RuntimeUtils.getAvailableMemory();
         int MB = 1000000;
@@ -154,17 +206,18 @@ public class Main {
 
         DirectoryManager.initializeLog();
         log.info("Startup  " + Globals.applicationString());
-        log.info("Java " + System.getProperty(Globals.JAVA_VERSION_STRING) 
-            + " (build " + System.getProperty("java.vm.version")
-            + ") " + System.getProperty("java.version.date", ""));
+        log.info("Java " + System.getProperty(Globals.JAVA_VERSION_STRING)
+                + " (build " + System.getProperty("java.vm.version")
+                + ") " + System.getProperty("java.version.date", ""));
         log.info("Java Vendor: " + System.getProperty("java.vendor")
-            + " " + System.getProperty("java.vendor.url", ""));
+                + " " + System.getProperty("java.vendor.url", ""));
         log.info("JVM: " + System.getProperty("java.vm.name", "")
-            + " " + System.getProperty("java.vendor.version", "")
-            + "   " + System.getProperty("java.compiler", ""));
-        log.info("Default User Directory: " + DirectoryManager.getUserDirectory());
-        log.info("OS: " + System.getProperty("os.name") + " " + System.getProperty("os.version")
-            + " " + System.getProperty("os.arch"));
+                + " " + System.getProperty("java.vendor.version", "")
+                + "   " + System.getProperty("java.compiler", ""));
+         log.info("OS: " + System.getProperty("os.name") + " " + System.getProperty("os.version")
+                + " " + System.getProperty("os.arch"));
+        log.info("IGV Directory: " + DirectoryManager.getIgvDirectory().getAbsolutePath());
+
         System.setProperty("http.agent", Globals.applicationString());
 
         Runtime.getRuntime().addShutdownHook(new ShutdownThread());
@@ -176,8 +229,6 @@ public class Main {
         System.setProperty("swing.aatext", "true");
 
         checkVersion();
-
-
     }
 
     public static void updateTooltipSettings() {
@@ -199,26 +250,34 @@ public class Main {
                     final String serverVersionString = HttpUtils.getInstance().getContentsAsString(new URL(Globals.getVersionURL())).trim();
                     // See if user has specified to skip this update
 
-                    final String skipString = PreferencesManager.getPreferences().get(SKIP_VERSION);
-                    if (skipString != null) {
-                        HashSet<String> skipVersion = new HashSet<>(Arrays.asList(skipString.split(",")));
-                        if (skipVersion.contains(serverVersionString)) return;
-                    }
-
                     Version serverVersion = Version.getVersion(serverVersionString.trim());
                     if (serverVersion == null) return;
 
                     if (thisVersion.lessThan(serverVersion)) {
-                        log.info("A later version of IGV is available (" + serverVersionString + ")");
-                    }
-                } else if (Globals.VERSION.contains("3.0_beta") || Globals.VERSION.contains("snapshot")) {
-                    HttpUtils.getInstance().getContentsAsString(new URL(Globals.getVersionURL())).trim();
-                } else {
-                    log.info("Unknown version: " + Globals.VERSION);
-                }
 
+                        log.info("A later version of IGV is available (" + serverVersionString + ")");
+                        final String skipString = PreferencesManager.getPreferences().get(SKIP_VERSION);
+                        boolean skip = false;
+                        if (skipString != null) {
+                            HashSet<String> skipVersion = new HashSet<>(Arrays.asList(skipString.split(",")));
+                            skip = (skipVersion.contains(serverVersionString));
+                        }
+
+                        if (!(skip || Globals.isBatch() || Globals.isHeadless() || Globals.isSuppressMessages())) {
+                            // Inform user, do this only once
+                            final VersionUpdateDialog dlg = new VersionUpdateDialog(serverVersionString);
+                            UIUtilities.invokeOnEventThread(() -> {
+                                dlg.setVisible(true);
+                                if (dlg.isSkipVersion()) {
+                                    String newSkipString = skipString + "," + serverVersionString;
+                                    PreferencesManager.getPreferences().put(SKIP_VERSION, newSkipString);
+                                }
+                            });
+                        }
+                    }
+                }
             } catch (Exception e) {
-                log.error("Error checking version", e);
+                log.error("Error checking IGV version ");
             } finally {
 
             }
@@ -289,9 +348,10 @@ public class Main {
 
         SeekableStreamFactory.setInstance(IGVSeekableStreamFactory.getInstance());
 
-        RuntimeUtils.loadPluginJars();
+        // Start IGV's UI itself (frame) and other components
+       IGV igv = IGV.createInstance(frame, igvArgs);
 
-        IGV.createInstance(frame).startUp(igvArgs);
+       igv.startUp(igvArgs);
 
         // TODO Should this be done here?  Will this step on other key dispatchers?
         KeyboardFocusManager.getCurrentKeyboardFocusManager().addKeyEventDispatcher(new GlobalKeyDispatcher());
@@ -304,10 +364,14 @@ public class Main {
             UIManager.setLookAndFeel(lnf);
 
         } catch (Exception e) {
-            e.printStackTrace();
+            log.error("Error setting look and feel", e);
         }
 
         double resolutionScale = Toolkit.getDefaultToolkit().getScreenResolution() / Globals.DESIGN_DPI;
+        if(resolutionScale != 1.0) {
+            log.info("Resoluction scale = " + resolutionScale);
+        }
+
         final IGVPreferences prefMgr = PreferencesManager.getPreferences();
         if (resolutionScale > 1.5) {
             if (prefMgr.getAsBoolean(SCALE_FONTS)) {
@@ -360,7 +424,7 @@ public class Main {
                     return new Version(major, minor, build);
 
                 } catch (NumberFormatException e) {
-                    log.error("Error parsing version string: " + versionString);
+                    log.error("Error parsing version string");
                     return null;
                 }
             }
@@ -410,7 +474,7 @@ public class Main {
         private String coverageFile = null;
         private String name = null;
         public String igvDirectory = null;
-        public String forceVersion = null;
+        public Collection<String> httpHeader = null;
 
         IGVArgs(String[] args) {
             if (args != null) {
@@ -435,14 +499,16 @@ public class Main {
             CmdLineParser.Option nameOption = parser.addStringOption('n', "name");
             CmdLineParser.Option locusOption = parser.addStringOption('l', "locus");
             CmdLineParser.Option igvDirectoryOption = parser.addStringOption("igvDirectory");
-            CmdLineParser.Option forceVersionOption = parser.addStringOption("forceVersion");
             CmdLineParser.Option versionOption = parser.addBooleanOption("version");
+            CmdLineParser.Option helpOption = parser.addBooleanOption("help");
+            CmdLineParser.Option headerOption = parser.addStringOption('H', "header");
+
 
             try {
                 parser.parse(args);
             } catch (Exception e) {
                 e.printStackTrace();  // This is not logged because the logger is not initialized yet.
-                JOptionPane.showMessageDialog(null, "Error parsing command line argments: " + e.getMessage());
+                JOptionPane.showMessageDialog(null, "Error parsing command line arguments: " + e.getMessage());
                 return;
             }
 
@@ -454,6 +520,7 @@ public class Main {
             genomeServerURL = getDecodedValue(parser, genomeServerOption);
             name = (String) parser.getOptionValue(nameOption);
             locusString = (String) parser.getOptionValue(locusOption);
+            httpHeader = parser.getOptionValues(headerOption);
 
             String indexFilePath = (String) parser.getOptionValue(indexFileOption);
             if (indexFilePath != null) {
@@ -471,18 +538,18 @@ public class Main {
                 igvDirectory = maybeDecodePath(igvDirectoryPath);
             }
 
-            String forceVersion = (String) parser.getOptionValue(forceVersionOption);
-            if (forceVersion != null) {
-                Globals.VERSION = forceVersion;
-            }
-
             String[] nonOptionArgs = parser.getRemainingArgs();
 
-            if(parser.getOptionValue(versionOption) != null) {
+            if (parser.getOptionValue(versionOption) != null) {
                 System.out.println(Globals.VERSION);
                 System.exit(0);
             }
 
+            if (parser.getOptionValue(helpOption) != null) {
+                printHelp();
+                System.exit(0);
+            }
+
             // The Mac app launcher sometimes inserts "" into the command line.  Filter empty strings
             if (nonOptionArgs.length > 0) {
                 nonOptionArgs = Arrays.stream(nonOptionArgs).filter(s -> !s.isEmpty()).toArray(String[]::new);
@@ -496,7 +563,7 @@ public class Main {
 
                 } else if (nonOptionArgs.length == 1) {
 
-                        decodeNonOptionArgsLegacy(nonOptionArgs);
+                    decodeNonOptionArgsLegacy(nonOptionArgs);
 
                 } else {  //nonOptionArgs.length == 2
 
@@ -550,7 +617,7 @@ public class Main {
 
         private String maybeDecodePath(String path) {
 
-            if (FileUtils.resourceExists(path)) {
+            if ((new File(path)).exists()) {
                 return path;
             } else if (FileUtils.isRemote(path)) {
                 return URLDecoder.decode(path);
@@ -623,6 +690,27 @@ public class Main {
             return name;
         }
 
-    }
+        public Collection<String> getHttpHeader() {
+            return httpHeader;
+        }
 
+        private void printHelp() {
+            System.out.println("Command line options:");
+            System.out.println("Space delimited list of data files to load");
+            System.out.println("--preferences, -o  Path or URL to a preference property file");
+            System.out.println("--batch. -b  Path or url to a batch command file");
+            System.out.println("--port, -p  IGV command port number (defaults to 60151)");
+            System.out.println("--genome, -g  Genome ID (e.g hg19) or path or url to .genome or indexed fasta file");
+            System.out.println("--dataServerURL, -d  Path or url to a data server registry file");
+            System.out.println("--genomeServerURL, -u  Path or url to a genome server registry file");
+            System.out.println("--indexFile, -i  Index file or comma delimited list of index files corresponding to data files");
+            System.out.println("--coverageFile, -c  Coverage file or comma delimited list of coverage files corresponding to data files");
+            System.out.println("--name, -n  Name or comma-delimited list of names for tracks corresponding to data files");
+            System.out.println("--locus, -l  Initial locus");
+            System.out.println("--header, -H http header to include with all requests for list of data files");
+            System.out.println("--igvDirectory Path to the local igv directory.  Defaults to <user home>/igv");
+            System.out.println("--version  Print the IGV version and exit");
+            System.out.println("--help Print this output and exit");
+        }
+    }
 }
