@@ -29,18 +29,20 @@
  */
 package org.broad.igv.ui.action;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.exceptions.HttpResponseException;
 import org.broad.igv.feature.genome.GenomeManager;
 import org.broad.igv.google.GoogleUtils;
-import org.broad.igv.google.OAuthUtils;
 import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.PreferencesManager;
+import org.broad.igv.session.SessionReader;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.IGVMenuBar;
 import org.broad.igv.ui.util.LoadFromURLDialog;
 import org.broad.igv.ui.util.MessageUtils;
+import org.broad.igv.util.AmazonUtils;
 import org.broad.igv.util.HttpUtils;
+import org.broad.igv.util.LongRunningTask;
 import org.broad.igv.util.ResourceLocator;
 
 import javax.swing.*;
@@ -52,12 +54,14 @@ import java.util.Arrays;
 import java.util.HashMap;
 import java.util.Map;
 
+import static org.broad.igv.util.AmazonUtils.isObjectAccessible;
+
 /**
  * @author jrobinso
  */
 public class LoadFromURLMenuAction extends MenuAction {
 
-    static Logger log = Logger.getLogger(LoadFilesMenuAction.class);
+    static Logger log = LogManager.getLogger(LoadFilesMenuAction.class);
     public static final String LOAD_FROM_DAS = "Load from DAS...";
     public static final String LOAD_FROM_URL = "Load from URL...";
     public static final String LOAD_FILE_AND_INDEX_FROM_URLS = "Load file and index from URLs...";
@@ -76,25 +80,25 @@ public class LoadFromURLMenuAction extends MenuAction {
         ta.setPreferredSize(new Dimension(600, 20));
         if (e.getActionCommand().equalsIgnoreCase(LOAD_FROM_URL)) {
 
-            LoadFromURLDialog dlg = new LoadFromURLDialog(IGV.getMainFrame());
+            LoadFromURLDialog dlg = new LoadFromURLDialog(IGV.getInstance().getMainFrame());
             dlg.setVisible(true);
 
             if (!dlg.isCanceled()) {
 
-                String url = dlg.getFileURL();
+                String inputURL = dlg.getFileURL();
 
-                if (url != null && url.trim().length() > 0) {
+                if (inputURL != null && inputURL.trim().length() > 0) {
 
-                    url = mapURL(url.trim());
+                    final String url = mapURL(inputURL.trim());
 
-                    if (url.endsWith(".xml") || url.endsWith(".session")) {
+                    if (SessionReader.isSessionFile(url)) {
                         try {
-                            boolean merge = false;
-                            String locus = null;
-                            igv.doRestoreSession(url, locus, merge);
+                            LongRunningTask.submit(() -> this.igv.loadSession(url, null));
                         } catch (Exception ex) {
                             MessageUtils.showMessage("Error loading url: " + url + " (" + ex.toString() + ")");
                         }
+                    } else if (url.startsWith("s3://")) {
+                        checkAWSAccessbility(url);
                     } else {
                         ResourceLocator rl = new ResourceLocator(url.trim());
 
@@ -113,24 +117,29 @@ public class LoadFromURLMenuAction extends MenuAction {
                 }
             }
         } else if ((e.getActionCommand().equalsIgnoreCase(LOAD_FROM_DAS))) {
-            String url = JOptionPane.showInputDialog(IGV.getMainFrame(), ta, "Enter DAS feature source URL",
+            String url = JOptionPane.showInputDialog(IGV.getInstance().getMainFrame(), ta, "Enter DAS feature source URL",
                     JOptionPane.QUESTION_MESSAGE);
             if (url != null && url.trim().length() > 0) {
                 ResourceLocator rl = new ResourceLocator(url.trim());
-                rl.setType("das");
+                rl.setFormat("das");
                 igv.loadTracks(Arrays.asList(rl));
             }
         } else if ((e.getActionCommand().equalsIgnoreCase(LOAD_GENOME_FROM_URL))) {
-
-            String url = JOptionPane.showInputDialog(IGV.getMainFrame(), ta, "Enter URL to .genome or FASTA file",
+            String url = JOptionPane.showInputDialog(IGV.getInstance().getMainFrame(), ta, "Enter URL to .genome or FASTA file",
                     JOptionPane.QUESTION_MESSAGE);
             if (url != null && url.trim().length() > 0) {
+                if (url.startsWith("s3://")) {
+                    checkAWSAccessbility(url);
+                } else if (url.startsWith("ftp://")) {
+                    MessageUtils.showMessage("FTP protocol is not supported");
+                }
                 try {
                     url = mapURL(url);
                     GenomeManager.getInstance().loadGenome(url.trim(), null);
                 } catch (Exception e1) {
                     MessageUtils.showMessage("Error loading genome: " + e1.getMessage());
                 }
+
             }
         }
     }
@@ -144,11 +153,11 @@ public class LoadFromURLMenuAction extends MenuAction {
             enableGoogleMenu();
             // if user is not currently logged in, attempt to
             // log in user
-            try {
-                OAuthUtils.getInstance().doSecureLogin();
-            } catch (IOException e) {
-                log.error("Error connecting to OAuth: " + e.getMessage());
-            }
+//            try {
+//               OAuthUtils.getInstance().getProvider().doSecureLogin();
+//            } catch (IOException e) {
+//                log.error("Error connecting to OAuth: " + e.getMessage());
+//            }
 
         }
 
@@ -164,6 +173,24 @@ public class LoadFromURLMenuAction extends MenuAction {
     }
 
 
+    private void checkAWSAccessbility(String url) {
+        try {
+            // If AWS support is active, check if objects are in accessible tiers via Load URL menu...
+            if (AmazonUtils.isAwsS3Path(url)) {
+                String bucket = AmazonUtils.getBucketFromS3URL(url);
+                String key = AmazonUtils.getKeyFromS3URL(url);
+                AmazonUtils.s3ObjectAccessResult res = isObjectAccessible(bucket, key);
+                if (!res.isObjectAvailable()) {
+                    MessageUtils.showErrorMessage(res.getErrorReason(), null);
+                    return;
+                }
+            }
+        } catch (NullPointerException npe) {
+            // User has not yet done Amazon->Login sequence
+            AmazonUtils.checkLogin();
+        }
+    }
+
     private boolean ping(String url) {
         InputStream is = null;
         try {
