@@ -34,8 +34,10 @@
 package org.broad.igv.ui.panel;
 
 import com.google.common.base.Objects;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
+import org.broad.igv.event.DataLoadedEvent;
+import org.broad.igv.event.IGVEventObserver;
 import org.broad.igv.feature.RegionOfInterest;
 import org.broad.igv.prefs.Constants;
 import org.broad.igv.prefs.PreferencesManager;
@@ -47,9 +49,8 @@ import org.broad.igv.ui.AbstractDataPanelTool;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.UIConstants;
 import org.broad.igv.ui.WaitCursorManager;
-import org.broad.igv.event.DataLoadedEvent;
-import org.broad.igv.event.IGVEventObserver;
 import org.broad.igv.ui.util.DataPanelTool;
+import org.broad.igv.ui.util.MessageUtils;
 
 import javax.swing.*;
 import javax.swing.event.MouseInputAdapter;
@@ -59,8 +60,8 @@ import java.awt.event.KeyEvent;
 import java.awt.event.MouseEvent;
 import java.awt.event.MouseWheelEvent;
 import java.text.DecimalFormat;
-import java.util.*;
 import java.util.List;
+import java.util.*;
 import java.util.concurrent.CompletableFuture;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
@@ -75,23 +76,15 @@ import java.util.stream.Collectors;
  */
 public class DataPanel extends JComponent implements Paintable, IGVEventObserver {
 
-    private static Logger log = Logger.getLogger(DataPanel.class);
-
-    // Thread pool for loading data
-    private static final ExecutorService threadExecutor = Executors.newFixedThreadPool(5);
-
-    private boolean isWaitingForToolTipText = false;
+    private static Logger log = LogManager.getLogger(DataPanel.class);
 
     private DataPanelTool defaultTool;
     private DataPanelTool currentTool;
-    // private Point tooltipTextPosition;
     private ReferenceFrame frame;
     private DataPanelContainer parent;
     private DataPanelPainter painter;
     private String tooltipText = "";
 
-    private boolean loadInProgress = false;
-
     public DataPanel(ReferenceFrame frame, DataPanelContainer parent) {
         init();
         this.defaultTool = new PanTool(this);
@@ -103,19 +96,14 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
         setToolTipText("");
         painter = new DataPanelPainter();
         setBackground(PreferencesManager.getPreferences().getAsColor(Constants.BACKGROUND_COLOR));
-
         ToolTipManager.sharedInstance().registerComponent(this);
-
-
-        //    IGVEventBus.getInstance().subscribe(DataLoadedEvent.class, this);
     }
 
     @Override
     public void receiveEvent(Object event) {
-
         if (event instanceof DataLoadedEvent) {
             if (((DataLoadedEvent) event).referenceFrame == frame) {
-                log.info("Data loaded repaint " + frame);
+                log.debug("Data loaded repaint " + frame);
                 repaint();
             }
         }
@@ -132,7 +120,6 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
         return sp == null ? null : ((JScrollPane) sp).getVerticalScrollBar();
     }
 
-
     public void setCurrentTool(final AbstractDataPanelTool tool) {
         this.currentTool = (tool == null) ? defaultTool : tool;
         if (currentTool != null) {
@@ -140,6 +127,7 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
         }
     }
 
+    long lastPaintTime = 0;
 
     @Override
     public void paintComponent(final Graphics g) {
@@ -148,15 +136,7 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
         RenderContext context = null;
         try {
 
-            long t0 = System.currentTimeMillis();
-
-            if (!allTracksLoaded()) {
-                if (!loadInProgress) {
-                    loadInProgress = true;
-                    load();
-                }
-                if (!Globals.isBatch()) return;
-            }
+            lastPaintTime = System.currentTimeMillis();
 
             Rectangle clipBounds = g.getClipBounds();
             final Rectangle visibleRect = getVisibleRect();
@@ -186,77 +166,29 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
             drawAllRegions(g);
 
 
-            long dt = System.currentTimeMillis() - t0;
+            long dt = System.currentTimeMillis() - lastPaintTime;
             PanTool.repaintTime(dt);
 
+        } catch (Exception e) {
+            MessageUtils.showMessage("Unexpected error repainting view.  See igv.log for details.");
+            log.error("Error painting DataPanel", e);
         } finally {
-
             if (context != null) {
                 context.dispose();
             }
         }
     }
 
-
-    public boolean allTracksLoaded() {
-        return parent.getTrackGroups().stream().
-                filter(TrackGroup::isVisible).
-                flatMap(trackGroup -> trackGroup.getVisibleTracks().stream()).
-                allMatch(track -> track.isReadyToPaint(frame));
+    @Override
+    public Dimension getMaximumSize() {
+        return new Dimension(Short.MAX_VALUE, Integer.MAX_VALUE);
     }
 
-
-    public List<Track> visibleTracks() {
-        return parent.getTrackGroups().stream().
-                filter(TrackGroup::isVisible).
-                flatMap(trackGroup -> trackGroup.getVisibleTracks().stream()).
-                collect(Collectors.toList());
-    }
-
-    private void load() {
-
-        ReferenceFrame frame = getFrame();
-        Collection<Track> trackList = visibleTracks();
-        List<CompletableFuture> futures = new ArrayList(trackList.size());
-        boolean batchLoaded = false;
-        for (Track track : trackList) {
-            if (track.isReadyToPaint(frame) == false) {
-                final Runnable runnable = () -> {
-                    track.load(frame);
-                    this.revalidate();
-                };
-
-                if (Globals.isBatch()) {
-                    runnable.run();
-                    batchLoaded = true;
-                } else {
-                    futures.add(CompletableFuture.runAsync(runnable, threadExecutor));
-                }
-            }
-        }
-
-        if (futures.size() > 0 || batchLoaded) {
-
-            final CompletableFuture[] futureArray = futures.toArray(new CompletableFuture[futures.size()]);
-
-            WaitCursorManager.CursorToken token = WaitCursorManager.showWaitCursor();
-
-            CompletableFuture.allOf(futureArray)
-                    .thenRun(() -> {
-                        //log.info("Call repaint " + dataPanel.hashCode() + " " + dataPanel.allTracksLoaded());
-                        loadInProgress = false;
-                        WaitCursorManager.removeWaitCursor(token);
-                        repaint();
-
-                    })
-                    .exceptionally(e -> {
-                        log.error("Error: ", e);
-                        loadInProgress = false;
-                        WaitCursorManager.removeWaitCursor(token);
-                        return null;
-                    });
-
-        }
+    @Override
+    public void setBounds(int x, int y, int width, int height) {
+        super.setBounds(x, y, width, height);
+        Insets insets = this.getInsets();
+        frame.setBounds(x + insets.left, width - insets.left - insets.right);
     }
 
     /**
@@ -303,31 +235,40 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
      * @param rect
      */
 
-    public void paintOffscreen(final Graphics2D g, Rectangle rect) {
+    public void paintOffscreen(final Graphics2D g, Rectangle rect, boolean batch) {
 
         RenderContext context = null;
-        try {
+        Graphics2D borderGraphics = (Graphics2D) g.create();
+        borderGraphics.setColor(Color.lightGray);
 
+        try {
             context = new RenderContext(null, g, frame, rect);
             final Collection<TrackGroup> groups = new ArrayList(parent.getTrackGroups());
-            int width = rect.width;
-            painter.paint(groups, context, width, getBackground(), rect);
-
+            Insets insets = getInsets();
+            Rectangle contentRect =  new Rectangle(
+                    rect.x + insets.left,
+                    rect.y + insets.top,
+                    rect.width - (insets.left + insets.right),
+                    rect.height - (insets.top + insets.bottom));
+            context.getGraphics().setClip(contentRect);
+            painter.paint(groups, context, contentRect.width, getBackground(), contentRect);
             drawAllRegions(g);
 
-            Color c = g.getColor();
-            g.setColor(Color.darkGray);
-            g.drawRect(rect.x, rect.y, rect.width, rect.height);
-            g.setColor(c);            //super.paintBorder(g);
+            borderGraphics.drawRect(0, rect.y, rect.width, rect.height-1);
 
         } finally {
-
             if (context != null) {
                 context.dispose();
             }
+            borderGraphics.dispose();
         }
     }
 
+    @Override
+    public int getSnapshotHeight(boolean batch) {
+        return getHeight();
+    }
+
 
     /**
      * Draw vertical lines demarcating regions of interest.
@@ -380,15 +321,6 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
         return false;
     }
 
-    protected String generateTileKey(final String chr, int t,
-                                     final int zoomLevel) {
-
-        // Fetch image for this chromosome, zoomlevel, and tile.  If found
-        // draw immediately
-        final String key = chr + "_z_" + zoomLevel + "_t_" + t;
-        return key;
-    }
-
 
     /**
      * Do not remove - Used for debugging only
@@ -547,7 +479,7 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
 
 
     private void init() {
-        setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
+        setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(200, 200, 200)));
         setRequestFocusEnabled(false);
 
         // Key Events
@@ -632,6 +564,7 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
         private ClickTaskScheduler clickScheduler = new ClickTaskScheduler();
 
         long lastClickTime = 0;
+        MouseEvent mouseDown = null;
 
 
         @Override
@@ -645,7 +578,7 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
             updateTooltipText(e.getX(), e.getY());
 
             if (IGV.getInstance().isRulerEnabled()) {
-                IGV.getInstance().revalidateTrackPanels();
+                IGV.getInstance().repaint();
             }
 
         }
@@ -660,14 +593,14 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
             if (SwingUtilities.getWindowAncestor(DataPanel.this).isActive()) {
                 DataPanel.this.requestFocus();
             }
-
             if (e.isPopupTrigger()) {
                 doPopupMenu(e);
+                e.consume();
             } else {
                 if (currentTool != null)
                     currentTool.mousePressed(e);
+                mouseDown = e;
             }
-
         }
 
         /**
@@ -676,13 +609,16 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
          */
         @Override
         public void mouseReleased(MouseEvent e) {
-
             if (e.isPopupTrigger()) {
                 doPopupMenu(e);
+                e.consume();
             } else {
-                if (currentTool != null)
+                if (mouseDown != null && distance(mouseDown, e) < 5) {
+                    doMouseClick(e);
+                } else if (currentTool != null)
                     currentTool.mouseReleased(e);
             }
+            mouseDown = null;
         }
 
         private void doPopupMenu(MouseEvent e) {
@@ -692,6 +628,12 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
             parent.openPopupMenu(te);
         }
 
+        private double distance(MouseEvent e1, MouseEvent e2) {
+            double dx = e1.getX() - e2.getX();
+            double dy = e1.getY() - e2.getY();
+            return Math.sqrt(dx * dx + dy * dy);
+        }
+
 
         /**
          * The mouse has been dragged.  Delegate to current tool.
@@ -700,15 +642,54 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
          */
         @Override
         public void mouseDragged(MouseEvent e) {
-            if (currentTool != null)
+            if (mouseDown != null && currentTool != null)
                 currentTool.mouseDragged(e);
         }
 
+        @Override
+        public void mouseEntered(MouseEvent e) {
+            mouseDown = null;
+        }
+
+        @Override
+        public void mouseExited(MouseEvent e) {
+            mouseDown = null;
+        }
+
+        /**
+         * Zoom in/out when modifier + scroll wheel used
+         *
+         * @param e
+         */
+        @Override
+        public void mouseWheelMoved(MouseWheelEvent e) {
+            //we use either ctrl or meta to deal with PCs and Macs
+            if (e.isControlDown() || e.isMetaDown()) {
+                int wheelRotation = e.getWheelRotation();
+                //Mouse move up is negative, that should zoom in
+                int zoomIncr = -wheelRotation / 2;
+                getFrame().doZoomIncrement(zoomIncr);
+            }
+            //TODO Use this to pan. Seems weird, but it's how side scrolling on my mouse gets interpreted,
+            //so could be handy for people with 2D wheels
+//            else if(e.isShiftDown()){
+//                System.out.println(e);
+//            }
+            else {
+                //Default action if no modifier
+                e.getComponent().getParent().dispatchEvent(e);
+            }
+        }
+
 
         /**
          * The mouse was clicked. If this is the second click of a double click, cancel the scheduled single click task.
          * The shift and alt keys are alternative  zoom options
          * shift zooms in by 8x,  alt zooms out by 2x
+         * <p>
+         * NOTE: mouseClick is not used because in Java a mouseClick event is emitted only if the mouse has not
+         * moved at all between press and release.  This is difficult to do, even when trying.
+         * <p>
          * <p/>
          * TODO -- the "currentTool" is also a mouselistener, so there are two.  This makes mouse event handling
          * TODO -- needlessly complicated, which handler has preference, etc.  Move this code to the default
@@ -716,13 +697,12 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
          *
          * @param e
          */
-        @Override
-        public void mouseClicked(final MouseEvent e) {
+
+        public void doMouseClick(final MouseEvent e) {
 
             long clickTime = System.currentTimeMillis();
 
-            // ctrl-mouse down is the mac popup trigger, but you will also get a clck even.  Ignore the click.
-            if (Globals.IS_MAC && e.isControlDown()) {
+            if (e.isPopupTrigger()) {
                 return;
             }
 
@@ -814,30 +794,6 @@ public class DataPanel extends JComponent implements Paintable, IGVEventObserver
             }
         }
 
-        /**
-         * Zoom in/out when modifier + scroll wheel used
-         *
-         * @param e
-         */
-        @Override
-        public void mouseWheelMoved(MouseWheelEvent e) {
-            //we use either ctrl or meta to deal with PCs and Macs
-            if (e.isControlDown() || e.isMetaDown()) {
-                int wheelRotation = e.getWheelRotation();
-                //Mouse move up is negative, that should zoom in
-                int zoomIncr = -wheelRotation / 2;
-                getFrame().doZoomIncrement(zoomIncr);
-            }
-            //TODO Use this to pan. Seems weird, but it's how side scrolling on my mouse gets interpreted,
-            //so could be handy for people with 2D wheels
-//            else if(e.isShiftDown()){
-//                System.out.println(e);
-//            }
-            else {
-                //Default action if no modifier
-                e.getComponent().getParent().dispatchEvent(e);
-            }
-        }
     }
 
     /**
