@@ -26,20 +26,19 @@
 package org.broad.igv.ui.panel;
 
 import com.jidesoft.swing.JideSplitPane;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.LogManager;
+import org.broad.igv.logging.Logger;
 import org.broad.igv.prefs.PreferencesManager;
-import org.broad.igv.session.Session;
 import org.broad.igv.track.AttributeManager;
 import org.broad.igv.ui.IGV;
-import org.broad.igv.ui.util.SnapshotUtilities;
 import org.broad.igv.ui.util.UIUtilities;
 
 import javax.swing.*;
 import java.awt.*;
 import java.awt.event.ComponentEvent;
 import java.awt.event.ComponentListener;
-import java.util.*;
 import java.util.List;
+import java.util.*;
 
 import static org.broad.igv.prefs.Constants.*;
 
@@ -49,7 +48,7 @@ import static org.broad.igv.prefs.Constants.*;
  */
 public class MainPanel extends JPanel implements Paintable {
 
-    private static Logger log = Logger.getLogger(MainPanel.class);
+    private static Logger log = LogManager.getLogger(MainPanel.class);
 
     IGV igv;
 
@@ -92,8 +91,8 @@ public class MainPanel extends JPanel implements Paintable {
         addComponentListener(new ComponentListener() {
 
             public void componentResized(ComponentEvent componentEvent) {
-                revalidate();
-                repaint();
+                revalidateTrackPanels();
+                igv.repaint();
             }
 
             public void componentMoved(ComponentEvent componentEvent) {
@@ -132,17 +131,29 @@ public class MainPanel extends JPanel implements Paintable {
 
     public void collapseNamePanel() {
         namePanelWidth = 0;
-        revalidate();
+        revalidateTrackPanels();
     }
 
     public void expandNamePanel() {
         namePanelWidth = PreferencesManager.getPreferences().getAsInt(NAME_PANEL_WIDTH);
-        revalidate();
+        revalidateTrackPanels();
     }
 
     public void setNamePanelWidth(int width) {
         this.namePanelWidth = width;
-        revalidate();
+        revalidateTrackPanels();
+    }
+
+    public void revalidateTrackPanels() {
+        updatePanelDimensions();
+        UIUtilities.invokeOnEventThread(() -> {
+            this.applicationHeaderPanel.invalidate();
+            for (TrackPanel tp : this.getTrackPanels()) {
+                tp.invalidate();
+            }
+            this.invalidate(); // this should not be neccessary, but is harmless
+            this.validate();
+        });
     }
 
     public void removeHeader() {
@@ -158,7 +169,6 @@ public class MainPanel extends JPanel implements Paintable {
 
     @Override
     public void doLayout() {
-        layoutFrames();
         super.doLayout();
         applicationHeaderPanel.doLayout();
         for (TrackPanel tp : getTrackPanels()) {
@@ -190,22 +200,16 @@ public class MainPanel extends JPanel implements Paintable {
 
         nameHeaderPanel = new NameHeaderPanel();
         nameHeaderPanel.setBackground(new java.awt.Color(255, 255, 255));
-        nameHeaderPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
         nameHeaderPanel.setMinimumSize(new java.awt.Dimension(0, 0));
         nameHeaderPanel.setPreferredSize(new java.awt.Dimension(0, 0));
-        nameHeaderPanel.setLayout(new BorderLayout());
 
         attributeHeaderPanel = new AttributeHeaderPanel();
-        attributeHeaderPanel.setBackground(new java.awt.Color(255, 255, 255));
-        attributeHeaderPanel.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(0, 0, 0)));
         attributeHeaderPanel.setDebugGraphicsOptions(javax.swing.DebugGraphics.NONE_OPTION);
         attributeHeaderPanel.setMinimumSize(new java.awt.Dimension(0, 0));
         attributeHeaderPanel.setPreferredSize(new java.awt.Dimension(0, 0));
 
 
         headerPanelContainer = new HeaderPanelContainer();
-
-
         headerScrollPane = new JScrollPane();
         headerScrollPane.setBorder(javax.swing.BorderFactory.createLineBorder(new java.awt.Color(102, 102, 102)));
         headerScrollPane.setForeground(new java.awt.Color(153, 153, 153));
@@ -230,7 +234,7 @@ public class MainPanel extends JPanel implements Paintable {
             featureTrackScrollPane = new TrackPanelScrollPane();
             featureTrackScrollPane.setPreferredSize(new java.awt.Dimension(1021, 50));
             featureTrackScrollPane.setViewportView(new TrackPanel(IGV.FEATURE_PANEL_NAME, this));
-            add(featureTrackScrollPane, java.awt.BorderLayout.SOUTH);
+            // add(featureTrackScrollPane, java.awt.BorderLayout.SOUTH);
         }
 
 
@@ -259,11 +263,13 @@ public class MainPanel extends JPanel implements Paintable {
 
 
     /**
-     * Removes user added panels. Used for resetting sessions
+     * Reset panels to initial state
      */
     public void resetPanels() {
         for (TrackPanel tp : getTrackPanels()) {
-            tp.clearTracks();
+            tp.reset();
+        }
+        for (TrackPanel tp : getTrackPanels()) {
             final TrackPanelScrollPane tsp = tp.getScrollPane();
             if (tsp == dataTrackScrollPane || tsp == featureTrackScrollPane) {
                 continue;
@@ -271,8 +277,6 @@ public class MainPanel extends JPanel implements Paintable {
             centerSplitPane.remove(tsp);
             TrackNamePanel.removeDropListenerFor(tsp.getNamePanel());
         }
-
-        igv.reset();
     }
 
     /**
@@ -426,55 +430,20 @@ public class MainPanel extends JPanel implements Paintable {
         }
     }
 
-    public void layoutFrames() {
-        synchronized (getTreeLock()) {
-
-            Insets insets = applicationHeaderPanel.getInsets();
-            namePanelX = insets.left;
-
-            attributePanelX = namePanelX + namePanelWidth + hgap;
-            attributePanelWidth = calculateAttributeWidth();
-
-            dataPanelX = attributePanelX + attributePanelWidth + hgap;
-
-            java.util.List<ReferenceFrame> frames = FrameManager.getFrames();
-            dataPanelWidth = applicationHeaderPanel.getWidth() - insets.right - dataPanelX;
-
-            if (frames.size() == 1) {
-                frames.get(0).setBounds(0, dataPanelWidth);
-            } else {
-
-                float gap = Math.min(1, 20.0f / ((int) (1.5 * frames.size()))) * hgap;
-                int x = 0;
-
-                // Width is in floating point because we need to fill data panel,  going straight to an "int" here
-                // would cause truncation
-                Session.GeneListMode mode = IGV.hasInstance() ?
-                        IGV.getInstance().getSession().getGeneListMode() :
-                        Session.GeneListMode.NORMAL;
-
-                float wc = mode == Session.GeneListMode.NORMAL ?
-                        ((float) dataPanelWidth - (frames.size() - 1) * gap) / frames.size() :
-                        20;
-
-                for (int i = 0; i < frames.size(); i++) {
-                    ReferenceFrame frame = frames.get(i);
-                    int nextX = (int) ((i + 1) * (wc + gap));
-                    int w = nextX - x;
-                    frame.setBounds(x, w);
-                    x = nextX;
-                }
-            }
-        }
+    public void updatePanelDimensions() {
+        Insets insets = applicationHeaderPanel.getInsets();
+        namePanelX = insets.left;
+        attributePanelX = namePanelX + namePanelWidth + hgap;
+        attributePanelWidth = calculateAttributeWidth();
+        dataPanelX = attributePanelX + attributePanelWidth + hgap;
+        dataPanelWidth = applicationHeaderPanel.getWidth() - insets.right - dataPanelX;
     }
 
-
-    private int calculateAttributeWidth() {
+    public int calculateAttributeWidth() {
 
         if (!PreferencesManager.getPreferences().getAsBoolean(SHOW_ATTRIBUTE_VIEWS_KEY)) {
             return 0;
         }
-
         Collection<String> attributeKeys = AttributeManager.getInstance().getVisibleAttributes();
         int attributeCount = attributeKeys.size();
         int packWidth = (attributeCount) * (AttributeHeaderPanel.ATTRIBUTE_COLUMN_WIDTH +
@@ -517,26 +486,17 @@ public class MainPanel extends JPanel implements Paintable {
     }
 
     static class SplitPane extends JideSplitPane {
-        @Override
-        public void doLayout() {
-            if (log.isTraceEnabled()) {
-                log.trace("Layout");
-            }
-            super.doLayout();    //To change body of overridden methods use File | Settings | File Templates.
-        }
+//        @Override
+//        public void doLayout() {
+//            if (log.isTraceEnabled()) {
+//                log.trace("Layout");
+//            }
+//            super.doLayout();    //To change body of overridden methods use File | Settings | File Templates.
+//        }
     }
 
 
-    public void paintOffscreen(Graphics2D g, Rectangle rect) {
-
-        // A hack -- we don't want to paint the background for vector graphics output (EPS and SVG)
-        String graphicsClassName = g.getClass().getName().toLowerCase();
-        if (!(graphicsClassName.contains("epd") || graphicsClassName.contains("svg"))) {
-            Graphics2D backgroundGraphics = (Graphics2D) g.create();
-            backgroundGraphics.setColor(Color.white);
-            backgroundGraphics.fill(rect);
-            backgroundGraphics.dispose();
-        }
+    public void paintOffscreen(Graphics2D g, Rectangle rect, boolean batch) {
 
         // Header
         int width = applicationHeaderPanel.getWidth();
@@ -544,20 +504,16 @@ public class MainPanel extends JPanel implements Paintable {
 
         Graphics2D headerGraphics = (Graphics2D) g.create();
         Rectangle headerRect = new Rectangle(0, 0, width, height);
-        applicationHeaderPanel.paintOffscreen(headerGraphics, headerRect);
+        applicationHeaderPanel.paintOffscreen(headerGraphics, headerRect, batch);
         headerGraphics.dispose();
 
-        // Now loop through track panel
+        // Now loop through track panels
         Rectangle r = centerSplitPane.getBounds();
         g.translate(0, r.y);
 
         // Get the components of the center pane and sort by Y position.
         Component[] components = centerSplitPane.getComponents();
-        Arrays.sort(components, new Comparator<Component>() {
-            public int compare(Component component, Component component1) {
-                return component.getY() - component1.getY();
-            }
-        });
+        Arrays.sort(components, Comparator.comparingInt(Component::getY));
 
         int dy = components[0].getY();
         for (Component c : components) {
@@ -566,7 +522,6 @@ public class MainPanel extends JPanel implements Paintable {
             g2d.translate(0, dy);
 
             if (c instanceof TrackPanelScrollPane) {
-
                 TrackPanelScrollPane tsp = (TrackPanelScrollPane) c;
 
                 //Skip if panel has no tracks
@@ -574,14 +529,12 @@ public class MainPanel extends JPanel implements Paintable {
                     continue;
                 }
 
-                int panelHeight = getOffscreenImagePanelHeight(tsp);
+                int panelHeight = tsp.getSnapshotHeight(batch);
 
-                Rectangle tspRect = new Rectangle(tsp.getBounds());
-                tspRect.height = panelHeight;
-
-                g2d.setClip(new Rectangle(0, 0, tsp.getWidth(), tspRect.height));
-                tsp.paintOffscreen(g2d, tspRect);
+                Rectangle tspRect = new Rectangle(0, 0, tsp.getWidth(), panelHeight);
 
+                g2d.setClip(tspRect);
+                tsp.paintOffscreen(g2d, tspRect, batch);
                 dy += tspRect.height;
 
             } else {
@@ -594,7 +547,7 @@ public class MainPanel extends JPanel implements Paintable {
 
         }
 
-        //super.paintBorder(g);
+//        //super.paintBorder(g);
 
     }
 
@@ -604,48 +557,34 @@ public class MainPanel extends JPanel implements Paintable {
      *
      * @return
      */
-    public int getOffscreenImageHeight() {
-        int height = centerSplitPane.getBounds().y;
-        for (Component c : centerSplitPane.getComponents()) {
+    @Override
+    public int getSnapshotHeight(boolean batch) {
 
-            if (c instanceof TrackPanelScrollPane) {
+        if (batch) {
+            int height = applicationHeaderPanel.getHeight();
 
-                TrackPanelScrollPane tsp = (TrackPanelScrollPane) c;
+            for (Component c : centerSplitPane.getComponents()) {
 
-                //Skip if panel has no tracks
-                if (tsp.getTrackPanel().getTracks().size() == 0) {
-                    continue;
+                if (c instanceof TrackPanelScrollPane) {
+
+                    TrackPanelScrollPane tsp = (TrackPanelScrollPane) c;
+
+                    //Skip if panel has no tracks
+                    if (tsp.getTrackPanel().getTracks().size() == 0) {
+                        continue;
+                    }
+
+                    height += tsp.getSnapshotHeight(batch);
+
+                } else {
+                    height += c.getHeight();
                 }
 
-                int panelHeight = getOffscreenImagePanelHeight(tsp);
-
-                Rectangle tspRect = new Rectangle(tsp.getBounds());
-                tspRect.height = panelHeight;
-
-
-                height += tspRect.height;
-            } else {
-                height += c.getHeight();
             }
-
-        }
-        // TODO Not sure why this is neccessary
-        height += 35;
-        return height;
-
-    }
-
-    private int getOffscreenImagePanelHeight(TrackPanelScrollPane tsp) {
-
-        int panelHeight;
-        int maxPanelHeight = SnapshotUtilities.getMaxPanelHeight();
-        final int visibleHeight = tsp.getVisibleRect().height;
-        if (maxPanelHeight < 0) {
-            panelHeight = visibleHeight;
+            return height;
         } else {
-            panelHeight = Math.min(maxPanelHeight, Math.max(visibleHeight, tsp.getDataPanel().getHeight()));
+            return getHeight();
         }
-        return panelHeight;
     }
 
 
