@@ -26,7 +26,7 @@
 
 package org.broad.igv.ui.panel;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.feature.Range;
 import org.broad.igv.feature.RegionOfInterest;
 import org.broad.igv.feature.Strand;
@@ -36,6 +36,7 @@ import org.broad.igv.ui.IGV;
 import org.broad.igv.event.IGVEventBus;
 import org.broad.igv.event.IGVEventObserver;
 import org.broad.igv.event.ViewChange;
+import org.broad.igv.ui.util.IGVMouseInputAdapter;
 import org.broad.igv.util.StringUtils;
 
 import javax.swing.*;
@@ -61,7 +62,7 @@ import java.util.List;
  */
 public class RegionNavigatorDialog extends JDialog implements Observer, IGVEventObserver {
 
-    private static Logger log = Logger.getLogger(AttributePanel.class);
+    private static Logger log = LogManager.getLogger(AttributePanel.class);
 
     //Column indexes, in case table structure changes
     private static final int TABLE_COLINDEX_CHR = 0;
@@ -566,9 +567,9 @@ public class RegionNavigatorDialog extends JDialog implements Observer, IGVEvent
                         cm.getColumn(3).setPreferredWidth(200);
                     }
                     regionTable.setAutoCreateRowSorter(true);
-                    regionTable.addMouseListener(new MouseAdapter() {
+                    regionTable.addMouseListener(new IGVMouseInputAdapter() {
                         @Override
-                        public void mouseClicked(MouseEvent e) {
+                        public void igvMouseClicked(MouseEvent e) {
                             regionTableMouseClicked(e);
                         }
                     });
@@ -685,16 +686,16 @@ public class RegionNavigatorDialog extends JDialog implements Observer, IGVEvent
         public void actionPerformed(ActionEvent e) {
             String chr = FrameManager.getDefaultFrame().getChrName();
             if (FrameManager.isGeneListMode()) {
-                JOptionPane.showMessageDialog(IGV.getMainFrame(),
+                JOptionPane.showMessageDialog(IGV.getInstance().getMainFrame(),
                         "Regions cannot be created in gene list or split-screen views.",
                         "Error", JOptionPane.INFORMATION_MESSAGE);
 
             } else if (chr == null || chr.isEmpty()) {
-                JOptionPane.showMessageDialog(IGV.getMainFrame(),
+                JOptionPane.showMessageDialog(IGV.getInstance().getMainFrame(),
                         "No chromosome is specified. Can't create a region without a chromosome.",
                         "Error", JOptionPane.INFORMATION_MESSAGE);
             } else if (chr.equalsIgnoreCase("All")) {
-                JOptionPane.showMessageDialog(IGV.getMainFrame(),
+                JOptionPane.showMessageDialog(IGV.getInstance().getMainFrame(),
                         "Regions cannot be created in the All Chromosomes view.",
                         "Error", JOptionPane.INFORMATION_MESSAGE);
             } else {
@@ -723,7 +724,7 @@ public class RegionNavigatorDialog extends JDialog implements Observer, IGVEvent
                 synchRegions();
             } else {
                 //todo dhmay -- I don't fully understand this call.  Clean this up.
-                JOptionPane.showMessageDialog(IGV.getMainFrame(), "No regions have been selected for removal.",
+                JOptionPane.showMessageDialog(IGV.getInstance().getMainFrame(), "No regions have been selected for removal.",
                         "Error", JOptionPane.INFORMATION_MESSAGE);
             }
             updateButtonsEnabled();
