@@ -33,7 +33,7 @@
  */
 package org.broad.igv.ui.panel;
 
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
 import org.broad.igv.feature.Chromosome;
 import org.broad.igv.feature.genome.ChromosomeCoordinate;
@@ -64,7 +64,7 @@ import static org.broad.igv.prefs.Constants.ENABLE_ANTIALISING;
  */
 public class RulerPanel extends JPanel {
 
-    private static Logger log = Logger.getLogger(RulerPanel.class);
+    private static Logger log = LogManager.getLogger(RulerPanel.class);
 
     private static final DecimalFormat DECIMAL_FORMAT = new DecimalFormat();
 
@@ -208,14 +208,15 @@ public class RulerPanel extends JPanel {
                     " " + ts.getMajorUnit();
             int strWidth = g.getFontMetrics().stringWidth(chrPosition);
             int strPosition = x - strWidth / 2;
-            //if (strPosition > strEnd) {
-
             final int height = getHeight();
-            if (nTick % 2 == 0) {
+
+            if (nTick % 2 == 0 && strPosition > 0) {
                 g.drawString(chrPosition, strPosition, height - 15);
             }
+            if(x > 0) {
+                g.drawLine(x, height - 10, x, height - 2);
+            }
 
-            g.drawLine(x, height - 10, x, height - 2);
             nTick++;
         }
     }
@@ -264,8 +265,9 @@ public class RulerPanel extends JPanel {
             int x = (int) (gStart / scale);
             int dw = (int) (chrLength / (locationUnit * scale));
 
-
-            g.drawLine(x, getHeight() - 10, x, getHeight() - 2);
+            if(x > 0) {
+                g.drawLine(x, getHeight() - 10, x, getHeight() - 2);
+            }
 
             // Don't label chromosome if its width is < 5 pixels
             if (dw > 5) {
@@ -337,8 +339,7 @@ public class RulerPanel extends JPanel {
 
     private void init() {
 
-        setBorder(BorderFactory.createLineBorder(UIConstants.TRACK_BORDER_GRAY));
-
+        //setBorder(BorderFactory.createLineBorder(UIConstants.TRACK_BORDER_GRAY));
         setCursor(Cursor.getDefaultCursor());
         if (isWholeGenomeView()) {
             this.setToolTipText(WHOLE_GENOME_TOOLTIP);
@@ -348,31 +349,7 @@ public class RulerPanel extends JPanel {
 
         MouseInputAdapter mouseAdapter = new MouseInputAdapter() {
 
-            int lastMousePressX;
-
-            @Override
-            public void mouseClicked(MouseEvent evt) {
-                final MouseEvent e = evt;
-                setCursor(Cursor.getDefaultCursor());
-                WaitCursorManager.CursorToken token = WaitCursorManager.showWaitCursor();
-                try {
-
-                    if (!isWholeGenomeView()) {
-                        double newLocation = frame.getChromosomePosition(e.getX());
-                        frame.centerOnLocation(newLocation);
-                    } else {
-                        for (final ClickLink link : chromosomeRects) {
-                            if (link.region.contains(e.getPoint())) {
-                                final String chrName = link.value;
-                                frame.changeChromosome(chrName, true);
-                            }
-                        }
-                    }
-                } finally {
-                    WaitCursorManager.removeWaitCursor(token);
-                }
-
-            }
+            private MouseEvent mouseDown;
 
             @Override
             public void mouseMoved(MouseEvent e) {
@@ -410,7 +387,6 @@ public class RulerPanel extends JPanel {
 
             }
 
-
             @Override
             public void mouseDragged(MouseEvent e) {
                 if (Math.abs(e.getPoint().getX() - dragStart) > 1) {
@@ -421,23 +397,69 @@ public class RulerPanel extends JPanel {
             }
 
             @Override
-            public void mousePressed(MouseEvent e) {
-                dragStart = e.getX();
-            }
-
-            @Override
-            public void mouseReleased(MouseEvent e) {
-                if (dragging) {
-                    dragEnd = e.getX();
-                    dragging = false;
-                    zoom();
+            public void mousePressed(final MouseEvent e) {
+                if (e.isPopupTrigger()) {
+                    // ignore
+                } else {
+                    dragStart = e.getX();
+                    mouseDown = e;
                 }
             }
 
-            //@Override
-            //public void mouseExited(MouseEvent e) {
-            //dragging = false;
-            //}
+
+            @Override
+            public void mouseReleased(MouseEvent e) {
+                if (e.isPopupTrigger()) {
+                    // ignore
+                } else {
+                    if (dragging) {
+                        dragEnd = e.getX();
+                        dragging = false;
+                        zoom();
+                    } else {
+                        if (mouseDown != null && distance(mouseDown, e) < 5) {
+                            doMouseClick(e);
+                        }
+                    }
+                }
+            }
+
+            private double distance(MouseEvent e1, MouseEvent e2) {
+                double dx = e1.getX() - e2.getX();
+                double dy = e1.getY() - e2.getY();
+                return Math.sqrt(dx * dx + dy * dy);
+            }
+
+
+            /**
+             * mouseClick is not used because in Java a mouseClick is emitted ONLY if the mouse has not moved
+             * between press and release.  This is really difficult to achieve, even if trying.
+             *
+             * @param evt
+             */
+            public void doMouseClick(MouseEvent evt) {
+                final MouseEvent e = evt;
+                setCursor(Cursor.getDefaultCursor());
+                WaitCursorManager.CursorToken token = WaitCursorManager.showWaitCursor();
+                try {
+
+                    if (!isWholeGenomeView()) {
+                        double newLocation = frame.getChromosomePosition(e.getX());
+                        frame.centerOnLocation(newLocation);
+                    } else {
+                        for (final ClickLink link : chromosomeRects) {
+                            if (link.region.contains(e.getPoint())) {
+                                final String chrName = link.value;
+                                frame.changeChromosome(chrName, true);
+                            }
+                        }
+                    }
+                } finally {
+                    WaitCursorManager.removeWaitCursor(token);
+                }
+
+            }
+
         };
 
         addMouseMotionListener(mouseAdapter);
