@@ -26,23 +26,20 @@
 package org.broad.igv.util;
 
 import biz.source_code.base64Coder.Base64Coder;
-import htsjdk.samtools.seekablestream.SeekableStream;
 import htsjdk.samtools.util.ftp.FTPClient;
 import htsjdk.samtools.util.ftp.FTPStream;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.apache.tomcat.util.HttpDate;
 import org.broad.igv.Globals;
 import org.broad.igv.exceptions.HttpResponseException;
-import org.broad.igv.ga4gh.GoogleUtils;
-import org.broad.igv.ga4gh.OAuthUtils;
-import org.broad.igv.gs.GSUtils;
+import org.broad.igv.google.GoogleUtils;
+import org.broad.igv.google.OAuthUtils;
 import org.broad.igv.prefs.IGVPreferences;
 import org.broad.igv.prefs.PreferencesManager;
 import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.util.MessageUtils;
 import org.broad.igv.util.collections.CI;
 import org.broad.igv.util.ftp.FTPUtils;
-import org.broad.igv.util.stream.IGVUrlHelper;
 
 import javax.net.ssl.HttpsURLConnection;
 import javax.net.ssl.SSLContext;
@@ -56,9 +53,10 @@ import java.io.*;
 import java.net.*;
 import java.security.KeyManagementException;
 import java.security.NoSuchAlgorithmException;
-import java.util.*;
+import java.time.ZonedDateTime;
+import java.time.format.DateTimeParseException;
 import java.util.List;
-import java.util.regex.Pattern;
+import java.util.*;
 import java.util.zip.GZIPInputStream;
 
 import static org.broad.igv.prefs.Constants.*;
@@ -72,7 +70,7 @@ import static org.broad.igv.util.stream.SeekableServiceStream.WEBSERVICE_URL;
  */
 public class HttpUtils {
 
-    private static Logger log = Logger.getLogger(HttpUtils.class);
+    private static Logger log = LogManager.getLogger(HttpUtils.class);
 
     private static HttpUtils instance;
 
@@ -83,12 +81,22 @@ public class HttpUtils {
 
     private String defaultUserName = null;
     private char[] defaultPassword = null;
-    private static Pattern URLmatcher = Pattern.compile(".{1,8}://.*");
+
+    private Map<String, Collection<String>> headerMap = new HashMap<>();
 
     // static provided to support unit testing
     private static boolean BYTE_RANGE_DISABLED = false;
     private Map<URL, Boolean> headURLCache = new HashMap<URL, Boolean>();
 
+    private class CachedRedirect {
+        private URL url = null;
+        private ZonedDateTime expires = null;
+    }
+
+    // remember HTTP redirects
+    private final int DEFAULT_REDIRECT_EXPIRATION_MIN = 15;
+    private Map<URL, CachedRedirect> redirectCache = new HashMap<URL, CachedRedirect>();
+
     /**
      * @return the single instance
      */
@@ -101,18 +109,13 @@ public class HttpUtils {
 
     private HttpUtils() {
 
-        htsjdk.tribble.util.ParsingUtils.registerHelperClass(IGVUrlHelper.class);
-
-        // if (!Globals.checkJavaVersion("1.8")) {
         disableCertificateValidation();
-        // }
-        CookieHandler.setDefault(new IGVCookieManager());
         Authenticator.setDefault(new IGVAuthenticator());
 
         try {
             System.setProperty("java.net.useSystemProxies", "true");
         } catch (Exception e) {
-            log.info("Couldn't set useSystemProxies=true");
+            log.warn("Couldn't set useSystemProxies=true");
         }
 
         byteRangeTestMap = Collections.synchronizedMap(new HashMap());
@@ -127,33 +130,59 @@ public class HttpUtils {
      * @throws MalformedURLException
      */
     public static URL createURL(String urlString) throws MalformedURLException {
+        if (AmazonUtils.isAwsS3Path(urlString)) {
+            try {
+                urlString = AmazonUtils.translateAmazonCloudURL(urlString);
+            } catch (IOException e) {
+                log.error("Error translating amazon cloud URL: " + urlString, e);
+                throw new RuntimeException(e);
+            }
+        } else {
+            urlString = mapURL(urlString.trim());
+        }
+        return new URL(urlString);
+    }
 
-        urlString = urlString.trim();
+    /**
+     * Map a URL, for example from a deprecated host or non-http scheme, to a stable newer form. Sort of a pre-request
+     * redirect.  This should be used to map to a stable, long-term URL, not to for example time-limited signed URLs.
+     *
+     * @param urlString
+     * @return
+     * @throws MalformedURLException
+     */
+    public static String mapURL(String urlString) throws MalformedURLException {
 
-        if (urlString.startsWith("gs://")) {
+        if (urlString.startsWith("htsget://")) {
+            urlString = urlString.replace("htsget://", "https://");
+        } else if (urlString.startsWith("gs://")) {
             urlString = GoogleUtils.translateGoogleCloudURL(urlString);
         }
-        if (OAuthUtils.isGoogleCloud(urlString)) {
+
+        if (GoogleUtils.isGoogleCloud(urlString)) {
             if (urlString.indexOf("alt=media") < 0) {
-                urlString = urlString + (urlString.indexOf('?') > 0 ? "&" : "?") + "alt=media";
+                urlString = URLUtils.addParameter(urlString, "alt=media");
             }
         }
-
-        String host = new URL(urlString).getHost();
+        String host = URLUtils.getHost(urlString);
         if (host.equals("igv.broadinstitute.org")) {
             urlString = urlString.replace("igv.broadinstitute.org", "s3.amazonaws.com/igv.broadinstitute.org");
         } else if (host.equals("igvdata.broadinstitute.org")) {
-            // Cloudfront server
-            urlString = urlString.replace("igvdata.broadinstitute.org", "dn7ywbm9isq8j.cloudfront.net");
+            urlString = urlString.replace("igvdata.broadinstitute.org", "s3.amazonaws.com/igv.broadinstitute.org");
+        } else if (host.equals("dn7ywbm9isq8j.cloudfront.net")) {
+            urlString = urlString.replace("dn7ywbm9isq8j.cloudfront.net", "s3.amazonaws.com/igv.broadinstitute.org");
         } else if (host.equals("www.broadinstitute.org")) {
             urlString = urlString.replace("www.broadinstitute.org/igvdata", "data.broadinstitute.org/igvdata");
+        } else if (host.equals("www.dropbox.com")) {
+            urlString = urlString.replace("//www.dropbox.com", "//dl.dropboxusercontent.com");
+        } else if (host.equals("drive.google.com")) {
+            urlString = GoogleUtils.driveDownloadURL(urlString);
         }
 
         // data.broadinstitute.org requires https
         urlString = urlString.replace("http://data.broadinstitute.org", "https://data.broadinstitute.org");
 
-
-        return new URL(urlString);
+        return urlString;
     }
 
     public static boolean isRemoteURL(String string) {
@@ -183,19 +212,8 @@ public class HttpUtils {
 
 
     public String getContentsAsString(URL url, Map<String, String> headers) throws IOException {
-
-        InputStream is = null;
-
-        HttpURLConnection conn = openConnection(url, headers);
-        try {
-            is = conn.getInputStream();
-            return readContents(is);
-        } catch (IOException e) {
-            readErrorStream(conn);  // Consume content
-            throw e;
-        } finally {
-            if (is != null) is.close();
-        }
+        byte[] bytes = this.getContentsAsBytes(url, headers);
+        return new String(bytes, "UTF-8");
     }
 
     public String getContentsAsGzippedString(URL url) throws IOException {
@@ -212,6 +230,20 @@ public class HttpUtils {
         }
     }
 
+    public byte[] getContentsAsBytes(URL url, Map<String, String> headers) throws IOException {
+        InputStream is = null;
+        HttpURLConnection conn = openConnection(url, headers);
+        try {
+            is = conn.getInputStream();
+            return is.readAllBytes();
+        } catch (IOException e) {
+            readErrorStream(conn);  // Consume content
+            throw e;
+        } finally {
+            if (is != null) is.close();
+        }
+    }
+
 
     public String getContentsAsJSON(URL url) throws IOException {
 
@@ -243,6 +275,8 @@ public class HttpUtils {
         }
         byte[] postDataBytes = postData.toString().getBytes();
 
+        log.debug("Raw POST request: " + postData.toString());
+
         HttpURLConnection conn = (HttpURLConnection) url.openConnection();
         conn.setRequestMethod("POST");
         conn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
@@ -254,6 +288,9 @@ public class HttpUtils {
         for (int c; (c = in.read()) >= 0; ) {
             response.append((char) c);
         }
+
+        in.close();
+
         return response.toString();
 
     }
@@ -284,6 +321,7 @@ public class HttpUtils {
     public InputStream openConnectionStream(URL url, Map<String, String> requestProperties) throws IOException {
 
         HttpURLConnection conn = openConnection(url, requestProperties);
+
         if (conn == null) {
             return null;
         }
@@ -328,6 +366,13 @@ public class HttpUtils {
                         e1.printStackTrace();
                     }
                 return false;
+            } finally {
+                if (conn != null) {
+                    try {
+                        conn.disconnect();
+                    } catch (Exception e) {
+                    }
+                }
             }
         }
     }
@@ -343,7 +388,10 @@ public class HttpUtils {
     private HttpURLConnection openConnectionHeadOrGet(URL url) throws IOException {
 
         // Keep track of urls for which "HEAD" does not work (e.g. Amazon with signed urls).
-        boolean tryHead = headURLCache.containsKey(url) ? headURLCache.get(url) : true;
+        String urlString = url.toString();
+        boolean isAWS = urlString.contains("AWSAccessKeyId");
+        boolean tryHead =
+                isAWS == false && (headURLCache.containsKey(url) ? headURLCache.get(url) : true);
 
         if (tryHead) {
             try {
@@ -354,7 +402,8 @@ public class HttpUtils {
                 if (e instanceof FileNotFoundException) {
                     throw e;
                 }
-                log.info("HEAD request failed for url: " + url.toExternalForm() + ".  Trying GET");
+                log.debug("HEAD request failed for url: " + url.toExternalForm());
+                log.debug("Trying GET instead for url: " + url.toExternalForm());
                 headURLCache.put(url, false);
             }
         }
@@ -430,13 +479,13 @@ public class HttpUtils {
         String pw = null;
 
         IGVPreferences prefMgr = PreferencesManager.getPreferences();
-        useProxy = prefMgr.getAsBoolean(USE_PROXY);
         proxyHost = prefMgr.get(PROXY_HOST, null);
         try {
             proxyPort = Integer.parseInt(prefMgr.get(PROXY_PORT, "-1"));
         } catch (NumberFormatException e) {
             proxyPort = -1;
         }
+        useProxy = prefMgr.getAsBoolean(USE_PROXY) && proxyHost != null && proxyHost.trim().length() > 0;
         auth = prefMgr.getAsBoolean(PROXY_AUTHENTICATE);
         user = prefMgr.get(PROXY_USER, null);
         String pwString = prefMgr.get(PROXY_PW, null);
@@ -534,76 +583,6 @@ public class HttpUtils {
     }
 
 
-    public void uploadGenomeSpaceFile(String uri, File file, Map<String, String> headers) throws IOException {
-
-        HttpURLConnection urlconnection = null;
-        OutputStream bos = null;
-
-        URL url = HttpUtils.createURL(uri);
-        urlconnection = openConnection(url, headers, "PUT");
-        urlconnection.setDoOutput(true);
-        urlconnection.setDoInput(true);
-
-        bos = new BufferedOutputStream(urlconnection.getOutputStream());
-        BufferedInputStream bis = new BufferedInputStream(new FileInputStream(file));
-        int i;
-        // read byte by byte until end of stream
-        while ((i = bis.read()) > 0) {
-            bos.write(i);
-        }
-        bos.close();
-        int responseCode = urlconnection.getResponseCode();
-
-        // Error messages below.
-        if (responseCode >= 400) {
-            String message = readErrorStream(urlconnection);
-            throw new IOException("Error uploading " + file.getName() + " : " + message);
-        }
-    }
-
-
-    public String createGenomeSpaceDirectory(URL url, String body) throws IOException {
-
-        HttpURLConnection urlconnection = null;
-        OutputStream bos = null;
-
-        Map<String, String> headers = new HashMap<String, String>();
-        headers.put("Content-Type", "application/json");
-        headers.put("Content-Length", String.valueOf(body.getBytes().length));
-
-        urlconnection = openConnection(url, headers, "PUT");
-        urlconnection.setDoOutput(true);
-        urlconnection.setDoInput(true);
-
-        bos = new BufferedOutputStream(urlconnection.getOutputStream());
-        bos.write(body.getBytes());
-        bos.close();
-        int responseCode = urlconnection.getResponseCode();
-
-        // Error messages below.
-        StringBuffer buf = new StringBuffer();
-        InputStream inputStream;
-
-        if (responseCode >= 200 && responseCode < 300) {
-            inputStream = urlconnection.getInputStream();
-        } else {
-            inputStream = urlconnection.getErrorStream();
-        }
-        BufferedReader br = new BufferedReader(new InputStreamReader(inputStream));
-        String nextLine;
-        while ((nextLine = br.readLine()) != null) {
-            buf.append(nextLine);
-            buf.append('\n');
-        }
-        inputStream.close();
-
-        if (responseCode >= 200 && responseCode < 300) {
-            return buf.toString();
-        } else {
-            throw new IOException("Error creating GS directory: " + buf.toString());
-        }
-    }
-
     /**
      * Code for disabling SSL certification
      */
@@ -638,13 +617,7 @@ public class HttpUtils {
     }
 
     private String readContents(InputStream is) throws IOException {
-        BufferedInputStream bis = new BufferedInputStream(is);
-        ByteArrayOutputStream bos = new ByteArrayOutputStream();
-        int b;
-        while ((b = bis.read()) >= 0) {
-            bos.write(b);
-        }
-        return new String(bos.toByteArray());
+        return ParsingUtils.readContentsFromStream(is);
     }
 
     public String readErrorStream(HttpURLConnection connection) throws IOException {
@@ -672,7 +645,7 @@ public class HttpUtils {
     }
 
     private HttpURLConnection openConnection(URL url, Map<String, String> requestProperties, String method) throws IOException {
-        return openConnection(url, requestProperties, method, 0);
+        return openConnection(url, requestProperties, method, 0, 0);
     }
 
     /**
@@ -685,15 +658,37 @@ public class HttpUtils {
      * @throws java.io.IOException
      */
     private HttpURLConnection openConnection(
+            URL url, Map<String, String> requestProperties, String method, int redirectCount, int retries) throws IOException {
 
-            URL url, Map<String, String> requestProperties, String method, int redirectCount) throws IOException {
+        // if we're already seen a redirect for this URL, use the updated one
+        if (redirectCache.containsKey(url)) {
+            CachedRedirect cr = redirectCache.get(url);
+            if (ZonedDateTime.now().compareTo(cr.expires) < 0.0) {
+                // now() is before our expiration
+                log.debug("Found URL in redirection cache: " + url + " ->" + redirectCache.get(url).url);
+                url = cr.url;
+            } else {
+                log.debug("Removing expired URL from redirection cache: " + url);
+                redirectCache.remove(url);
+            }
+        }
 
         // if the url points to a openid location instead of a oauth2.0 location, used the fina and replace
         // string to dynamically map url - dwm08
-        if (url.getHost().equals(OAuthUtils.GS_HOST) && OAuthUtils.findString != null && OAuthUtils.replaceString != null) {
+        if (url.getHost().equals(GoogleUtils.GOOGLE_API_HOST) && OAuthUtils.findString != null && OAuthUtils.replaceString != null) {
             url = HttpUtils.createURL(url.toExternalForm().replaceFirst(OAuthUtils.findString, OAuthUtils.replaceString));
         }
 
+        // If a presigned URL, check its validity and update if needed
+        if (AmazonUtils.isPresignedURL(url.toExternalForm())) {
+            url = new URL(AmazonUtils.updatePresignedURL(url.toExternalForm()));
+        }
+
+        // If an S3 url, obtain a signed https url
+        if (AmazonUtils.isAwsS3Path(url.toExternalForm())) {
+            url = new URL(AmazonUtils.translateAmazonCloudURL(url.toExternalForm()));
+        }
+
         //Encode query string portions
         url = StringUtils.encodeURLQueryString(url);
         if (log.isTraceEnabled()) {
@@ -701,98 +696,86 @@ public class HttpUtils {
         }
 
         //Encode base portions. Right now just spaces, most common case
-        //TODO This is a hack and doesn't work for all characters which need it
         if (StringUtils.countChar(url.toExternalForm(), ' ') > 0) {
             String newPath = url.toExternalForm().replaceAll(" ", "%20");
             url = HttpUtils.createURL(newPath);
         }
 
-
-        Proxy sysProxy = null;
-
-        boolean igvProxySettingsExist = proxySettings != null && proxySettings.useProxy;
-        boolean checkSystemProxy =
-                !PreferencesManager.getPreferences().getAsBoolean("PROXY.DISABLE_CHECK") && !igvProxySettingsExist;
-
-        //Only check for system proxy if igv proxy settings not found
-        if (checkSystemProxy) {
-            sysProxy = getSystemProxy(url.toExternalForm());
+        // If this is a Google URL and we have set a userProject ("requestor pays') use it.
+        if (GoogleUtils.isGoogleURL(url.toExternalForm()) &&
+                GoogleUtils.getProjectID() != null &&
+                GoogleUtils.getProjectID().length() > 0 &&
+                !hasQueryParameter(url, "userProject")) {
+            url = addQueryParameter(url, "userProject", GoogleUtils.getProjectID());
         }
 
+        HttpURLConnection conn = null;
+        if (proxySettings != null && proxySettings.isProxyDefined()) {
 
-        boolean useProxy =
-                (sysProxy != null && sysProxy.type() != Proxy.Type.DIRECT) ||
-                        (igvProxySettingsExist && !proxySettings.getWhitelist().contains(url.getHost()));
+            // NOTE: setting disabledSchemes to "" through System.setProperty does not work !!!
+             System.setProperty("jdk.http.auth.tunneling.disabledSchemes", "");
+             System.setProperty("jdk.http.auth.proxying.disabledSchemes", "");
 
-        HttpURLConnection conn;
-        if (useProxy) {
-            Proxy proxy = sysProxy;
-            if (igvProxySettingsExist) {
-                if (proxySettings.type == Proxy.Type.DIRECT) {
-
-                    if (PreferencesManager.getPreferences().getAsBoolean("DEBUG.PROXY")) {
-                        log.info("NO_PROXY");
-                    }
-
-                    proxy = Proxy.NO_PROXY;
-                } else {
-                    if (PreferencesManager.getPreferences().getAsBoolean("DEBUG.PROXY")) {
-                        log.info("PROXY " + proxySettings.proxyHost + "  " + proxySettings.proxyPort);
-                    }
-
-                    proxy = new Proxy(proxySettings.type, new InetSocketAddress(proxySettings.proxyHost, proxySettings.proxyPort));
+//            if (url.getProtocol().equals("https") && proxySettings.isUserPwDefined()) {
+//                conn = new ProxiedHttpsConnection(url, proxySettings.proxyHost, proxySettings.proxyPort,
+//                        proxySettings.user, proxySettings.pw);
+//            } else {
+                Proxy proxy = new Proxy(proxySettings.type, new InetSocketAddress(proxySettings.proxyHost, proxySettings.proxyPort));
+                conn = (HttpURLConnection) url.openConnection(proxy);
+                if (proxySettings.isUserPwDefined()) {
+                    byte[] bytes = (proxySettings.user + ":" + proxySettings.pw).getBytes();
+                    String encodedUserPwd = String.valueOf(Base64Coder.encode(bytes));
+                    conn.setRequestProperty("Proxy-Authorization", "Basic " + encodedUserPwd);
                 }
+//            }
+        }
+        if (conn == null && !PreferencesManager.getPreferences().getAsBoolean("PROXY.DISABLE_CHECK")) {
+            Proxy sysProxy = getSystemProxy(url.toExternalForm());
+            if (sysProxy != null && sysProxy.type() != Proxy.Type.DIRECT) {
+                conn = (HttpURLConnection) url.openConnection(sysProxy);
             }
-            conn = (HttpURLConnection) url.openConnection(proxy);
+        }
 
-            if (igvProxySettingsExist && proxySettings.auth && proxySettings.user != null && proxySettings.pw != null) {
-                byte[] bytes = (proxySettings.user + ":" + proxySettings.pw).getBytes();
-
-                String encodedUserPwd = String.valueOf(Base64Coder.encode(bytes));
-                conn.setRequestProperty("Proxy-Authorization", "Basic " + encodedUserPwd);
-            }
-        } else {
-            if (PreferencesManager.getPreferences().getAsBoolean("DEBUG.PROXY")) {
-                log.info("PROXY NOT USED ");
-                if (proxySettings.getWhitelist().contains(url.getHost())) {
-                    log.info(url.getHost() + " is whitelisted");
-                }
-                ;
-            }
+        // If connection is still null no proxy is used
+        if (conn == null) {
             conn = (HttpURLConnection) url.openConnection();
         }
 
-        if (GSUtils.isGenomeSpace(url)) {
-            conn.setRequestProperty("Accept", "application/json,text/plain");
-        } else {
-            if (!"HEAD".equals(method))
-                conn.setRequestProperty("Accept", "text/plain");
+        if (!"HEAD".equals(method)) {
+            conn.setRequestProperty("Accept", "text/plain");
         }
 
-        //There seems to be a bug with JWS caches, so we avoid caching
-        //This default is persistent, really should be available statically but isn't
-        conn.setDefaultUseCaches(false);
-        conn.setUseCaches(false);
-
-
         conn.setConnectTimeout(Globals.CONNECT_TIMEOUT);
         conn.setReadTimeout(Globals.READ_TIMEOUT);
         conn.setRequestMethod(method);
         conn.setRequestProperty("Connection", "Keep-Alive");
+        // we'll handle redirects manually, allowing us to cache the new URL
+        conn.setInstanceFollowRedirects(false);
+
         if (requestProperties != null) {
             for (Map.Entry<String, String> prop : requestProperties.entrySet()) {
                 conn.setRequestProperty(prop.getKey(), prop.getValue());
             }
         }
-        conn.setRequestProperty("User-Agent", Globals.applicationString());
 
-        if (url.getHost().equals(OAuthUtils.GS_HOST) || url.getHost().startsWith("igvweb02")) {
-            String token = OAuthUtils.getInstance().getAccessToken();
-            if (token != null) conn.setRequestProperty("Authorization", "Bearer " + token);
+        Collection<String> headers = headerMap.get(url.getHost());
+        if (headers != null) {
+            for (String h : headers) {
+                String[] kv = h.split(":");
+                if (kv.length == 2) {
+                    conn.setRequestProperty(kv[0], kv[1]);
+                }
+            }
         }
 
-        if(url.getHost().equals(GoogleUtils.GOOGLE_API_HOST) && GoogleUtils.getProjectID() != null && GoogleUtils.getProjectID().length() > 0) {
-            url = addQueryParameter(url, "userProject", GoogleUtils.getProjectID());
+        conn.setRequestProperty("User-Agent", Globals.applicationString());
+
+        // If this is a Google URL and we have an access token use it.
+        if (GoogleUtils.isGoogleURL(url.toExternalForm())) {
+            String token = OAuthUtils.getInstance().getProvider().getAccessToken();
+            if (token != null) {
+                conn.setRequestProperty("Authorization", "Bearer " + token);
+            }
         }
 
         if (method.equals("PUT")) {
@@ -801,7 +784,8 @@ public class HttpUtils {
 
             int code = conn.getResponseCode();
 
-            if (requestProperties != null && requestProperties.containsKey("Range") && code == 200 && method.equals("GET")) {
+            if (!isDropboxHost(url.getHost()) && requestProperties != null && requestProperties.containsKey("Range") && code == 200 && method.equals("GET")) {
+
                 log.error("Range header removed by client or ignored by server for url: " + url.toString());
 
                 if (!SwingUtilities.isEventDispatchThread()) {
@@ -813,7 +797,7 @@ public class HttpUtils {
                 int length = Integer.parseInt(positionString[1]) - Integer.parseInt(positionString[0]) + 1;
                 requestProperties.remove("Range"); // < VERY IMPORTANT
                 URL wsUrl = HttpUtils.createURL(WEBSERVICE_URL + "?file=" + url.toExternalForm() + "&position=" + positionString[0] + "&length=" + length);
-                return openConnection(wsUrl, requestProperties, "GET", redirectCount);
+                return openConnection(wsUrl, requestProperties, "GET", redirectCount, retries);
             }
 
             if (log.isDebugEnabled()) {
@@ -823,15 +807,46 @@ public class HttpUtils {
             // Redirects.  These can occur even if followRedirects == true if there is a change in protocol,
             // for example http -> https.
             if (code >= 300 && code < 400) {
-
                 if (redirectCount > MAX_REDIRECTS) {
                     throw new IOException("Too many redirects");
                 }
 
-                String newLocation = conn.getHeaderField("Location");
-                log.debug("Redirecting to " + newLocation);
+                CachedRedirect cr = new CachedRedirect();
+                cr.url = new URL(conn.getHeaderField("Location"));
+                if (cr.url != null) {
+                    cr.expires = ZonedDateTime.now().plusMinutes(DEFAULT_REDIRECT_EXPIRATION_MIN);
+                    String s;
+                    if ((s = conn.getHeaderField("Cache-Control")) != null) {
 
-                return openConnection(HttpUtils.createURL(newLocation), requestProperties, method, ++redirectCount);
+                        // cache-control takes priority
+                        CacheControl cc = null;
+                        try {
+                            cc = CacheControl.valueOf(s);
+                        } catch (IllegalArgumentException e) {
+                            // use default
+                        }
+                        if (cc != null) {
+                            if (cc.isNoCache()) {
+                                // set expires to null, preventing caching
+                                cr.expires = null;
+                            } else if (cc.getMaxAge() > 0) {
+                                cr.expires = ZonedDateTime.now().plusSeconds(cc.getMaxAge());
+                            }
+                        }
+                    } else if ((s = conn.getHeaderField("Expires")) != null) {
+                        // no cache-control header, so try "expires" next
+                        try {
+                            cr.expires = ZonedDateTime.parse(s);
+                        } catch (DateTimeParseException e) {
+                            // use default
+                        }
+                    }
+                    if (cr.expires != null) {
+                        redirectCache.put(url, cr);
+                        log.debug("Redirecting to " + cr.url);
+                        return openConnection(HttpUtils.createURL(cr.url.toString()), requestProperties, method, ++redirectCount, retries);
+                    }
+                }
             }
 
             // TODO -- handle other response codes.
@@ -845,6 +860,10 @@ public class HttpUtils {
                     message = "File not found: " + url.toString();
                     throw new FileNotFoundException(message);
                 } else if (code == 401) {
+                    if (GoogleUtils.isGoogleURL(url.toExternalForm()) && retries == 0) {
+                        GoogleUtils.checkLogin();
+                        return openConnection(url, requestProperties, method, redirectCount, ++retries);
+                    }
                     message = "You must log in to access this file";
                     throw new HttpResponseException(code, message, "");
                 } else if (code == 403) {
@@ -867,9 +886,13 @@ public class HttpUtils {
         return conn;
     }
 
-    private URL addQueryParameter(URL url, String userProject, String projectID) {
+    private boolean isDropboxHost(String host) {
+        return (host.equals("dl.dropboxusercontent.com") || host.equals("www.dropbox.com"));
+    }
+
+    private URL addQueryParameter(URL url, String param, String value) {
         String urlString = url.toExternalForm();
-        urlString = urlString + (urlString.contains("?") ? "&" : "?") + userProject + "=" + projectID;
+        urlString = urlString + (urlString.contains("?") ? "&" : "?") + param + "=" + value;
         try {
             return new URL(urlString);
         } catch (MalformedURLException e) {
@@ -878,6 +901,16 @@ public class HttpUtils {
         }
     }
 
+    private boolean hasQueryParameter(URL url, String parameter) {
+        String urlstring = url.toExternalForm();
+        if (urlstring.contains("?")) {
+            int idx = urlstring.indexOf('?');
+            return urlstring.substring(idx).contains(parameter + "=");
+        } else {
+            return false;
+        }
+    }
+
 
     //Used for testing sometimes, please do not delete
     private void logHeaders(HttpURLConnection conn) {
@@ -917,35 +950,11 @@ public class HttpUtils {
         // We can test byte-range success for hosts we can reach.
         synchronized (byteRangeTestMap) {
             final String host = url.getHost();
-
             if (byteRangeTestMap.containsKey(host)) {
                 return byteRangeTestMap.get(host);
             } else {
-                SeekableStream str = null;
-                try {
-
-                    boolean byteRangeTestSuccess = testByteRange(url);
-
-                    if (byteRangeTestSuccess) {
-                        log.info("Range-byte request succeeded");
-                    } else {
-                        log.info("Range-byte test failed -- Host: " + host +
-                                " does not support range-byte requests or there is a problem with client network environment.");
-                    }
-
-                    byteRangeTestMap.put(host, byteRangeTestSuccess);
-                    return byteRangeTestSuccess;
-
-
-                } finally {
-                    if (str != null) try {
-                        str.close();
-                    } catch (IOException e) {
-                        log.error("Error closing stream (" + url.toExternalForm() + ")", e);
-                    }
-                }
+                return true;  // Let's be optimistic
             }
-
         }
     }
 
@@ -960,31 +969,40 @@ public class HttpUtils {
         return byteRangeTestSuccess;
     }
 
-    public void shutdown() {
-        // Do any cleanup required here
-    }
-
     /**
-     * Checks if the string is a URL (not necessarily remote, can be any protocol)
+     * Add an http header string to be applied the the specified URLs.  Used to support command line specification
+     * of authentication headers
      *
-     * @param f
-     * @return
+     * @param headers
+     * @param urls
      */
-    public static boolean isURL(String f) {
-        return f.startsWith("http://") || f.startsWith("ftp://") || f.startsWith("https://") || f.startsWith("gs://")
-                || URLmatcher.matcher(f).matches();
-    }
+    public void addHeaders(Collection<String> headers, List<String> urls) {
+        for (String u : urls) {
+            if (isRemoteURL(u)) {
+                try {
+                    URL url = new URL(mapURL(u));
+                    headerMap.put(url.getHost(), headers);
+                    System.out.println("Added " + url.getHost() + " -> " + headers);
 
-    public static Map<String, String> parseQueryString(String query) {
-        String[] params = query.split("&");
-        Map<String, String> map = new HashMap<String, String>();
-        for (String param : params) {
-            String[] name_val = param.split("=", 2);
-            if (name_val.length == 2) {
-                map.put(name_val[0], name_val[1]);
+                } catch (MalformedURLException e) {
+                    log.error("Error parsing URL " + u, e);
+                }
             }
         }
-        return map;
+    }
+
+
+    private String stripParameters(String url) {
+        int idx = url.indexOf("?");
+        if (idx > 0) {
+            return url.substring(0, idx);
+        } else {
+            return url;
+        }
+    }
+
+    public void shutdown() {
+        // Do any cleanup required here
     }
 
     public static class ProxySettings {
@@ -1010,6 +1028,14 @@ public class HttpUtils {
             this.whitelist = whitelist;
         }
 
+        public boolean isProxyDefined() {
+            return useProxy && proxyHost != null && proxyPort > 0;
+        }
+
+        public boolean isUserPwDefined() {
+            return this.auth && this.user != null && this.pw != null;
+        }
+
         public Set<String> getWhitelist() {
             return whitelist;
         }
@@ -1060,15 +1086,9 @@ public class HttpUtils {
                 return new PasswordAuthentication(defaultUserName, defaultPassword);
             }
 
-            Frame owner = IGV.hasInstance() ? IGV.getMainFrame() : null;
+            Frame owner = IGV.hasInstance() ? IGV.getInstance().getMainFrame() : null;
 
-            boolean isGenomeSpace = GSUtils.isGenomeSpace(getRequestingURL());
-            if (isGenomeSpace) {
-                // If we are being challenged by GS the token must be bad/expired
-                GSUtils.logout();
-            }
-
-            LoginDialog dlg = new LoginDialog(owner, isGenomeSpace, urlString, isProxyChallenge);
+            LoginDialog dlg = new LoginDialog(owner, urlString, isProxyChallenge);
             dlg.setVisible(true);
             if (dlg.isCanceled()) {
                 return null;
@@ -1133,86 +1153,6 @@ public class HttpUtils {
         }
     }
 
-
-    /**
-     * Extension of CookieManager that grabs cookies from the GenomeSpace identity server to store locally.
-     * This is to support the GenomeSpace "single sign-on". Examples ...
-     * gs-username=igvtest; Domain=.genomespace.org; Expires=Mon, 21-Jul-2031 03:27:23 GMT; Path=/
-     * gs-token=HnR9rBShNO4dTXk8cKXVJT98Oe0jWVY+; Domain=.genomespace.org; Expires=Mon, 21-Jul-2031 03:27:23 GMT; Path=/
-     */
-
-    static class IGVCookieManager extends CookieHandler {
-
-
-        CookieManager wrappedManager;
-
-        public IGVCookieManager() {
-            wrappedManager = new CookieManager();
-        }
-
-        @Override
-        public Map<String, List<String>> get(URI uri, Map<String, List<String>> requestHeaders) throws IOException {
-
-            Map<String, List<String>> headers = new HashMap<String, List<String>>();
-            headers.putAll(wrappedManager.get(uri, requestHeaders));
-
-            if (GSUtils.isGenomeSpace(uri.toURL())) {
-                String token = GSUtils.getGSToken();
-                if (token != null) {
-                    List<String> cookieList = headers.get("Cookie");
-                    boolean needsTokenCookie = true;
-                    boolean needsToolCookie = true;
-                    if (cookieList == null) {
-                        cookieList = new ArrayList<String>(1);
-                        headers.put("Cookie", cookieList);
-                    }
-
-                    for (String cookie : cookieList) {
-                        if (cookie.startsWith("gs-token")) {
-                            needsTokenCookie = false;
-                        } else if (cookie.startsWith("gs-toolname")) {
-                            needsToolCookie = false;
-                        }
-                    }
-                    if (needsTokenCookie) {
-                        cookieList.add("gs-token=" + token);
-                    }
-                    if (needsToolCookie) {
-                        cookieList.add("gs-toolname=IGV");
-                    }
-                }
-            }
-
-            return Collections.unmodifiableMap(headers);
-        }
-
-        @Override
-        public void put(URI uri, Map<String, List<String>> responseHeaders) throws IOException {
-            String urilc = uri.toString().toLowerCase();
-            if (urilc.contains("identity") && urilc.contains("genomespace")) {
-                List<String> cookies = responseHeaders.get("Set-Cookie");
-                if (cookies != null) {
-                    for (String cstring : cookies) {
-                        List<HttpCookie> cookieList = HttpCookie.parse(cstring);
-                        for (HttpCookie cookie : cookieList) {
-                            String cookieName = cookie.getName();
-                            String value = cookie.getValue();
-                            if (cookieName.equals("gs-token")) {
-                                //log.debug("gs-token: " + value);
-                                GSUtils.setGSToken(value);
-                            } else if (cookieName.equals("gs-username")) {
-                                //log.debug("gs-username: " + value);
-                                GSUtils.setGSUser(value);
-                            }
-                        }
-                    }
-                }
-            }
-            wrappedManager.put(uri, responseHeaders);
-        }
-    }
-
-
     public class UnsatisfiableRangeException extends RuntimeException {
 
         String message;
@@ -1222,4 +1162,36 @@ public class HttpUtils {
             this.message = message;
         }
     }
+
+    static class CacheControl {
+
+        boolean noCache = false;
+        long maxAge = 0;
+
+        static CacheControl valueOf(String s) {
+            CacheControl cc = new CacheControl();
+            String[] tokens = Globals.commaPattern.split(s);
+            for (String t : tokens) {
+                t = t.trim().toLowerCase();
+                if (t.startsWith("no-cache")) {
+                    cc.noCache = true;
+                } else if (t.startsWith("max-age")) {
+                    String[] ma = Globals.equalPattern.split(t);
+                    cc.maxAge = Long.parseLong(ma[1].trim());
+                }
+            }
+            return cc;
+        }
+
+        private CacheControl() {
+        }
+
+        public boolean isNoCache() {
+            return noCache;
+        }
+
+        public long getMaxAge() {
+            return maxAge;
+        }
+    }
 }
