@@ -27,9 +27,13 @@ package org.broad.igv.util;
 
 import htsjdk.samtools.util.ftp.FTPClient;
 import htsjdk.samtools.util.ftp.FTPReply;
+import htsjdk.tribble.Feature;
 import htsjdk.tribble.readers.AsciiLineReader;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.Globals;
+import org.broad.igv.feature.AbstractFeatureParser;
+import org.broad.igv.feature.FeatureParser;
+import org.broad.igv.feature.genome.Genome;
 import org.broad.igv.renderer.*;
 import org.broad.igv.track.Track;
 import org.broad.igv.track.TrackProperties;
@@ -37,12 +41,15 @@ import org.broad.igv.track.WindowFunction;
 import org.broad.igv.ui.color.ColorUtilities;
 import org.broad.igv.ui.util.MessageUtils;
 import org.broad.igv.util.ftp.FTPUtils;
+import software.amazon.awssdk.utils.StringInputStream;
 
 import java.awt.*;
 import java.io.*;
 import java.net.MalformedURLException;
 import java.net.URL;
 import java.net.URLConnection;
+import java.net.URLDecoder;
+import java.nio.charset.StandardCharsets;
 import java.text.DateFormat;
 import java.text.ParseException;
 import java.text.SimpleDateFormat;
@@ -58,7 +65,7 @@ import java.util.zip.GZIPInputStream;
  */
 public class ParsingUtils {
 
-    private static Logger log = Logger.getLogger(ParsingUtils.class);
+    private static Logger log = LogManager.getLogger(ParsingUtils.class);
     public static final Pattern TAB_PATTERN = Pattern.compile("\t");
     public static final Pattern COMMA_PATTERN = Pattern.compile(",");
     public static final Pattern SEMI_COLON_PATTERN = Pattern.compile(";");
@@ -68,7 +75,8 @@ public class ParsingUtils {
 
 
     /**
-     * Open a BufferedReader on the path, which might be a local file or URL, and might be gzipped or not.
+     * Open a BufferedReader on the path, which might be
+     * a local file or URL, and might be gzipped or not.
      *
      * @param pathOrUrl
      * @return
@@ -85,7 +93,6 @@ public class ParsingUtils {
 
     }
 
-
     public static AsciiLineReader openAsciiReader(ResourceLocator locator) throws IOException {
         InputStream stream = openInputStreamGZ(locator);
         return new AsciiLineReader(stream);
@@ -106,7 +113,17 @@ public class ParsingUtils {
     public static InputStream openInputStreamGZ(ResourceLocator locator) throws IOException {
 
         InputStream inputStream = null;
-        if (HttpUtils.isRemoteURL(locator.getPath())) {
+
+        if (isDataURL(locator.getPath())) {
+            String dataURL = locator.getPath();
+            int commaIndex = dataURL.indexOf(',');
+            if (commaIndex < 0) {
+                throw new Error("dataURL missing commas");
+            }
+            // TODO -- check optional media type - only text/plain supported
+            String contents = URLDecoder.decode(dataURL.substring(commaIndex + 1), StandardCharsets.UTF_8);
+            return new ByteArrayInputStream(contents.getBytes(StandardCharsets.UTF_8));
+        } else if (HttpUtils.isRemoteURL(locator.getPath())) {
             URL url = HttpUtils.createURL(locator.getPath());
             inputStream = HttpUtils.getInstance().openConnectionStream(url);
         } else {
@@ -125,6 +142,11 @@ public class ParsingUtils {
         }
     }
 
+    public static String readContentsFromStream(InputStream is) throws IOException {
+        byte[] bytes = is.readAllBytes();
+        return new String(bytes, "UTF-8");
+    }
+
     /**
      * Parse the string and return the result as an integer.  This method supports scientific notation for integers,
      * which Integer.parseInt() does not.
@@ -453,7 +475,7 @@ public class ParsingUtils {
                                     float max = Float.parseFloat(limits[1].trim());
                                     trackProperties.setMinValue(min);
                                     trackProperties.setMaxValue(max);
-                                } else if(limits.length == 3) {
+                                } else if (limits.length == 3) {
                                     float min = Float.parseFloat(limits[0].trim());
                                     float base = Float.parseFloat(limits[1].trim());
                                     float max = Float.parseFloat(limits[2].trim());
@@ -574,4 +596,8 @@ public class ParsingUtils {
         int idx = path.lastIndexOf('.');
         return idx < 0 ? path : path.substring(idx + 1, path.length());
     }
+
+    public static boolean isDataURL(String url) {
+        return url != null && url.startsWith("data:") && !((new File(url)).exists());
+    }
 }
