@@ -28,6 +28,7 @@ package org.broad.igv.util.stream;
 import htsjdk.samtools.seekablestream.ISeekableStreamFactory;
 import htsjdk.samtools.seekablestream.SeekableFileStream;
 import htsjdk.samtools.seekablestream.SeekableStream;
+import org.broad.igv.util.FileUtils;
 import org.broad.igv.util.HttpUtils;
 
 import java.io.File;
@@ -58,17 +59,10 @@ public class IGVSeekableStreamFactory implements ISeekableStreamFactory {
 
         if (path.endsWith(".list")) {
             return new SeekableSplitStream(path);
-
         } else {
+            path = mapPath(path);
             SeekableStream is = null;
-
-//            if(path.startsWith("gs://")) {
-//                path = GoogleUtils.translateGoogleCloudURL(path);
-//            }
-
-            if (path.toLowerCase().startsWith("http://") || path.toLowerCase().startsWith("https://") ||
-                    path.toLowerCase().startsWith("gs://")) {
-
+            if (FileUtils.isRemote(path) ) {
                 final URL url = HttpUtils.createURL(path);
                 boolean useByteRange = HttpUtils.getInstance().useByteRange(url);
                 if (useByteRange) {
@@ -94,4 +88,12 @@ public class IGVSeekableStreamFactory implements ISeekableStreamFactory {
         return new IGVSeekableBufferedStream(stream, bufferSize);
     }
 
+    private String mapPath(String path) {
+        if(path.startsWith("ftp://ftp.ncbi.nlm.nih.gov/geo")) {
+            return path.replace("ftp://ftp.ncbi.nlm.nih.gov/geo", "https://ftp.ncbi.nlm.nih.gov/geo");
+        } else {
+            return path;
+        }
+    }
+
 }
