@@ -1,7 +1,7 @@
 package org.broad.igv.util.stream;
 
 import htsjdk.tribble.util.URLHelper;
-import org.apache.log4j.Logger;
+import org.broad.igv.logging.*;
 import org.broad.igv.util.HttpUtils;
 
 import java.io.IOException;
@@ -13,7 +13,7 @@ import java.util.Map;
 
 public class IGVUrlHelper implements URLHelper {
 
-    static Logger log = Logger.getLogger(IGVUrlHelperFactory.class);
+    static Logger log = LogManager.getLogger(IGVUrlHelperFactory.class);
 
     URL url;
 
@@ -81,9 +81,9 @@ public class IGVUrlHelper implements URLHelper {
     }
 
     public boolean exists() {
-        log.info("Checking resoure " + url.toExternalForm());
+        //log.warn("Checking resoure " + url.toExternalForm());
         boolean exists = HttpUtils.getInstance().resourceAvailable(url.toExternalForm());
-        log.info("Exists: " + exists);
+        //log.warn("Exists: " + exists);
         return exists;
     }
 }
