@@ -30,22 +30,24 @@ package org.broad.igv.variant;
 
 import htsjdk.tribble.Feature;
 import htsjdk.variant.variantcontext.GenotypeType;
-import org.apache.log4j.Logger;
-import org.broad.igv.feature.FeatureUtils;
-import org.broad.igv.feature.IGVFeature;
-import org.broad.igv.prefs.IGVPreferences;
-import org.broad.igv.prefs.PreferencesManager;
-import org.broad.igv.renderer.GraphicUtils;
-
-import org.broad.igv.track.*;
-import org.broad.igv.ui.FontManager;
-import org.broad.igv.ui.IGV;
+import org.broad.igv.Globals;
 import org.broad.igv.event.IGVEventBus;
 import org.broad.igv.event.IGVEventObserver;
 import org.broad.igv.event.TrackGroupEvent;
+import org.broad.igv.feature.FeatureUtils;
+import org.broad.igv.jbrowse.CircularViewUtilities;
+import org.broad.igv.logging.LogManager;
+import org.broad.igv.logging.Logger;
+import org.broad.igv.prefs.IGVPreferences;
+import org.broad.igv.prefs.PreferencesManager;
+import org.broad.igv.renderer.GraphicUtils;
+import org.broad.igv.track.*;
+import org.broad.igv.ui.FontManager;
+import org.broad.igv.ui.IGV;
 import org.broad.igv.ui.panel.*;
 import org.broad.igv.ui.util.MessageUtils;
 import org.broad.igv.util.*;
+import org.broad.igv.variant.vcf.MateVariant;
 import org.w3c.dom.Document;
 import org.w3c.dom.Element;
 
@@ -55,8 +57,9 @@ import java.io.BufferedReader;
 import java.io.File;
 import java.io.IOException;
 import java.text.DecimalFormat;
-import java.util.*;
 import java.util.List;
+import java.util.*;
+import java.util.stream.Collectors;
 
 import static org.broad.igv.prefs.Constants.*;
 
@@ -66,11 +69,12 @@ import static org.broad.igv.prefs.Constants.*;
 
 public class VariantTrack extends FeatureTrack implements IGVEventObserver {
 
-    private static Logger log = Logger.getLogger(VariantTrack.class);
+
+    private static Logger log = LogManager.getLogger(VariantTrack.class);
 
     static final DecimalFormat numFormat = new DecimalFormat("#.###");
 
-    private static final Color GREY_170 = new Color(170, 170, 170);
+    private static final Color CIRC_VIEW_DEFAULT_COLOR = new Color(27,192,249);
     private static final int GROUP_BORDER_WIDTH = 3;
     private static final Color BAND1_COLOR = new Color(245, 245, 245);
     private static final Color BAND2_COLOR = Color.white;
@@ -78,20 +82,21 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
     private static final Color borderGray = new Color(200, 200, 200);
 
     private final static int DEFAULT_EXPANDED_GENOTYPE_HEIGHT = 15;
-    private final int DEFAULT_SQUISHED_GENOTYPE_HEIGHT = 4;
-    private final static int DEFAULT_VARIANT_BAND_HEIGHT = 25;
+    private final int DEFAULT_SQUISHED_HEIGHT = 4;
+    private final static int DEFAULT_EXPANDED_VARIANT_HEIGHT = 25;
+    private final static int DEFAULT_SQUISHED_VARIANT_HEIGHT = 6;
     private final static int MAX_FILTER_LINES = 15;
+    private final static int WG_TRACK_HEIGHT = 40;
 
 
     // TODO -- this needs to be settable
     public static int METHYLATION_MIN_BASE_COUNT = 10;
 
-
-    public static boolean isVCF(String typeString) {
-        return (typeString.endsWith(".vcf3") ||
-                typeString.endsWith(".vcf4") ||
-                typeString.endsWith(".vcf") ||
-                typeString.endsWith(".bcf"));
+    public static boolean isVCF(String format) {
+        return (format.equals("vcf3") ||
+                format.equals("vcf4") ||
+                format.equals("vcf") ||
+                format.equals("bcf"));
     }
 
 
@@ -108,15 +113,12 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
      */
     private int top;
 
+    private boolean showGenotypes = true;
+
     /**
      * The height of a single row in in squished mode
      */
-    private int squishedHeight = DEFAULT_SQUISHED_GENOTYPE_HEIGHT;
-
-    /**
-     * The height of the top band representing the variant call
-     */
-    private int variantBandHeight = DEFAULT_VARIANT_BAND_HEIGHT;
+    private int squishedHeight = DEFAULT_SQUISHED_HEIGHT;
 
     /**
      * List of all samples, in the order they appear in the file.
@@ -143,7 +145,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
     /**
      * Current coloring option
      */
-    private ColorMode coloring = ColorMode.GENOTYPE;
+    private ColorMode genotypeColorMode = ColorMode.GENOTYPE;
 
 
     private ColorMode siteColorMode;
@@ -202,13 +204,12 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
 
         String path = locator == null ? null : locator.getPath();
 
-
         this.renderer = new VariantRenderer(this);
 
         this.enableMethylationRateSupport = enableMethylationRateSupport;
         if (enableMethylationRateSupport) {
             // also set the default color mode to Methylation rate:
-            coloring = ColorMode.METHYLATION_RATE;
+            genotypeColorMode = ColorMode.METHYLATION_RATE;
         }
 
         this.siteColorMode = prefMgr.getAsBoolean(VARIANT_COLOR_BY_ALLELE_FREQ) ?
@@ -225,8 +226,11 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
         int sampleCount = allSamples.size();
         final int groupCount = samplesByGroups.size();
         final int margins = (groupCount - 1) * 3;
-        squishedHeight = sampleCount == 0 ? DEFAULT_SQUISHED_GENOTYPE_HEIGHT :
-                Math.min(DEFAULT_SQUISHED_GENOTYPE_HEIGHT, Math.max(1, (height - variantBandHeight - margins) / sampleCount));
+        squishedHeight = sampleCount == 0 || showGenotypes == false ? DEFAULT_SQUISHED_HEIGHT :
+                Math.min(DEFAULT_SQUISHED_HEIGHT, Math.max(1, (height - getVariantBandHeight() - margins) / sampleCount));
+        if (sampleCount == 1) {
+            showGenotypes = false;
+        }
 
         // If sample->bam list file is supplied enable vcfToBamMode.
         String vcfToBamMapping = locator == null ? null : locator.getMappingPath();
@@ -250,7 +254,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
             if (defVisibilityWindow > 0) {
                 setVisibilityWindow(defVisibilityWindow * 1000);
             } else {
-                int vw = Math.max(10000, 2000000 - 2000 * allSamples.size());
+                int vw = Math.max(10000, (100000 - 100 * (allSamples.size() - 1)));
                 setVisibilityWindow(vw);
             }
         }
@@ -269,7 +273,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
             while ((nextLine = br.readLine()) != null) {
                 String[] tokens = ParsingUtils.TAB_PATTERN.split(nextLine);
                 if (tokens.length < 2) {
-                    log.info("Skipping bam mapping file line: " + nextLine);
+                    log.warn("Skipping bam mapping file line: " + nextLine);
                 } else {
 
                     String alignmentPath = tokens[1];
@@ -391,13 +395,15 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
      */
     public int getHeight() {
         int sampleCount = allSamples.size();
-        if (getDisplayMode() == DisplayMode.COLLAPSED || sampleCount == 0) {
-            return getVariantsHeight();
+        int h;
+        if (getDisplayMode() == DisplayMode.COLLAPSED || sampleCount == 0 || showGenotypes == false) {
+            h = getVariantsHeight();
         } else {
             final int groupCount = samplesByGroups.size();
             int margins = groupCount * 3;
-            return getVariantsHeight() + margins + (sampleCount * getGenotypeBandHeight());
+            h = getVariantsHeight() + margins + (sampleCount * getGenotypeBandHeight());
         }
+        return Math.max(WG_TRACK_HEIGHT, h);
     }
 
     public Object getHeader() {
@@ -413,7 +419,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
      * @return
      */
     private int getVariantsHeight() {
-        return variantBandHeight * Math.max(1, getNumberOfFeatureLevels());
+        return getVariantBandHeight() * getNumberOfFeatureLevels();
     }
 
     /**
@@ -433,8 +439,8 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
         // If height is < expanded height try "squishing" track, otherwise expand it
         final int groupCount = samplesByGroups.size();
         final int margins = (groupCount - 1) * 3;
-        int sampleCount = allSamples.size();
-        final int expandedHeight = variantBandHeight + margins + (sampleCount * getGenotypeBandHeight());
+        int sampleCount = showGenotypes == false ? 0 : allSamples.size();
+        final int expandedHeight = getVariantBandHeight() + margins + (sampleCount * getGenotypeBandHeight());
         if (height < expandedHeight) {
             setDisplayMode(DisplayMode.SQUISHED);
         } else {
@@ -443,7 +449,8 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
             }
         }
 
-        squishedHeight = Math.min(DEFAULT_SQUISHED_GENOTYPE_HEIGHT, Math.max(1, (height - variantBandHeight - margins) / sampleCount));
+        squishedHeight = showGenotypes == false ? DEFAULT_SQUISHED_HEIGHT :
+                Math.min(DEFAULT_SQUISHED_HEIGHT, Math.max(1, (height - getVariantBandHeight() - margins) / sampleCount));
     }
 
 
@@ -485,7 +492,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
 
             final double pXMin = tmpRect.getMinX();
             final double pXMax = tmpRect.getMaxX();
-            tmpRect.height = variantBandHeight;
+            tmpRect.height = getVariantBandHeight();
 
             int lastEndX = -1;
             int minSpacing = 3;
@@ -531,21 +538,23 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
                         lastEndX = x + w - 1;
                     }
 
-                    renderSamples(g2D, visibleRectangle, variant, context, overallSampleRect, x, w);
-                    boolean isSelected = selectedVariant != null && selectedVariant == variant;
-                    if (isSelected) {
-                        Graphics2D selectionGraphics = context.getGraphic2DForColor(Color.black);
-                        selectionGraphics.drawRect(x, curRowTop, w, getHeight());
+                    if (showGenotypes) {
+                        renderSamples(g2D, visibleRectangle, variant, context, overallSampleRect, x, w);
+                        boolean isSelected = selectedVariant != null && selectedVariant == variant;
+                        if (isSelected) {
+                            Graphics2D selectionGraphics = context.getGraphic2DForColor(Color.black);
+                            selectionGraphics.drawRect(x, curRowTop, w, getHeight());
+                        }
                     }
 
                 }
-                if (areFeaturesStacked()) {
-                    curRowTop += variantBandHeight;
-                    lastEndX = -1;
-                }
+
+                curRowTop += getVariantBandHeight();
+                lastEndX = -1;
+
             }
         } else {
-            tmpRect.height = variantBandHeight;
+            tmpRect.height = getVariantBandHeight();
             tmpRect.y = trackRectangle.y;
             g2D.setColor(Color.gray);
             GraphicUtils.drawCenteredText("No Variants Found", trackRectangle, g2D);
@@ -563,8 +572,8 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
     }
 
     private void drawVariantBandBorder(Graphics2D g2D, Rectangle visibleRectangle, int variantBandY, int left, int right) {
-        if (allSamples.size() > 0) {
-            drawLineIfVisible(g2D, visibleRectangle, Color.black, variantBandY, left, right);
+        if (allSamples.size() > 0 && showGenotypes) {
+            drawLineIfVisible(g2D, visibleRectangle, Color.lightGray, variantBandY, left, right);
         }
     }
 
@@ -576,7 +585,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
             for (Map.Entry<String, List<String>> entry : samplesByGroups.entrySet()) {
                 for (String sample : entry.getValue()) {
                     if (overallSampleRect.intersects(visibleRectangle)) {
-                        renderer.renderGenotypeBandSNP(variant, context, tmpRect, x, w, sample, coloring, hideFiltered);
+                        renderer.renderGenotypeBandSNP(variant, context, tmpRect, x, w, sample, genotypeColorMode, hideFiltered);
                     }
                     tmpRect.y += tmpRect.height;
                 }
@@ -587,7 +596,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
             for (String sample : allSamples) {
 
                 if (tmpRect.intersects(visibleRectangle)) {
-                    renderer.renderGenotypeBandSNP(variant, context, tmpRect, x, w, sample, coloring, hideFiltered);
+                    renderer.renderGenotypeBandSNP(variant, context, tmpRect, x, w, sample, genotypeColorMode, hideFiltered);
                 }
                 tmpRect.y += tmpRect.height;
             }
@@ -607,22 +616,24 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
         final int right = (int) trackRectangle.getMaxX();
 
         //Top line
-        drawLineIfVisible(g2D, visibleRectangle, Color.black, top + 1, left, right);
+        // drawLineIfVisible(g2D, visibleRectangle, Color.black, top + 1, left, right);
 
         // Bottom border
         int bottomY = trackRectangle.y + trackRectangle.height;
         drawLineIfVisible(g2D, visibleRectangle, borderGray, bottomY, left, right);
 
         // Variant / Genotype border
-        int variantGenotypeBorderY = trackRectangle.y + getVariantsHeight();
-        drawVariantBandBorder(g2D, visibleRectangle, variantGenotypeBorderY, left, right);
+        if (allSamples.size() > 0 && showGenotypes) {
+            int variantGenotypeBorderY = trackRectangle.y + getVariantsHeight();
+            drawVariantBandBorder(g2D, visibleRectangle, variantGenotypeBorderY, left, right);
 
-        if (grouped) {
-            g2D.setColor(Color.black);
-            int y = trackRectangle.y + getVariantsHeight();
-            for (Map.Entry<String, List<String>> entry : samplesByGroups.entrySet()) {
-                y += entry.getValue().size() * getGenotypeBandHeight() + GROUP_BORDER_WIDTH;
-                g2D.drawLine(trackRectangle.x, y, trackRectangle.x + trackRectangle.width, y);
+            if (grouped) {
+                g2D.setColor(Color.black);
+                int y = trackRectangle.y + getVariantsHeight();
+                for (Map.Entry<String, List<String>> entry : samplesByGroups.entrySet()) {
+                    y += entry.getValue().size() * getGenotypeBandHeight() + GROUP_BORDER_WIDTH;
+                    g2D.drawLine(trackRectangle.x, y, trackRectangle.x + trackRectangle.width, y);
+                }
             }
         }
     }
@@ -630,7 +641,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
     /**
      * Render the name panel.
      * <p/>
-     * NOTE:  The sample names are actually drawn in the drawBackground method!
+     * NOTE:  The sample names are actually drawn in the drawBackground method.
      *
      * @param g2D
      * @param trackRectangle
@@ -649,16 +660,17 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
         g2D.setColor(Color.black);
         rect.height = getVariantsHeight();
         if (rect.intersects(visibleRectangle)) {
-            GraphicUtils.drawWrappedText(getName(), rect, g2D, false);
+            Rectangle intersectedRect = rect.intersection(visibleRectangle);
+            GraphicUtils.drawWrappedText(getName(), intersectedRect, g2D, false);
         }
 
         rect.y += rect.height;
         rect.height = getGenotypeBandHeight();
-        if (areFeaturesStacked()) {
-            // The sample bounds list will get reset when  the names are drawn.
-            sampleBounds.clear();
-            drawBackground(g2D, rect, visibleRectangle, BackgroundType.NAME);
-        }
+
+        // The sample bounds list will get reset when  the names are drawn.
+        sampleBounds.clear();
+        drawBackground(g2D, rect, visibleRectangle, BackgroundType.NAME);
+
 
         renderBoundaryLines(g2D, trackRectangle, visibleRectangle);
 
@@ -677,6 +689,8 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
     public void renderAttributes(Graphics2D g2D, Rectangle trackRectangle, Rectangle visibleRectangle,
                                  List<String> attributeNames, List<MouseableRegion> mouseRegions) {
 
+        if (showGenotypes == false) return;
+
         top = trackRectangle.y;
         Rectangle rect = new Rectangle(trackRectangle);
 
@@ -765,7 +779,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
                                 BackgroundType type) {
 
 
-        if (getDisplayMode() == DisplayMode.COLLAPSED) {
+        if (getDisplayMode() == DisplayMode.COLLAPSED || showGenotypes == false) {
             return;
         }
 
@@ -858,12 +872,12 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
         this.hideFiltered = value;
     }
 
-    public ColorMode getColorMode() {
-        return coloring;
+    public ColorMode getGenotypeColorMode() {
+        return genotypeColorMode;
     }
 
-    public void setColorMode(ColorMode mode) {
-        this.coloring = mode;
+    public void setGenotypeColorMode(ColorMode mode) {
+        this.genotypeColorMode = mode;
     }
 
     public ColorMode getSiteColorMode() {
@@ -874,8 +888,16 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
         this.siteColorMode = siteColorMode;
     }
 
+    @Override
+    public void setColor(Color color) {
+        // Setting color implicitly turns of "color by" modes
+        this.genotypeColorMode = ColorMode.NONE;
+        this.siteColorMode = ColorMode.NONE;
+        super.setColor(color);
+    }
+
     public String getNameValueString(int y) {
-        if (y < top + variantBandHeight) {
+        if (y < top + getVariantBandHeight()) {
             return getName();
         } else {
             String sample = getSampleAtPosition(y);
@@ -896,7 +918,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
         try {
             double maxDistance = 10 * frame.getScale();
             if (mouseY < top + getVariantsHeight()) {
-                int modY = areFeaturesStacked() ? mouseY : -1;
+                int modY = mouseY;
                 Variant variant = getFeatureClosest(position, modY, frame.getName(), maxDistance);
                 if (variant == null) return null;
 
@@ -968,18 +990,18 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
      */
     protected Variant getFeatureClosest(double position, int y, String frameName, double maxDistance) {
 
-        PackedFeatures<IGVFeature> packedFeatures = packedFeaturesMap.get(frameName);
+        PackedFeatures<Feature> packedFeatures = packedFeaturesMap.get(frameName);
 
         if (packedFeatures == null) {
             return null;
         }
 
         Feature feature = null;
-        List<IGVFeature> features;
+        List<Feature> features;
 
         //We search only the specified row if y is a meaningful value.
         //Otherwise we search everything
-        int row = ((y - top) / variantBandHeight);
+        int row = ((y - top) / getVariantBandHeight());
         if (y < 0 || row >= getNumberOfFeatureLevels()) {
             features = packedFeatures.getFeatures();
         } else {
@@ -1014,7 +1036,9 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
             toolTip.append("<br>Alternate: " + alternateString);
         }
 
-        toolTip.append("<br>Qual: " + numFormat.format(variant.getPhredScaledQual()));
+        double qual = variant.getPhredScaledQual();
+        String qualString = variant.hasLog10PError() ? numFormat.format(qual) : ".";
+        toolTip.append("<br>Qual: " + qualString);
         toolTip.append("<br>Type: " + variant.getType());
         if (variant.isFiltered()) {
             toolTip.append("<br>Is Filtered Out: Yes</b>");
@@ -1137,7 +1161,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
 
     @Override
     public void receiveEvent(Object event) {
-        if(event instanceof  TrackGroupEvent) {
+        if (event instanceof TrackGroupEvent) {
             setupGroupsFromAttributes();
         }
     }
@@ -1150,8 +1174,23 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
         return selectedSamples;
     }
 
-    public static enum ColorMode {
-        GENOTYPE, METHYLATION_RATE, ALLELE_FREQUENCY, ALLELE_FRACTION
+    public boolean isShowGenotypes() {
+        return showGenotypes;
+    }
+
+    public void setShowGenotypes(boolean showGenotypes) {
+        this.showGenotypes = showGenotypes;
+    }
+
+    /**
+     * The height of the top band representing the variant call
+     */
+    public int getVariantBandHeight() {
+        return getDisplayMode() == DisplayMode.SQUISHED ? DEFAULT_SQUISHED_VARIANT_HEIGHT : DEFAULT_EXPANDED_VARIANT_HEIGHT;
+    }
+
+    public enum ColorMode {
+        GENOTYPE, METHYLATION_RATE, ALLELE_FREQUENCY, NONE, ALLELE_FRACTION
     }
 
     public static enum BackgroundType {
@@ -1253,9 +1292,9 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
     public IGVPopupMenu getPopupMenu(final TrackClickEvent te) {
         selectedVariant = getSelectedVariant(te);
         if (selectedVariant != null) {
-            IGV.getInstance().doRefresh();
+            repaint();
         }
-        return new VariantMenu(this, selectedVariant);
+        return new VariantMenu(this, selectedVariant, te);
     }
 
 
@@ -1295,15 +1334,15 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
             if (sampleAtPosition != null) {
                 if (selectedSamples.size() == 1 && selectedSamples.contains(sampleAtPosition)) {
                     selectedSamples.clear();
-                    IGV.getInstance().repaint();
-                    return;
+                    repaint();
+                    return;     // <-  TODO fix, bad practice
                 } else {
                     selectedSamples.clear();
                 }
                 selectedSamples.add(sampleAtPosition);
             }
         }
-        IGV.getInstance().repaint();
+        repaint();
 
     }
 
@@ -1368,7 +1407,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
                     }
                 }
             }
-            IGV.getInstance().doRefresh();
+            repaint();
         }
 
         if (IGV.getInstance().isShowDetailsOnClick()) {
@@ -1414,7 +1453,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
 
                 }
                 ResourceLocator loc = new ResourceLocator(bamList);
-                loc.setType("alist");
+                loc.setFormat("alist");
                 loc.setName(name);
                 List<Track> tracks = null;
                 try {
@@ -1441,7 +1480,7 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
      * @param center
      * @param forward
      * @return
-     * @throws java.io.IOException
+     * @throws IOException
      */
     @Override
     public Feature nextFeature(String chr, double center, boolean forward, ReferenceFrame frame) throws IOException {
@@ -1478,21 +1517,65 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
         }
     }
 
+    void sendToCircularView(TrackClickEvent e) {
+
+        List<Feature> visibleFeatures;
+        if (e.getFrame() == null) {
+            visibleFeatures = new ArrayList<>();
+            for (ReferenceFrame frame : FrameManager.getFrames()) {
+                visibleFeatures.addAll(getVisibleFeatures(frame));
+            }
+        } else {
+            visibleFeatures = getVisibleFeatures(e.getFrame());
+        }
+
+        List<Feature> svFeatures = visibleFeatures.stream().filter(f -> {
+            Variant v = f instanceof MateVariant ? ((MateVariant) f).mate : (Variant) f;
+            Map<String, Object> attrs = v.getAttributes();
+            return attrs.containsKey("CHR2") && attrs.containsKey("END");
+        }).collect(Collectors.toList());
+
+        if (svFeatures.isEmpty()) {
+            MessageUtils.showMessage("No structural variants found.");
+        } else {
+            CircularViewUtilities.sendVariantsToJBrowse(svFeatures, getName(), CIRC_VIEW_DEFAULT_COLOR);
+        }
+    }
+
+    @Override
+    public List<Feature> getVisibleFeatures(ReferenceFrame frame) {
+        if (frame.getChrName().equals(Globals.CHR_ALL) &&
+                this.source instanceof TribbleFeatureSource.NonIndexedFeatureSource) {
+            try {
+                return ((TribbleFeatureSource.NonIndexedFeatureSource) this.source).getAllFeatures();
+            } catch (IOException e) {
+                return Collections.emptyList();
+            }
+        } else {
+            return super.getVisibleFeatures(frame);
+        }
+    }
+
 
     @Override
     public void marshalXML(Document document, Element element) {
 
         super.marshalXML(document, element);
 
-        if(this.squishedHeight != DEFAULT_SQUISHED_GENOTYPE_HEIGHT) {
+        boolean defaultShowGenotypes = allSamples.size() == 1 ? false : true;
+        if (showGenotypes != defaultShowGenotypes) {
+            element.setAttribute("showGenotypes", String.valueOf(showGenotypes));
+        }
+
+        if (this.squishedHeight != DEFAULT_SQUISHED_HEIGHT) {
             element.setAttribute("squishedHeight", String.valueOf(squishedHeight));
         }
 
-        if(coloring != ColorMode.GENOTYPE) {
-            element.setAttribute("coloring", coloring.toString());
+        if (genotypeColorMode != ColorMode.GENOTYPE) {
+            element.setAttribute("genotypeColorMode", genotypeColorMode.toString());
         }
 
-        if(siteColorMode != null) {
+        if (siteColorMode != null) {
             element.setAttribute("siteColorMode", siteColorMode.toString());
         }
 
@@ -1503,15 +1586,22 @@ public class VariantTrack extends FeatureTrack implements IGVEventObserver {
 
         super.unmarshalXML(element, version);
 
-        if(element.hasAttribute("squishedHeight")) {
+        if (element.hasAttribute("showGenotypes")) {
+            this.showGenotypes = Boolean.getBoolean(element.getAttribute("showGenotypes"));
+        }
+
+        if (element.hasAttribute("squishedHeight")) {
             this.squishedHeight = Integer.parseInt(element.getAttribute("squishedHeight"));
         }
 
-        if(element.hasAttribute("coloring")) {
-            this.coloring = ColorMode.valueOf(element.getAttribute("coloring"));
+        if (element.hasAttribute("genotypeColorMode")) {
+            this.genotypeColorMode = ColorMode.valueOf(element.getAttribute("genotypeColorMode"));
+        } else if (element.hasAttribute("coloring")) {
+            // backward compatibility
+            this.genotypeColorMode = ColorMode.valueOf(element.getAttribute("coloring"));
         }
 
-        if(element.hasAttribute("siteColorMode")) {
+        if (element.hasAttribute("siteColorMode")) {
             this.siteColorMode = ColorMode.valueOf(element.getAttribute("siteColorMode"));
 
         }
