@@ -25,7 +25,6 @@
 
 package org.broad.igv.batch;
 
-import org.apache.commons.lang.StringUtils;
 import org.broad.igv.Globals;
 import org.broad.igv.prefs.Constants;
 import org.broad.igv.feature.RegionOfInterest;
@@ -62,7 +61,7 @@ import static org.junit.Assert.assertTrue;
  */
 public class CommandExecutorTest extends AbstractHeadedTest {
 
-    CommandExecutor exec = new CommandExecutor();
+    CommandExecutor exec = new CommandExecutor(IGV.getInstance());
     private final String snapshotDir = TestUtils.TMP_OUTPUT_DIR;
 
     @Rule
@@ -395,11 +394,13 @@ public class CommandExecutorTest extends AbstractHeadedTest {
         String coverage = null;
         String locus = null;
         boolean merge = true;
-        exec.loadFiles(urlPath, index, coverage, name, format, locus, merge, params);
+        String sort = null;
+        String sortTag = null;
+        exec.loadFiles(urlPath, index, coverage, name, format, locus, merge, params, sort, sortTag);
 
         name = null;
         String localPath = TestUtils.DATA_DIR + "bed/test.bed";
-        exec.loadFiles(localPath, index, coverage, name, format, locus, merge, params);
+        exec.loadFiles(localPath, index, coverage, name, format, locus, merge, params, sort, sortTag);
 
         assertEquals(2, igv.getAllTracks().size() - beginTracks);
     }
@@ -414,7 +415,9 @@ public class CommandExecutorTest extends AbstractHeadedTest {
         String coverage = null;
         String locus = null;
         boolean merge = true;
-        exec.loadFiles(dataFile, index, coverage, name, format, locus, merge, params);
+        String sort = null;
+        String sortTag = null;
+        exec.loadFiles(dataFile, index, coverage, name, format, locus, merge, params, sort, sortTag);
 
         String[] goodArgSet = new String[]{"0,5.0 ", "0,1,5", "-1,0,1", "-1.32,10.21"};
         for (String arg : goodArgSet) {
@@ -441,7 +444,9 @@ public class CommandExecutorTest extends AbstractHeadedTest {
         String coverage = null;
         String locus = null;
         boolean merge = true;
-        exec.loadFiles(dataFile, index, coverage, name, format, locus, merge, params);
+        String sort = null;
+        String sortTag = null;
+        exec.loadFiles(dataFile, index, coverage, name, format, locus, merge, params, sort, sortTag);
 
         String[] goodArgSet = new String[]{"true"};
         for (String arg : goodArgSet) {
@@ -514,7 +519,9 @@ public class CommandExecutorTest extends AbstractHeadedTest {
         String locus = "chr1:155,156,300-155,164,706";  //muc1
         boolean merge = true;
         Map<String, String> params = null;
-        exec.loadFiles(urlPath, index, coverage, name, format, locus, merge, params);
+        String sort = null;
+        String sortTag = null;
+        exec.loadFiles(urlPath, index, coverage, name, format, locus, merge, params, sort, sortTag);
         List<Track> tracks = igv.getAllTracks();
         // Find our alignment track
         boolean foundTrack = false;
@@ -561,8 +568,6 @@ public class CommandExecutorTest extends AbstractHeadedTest {
         File outputFile = new File(snapshotDir, outFileName);
         BufferedImage image = ImageIO.read(outputFile);
 
-        assertTrue("Output image height " + image.getHeight() + " is not at least " + minHeight, image.getHeight() > minHeight);
-
         int remAlphaMask = 0x00ffffff;
 
         int numBlackPix = 0;
